PROGRAM APEX4                                                        ! Start of main program
   ! This is the Fortran 90 source code for version 4.5 of the APEX model. APEX4 was developed for 
   ! EPA in October 2006, following from APEX, APEX2, and APEX3 - released in 2000, 2001, and 2002 
   ! respectively.  This program was developed and written by Alion Science and Technology, Inc.
   ! (formerly ManTech Environmental Technology, Inc.), under contract to the U.S. Environmental 
   ! Protection Agency.  Latest revisions to this code made by WGG on August 16, 2012.
   !
   ! The APEX source code consists of this file plus ten modules named below in USE statements.  
   ! Compile GlobalModule first, then UtilityModule, then DistributionModule, then PMModule.  
   ! Compile other modules in any order, except this file (APEX4.F90) must be compiled last of all.
   ! 
   USE GlobalModule                                                  ! In file GlobalModule.F90
   USE UtilityModule                                                 ! In file UtilityModule.F90
   USE DistributionModule                                            ! In file DistributionModule.F90 
   USE PMModule                                                      ! In file PMModule.F90
   USE ProfileModule                                                 ! In file ProfileModule.F90
   USE SpaceTimeModule                                               ! In file SpaceTimeModule.F90
   USE DiaryModule                                                   ! In file DiaryModule.F90
   USE MicroEnvModule                                                ! In file MicroEnvModule.F90
   USE ExposureDoseModule                                            ! In file ExposureDoseModule.F90
   USE OutputModule                                                  ! In file OutputModule.F90
   IMPLICIT NONE                                                     ! Must declare all variables 
   INTEGER (KIND=IK) :: P                                            ! Index for profile #              
   INTEGER (KIND=IK) :: PL                                           ! Index for pollutant #              
   ! Global constants: IK 
   ! Global vars used: FirstProfile, NumProfiles, NumPollutants
   ! APEX procedures : CalcDates, CompositeDiary, DiaryProbabilities, ExposureDose, FinalSummary
   !                   GenerateDailyVars, GenerateProfiles, GeneratePhysiology, MatchSites, MicroConcs, 
   !                   Output, OutputParameters, ReadAirQuality, ReadDiaries, ReadProfileFunctions, 
   !                   ReadEmployment, ReadMicroData, ReadParameters, ReadPopulation, ReadSiteLists,
   !                   ReadMeteorologyData, SetCVLevels, SetCVList, SetSeeds, SetupTables, StudyAreaPop,
   !                   Ventilation, WriteJobTime  
   CALL ReadParameters                                               ! In main APEX module
   CALL OutputParameters                                             ! In main APEX module
   CALL CalcDates                                                    ! In SpaceTimeModule
   CALL ReadSiteLists                                                ! In SpaceTimeModule
   CALL MatchSites                                                   ! In SpaceTimeModule
   CALL SetCVList                                                    ! In DistributionModule   
   CALL ReadProfileFunctions                                         ! In DistributionModule
   CALL SetCVLevels                                                  ! In DistributionModule
   CALL ReadMeteorologyData                                          ! In SpaceTimeModule
   CALL ReadAirQuality                                               ! In SpaceTimeModule
   CALL ReadPopulation                                               ! In ProfileModule
   CALL ReadEmployment                                               ! In ProfileModule
   CALL ReadDiseaseRate                                              ! In ProfileModule
   IF (UseProfileFactors == YES) CALL ReadProfileFactors             ! In ProfileModule
   CALL StudyAreaPop                                                 ! In ProfileModule
   CALL GenerateProfiles                                             ! In ProfileModule
   CALL GeneratePhysiology                                           ! In ProfileModule
   CALL ReadDiaries                                                  ! In DiaryModule
   CALL ReadMicroData                                                ! In MicroEnvModule
   CALL SetupOutput                                                  ! In OutputModule
   CALL WriteJobTime(0)                                              ! In UtilityModule 
   DO P = FirstProfile, NumProfiles                                  ! Loop over profile
      CALL SetSeeds(P)                                               ! In UtilityModule  
      CALL GenerateDailyVars(P)                                      ! In ProfileModule         
      CALL DiaryProbabilities(P)                                     ! In DiaryModule         
      CALL CompositeDiary(P)                                         ! In DiaryModule        
      CALL Ventilation(P)                                            ! In ExposureDoseModule      
      DO PL = 1, NumPollutants                                       ! Loop over pollutants           
         CALL GenerateAirQuality(P,PL)                               ! In SpaceTimeModule         
         CALL MicroConcs(P,PL)                                       ! In MicroEnvModule            
         CALL Exposure(P,PL)                                         ! In ExposureDoseModule          
         CALL Dose(P,PL)                                             ! In ExposureDoseModule           
         CALL Output(P,PL)                                           ! In OutputModule          
      ENDDO                                                          ! Finished pollutant
   ENDDO                                                             ! Finished profile
   DO PL = 1, NumPollutants                                          ! Loop over pollutants   
     CALL WriteTablesFile(PL)                                        ! Write tables files
   ENDDO                                                             ! End loop over pollutants
   CALL FinalSummary                                                 ! In OutputModule
   STOP "Finished APEX model run"                                    ! Succesful job termination
CONTAINS                                                             ! Routines internal to this module

SUBROUTINE ReadParameters                                            ! Called from main APEX program
   IMPLICIT NONE                                                     ! Must declare all variables 
   CHARACTER(LEN=LEN4) :: ParamLine, VarName, Key, VC, UC, PName     ! Buffers for storing records
   CHARACTER(LEN=LEN1) :: Mic                                        ! Buffers for storing records
   INTEGER  (KIND=IK)  :: I=0, J, VI, NumLev, LType,K=0, H=0         ! Temporary integer variables
   INTEGER  (KIND=IK)  :: PL=0, NP, AE, FEV1Risk=NO                  ! Temporary integer variables
   INTEGER  (KIND=IK)  :: MFound=NO,FFound=NO,KeyFound=0,LineNum=0   ! Temporary integer variables
   REAL     (KIND=RK)  :: VR, Buf(0:100)                             ! Temporary real variables
   CHARACTER(LEN=1)    :: G, U1                                      ! Buffers for single character
   ! Global constants: AQU, ChEvU, ChMapU, ChQuU, CommU, DistU, EventU, HDoseU, HExpU, IK, LENTAB 
   !                   LEN3, LogU, MAXCOUNTY, MetsU, MicroU, MonU, MResU, MSumU, NO, ParFile, ParU, PhysU,
   !                   PopU, PSumU, RK, SiteU, TabU, TempU, UGM3, VentU, YES, ZoneU
   ! Global vars set : AirHeader,  AirRadius, AgeCutPct, AgeMin, AgeMax, Age2Probab, Altitude, Center, 
   !                   CityRadius, CList, Clock, ClockSeed, COHbFact, CommuteFlag, AlertThresh, 
   !                   CountyList, CustProfs, DailyOut, DailyList, DateTime, DAvgExpLev, DAvgDoseLev, DebugLevel, Disease, 
   !                   DM1HDoseLev, DM1HExpLev, DM8HDoseLev, DM8HExpLev, DMEHDoseLev, DoCustProfs, DoDose, DSTAdjust,
   !                   EndDate, FName, H_EHDoseLev,InUnits, Ios, ISdAct, ISdMts, ISdPrs, HourlyOut, HourFlags
   !                   ISeed, KeepLeavers, LeaverAdd, LeaverMult, Location, LogDistrict, LogPopulate,
   !                   LogProfiles, LogSectors, LogTables, LogZones, MissAge, MissEmpl, MissGender, MResOut, MSumOut,
   !                   NumCounties, NumCustProfs, NumMicros, NumPopTypes, NumPollutants, NumProfiles, NumSources, 
   !                   NumTracts, OutUnits, ParamsHeader, Pctl, Pollut, PopCat, PPMFact, ProcLevel, ProcName, 
   !                   PSumFlags, RBFactor, RBBkgnd, Rollback, SAvgDoseLev, SAvgExpLev, Scenario, StartDate, 
   !                   TimeExpLev, TimeDoseLev, TList, Tractlist, VAoutput, ZoneRadius
   ! Intrinsic procs : ADJUSTL, ALLOCATE, ANY, CLOSE, COUNT, DATE_AND_TIME, INDEX, INQUIRE, LEN_TRIM,
   !                   OPEN, PRINT, RANDOM_SEED, READ, REAL, REWIND, SYSTEM_CLOCK, TRIM, WRITE
   ! APEX procedures : CheckAllocation, CheckKey, CheckParameters, Fatal, GetLine, ManageFiles
   !                   OpenInput, OpenOutput,Up, Warning    
   ProcLevel = ProcLevel+1                                           ! Increment procedure depth 
   ProcName(ProcLevel) = "ReadParameters"                            ! Procedure name for messages
   CALL GETARG(1, ParFile)
   OPEN(ParU,FILE=ParFile,RECL=LEN3,IOSTAT=Ios(ParU),ACTION='READ')  ! Open PARAMS file 
   INQUIRE(ParU,Name=FName(ParU))                                    ! Obtain name of PARAMS file
   IF (Ios(ParU)>0) CALL Fatal(1)                                    ! Must be able to open PARAMS fl   
   READ(ParU,FMT3) ParamsHeader                                      ! Save first line from PARAMS
   CALL SYSTEM_CLOCK(Clock(1))                                       ! Start job timing 
   CALL DATE_AND_TIME(DateTime(1),DateTime(2),DateTime(3))           ! Set date and time variables
   DO                                                                ! Loop over PARAMS records
      CALL GetLine(ParU,ParamLine,LType)                             ! Get one record from input file 
      IF (Ios(ParU)<0) EXIT                                          ! Exit DO loop on end of file
      IF (LType<=0) CYCLE                                            ! If no equals sign, discard
      Key  = TRIM(Up(ADJUSTL(ParamLine(1:LType-1))))                 ! Part on left of "=" (uppercase)
      VC = TRIM(ADJUSTL(ParamLine(LType+1:)))                        ! Part on right of "="
      UC = TRIM(Up(VC))                                              ! Part on right of "=" (uppercase)
      U1 = UC(1:1)                                                   ! First letter of UC
      NumLev  = 2+COUNT((/(UC(I:I)==",",I=1,LEN_TRIM(UC))/))         ! = 2 + #commas in VC
      IF (INDEX(Key,"SCENARIO")>0) Scenario  = VC                    ! Set the scenario description
      IF (INDEX(Key,"LOCATION")>0) Location  = VC                    ! Set the study location
      IF (INDEX(Key,"#MICROS")>0) THEN                               ! If reading # micros
         READ(VC,*,IOSTAT=Ios(2)) VI                                 ! Convert string to integer 
         NumMicros = VI                                              ! Set number of micros
         ALLOCATE (HourFlags(19+2*(NumMicros+1)),STAT=AllocErr)      ! Space for hour file flags 
         CALL CheckAllocation(AllocErr)                              ! Check the success of the alloc
      ENDIF                                                          ! End #micros logic
      IF (INDEX(Key,"#POLL")>0) THEN                                 ! If reading # pollutants key    
         READ(VC,*,IOSTAT=Ios(2)) VI                                 ! Convert string to integer 
         NumPollutants = VI                                          ! Set number of pollutants
         ALLOCATE (Pol(NumPollutants),STAT=AllocErr)                 ! Space for pollutant information 
         CALL CheckAllocation(AllocErr)                              ! Check the success of the alloc
         ALLOCATE (AirHeader(NumPollutants),STAT=AllocErr)           ! Space for air quality headers
         CALL CheckAllocation(AllocErr)                              ! Check the success of the alloc
      ENDIF                                                          ! End of # pollutants logic    
      IF (INDEX(Key,"POLLUTANT")>0.AND.INDEX(Key,"#")==0 )   THEN    ! If pollutant keyword
          PolList = TRIM(ADJUSTL(PolList))//" "//ADJUSTL(TRIM(VC))   ! Update the pollutant list
          PL=PL+1                                                    ! Update pollutant #
          IF (PL > NumPollutants) CALL Fatal(2)                      ! Pollutant number out of range
      ENDIF                                                          ! End pollutant name logic
      IF (INDEX(Key,"TIMESTEPSPERDAY")>0) THEN                       ! If reading # ts per day
         READ(VC,*,IOSTAT=Ios(2)) VI                                 ! Convert string to integer 
         TimeStepsPerDay = VI                                        ! Set number ts/day
      ENDIF                                                          ! End timestep logic
      IF (INDEX(Key,"PERCENTILE")>0)POL(PL)%N_Pctl = NumLev-1        ! Found keyword for percentiles       
      IF (INDEX(Key,"DM1HEXP")>0)   POL(PL)%N_DM1HExpLev = NumLev    ! Found DM1HExp levels keyword
      IF (INDEX(Key,"DM8HEXP")>0)   POL(PL)%N_DM8HExpLev = NumLev    ! Found DM8HExp levels keyword 
      IF (INDEX(Key,"DAVGEXP")>0)   POL(PL)%N_DAvgExpLev = NumLev    ! Found DAvgExp levels keyword
      IF (INDEX(Key,"SAVGEXP")>0)   POL(PL)%N_SAvgExpLev = NumLev    ! Found SAvgExp levels keyword 
      IF (INDEX(Key,"TIMEEXP")>0)   POL(PL)%N_TimeExpLev = NumLev    ! Found TimeExp levels keyword   
      IF (INDEX(Key,"DM1HDOSE")>0)  POL(PL)%N_DM1HDoseLev = NumLev   ! Found DM1HDose levels keyword 
      IF (INDEX(Key,"DM8HDOSE")>0)  POL(PL)%N_DM8HDoseLev = NumLev   ! Found DM8HDose levels keyword 
      IF (INDEX(Key,"DAVGDOSE")>0)  POL(PL)%N_DAvgDoseLev = NumLev   ! Found DAvgDose levels keyword 
      IF (INDEX(Key,"SAVGDOSE")>0)  POL(PL)%N_SAvgDoseLev = NumLev   ! Found SAvgDose levels keyword 
      IF (INDEX(Key,"DMEHDOSE")>0)  POL(PL)%N_DMEHDoseLev = NumLev   ! Found DMEHDose levels keyword 
      IF (INDEX(Key,"TIMEDOSE")>0)  POL(PL)%N_TimeDoseLev = NumLev   ! Found TimeDose levels keyword  
      IF (INDEX(Key,"H_EHDOSE")>0)  POL(PL)%N_H_EHDoseLev = NumLev   ! Found H_EHDose levels keyword
      IF (INDEX(Key,"DMTSEXP")>0)   POL(PL)%N_DMTSExpLev  = NumLev   ! Found DMTSExp levels keyword
      IF (INDEX(Key,"DMTSDOSE")>0)  POL(PL)%N_DMTSDoseLev = NumLev   ! Found DMTSDose levels keyword
      IF (INDEX(Key,"TSMULTI")>0)   POL(PL)%N_TSMultiLev  = NumLev   ! Found Multi levels keyword
      IF (INDEX(Key,"TSEXP")>0 .AND.INDEX(Key,"DM")==0)             &
         POL(PL)%N_TSExpLev    = NumLev                              ! Found TSExp levels keyword
      IF (INDEX(Key,"TSDOSE")>0.AND.INDEX(Key,"DM")==0)             & 
         POL(PL)%N_TSDoseLev   = NumLev                              ! Found TSDose levels keyword
      IF (INDEX(Key,"FEV1RISK")>0.OR.INDEX(Key,"FEVRISK")>0)FEV1RISK=YES   ! Found FEV1Risk probabilities
      IF (INDEX(Key," FILE")>0) THEN                                 ! For file names...
         IF (INDEX(Key,"LOG")>0 .AND. INDEX(Key,"PHYS")==0)         &
            FName(LogU) = VC                                         ! Set log filename
         IF (INDEX(Key,"DISTRICT")>0) FName(MonU)  = VC              ! Set air districts filename
         IF (INDEX(Key,"POP")>0)      NumPopTypes   = NumPopTypes+1  ! Increment pop type counter
      ENDIF                                                          ! End of file processing 
   ENDDO                                                             ! Continue with next record  
   CLOSE(ParU)                                                       ! Close PARAMS file 
   IF(NumMicros==0)     CALL Fatal(3)                                ! Number of micros missing or 0           
   IF(.NOT. ALLOCATED(Pol)) CALL Fatal(4)                            ! Number of micros missing or 0           
   PSumFlags(:)    = 0                                               ! Initialize psum flags
   PSumFlags(1:12) = 1                                               ! Always print first 12 vars
   HourFlags(:)    = 0                                               ! Initialize hour flags
   HourFlags(1:2)  = 1                                               ! Always print first 2 vars
   NP=NumPollutants                                                  ! Short form of # pollutants
   ALLOCATE (Pctl(NP,MAXVAL(POL(:)%N_Pctl)), STAT=AE)                ! Make space for percentiles 
   CALL CheckAllocation(AE)                                          ! Check the success of the alloc     
   ALLOCATE (DM1HExpLev(NP,(MAXVAL(POL(:)%N_DM1HExpLev))+1),STAT=AE) ! Space for exposure levels
   CALL CheckAllocation(AE)                                          ! Check the success of the alloc
   ALLOCATE (DM8HExpLev(NP,(MAXVAL(POL(:)%N_DM8HExpLev))+1),STAT=AE) ! Space for exposure levels
   CALL CheckAllocation(AE)                                          ! Check the success of the alloc
   ALLOCATE (DAvgExpLev(NP,(MAXVAL(POL(:)%N_DAvgExpLev))+1),STAT=AE) ! Space for exposure levels
   CALL CheckAllocation(AE)                                          ! Check the success of the alloc
   ALLOCATE (SAvgExpLev(NP,(MAXVAL(POL(:)%N_SAvgExpLev))+1),STAT=AE) ! Space for exposure levels
   CALL CheckAllocation(AE)                                          ! Check the success of the alloc
   ALLOCATE (TimeExpLev(NP,(MAXVAL(POL(:)%N_TimeExpLev))+1),STAT=AE) ! Space for exposure levels
   CALL CheckAllocation(AE)                                          ! Check the success of the alloc
   ALLOCATE(DM1HDoseLev(NP,(MAXVAL(POL(:)%N_DM1HDoseLev))+1),STAT=AE)! Space for dose levels
   CALL CheckAllocation(AllocErr)                                    ! Check the success of the alloc
   ALLOCATE(DM8HDoseLev(NP,(MAXVAL(POL(:)%N_DM8HDoseLev))+1),STAT=AE)! Space for dose levels
   CALL CheckAllocation(AllocErr)                                    ! Check the success of the alloc
   IF (FEV1RISK==YES) THEN
      ALLOCATE(FEVProbList(MAXFEV,(MAXVAL(POL(:)%N_DM8HExpLev))+1),STAT=AE)
      CALL CheckAllocation(AllocErr)  
   ENDIF
   ALLOCATE(DAvgDoseLev(NP,(MAXVAL(POL(:)%N_DAvgDoseLev))+1),STAT=AE)! Space for dose levels
   CALL CheckAllocation(AllocErr)                                    ! Check the success of the alloc
   ALLOCATE(SAvgDoseLev(NP,(MAXVAL(POL(:)%N_SAvgDoseLev))+1),STAT=AE)! Space for dose levels
   CALL CheckAllocation(AllocErr)                                    ! Check the success of the alloc
   ALLOCATE(DMEHDoseLev(NP,(MAXVAL(POL(:)%N_DMEHDoseLev))+1),STAT=AE)! Space for dose levels
   CALL CheckAllocation(AllocErr)                                    ! Check the success of the alloc
   ALLOCATE(TimeDoseLev(NP,(MAXVAL(POL(:)%N_TimeDoseLev))+1),STAT=AE)! Space for dose levels
   CALL CheckAllocation(AllocErr)                                    ! Check the success of the alloc
   ALLOCATE(H_EHDoseLev(NP,(MAXVAL(POL(:)%N_H_EHDoseLev))+1),STAT=AE)! Space for dose levels
   CALL CheckAllocation(AE)                                          ! Check the success of the alloc
   ALLOCATE(TSExpLev(NP,(MAXVAL(POL(:)%N_TSExpLev))+1),STAT=AE)      ! Space for time step exp levels
   CALL CheckAllocation(AE)                                          ! Check the success of the alloc
   ALLOCATE(DMTSExpLev(NP,(MAXVAL(POL(:)%N_DMTSExpLev))+1),STAT=AE)  ! Space for time step exp levels
   CALL CheckAllocation(AE)                                          ! Check the success of the alloc
   ALLOCATE(TSDoseLev(NP,(MAXVAL(POL(:)%N_TSDoseLev))+1),STAT=AE)    ! Space for time step dose levels
   CALL CheckAllocation(AE)                                          ! Check the success of the alloc
   ALLOCATE(DMTSDoseLev(NP,(MAXVAL(POL(:)%N_DMTSDoseLev))+1),STAT=AE)! Space for time step dose levels
   CALL CheckAllocation(AE)                                          ! Check the success of the alloc
   ALLOCATE(TSMultiLev(NP,(MAXVAL(POL(:)%N_TSMultiLev))+1),STAT=AE)  ! Space for multi exceed levels
   CALL CheckAllocation(AE)                                          ! Check the success of the alloc
   ALLOCATE (PopCat(NumPopTypes),STAT=AE)                            ! Space for population categories
   CALL CheckAllocation(AE)                                          ! Check the success of the alloc
   OPEN(ParU,FILE=FName(ParU),RECL=LEN3,                            &
      IOSTAT=Ios(ParU),ACTION='READ')                                ! Open the PARAMS file at start   
   PL = 0                                                            ! Reset number of pollutants
   DO                                                                ! Loop over parameter records
      CALL GetLine(ParU,ParamLine,LType)                             ! Get one record from input file
      LineNum = LineNum+1                                            ! Increment params file line num    
      IF (Ios(ParU)<0) EXIT                                          ! Exit DO loop on end of file
      IF (LType<=0) CYCLE                                            ! Discard non-keyword lines
      VarName = TRIM((ADJUSTL(ParamLine(1:LType-1))))                ! Part on left of "="
      Key  = TRIM(Up(VarName))                                       ! Part on left of "=" (uppercase)
      VC = TRIM(ADJUSTL(ParamLine(LType+1:)))                        ! Part on right of "="
      UC = TRIM(Up(VC))                                              ! Part on right of "=" (uppercase)
      U1 = UC(1:1)                                                   ! First letter of UC
      NumLev  = 2+COUNT((/(UC(I:I)==",",I=1,LEN_TRIM(UC))/))         ! = 2 + #commas in VC
      KeyFound = CheckKey(Key)                                       ! Test for recognizable key
      IF (KeyFound==0) THEN                                          ! If not a recognized keyword
         PRINT*, TRIM(ADJUSTL(ParamLine))
         CALL Warning(LineNum)                                       ! Write warning     
         CYCLE                                                       ! Discard and go to next line
      ENDIF                                                          ! End unrecognized key logic   
      IF (INDEX(Key,"FILE")==0 .AND. (INDEX(Key,"EXP")>0 .OR.       &     
         INDEX(Key,"DOSE")>0 .OR. INDEX(Key,"PERC")>0) .OR.         & 
         INDEX(Key,"CUSTOMSA")>0.OR.INDEX(Key,"MRESMICROS")>0       &
         .OR. INDEX(Key,"TSMULT")>0) THEN                            ! If variable with a list     
         Buf(0)  = REAL(NumLev)                                      ! Store # of levels in buffer
         Buf(1)  = 0.                                                ! Lowest level is zero by default
         DO I=1, NumLev-1                                            ! Loop over levels
            READ(UC,*,IOSTAT=Ios(1)) Buf(I+1)                        ! Read first item in list
            UC = UC(INDEX(UC,",")+1:)                                ! Remove first item from list
            IF (I>1.AND.Buf(I+1)<=Buf(I)) CALL Fatal(5)              ! Levels must be ascending
         ENDDO                                                       ! Continue with next level
      ENDIF                                                          ! End of Table levels logic block
      IF (INDEX(Key," FILE")>0) THEN                                 ! For file names...
         IF (INDEX(Key,"TABLE")>0)    FName(TabU)  =VC               ! Set output tables filename
         IF (INDEX(Key,"HOUR")>0)     FName(HourU) =VC               ! Set hourly filename
         IF (INDEX(Key,"DAILY")>0)    FName(DayU)  =VC               ! Set daily filename
         IF (INDEX(Key,"TIMESTEP")>0) FName(TSU)   =VC               ! Set timestep filename
         IF (INDEX(Key,"PERSON")>0)   FName(PSumU) =VC               ! Set personal data filename
         IF (INDEX(Key,"SITE")>0)     FName(SiteU) =VC               ! Set site list filename
         IF (INDEX(Key,"EVENT")>0 .AND. INDEX(Key,"DIARY")==0)      &
            FName(EventU)=VC                                         ! Set events output filename
         IF (INDEX(Key,"PREV")>0)     FName(PrevU)=VC                ! Set prevalence filename 
         IF (INDEX(Key,"MICROSUM")>0) FName(MSumU) =VC               ! Set micro. summary filename
         IF (INDEX(Key,"MICRORES")>0) FName(MResU) =VC               ! Set micro. results filename
         IF (INDEX(Key,"MICROENV")>0) FName(MicroU)=VC               ! Set microenvironment filename
         IF (INDEX(Key,"DIARYQUEST")>0) FName(ChQuU) =VC             ! Set diary quest. filename
         IF (INDEX(Key,"DIARYEVE")>0) FName(ChEvU) =VC               ! Set diary event filename
         IF (INDEX(Key,"DIARYSTA")>0) FName(StatU) =VC               ! Set diary statistics filename
         IF (INDEX(Key,"DIARYOCC")>0) THEN                           ! If an employment file is present
            FName(DEmpU) =VC                                         ! Set diary employment filename
            UseCHADOccFile = YES                                     ! Setting empl file flag
         ENDIF                                                       ! End empl file logic
         IF (INDEX(Key,"MEMAP")>0)    FName(ChMapU)=VC               ! Set location mapping filename 
		 IF (INDEX(Key,"DIARYCLU")>0) FName(ClMapU)=VC               ! Set cluster location mapping filename         
         IF (INDEX(Key,"FUNCTIONS")>0) FName(DistU) =VC              ! Set distribution filename
         IF (INDEX(Key,"PHYSIOL")>0)  FName(PhysU) =VC               ! Set physiology filename
         IF (INDEX(Key,"METSMAP")>0)  FName(MMapU) =VC               ! Set metabolic filename
         IF (INDEX(Key,"DISTRIB")>0)  FName(MetsU) =VC               ! Set metabolic filename
         IF (INDEX(Key,"COMMUT")>0)   FName(CommU) =VC               ! Set commuting filename
         IF (INDEX(Key,"COMMTIME")>0)  FName(CommTU) =VC             ! Set commuting time filename         
         IF (INDEX(Key,"METEOR")>0)   FName(TempU) =VC               ! Set temperature filename
         IF (INDEX(Key,"SECTOR")>0)   FName(SectU) =VC               ! Set pop. sectors filename
         IF (INDEX(Key,"DISTRICTS")>0) FName(MonU)  =VC              ! Set air districts filename
         IF (INDEX(Key,"ZONE")>0 .AND. INDEX(Key,"QUALITY")==0)     &
            FName(ZoneU) =VC                                         ! Set temperature zones filename
         IF (INDEX(Key,"EMPLOY")>0)   FName(EmpU)  =VC               ! Set employment ranges filename
         IF (INDEX(Key,"PROFIL")>0)   THEN                           ! If Profile Factors file is specified
            FName(ProU)  =VC                                         ! Set profile ranges filename         
            UseProfileFactors = YES                                  ! APEX will include profile factor code
         ENDIF
         IF (INDEX(Key,"VENTIL")>0)   FName(VentU) =VC               ! Set ventilation rates filename                                                         
         IF (INDEX(Key,"QUALITY")>0)  THEN                           ! There may be many AQ files 
            I = INDEX(VarName,",")                                   ! Location of comma 
            IF (I==0) CALL Fatal(6)                                  ! Stop if comma missing
            H = H+1                                                  ! Increment AQ file counter
            IF (H > NumPollutants) CALL Fatal(14)                    ! Too many Air Quality files
            PName = TRIM(ADJUSTL(Key(I+1:I+3)))                      ! Extract pollutant name     
            IF (INDEX(UP(PolList),TRIM(UP(PName)))==0) CALL Fatal(7) ! Pol name must be in list
            Pol(H)%AQName = VC                                       ! Set AQ filename for pollutant
         ENDIF                                                       ! End of air quality file logic      
         IF (INDEX(Key,"POP")>0)  THEN                               ! There are many pop files 
            I = INDEX(VarName,",")                                   ! Location of first comma
            J = INDEX(VarName(I+1:),",")                             ! Location of second comma 
            IF (I*J==0) CALL Fatal(8)                                ! Stop if either comma missing
            K = K+1                                                  ! Increment pop type counter
            G = ADJUSTL(Key(I+1:I+J-1))                              ! Extract first letter of gender
            IF (INDEX('MF',G)==0) CALL Fatal(9)                      ! Gender must be male or female
            PopCat(K)%Gender = ADJUSTL(VarName(I+1:I+J-1))           ! Extract gender of this type
            PopCat(K)%Race   = ADJUSTL(VarName(I+J+1:))              ! Extract race of this type
            IF (LEN_TRIM(PopCat(K)%Race)==0) CALL Fatal(10)          ! Race must not be blank
            PopCat(K)%FName  = VC                                    ! Set filename for this type
         ENDIF                                                       ! End of population file logic
      ENDIF                                                          ! End of filename logic
      IF (INDEX(Key,"POLLUTANT")>0.AND.INDEX(Key,"#")==0 )   THEN    ! If pollutant keyword
          PL      = PL+1                                             ! Update pollutant #
          Pol(PL)%Name      = VC                                     ! Set the pollutant name
          IF (INDEX(Up(VC),"PM")==1)  Pol(PL)%PM=YES                 ! This pollutant is a particle
      ENDIF                                                          ! End pollutant name logic      
      IF (INDEX(Key,"DISEASE")>0)                 Disease     = VC   ! Set the disease name
      IF (INDEX(Key,"LOCATION")>0)                Location    = VC   ! Set the study area location
      IF (INDEX(Key,"SCENARIO")>0)                Scenario    = VC   ! Set the scenario description
      IF (INDEX(Key,"INPUTUNIT")>0) THEN                             ! Declaring input unit
         IF (INDEX(Up(VC),"PPB")>0) Pol(PL)%InUnits = PPB            ! Set to PPB
         IF (U1/="P") Pol(PL)%InUnits = UGM3                         ! Or set to ug/m3
      ENDIF                                                          ! End input unit assignment
      IF (INDEX(Key,"OUTPUTUNI")>0) THEN                             ! Declaring output unit
         IF (INDEX(Up(VC),"PPB")>0) Pol(PL)%OutUnits  = PPB          ! Set to PPB                                
         IF (U1/="P") Pol(PL)%OutUnits = UGM3                        ! Set to ug/m3 if not PPM
      ENDIF                                                          ! End output unit assignment
      IF (INDEX(Key,"ROADWAY")>0   .AND. U1=="Y") THEN               ! If roadway concentrations
         Pol(PL)%UseRoadAQ = YES                                     ! Set flag for roadway concs           
         RoadWayFlag = YES                                           ! Setting global roadwayflag
      ENDIF                                                          ! End roadway assignments
      IF (INDEX(Key,"ROADLAST")>0  .AND. U1=="Y") THEN               ! If entry roadlast
         Pol(PL)%RoadLast = YES                                      ! Set roadlast flag  
         RoadLastFlag = YES                                          ! Set global roadlast flag
      ENDIF                                                          ! End roadlast assignments
      IF (INDEX(Key,"SAMPLEOT")>0  .AND. U1=="Y") SampleLocs  = YES  ! Set sample "other" flag
      IF (INDEX(Key,"ROLLBACK")>0  .AND. U1=="Y") Rollback    = YES  ! Set ambient air Rollback flag
      IF (INDEX(Key,"DSTADJUST")>0 .AND. U1=="N") DSTAdjust   = NO   ! Set Daylight Savings flag
      IF (INDEX(Key,"HOURLYOUT")>0 .AND. U1=="Y") HourlyOut   = YES  ! Set Hourly file output flag
      IF (INDEX(Key,"DAILYOUT")>0  .AND. U1=="Y") DailyOut    = YES  ! Set Daily file output flag
      IF (INDEX(Key,"MODELAQVAR")>0.AND. U1=="Y") ModelAQVar  = YES  ! Set AQ variability flag
      IF (INDEX(Key,"MSUMOUT")>0   .AND. U1=="N") MSumOut     = NO   ! Set Micro Sum output flag
      IF (INDEX(Key,"MRESOUT")>0   .AND. U1=="Y") MResOut     = YES  ! Set Micro Res output flag
      IF (INDEX(Key,"EVENTSOUT")>0 .AND. U1=="Y") EventsOut   = YES  ! Set Events output flag
      IF (INDEX(Key,"TIMESTEPOUT")>0.AND. U1=="Y")TimeStepOut = YES  ! Set timestep output flag
      IF (INDEX(Key,"DODOSE")>0 .AND. U1=="Y") Pol(PL)%DoDose = YES  ! Set Dose calculation flag
      IF (INDEX(Key,"COMMUT")>0    .AND. U1=="Y") CommuteFlag = YES  ! Set flag for commuting    
      IF (INDEX(Key,"KEEPLEAV")>0  .AND. U1=="Y") KeepLeavers = YES  ! Set flag for commuters outside
      IF (INDEX(Key,"LOGDISTR")>0  .AND. U1=="N") LogDistrict = NO   ! Set flag for printing districts
      IF (INDEX(Key,"LOGPOPUL")>0  .AND. U1=="N") LogPopulate = NO   ! Set flag for printing population
      IF (INDEX(Key,"LOGPROFIL")>0 .AND. U1=="N") LogProfiles = NO   ! Set flag for printing profiles
      IF (INDEX(Key,"LOGSECTOR")>0 .AND. U1=="Y") LogSectors  = YES  ! Set flag for printing sectors
      IF (INDEX(Key,"LOGTABLE")>0  .AND. U1=="Y") LogTables   = YES  ! Set flag for printing tables
      IF (INDEX(Key,"LOGZONES")>0  .AND. U1=="N") LogZones    = NO   ! Set flag for printing zones
      IF (INDEX(Key,"LONGITDIA")>0 .AND. U1=="Y") LongitDiary = YES  ! Set flag for longitud. diaries
      IF (INDEX(Key,"CLUSTDIARYB")>0.AND. U1=="Y") ClustDiaryB = YES ! Set flag for cluster. diaries B
      IF (INDEX(Key,"CLUSTDIARYA")>0.AND. U1=="Y") ClustDiaryA = YES ! Set flag for cluster. diaries A
      IF (INDEX(Key,"OCCFACTOR")>0 .AND. U1=="Y") OccFactor = YES    ! Set flag for factor tied to employment
      IF (INDEX(Key,"ONEPERCENT")>0 .AND. U1=="Y") OnePercent = YES
	  IF (INDEX(Key,"RERUNCLUS")>0  .AND. U1=="Y") RerunClus = YES   ! Set flag for cluster. diaries A
      IF (INDEX(Key,"COUNTYLIS")>0 .AND. U1=="Y") CountyList  = YES  ! Set flag for using a county list
      IF (INDEX(Key,"RESAMPLEN")>0 .AND. U1=="Y") ResampleNear = YES ! Set flag for using a county list      
      IF (INDEX(Key,"TRACTLIST")>0 .AND. U1=="Y") TractList   = YES  ! Set flag for using a tract list
      IF (INDEX(Key,"MRESHOME") >0 .AND. U1=="Y") MResHome    = YES  ! Set flag for using a tract list
      IF (INDEX(Key,"TRACTLIST")>0 .AND. U1=="Y") TractList   = YES  ! Set flag for using a tract list
      IF (INDEX(Key,"HOURLYFEVE1")>0 .AND.U1=="Y")HourlyFEVE1Flag=YES  ! Set flag for using hourly FEV E1      
      IF (INDEX(Key,"HOURLYFEVE2")>0 .AND.U1=="Y")HourlyFEVE2Flag=YES  ! Set flag for using hourly FEV E2            
      IF (INDEX(Key,"USEOCCGROUPS")>0.AND.U1=="Y")UseOccGroups=YES   ! Set flag for matching based on emp group
      IF (INDEX(Key,"COUNTY ")>0) THEN                               ! Found keyword for county FIPS
         NumCounties = NumCounties+1                                 ! Increment list of counties
         IF (NumCounties > MAXCOUNTY) CALL Fatal(11)                 ! Can't exceed max counties
         CList(NumCounties) = VC(1:5)                                ! Add county FIPS code to list
      ELSE IF (INDEX(Key,"TRACT ")>0)  THEN                          ! Found keyword for tract #
         NumTracts = NumTracts + 1                                   ! Increment list of tracts
         IF (NumTracts > MAXTRACT) CALL Fatal(11)                    ! Can't exceed max tracts
         TList(NumTracts)   = VC(1:11)                               ! Add tract # to list
       ELSE IF (INDEX(Key,"MRESMICROS")>0) THEN                      ! Found keyword for mres mics  
         NumMicrosPr     = NumLev-1                                  ! # micros to print to MRes file
         MList(1:NumLev-1) = Buf(2:NumLev)                           ! Load mic vals from buffer
      ELSE IF (INDEX(Key,"PSUMLIST")>0) THEN                         ! Found keyword for psums vars  
         IF (INDEX(Up(VC),"ACCAR")   >0) PSumFlags(13)=1             ! Update flag for ACCAR
         IF (INDEX(Up(VC),"ACHOM")   >0) PSumFlags(14)=1             ! Update flag for ACHOME
         IF (INDEX(Up(VC),"DISEASE") >0) PSumFlags(15)=1             ! Update flag for disease
         IF (INDEX(Up(VC),"PILOT")   >0) PSumFlags(16)=1             ! Update flag for pilot
         IF (INDEX(Up(VC),"STOVE")   >0) PSumFlags(17)=1             ! Update flag for stove
         IF (INDEX(Up(VC),"EVENTS")  >0) PSumFlags(18)=1             ! Update flag for # events
         IF (INDEX(Up(VC),"BLOODVOL")>0) PSumFlags(19)=1             ! Update flag for blood vol
         IF (INDEX(Up(VC),"BSA")     >0) PSumFlags(20)=1             ! Update flag for body surf area
         IF (INDEX(Up(VC),"ECF")     >0) PSumFlags(21)=1             ! Update flag for ECF
         IF (INDEX(Up(VC),"DIFFUS")  >0) PSumFlags(22)=1             ! Update flag for diffusivity
         IF (INDEX(Up(VC),"ENDGN1")  >0) PSumFlags(23)=1             ! Update flag for ENDGN1
         IF (INDEX(Up(VC),"ENDGN2")  >0) PSumFlags(24)=1             ! Update flag for ENDGN2
         IF (INDEX(Up(VC),"HEMOGLOB")>0) PSumFlags(25)=1             ! Update flag for hemoglobin
         IF (INDEX(Up(VC),"METSMAX") >0) PSumFlags(26)=1             ! Update flag for metsmax
         IF (INDEX(Up(VC),"VO2MAX")  >0) PSumFlags(27)=1             ! Update flag for vo2max 
         IF (INDEX(Up(VC),"MOXD")    >0) PSumFlags(28)=1             ! Update flag for max O2 deficit
         IF (INDEX(Up(VC),"PAI")     >0) PSumFlags(29)=1             ! Update flag for PAI
         IF (INDEX(Up(VC),"RECTIME") >0) PSumFlags(30)=1             ! Update flag for recovery time
         IF (INDEX(Up(VC),"RMR")     >0) PSumFlags(31)=1             ! Update flag for RMR
         IF (INDEX(Up(VC),"VEINTER") >0) PSumFlags(32)=1             ! Update flag for VEINTER
         IF (INDEX(Up(VC),"VERESID") >0) PSumFlags(33)=1             ! Update flag for VERESID
         IF (INDEX(Up(VC),"VESLOPE") >0) PSumFlags(34)=1             ! Update flag for VESLOPE
         IF (INDEX(Up(VC),"BMI")     >0) PSumFlags(35)=1             ! Update flag for BMI
         IF (INDEX(Up(VC),"B1")      >0) PSumFlags(36)=1             ! Update flag for B1   
         IF (INDEX(Up(VC),"B2")      >0) PSumFlags(37)=1             ! Update flag for B2  
         IF (INDEX(Up(VC),"B3")      >0) PSumFlags(38)=1             ! Update flag for B3  
         IF (INDEX(Up(VC),"B4")      >0) PSumFlags(39)=1             ! Update flag for B4  
         IF (INDEX(Up(VC),"B5")      >0) PSumFlags(40)=1             ! Update flag for B5  
         IF (INDEX(Up(VC),"B6")      >0) PSumFlags(41)=1             ! Update flag for B6  
         IF (INDEX(Up(VC),"B7")      >0) PSumFlags(42)=1             ! Update flag for B7
         IF (INDEX(Up(VC),"FEVU")    >0) PSumFlags(43)=1             ! Update flag for FEVU                                                                  
         IF (INDEX(Up(VC),"FEVSLP")  >0) PSumFlags(44)=1             ! Update flag for FEVSLP
         IF (INDEX(Up(VC),"FEVINT")  >0) PSumFlags(45)=1             ! Update flag for FEVINT
         IF (INDEX(Up(VC),"#DIARIES")>0) PSumFlags(46)=1             ! Update flag for # diaries
         IF (INDEX(Up(VC),"PCOND1")>0)   PSumFlags(47)=1             ! Update flag for pcond1
         IF (INDEX(Up(VC),"PCOND2")>0)   PSumFlags(48)=1             ! Update flag for pcond2
         IF (INDEX(Up(VC),"PCOND3")>0)   PSumFlags(49)=1             ! Update flag for pcond3
         IF (INDEX(Up(VC),"PCOND4")>0)   PSumFlags(50)=1             ! Update flag for pcond3
         IF (INDEX(Up(VC),"PCOND5")>0)   PSumFlags(51)=1             ! Update flag for pcond3   
         IF (INDEX(Up(VC),"RCOND1")>0)   PSumFlags(52)=1             ! Update flag for rcond1
         IF (INDEX(Up(VC),"RCOND2")>0)   PSumFlags(53)=1             ! Update flag for rcond2
         IF (INDEX(Up(VC),"RCOND3")>0)   PSumFlags(54)=1             ! Update flag for rcond3
         IF (INDEX(Up(VC),"RCOND4")>0)   PSumFlags(55)=1             ! Update flag for rcond3
         IF (INDEX(Up(VC),"RCOND5")>0)   PSumFlags(56)=1             ! Update flag for rcond3   
         IF (INDEX(Up(VC),"COMMDIST") >0)PSumFlags(57)=1             ! Update flag for com distance
         IF (INDEX(Up(VC),"COMMTIME") >0)PSumFlags(58)=1             ! Update flag for com time   
         IF (INDEX(Up(VC),"ROADDIST") >0)PSumFlags(59)=1             ! Update flag for roadway district                  
         IF (INDEX(Up(VC),"RWDIST")   >0)PSumFlags(60)=1             ! Update flag for work road district          
         IF (INDEX(Up(VC),"FGROUP")   >0)PSumFlags(61)=1             ! Update flag for factor group
         IF (INDEX(Up(VC),"GROUPNAME")>0)PSumFlags(62)=1             ! Update flag for factor name
         IF (INDEX(Up(VC),"MAXDFEV") >0) PSumFlags(63)=1             ! Update flag for max DFEV1
         IF (INDEX(Up(VC),"AVGEXP")  >0) PSumFlags(NUMPSUMVARS+1)=1  ! Update flag for avg exposure
         IF (INDEX(Up(VC),"MAXEXP")  >0) PSumFlags(NUMPSUMVARS+2)=1  ! Update flag for max exposure     
         IF (INDEX(Up(VC),"AVGDOSE") >0) PSumFlags(NUMPSUMVARS+3)=1  ! Update flag for avg dose      
         IF (INDEX(Up(VC),"MAXDOSE") >0) PSumFlags(NUMPSUMVARS+4)=1  ! Update flag for max dose    
       ELSE IF (INDEX(Key,"TABLESLIST")>0) THEN                      ! Found keyword for tables vars  
         IF (INDEX(Up(VC),"EXP1H")  >0) TableFlags(1)=1              ! Update flag for exp 1-hour
         IF (INDEX(Up(VC),"EXP8H")  >0) TableFlags(2)=1              ! Update flag for exp 8-hour
         IF (INDEX(Up(VC),"EXPTS")  >0) TableFlags(3)=1              ! Update flag for exp timestep
         IF (INDEX(Up(VC),"DOSE1H") >0) TableFlags(4)=1              ! Update flag for dose 1-hour
         IF (INDEX(Up(VC),"DOSE8H") >0) TableFlags(5)=1              ! Update flag for dose 8-hour
         IF (INDEX(Up(VC),"DOSE1EH")>0) TableFlags(6)=1              ! Update flag for dose end-of-hour
         IF (INDEX(Up(VC),"DOSETS") >0) TableFlags(7)=1              ! Update flag for dose timestep
         IF (INDEX(Up(VC),"ACTIVE") >0) TableFlags(8)=1              ! Update flag for active
         IF (INDEX(Up(VC),"ACTCHILD")  >0) TableFlags(9)=1           ! Update flag for activechild
         IF (INDEX(Up(VC),"ILLNESS")   >0) TableFlags(10)=1          ! Update flag for illness
         IF (INDEX(Up(VC),"ILLCHILD")  >0) TableFlags(11)=1          ! Update flag for illchild
         IF (INDEX(Up(VC),"MOD")    >0) TableFlags(12)=1             ! Update flag for moderate
         IF (INDEX(Up(VC),"HEAVY")  >0) TableFlags(13)=1             ! Update flag for heavy     
         IF (INDEX(Up(VC),"CHILDREN")  >0) TableFlags(14)=1          ! Update flag for child  
         IF (INDEX(Up(VC),"MICROTIME") >0) TableFlags(15)=1          ! Update flag for time in ue   
         IF (INDEX(Up(VC),"EXPAVG") >0) TableFlags(16)=1             ! Update flag for average exposure
         IF (INDEX(Up(VC),"DOSEAVG")   >0) TableFlags(17)=1          ! Update flag for average dose
         IF (INDEX(Up(VC),"DOSETIME")  >0) TableFlags(18)=1          ! Update flag for time in each dose
         IF (INDEX(Up(VC),"EMPLOYED")  >0) TableFlags(19)=1          ! Update flag for employed people
       ELSE IF (INDEX(Key,"EVENTSLIST")>0) THEN                      ! Found keyword for event vars
         IF (INDEX(Up(VC),"UMET")    >0) EventFlags(12)=1            ! Update flag for uncorrected METS
         IF (INDEX(Up(VC),"METS")    >0) EventFlags(13)=1            ! Update flag for METS 
         IF (INDEX(Up(VC),"VA")      >0) THEN                        ! If VA found in list
             EventFlags(14)=1                                        ! Update flag for VA 
             VAoutput    = YES                                       ! Set VAOutput to Yes
         ENDIF                                                       ! End VA Output logic
         IF (INDEX(Up(VC),"VE")      >0) EventFlags(15)=1            ! Update flag for VE 
         IF (INDEX(Up(VC),"DEFICIT") >0) EventFlags(16)=1            ! Update flag for deficit 
         IF (INDEX(Up(VC),"EVR")     >0) EventFlags(17)=1            ! Update flag for EVR    
         IF (INDEX(Up(VC),"DFEV1")   >0) THEN                        ! If DFEV1 found
             EventFlags(18)=1                                        ! Update flag for dFEV1            
             FEVFlag = 1                                             ! Calculate dFEV1 = YES
         ENDIF                                                       ! End dFEV logic
         IF (INDEX(Up(VC),"FEVX")    >0) THEN                        ! If FEVX found
             EventFlags(19)=1                                        ! Update flag for FEVX
             FEVFlag = 1                                             ! Calculate dFEV1 = YES
         ENDIF                                                       ! End dFEV logic         
         IF (INDEX(Up(VC),"FEVE1")   >0) EventFlags(20)=1            ! Update flag for FEV E term         
         IF (INDEX(Up(VC),"FEVE2")   >0) EventFlags(21)=1            ! Update flag for FEV E term                           
         IF (INDEX(Up(VC),"AMB")     >0) EventFlags(22)=1            ! Update flag for AMB           
         IF (INDEX(Up(VC),"HOMEAMB") >0) EventFlags(23)=1            ! Update flag for HOMEAMB
       ELSE IF (INDEX(Key,"MRESLIST")>0) THEN                        ! Found keyword for mres vars  
         IF (INDEX(Up(VC),"PRX")     >0) MResFlags(6)=1              ! Update flag for proximity
         IF (INDEX(Up(VC),"PEN")     >0) MResFlags(7)=1              ! Update flag for penetration
         IF (INDEX(Up(VC),"CSUM")    >0) MResFlags(8)=1              ! Update flag for csum
         IF (INDEX(Up(VC),"AMB")     >0) MResFlags(9)=1              ! Update flag for amb
         IF (INDEX(Up(VC),"CONC")    >0) MResFlags(10)=1             ! Update flag for conc
         IF (INDEX(Up(VC),"ESUM")    >0) MResFlags(11)=1             ! Update flag for esum
         IF (INDEX(Up(VC),"SOURCE")  >0) MResFlags(12)=1             ! Update flag for source
         IF (INDEX(Up(VC),"VOL")     >0) MResFlags(13)=1             ! Update flag for volume
         IF (INDEX(Up(VC),"DEC")     >0) MResFlags(14)=1             ! Update flag for decay
         IF (INDEX(Up(VC),"AER")     >0) MResFlags(15)=1             ! Update flag for AER
         IF (INDEX(Up(VC),"RR")      >0) MResFlags(16)=1             ! Update flag for rem rate
         IF (INDEX(Up(VC),"WINDOWRES")  >0) MResFlags(17)=1          ! Update flag for windowres
         IF (INDEX(Up(VC),"WINDOWCAR")  >0) MResFlags(18)=1          ! Update flag for windowcar
         IF (INDEX(Up(VC),"MAXTEMPCAT") >0) MResFlags(19)=1          ! Update flag for maxtempcat
         IF (INDEX(Up(VC),"AVGTEMPCAT") >0) MResFlags(20)=1          ! Update flag for avgtempcat 
         IF (INDEX(Up(VC),"SPEEDCAT")   >0) MResFlags(21)=1          ! Update flag for speedcat
         IF (INDEX(Up(VC),"COND1")      >0) MResFlags(22)=1          ! Update flag for cond1
         IF (INDEX(Up(VC),"COND2")      >0) MResFlags(23)=1          ! Update flag for cond2
         IF (INDEX(Up(VC),"COND3")      >0) MResFlags(24)=1          ! Update flag for cond3
         IF (INDEX(Up(VC),"TEMPCAT")    >0) MResFlags(25)=1          ! Update flag for tempcat
         IF (INDEX(Up(VC),"HUMIDCAT")   >0) MResFlags(26)=1          ! Update flag for humidcat
         IF (INDEX(Up(VC),"PRECIPCAT")  >0) MResFlags(27)=1          ! Update flag for precipcat
         IF (INDEX(Up(VC),"WINDCAT")    >0) MResFlags(28)=1          ! Update flag for windcat
         IF (INDEX(Up(VC),"DIRCAT")     >0) MResFlags(29)=1          ! Update flag for dircat
         IF (INDEX(Up(VC),"DAY")        >0) MResFlags(30)=1          ! Update flag for day
         IF (INDEX(Up(VC),"MONTH")      >0) MResFlags(31)=1          ! Update flag for month
         IF (INDEX(Up(VC),"DISTRICT")   >0) MResFlags(32)=1          ! Update flag for district
         IF (INDEX(Up(VC),"DAYWEEK")    >0) MResFlags(33)=1          ! Update flag for district
         IF (INDEX(Up(VC),"AQCOND1")    >0) MResFlags(34)=1          ! Update flag for AQ1 Conditional 
         IF (INDEX(Up(VC),"AQCOND2")    >0) MResFlags(35)=1          ! Update flag for AQ2 Conditional 
         IF (INDEX(Up(VC),"AQCOND3")    >0) MResFlags(36)=1          ! Update flag for AQ3 Conditional 
         IF (INDEX(Up(VC),"AQCOND4")    >0) MResFlags(37)=1          ! Update flag for AQ4 Conditional 
         IF (INDEX(Up(VC),"AQCOND5")    >0) MResFlags(38)=1          ! Update flag for AQ5 Conditional         
      ELSE IF (INDEX(Key,"HOURLYLIST")>0) THEN                       ! Found keyword for hourly vars  
         IF (INDEX(Up(VC),"VE")   >0)       HourFlags(3)=1           ! Update flag for hourly Ve
         IF (INDEX(Up(VC),"VA")   >0)       HourFlags(4)=1           ! Update flag for hourly Va
         IF (INDEX(Up(VC),"EVR")  >0)       HourFlags(5)=1           ! Update flag for hourly EVR
         IF (INDEX(Up(VC),"METS") >0)       HourFlags(6)=1           ! Update flag for hourly METS
         IF (INDEX(Up(VC),"EE")   >0)       HourFlags(7)=1           ! Update flag for hourly EE
         IF (INDEX(Up(VC),"DFEV1")>0) THEN                           ! If dFEV1 found...
            HourFlags(8)=1                                           ! Update flag for dFEV1         
            FEVFlag = 1                                              ! FEVFlag = 1
         ENDIF                                                       ! End dFEV1 logic
         IF (INDEX(Up(VC),"FEVE1")   >0)       HourFlags(9)=1        ! Update flag for hourly FEVE1
         IF (INDEX(Up(VC),"FEVE2")   >0)       HourFlags(10)=1       ! Update flag for hourly FEVE2         
         DO I=1, NumMicros+1                                         ! Loop over micros (+1 for Mic 0)
            WRITE(Mic,*) I-1                                         ! mic num to character           
            IF (INDEX(Up(VC),"TIME"//TRIM(ADJUSTL(Mic))//",") >0    &
               .OR. INDEX(Up(VC),"TIME"//TRIM(ADJUSTL(Mic))//" ")>0)&
              HourFlags(11+I)=1                                      ! Update flag for time in micro
            IF (INDEX(Up(VC),"EXP"//TRIM(ADJUSTL(Mic))//",") >0     &
               .OR. INDEX(Up(VC),"EXP"//TRIM(ADJUSTL(Mic))//" ")>0) &
              HourFlags(11+NumMicros+1+I)=1                          ! Update flag for micro conc
         ENDDO                                                       ! End loop over micros
         IF (INDEX(Up(VC),"AMB") >0) HourFlags(12+2*(NumMicros+1))=1 ! Update flag for hourly ambient
         IF (INDEX(Up(VC),"EXP") >0) HourFlags(13+2*(NumMicros+1))=1 ! Update flag for hourly exposure
         IF (INDEX(Up(VC),"DOSE")>0.AND.INDEX(Up(VC),"INTAKE")==0   &
            .AND.INDEX(Up(VC),"DEP")==0)                            &
            HourFlags(14+2*(NumMicros+1))=1                          ! Update flag for hourly dose
         IF (INDEX(Up(VC),"EF") >0)  HourFlags(15+2*(NumMicros+1))=1 ! Update flag for hourly EF
         IF (INDEX(Up(VC),"INTAKE") >0)                             &  
            HourFlags(16+2*(NumMicros+1))=1                          ! Update flag for intake dose
         IF (INDEX(Up(VC),"DEPDOSE") >0)                            &  
            HourFlags(17+2*(NumMicros+1))=1                          ! Update flag for deposited dose               
         IF (INDEX(Up(VC),"HOMEAMB") >0)                            &  
            HourFlags(18+2*(NumMicros+1))=1                          ! Update flag for home ambient 
         IF (INDEX(Up(VC),"HOMEEF") >0)                             &  
            HourFlags(19+2*(NumMicros+1))=1                          ! Update flag for home EF ratio 
     ELSE IF (INDEX(Key,"TIMESTEPLIST")>0) THEN                      ! Found keyword for ts vars  
         IF (INDEX(Up(VC),"VE")   >0)  TSFlags(4)=1                  ! Update flag for ts Ve
         IF (INDEX(Up(VC),"VA")   >0)  TSFlags(5)=1                  ! Update flag for ts Va
         IF (INDEX(Up(VC),"EVR")  >0)  TSFlags(6)=1                  ! Update flag for ts EVR
         IF (INDEX(Up(VC),"METS") >0)  TSFlags(7)=1                  ! Update flag for ts METS
         IF (INDEX(Up(VC),"EE")   >0)  TSFlags(8)=1                  ! Update flag for ts EE
         IF (INDEX(Up(VC),"DFEV1") >0) THEN                          ! If DFEV1 found...
             TSFlags(9)=1                                            ! Update flag for dFEV1%          
             FEVFlag = 1                                             ! Update FEVFlag = YES
         ENDIF                                                       ! End dFEV1 logic
         IF (INDEX(Up(VC),"AMB") >0)   TSFlags(10)=1                 ! Update flag for ts ambient
         IF (INDEX(Up(VC),"EXP") >0)   TSFlags(11)=1                 ! Update flag for ts exposure
         IF (INDEX(Up(VC),"DOSE") >0.AND.INDEX(Up(VC),"INTAKE")==0  &
            .AND.INDEX(Up(VC),"DEP")==0)   TSFlags(12)=1             ! Update flag for ts dose
         IF (INDEX(Up(VC),"EF") >0)        TSFlags(13)=1             ! Update flag for ts EF
         IF (INDEX(Up(VC),"INTAKE") >0)    TSFlags(14)=1             ! Update flag for intake dose
         IF (INDEX(Up(VC),"DEPDOSE") >0)   TSFlags(15)=1             ! Update flag for deposited dose                       
      ELSE IF (INDEX(Key,"DAILYLIST")>0) THEN                        ! Found keyword for daily vars  
         IF (INDEX(Up(VC),"CHADID")    >0)     DayFlags(3)=1         ! Update flag for CHADID
         IF (INDEX(Up(VC),"CHADAGE")   >0)     DayFlags(4)=1         ! Update flag for diary age
         IF (INDEX(Up(VC),"CHADOCC")   >0)     DayFlags(5)=1         ! Update flag for diary empl 
         IF (INDEX(Up(VC),"DIARYPOOL") >0)     DayFlags(6)=1         ! Update flag for dairypool
         IF (INDEX(Up(VC),"PAI")       >0)     DayFlags(7)=1         ! Update flag for PAI
         IF (INDEX(Up(VC),"KEYVAR")    >0)     DayFlags(8)=1         ! Update flag for the key var
         IF (INDEX(Up(VC),"WINDOWRES") >0)     DayFlags(9)=1         ! Update flag for windowres
         IF (INDEX(Up(VC),"WINDOWCAR") >0)     DayFlags(10)=1        ! Update flag for windowcar
         IF (INDEX(Up(VC),"SPEEDCAT")  >0)     DayFlags(11)=1        ! Update flag for speedcat
         IF (INDEX(Up(VC),"DCOND1")    >0)     DayFlags(12)=1        ! Update flag for cond1
         IF (INDEX(Up(VC),"DCOND2")    >0)     DayFlags(13)=1        ! Update flag for cond2
         IF (INDEX(Up(VC),"DCOND3")    >0)     DayFlags(14)=1        ! Update flag for cond3
         IF (INDEX(Up(VC),"MAXTEMPCAT")>0)     DayFlags(15)=1        ! Update flag for maxtempcat
         IF (INDEX(Up(VC),"AVGTEMPCAT")>0)     DayFlags(16)=1        ! Update flag for avgtempcat
         IF (INDEX(Up(VC),"AVGTEMP")   >0)     DayFlags(17)=1        ! Update flag for avgtemp
         IF (INDEX(Up(VC),"MAXTEMP")   >0)     DayFlags(18)=1        ! Update flag for maxtemp
         IF (INDEX(Up(VC),"DFEV1")     >0) THEN                      ! If dFEV1 asked for
             DayFlags(19)=1                                          ! Update flag for %dFEV1     
             FEVFlag = 1                                             ! Update FEVFlag = YES
         ENDIF                                                       ! End dFEV1 logic
         IF (INDEX(Up(VC),"CHADOCC")   >0)     DayFlags(20)=1        ! Update flag for CHAD Occupation          
         IF (INDEX(Up(VC),"AVGEXP")    >0)     DayFlags(21)=1        ! Update flag for avgexp
         IF (INDEX(Up(VC),"MAX1EXP")   >0)     DayFlags(22)=1        ! Update flag for max1exp
         IF (INDEX(Up(VC),"MAX8EXP")   >0)     DayFlags(23)=1        ! Update flag for max8exp
         IF (INDEX(Up(VC),"AVGDOSE")   >0)     DayFlags(24)=1        ! Update flag for avgdose
         IF (INDEX(Up(VC),"MAX1DOSE")  >0)     DayFlags(25)=1        ! Update flag for max1dose
         IF (INDEX(Up(VC),"MAX8DOSE")  >0)     DayFlags(26)=1        ! Update flag for max8dose
         IF (INDEX(Up(VC),"MAX1FDOSE") >0)     DayFlags(27)=1        ! Update flag for maxfdose
         IF (INDEX(Up(VC),"INTAKE")    >0)     DayFlags(28)=1        ! Update flag for intake dose
         IF (INDEX(Up(VC),"DEPDOSE")   >0)     DayFlags(29)=1        ! Update flag for deposited dose
         IF (INDEX(Up(VC),"HOME8MAXEF")>0)     DayFlags(30)=1        ! Update flag for home 8-hour E/C ratio
      ELSE IF (INDEX(Key,"PERCENTILE")>0) THEN                       ! Found keyword for percentiles  
         Pctl(PL,1:NumLev-1) = Buf(2:NumLev)                         ! Load values from list buffer
         IF (ANY(Pctl(PL,:)<=0.).OR.ANY(Pctl(PL,:)>=100.))          &
            CALL Fatal(12)                                           ! Stop if percentiles invalid
      ELSE IF (INDEX(Key,"CUSTOMSA")>0) THEN                         ! Found custom profile keyword
         IF (VC=="")  CYCLE                                          ! If no custom samples listed  
         NumCustProfs= NumLev-1                                      ! Number of custom profiles    
         ALLOCATE (CustProfs(1:NumCustProfs),STAT=AllocErr)          ! Make space for custom profiles
         CALL CheckAllocation(AllocErr)                              ! Check the success of the alloc
         CustProfs   = Buf(2:NumLev)                                 ! Load custom profiles from buffer
         DoCustProfs = YES                                           ! Set custom profile flag    
      ELSE IF (INDEX(Key,"CUSTOMWO")>0) THEN                         ! Found custom work keyword
         CustomWork  = YES                                           ! Set custom work flag    
         IF (VC=="")  CYCLE                                          ! If no custom work acts listed  
         NumWork  = NumLev-1                                         ! Number of custom activites    
         ALLOCATE (WorkActs(1:NumWork),STAT=AllocErr)                ! Make space for custom activities
         CALL CheckAllocation(AllocErr)                              ! Check the success of the alloc
         CustProfs   = Buf(2:NumLev)                                 ! Load custom acts from buffer
      ELSE IF (INDEX(Key,"DM1HEXP")>0) THEN                          ! Found DM1HExp levels keyword
         MaxLevels = MAX(MaxLevels, NumLev)                          ! Update maximum number of levels
         DM1HExpLev(PL,1:NumLev) = Buf(1:NumLev)                     ! Load DM1HExp levels from buffer
      ELSE IF (INDEX(Key,"DM8HEXP")>0) THEN                          ! Found DM8HExp levels keyword 
         MaxLevels = MAX(MaxLevels, NumLev)                          ! Update maximum number of levels                     
         DM8HExpLev(PL,1:NumLev) = Buf(1:NumLev)                     ! Load DM8HExp levels from buffer
      ELSE IF (INDEX(Key,"DAVGEXP")>0) THEN                          ! Found DAvgExp levels keyword
         MaxLevels = MAX(MaxLevels, NumLev)                          ! Update maximum number of levels)
         DAvgExpLev(PL,1:NumLev) = Buf(1:NumLev)                     ! Load DAvgExp levels from buffer
      ELSE IF (INDEX(Key,"SAVGEXP")>0) THEN                          ! Found SAvgExp levels keyword 
         MaxLevels = MAX(MaxLevels, NumLev)                          ! Update maximum number of levels
         SAvgExpLev(PL,1:NumLev) = Buf(1:NumLev)                     ! Load SAvgExp levels from buffer
      ELSE IF (INDEX(Key,"TIMEEXP")>0) THEN                          ! Found TimeExp levels keyword   
         MaxLevels = MAX(MaxLevels, NumLev)                          ! Update maximum number of levels
         TimeExpLev(PL,1:NumLev) = Buf(1:NumLev)                     ! Load TimeExp levels from buffer
      ELSE IF (INDEX(Key,"DM1HDOSE")>0) THEN                         ! Found DM1HDose levels keyword 
         MaxLevels = MAX(MaxLevels, NumLev)                          ! Update maximum number of levels
         DM1HDoseLev(PL,1:NumLev) = Buf(1:NumLev)                    ! Load DM1HDose levels from buffer
      ELSE IF (INDEX(Key,"DM8HDOSE")>0) THEN                         ! Found DM8HDose levels keyword 
         MaxLevels = MAX(MaxLevels, NumLev)                          ! Update maximum number of levels
         DM8HDoseLev(PL,1:NumLev) = Buf(1:NumLev)                    ! Load DM8HDose levels from buffer
      ELSE IF (INDEX(Key,"FEV1RISK")>0.OR.INDEX(Key,"FEVRISK")>0) THEN   ! Found Keyword for FEV1Risk levels
         NumFEV = NumFEV+1                                           ! Add another list of probs.
         IF (NumFEV > MAXFEV) CALL Fatal(11)                         ! Error if more than MAXLEV lists
         MaxLevels = MAX(MaxLevels, NumLev)                          ! Update maximum number of levels
         FEVDescList(NumFEV) = VC(1:20)
         UC = Up(VC(22:))
         NumLev  = 1+COUNT((/(UC(I:I)==",",I=1,LEN_TRIM(UC))/))      ! = 1 + #commas in UC
         Buf(0)  = REAL(NumLev)                                      ! Store # of levels in buffer
         DO I=1, NumLev-1                                            ! Loop over levels
            READ(UC,*,IOSTAT=Ios(1)) Buf(I)                          ! Read first item in list
            UC = UC(INDEX(UC,",")+1:)                                ! Remove first item from list
            IF (I>1.AND.Buf(I)<Buf(I-1)) CALL Fatal(5)               ! Levels must be ascending
         ENDDO   
         READ(UC,*,IOSTAT=Ios(1)) Buf(NumLev)                        ! Continue with next level
         FEVProbList(NumFEV,1:NumLev) = Buf(1:NumLev)                ! Load FEV1RISK probabilities
      ELSE IF (INDEX(Key,"DAVGDOSE")>0) THEN                         ! Found DAvgDose levels keyword 
         MaxLevels = MAX(MaxLevels, NumLev)                          ! Update maximum number of levels
         DAvgDoseLev(PL,1:NumLev) = Buf(1:NumLev)                    ! Load DAvgDose levels from buffer
      ELSE IF (INDEX(Key,"SAVGDOSE")>0) THEN                         ! Found SAvgDose levels keyword 
         MaxLevels = MAX(MaxLevels, NumLev)                          ! Update maximum number of levels
         SAvgDoseLev(PL,1:NumLev) = Buf(1:NumLev)                    ! Load SAvgDose levels from buffer
      ELSE IF (INDEX(Key,"DMEHDOSE")>0) THEN                         ! Found DMEHDose levels keyword 
         MaxLevels = MAX(MaxLevels, NumLev)                          ! Update maximum number of levels
         DMEHDoseLev(PL,1:NumLev) = Buf(1:NumLev)                    ! Load DMEHDose levels from buffer
      ELSE IF (INDEX(Key,"TIMEDOSE")>0) THEN                         ! Found TimeDose levels keyword  
         MaxLevels = MAX(MaxLevels, NumLev)                          ! Update maximum number of levels
         TimeDoseLev(PL,1:NumLev) = Buf(1:NumLev)                    ! Load TimeDose levels from buffer
      ELSE IF (INDEX(Key,"H_EHDOSE")>0) THEN                         ! Found H_EHDose levels keyword
         MaxLevels = MAX(MaxLevels, NumLev)                          ! Update maximum number of levels
         H_EHDoseLev(PL,1:NumLev) = Buf(1:NumLev)                    ! Load H_EHDose levels from buffer
      ELSE IF (INDEX(Key,"DMTSEXP")>0) THEN                          ! Found DMTSExp levels keyword
         MaxLevels = MAX(MaxLevels, NumLev)                          ! Update maximum number of levels
         DMTSExpLev(PL,1:NumLev) = Buf(1:NumLev)                     ! Load DMTSExp levels from buffer
      ELSE IF (INDEX(Key,"TSEXP")>0 .AND. INDEX(Key,"DM")==0 ) THEN  ! Found TSExp levels keyword
         MaxLevels = MAX(MaxLevels, NumLev)                          ! Update maximum number of levels
         TSExpLev(PL,1:NumLev) = Buf(1:NumLev)                       ! Load TSExp levels from buffer
      ELSE IF (INDEX(Key,"DMTSDOSE")>0) THEN                         ! Found DMTSDose levels keyword
         MaxLevels = MAX(MaxLevels, NumLev)                          ! Update maximum number of levels
         DMTSDoseLev(PL,1:NumLev) = Buf(1:NumLev)                    ! Load DMTSDose levels from buffer
      ELSE IF (INDEX(Key,"TSDOSE")>0 .AND. INDEX(Key,"DM")==0 ) THEN ! Found TSDose levels keyword
         MaxLevels = MAX(MaxLevels, NumLev)                          ! Update maximum number of levels
         TSDoseLev(PL,1:NumLev) = Buf(1:NumLev)                      ! Load TSDose levels from buffer
      ELSE IF (INDEX(Key,"TSMULT")>0) THEN                           ! Found TS multi levels keyword
         MaxLevels = MAX(MaxLevels, NumLev)                          ! Update maximum number of levels
         TSMultiLev(PL,1:NumLev) = INT(Buf(1:NumLev))                ! Load TSDose levels from buffer
      ELSE                                                           ! Or else set parameters...
         READ(VC,*,IOSTAT=Ios(1)) VR                                 ! Convert string into real
         IF (Ios(1)/=0) CYCLE                                        ! Discard if not numeric 
         READ(VC,*,IOSTAT=Ios(2)) VI                                 ! Convert string to integer 
         IF (INDEX(Key,"START_DATE")>0)       StartDate = VI         ! Set simulation start date 
         IF (INDEX(Key,"END_DATE")>0)           EndDate = VI         ! Set simulation end date
         IF (INDEX(Key,"#PROFILES")>0)      NumProfiles = VI         ! Set number of profiles
         IF (INDEX(Key,"FIRSTPROFILE")>0)  FirstProfile = VI         ! Set first profile to simulate
         IF (INDEX(Key,"AGEMIN")>0)              AgeMin = VI         ! Set minimum age for profiles
         IF (INDEX(Key,"AGEMAX")>0)              AgeMax = VI         ! Set maximum age for profiles
         IF (INDEX(Key,"RANDOM")>0) THEN                             ! Randomseed flag
            ISeed(1) = VI                                            ! Set random number seed
            IF (VC /= "0")  ClockSeed = NO                           ! Seed set by user
            IF (ISeed(1)==0 .AND. Clockseed==NO) CALL Fatal(13)      ! Error if seed to large
         ENDIF                                                       ! End randomseed logic                
         IF (INDEX(Key,"DEBUGLEVEL")>0)      DebugLevel = VI         ! Set debug message level
         IF (INDEX(Key,"EVENTSAMPLE")>0)    EventSample = VI         ! Set number of event profiles
         IF (INDEX(Key,"LATITUDE")>0)        Center%Lat = VR         ! Set city center latitude
         IF (INDEX(Key,"LONGITUDE")>0)       Center%Lon = VR         ! Set city center longitude
         IF (INDEX(Key,"CITYRADIUS")>0)      CityRadius = VR         ! Set radius of study area in km
         IF (INDEX(Key,"AIRRADIUS")>0)        AirRadius = VR         ! Set radius of air districts (km)
         IF (INDEX(Key,"NEARBYRADIUS")>0)  NearbyRadius = VR         ! Set radius for nearby sampling (km)
         IF (INDEX(Key,"ZONERADIUS")>0)      ZoneRadius = VR         ! Set radius of temp. zones in km 
         IF (INDEX(Key,"RBTARGET")>0)          RBTarget = VR         ! Rollback target value
         IF (INDEX(Key,"RBBACK")>0)             RBBkgnd = VR         ! Rollback background value
         IF (INDEX(Key,"RBMAX")>0)                RBMax = VR         ! Rollback maximum value
         IF (INDEX(Key,"MISSGENDER")>0)      MissGender = VR         ! Probability of diary selection
         IF (INDEX(Key,"MISSEMPL")>0)          MissEmpl = VR         ! Probability of diary selection
         IF (INDEX(Key,"MISSOCCUP")>0)          MissOcc = VR         ! Probability of diary selection
         IF (INDEX(Key,"MISSAGE")>0)            MissAge = VR         ! Probability of diary selection 
         IF (INDEX(Key,"AGECUT")>0)           AgeCutPct = VR         ! Width of age window in percent
         IF (INDEX(Key,"AGE2PROB")>0)        Age2Probab = VR         ! Shoulder probability for age
         IF (INDEX(Key,"COMPROBAB1")>0)      ComProbab1 = VR         ! Shoulder probability for comm1
         IF (INDEX(Key,"COMPROBAB2")>0)      ComProbab2 = VR         ! Shoulder probability for comm2
         IF (INDEX(Key,"COMCUT1")>0)         ComWindow1 = VR         ! Width of comm1 window in percent
         IF (INDEX(Key,"COMCUT2")>0)         ComWindow2 = VR         ! Width of comm2 window in percent
         IF (INDEX(Key,"ALTITUDE")>0)          Altitude = VR         ! Altitude in feet of study area
         IF (INDEX(Key,"COHBFACT")>0)          COHbFact = VR         ! COHb convergence safety factor
         IF (INDEX(Key,"HOMEPROB")>0)          HomeProb = VR         ! Probability of home loc in "O"
         IF (INDEX(Key,"LEAVERMULT")>0)      LeaverMult = VR         ! Mult. factor for commuters out
         IF (INDEX(Key,"LEAVERADD")>0)        LeaverAdd = VR         ! Additive term for commuters out
         IF (INDEX(Key,"DIARYD")>0)           DiaryD    = VR         ! Longitudinal diary divers stat   
         IF (INDEX(Key,"DIARYAUTOC")>0)       AutoC     = VR         ! Longitudinal diary autocorr
         IF (INDEX(Key,"HEAVYEVR1")>0)        HeavyEVR1 = VR         ! Heavy 1 hour EVR for tables 
         IF (INDEX(Key,"HEAVYEVR8")>0)        HeavyEVR8 = VR         ! Heavy 8 hour EVR for tables
         IF (INDEX(Key,"HEAVYEVRTS")>0)       HeavyEVRTS= VR         ! Heavy timestep EVR for tables
         IF (INDEX(Key,"MODEVR1")>0)          ModEVR1   = VR         ! Moderate 1 hour EVR for tables
         IF (INDEX(Key,"MODEVR8")>0)          ModEVR8   = VR         ! Moderate 8 hour EVR for tables
         IF (INDEX(Key,"MODEVRTS")>0)         ModEVRTS  = VR         ! Mod timestep EVR for tables
         IF (INDEX(Key,"CHILDMIN")>0)         ChildMin  = VR         ! Min age for children for tables
         IF (INDEX(Key,"CHILDMAX")>0)         ChildMax  = VR         ! Max age for children for tables
         IF (INDEX(Key,"ACTIVE")>0)           ActivePAI = VR         ! Median PAI for active persons     
         IF (INDEX(Key,"#OTHER")>0)       NumODistricts = VR         ! Number of "O" districts to use     
         IF (PL>0 .AND. INDEX(Key,"#SOURCES")>0) Pol(PL)%NumSources = VI ! Set maximum source number
         IF (PL>0 .AND. INDEX(Key,"PPMFACTOR")>0)       Pol(PL)%PPMFact = VR ! Units conversion factor
         IF (PL>0 .AND. INDEX(Key,"ALERTTHRESH")>0) Pol(PL)%AlertThresh = VR ! Exposure threshold for warnings
         IF (PL>0 .AND. INDEX(Key,"SIZE")>0)               Pol(PL)%Size = VR ! PM diameter
         IF (PL>0 .AND. INDEX(Key,"DENSITY")>0)         Pol(PL)%Density = VR ! PM density
         IF (PL==0 .AND. INDEX(Key,"#SOURCES")>0) CALL Fatal(15)     ! Set maximum source number
         IF (PL==0 .AND. INDEX(Key,"PPMFACTOR")>0) CALL Fatal(15)    ! Units conversion factor
         IF (PL==0 .AND. INDEX(Key,"ALERTTHRESH")>0) CALL Fatal(15)  ! Exposure threshold for warnings
         IF (PL==0 .AND. INDEX(Key,"SIZE")>0)  CALL Fatal(15)        ! PM diameter
         IF (PL==0 .AND. INDEX(Key,"DENSITY")>0) CALL Fatal(15)      ! PM density         
      ENDIF                                                          ! Finished with line
   ENDDO                                                             ! Get next record
   CLOSE(ParU)                                                       ! Close ParFile
   DO I=1, NumPopTypes                                               ! Loop over all pop types
      DO J=1, NumPopTypes                                            ! Loop over all pop types                                                                          
         IF (Up(PopCat(J)%Race)==Up(PopCat(I)%Race)) THEN            ! If pop has current race                             
            IF (Up(PopCat(J)%Gender)=='MALE')   MFound=YES           ! Male file found for this race   
            IF (Up(PopCat(J)%Gender)=='FEMALE') FFound=YES           ! Female file found for race
         ENDIF                                                       ! End current race logic           
      ENDDO                                                          ! End NumPopTypes loop
   IF (MFound==NO .OR. FFound==NO) CALL Warning(0-I)                 ! Only one gender file for race
   MFound=NO                                                         ! Reset male indicator           
   FFound=NO                                                         ! Reset female indicator 
   ENDDO                                                             ! End NumPopTypes loop
   TimeStepsPerHour  =  TimeStepsPerDay/24                           ! Number of timesteps per hour
   IF (TimeStepsPerHour .NE. 1) DoTimeSteps = YES                    ! For writing ts files and tables       
   IF (TimeStepsPerDay < 24) DoHours = NO                            ! Don't do hour data for ts>1 hr       
   TSMinutes         =  1440/TimeStepsPerDay                         ! Length of TS in minutes
   CALL ManageFiles(OPN)                                             ! In UtilityModule   
   WRITE(LogU,*) " Job start   = ", Clock(1)                         ! Time at start of job execution  
   CALL CheckParameters                                              ! In main APEX module
   IF (Rollback==YES) THEN                                           ! Rollback air quality data 
      RBFactor = (RBTarget-RBBkgnd) / (RBMax-RBBkgnd)                ! Air quality reduction factor 
   ELSE                                                              ! If no air quality rollback...
      RBFactor = 1.0                                                 ! Factor of unity means no change
      RBBkgnd  = 0.0                                                 ! No background if no rollback
   ENDIF                                                             ! End of rollback logic
   IF (HourlyOut==NO) FName(HourU)   = ""                            ! Keep hourly filename only if used
   IF (DailyOut==NO)  FName(DayU)   = ""                             ! Keep daily filename only if used
   IF (MSumOut==NO)   FName(MSumU)  = ""                             ! Drop the micro sum filename
   IF (MResOut==NO)   FName(MResU)  = ""                             ! Drop the micro res filename
   IF (EventsOut==NO) FName(EventU) = ""                             ! Drop events filename
   IF (EventsOut==NO) EventSample   = 0                              ! Reset for OutputParameters
   IF (ISeed(1)==0) CALL SYSTEM_CLOCK(ISeed(1))                      ! If Seed=0, use clock for seed
   CALL RANDOM_SEED(SIZE=KSeed)                                      ! # words used to store seeds
   ISeed   = ISeed(1)                                                ! Set each word in ISeed  to base
   ProcLevel = ProcLevel-1                                           ! Decrement procedure depth
END SUBROUTINE ReadParameters                                        ! Return to main APEX program

SUBROUTINE CheckParameters                                           ! Called from ReadParameters 
   ! Global constants: LogU, ST_FMT, ST_MSG
   ! Global vars used: NumProfiles, NumSources, StartDate, EndDate, CityRadius, ZoneRadius
   !                   AirRadius, AgeMin, AgeMax, LeaverMult, LeaverAdd,PPMFact, AgeCutPct
   !                   AlertThresh,ISeed,EventSample, Center, CountyList, MissGender, MissAge,
   !                   MissEmpl, Age2Prob, Rollback, RBTarget, RBMax, RBBkgnd, DebugLevel
   !                   HeavyEVR1, HeavyEVR8, ModEVR1, ModEVR8, ActivePAI, ChildMin, ChildMax,
   !                   TractList  
   ! Global vars set : ProcName, ProcLevel
   ! Intrinsic procs : WRITE, ANY, TRIM, ADJUSTL
   ! APEX procedures : Fatal
   INTEGER  (KIND=IK)  :: I                                          ! Temporary integer variables
   ProcLevel = ProcLevel+1                                           ! Increment procedure depth 
   ProcName(ProcLevel) = "CheckParameters"                           ! Procedure name for messages
   IF (DebugLevel>0) WRITE(LogU,ST_FMT) ST_MSG, ProcName(ProcLevel)  ! Message for debugging
   IF (NumProfiles <= 0)                         CALL Fatal(1)       ! # profiles must be positive 
   IF (NumMicros   <= 0)                         CALL Fatal(2)       ! # micros must be positive  
   IF (StartDate   <= 0)                         CALL Fatal(4)       ! Dates must be positive
   IF (EndDate     <= 0)                         CALL Fatal(5)       ! Dates must be positive
   IF (StartDate   >  EndDate)                   CALL Fatal(6)       ! Start date must come first    
   IF (CityRadius  <= 0)                         CALL Fatal(7)       ! Radius must be greater than 0
   IF (ZoneRadius  <= 0)                         CALL Fatal(8)       ! Radius must be greater than 0
   IF (AirRadius   <= 0)                         CALL Fatal(9)       ! Radius must be greater than 0
   IF (NearbyRadius<= 0)                         CALL Fatal(44)      ! Radius must be greater than 0   
   IF (AgeMin      >  AgeMax)                    CALL Fatal(10)      ! Agemax can't be less than min
   IF (AgeMin      <  0)                         CALL Fatal(11)      ! Min age cannot be negative   
   IF (LeaverMult  <  0)                         CALL Fatal(12)      ! Must be non-negative 
   IF (LeaverAdd   <  0)                         CALL Fatal(13)      ! Must be non-negative 
   IF (ANY(Pol(:)%PPMFact <= 0))                 CALL Fatal(14)      ! Must be non-negative 
   IF (AgeCutPct   <  0)                         CALL Fatal(15)      ! Must be non-negative 
   IF (ANY(Pol(:)%AlertThresh <= 0))             CALL Fatal(16)      ! Must be non-negative 
   IF (ISeed(1)    <  0)                         CALL Fatal(17)      ! Must be non-negative 
   IF (EventSample <  0)                         CALL Fatal(18)      ! Must be non-negative 
   IF (Center%Lat  == 99999)                     CALL Fatal(19)      ! Latitude not set  
   IF (Center%Lon  == 99999)                     CALL Fatal(20)      ! Longitude not set  
   IF (Center%Lat  > 180 .OR. Center%Lat < -180) CALL Fatal(21)      ! Invalid latitude  
   IF (Center%Lon  > 180 .OR. Center%Lon < -180) CALL Fatal(22)      ! Invalid longitude  
   IF (CountyList  == YES .AND. NumCounties==0)  CALL Warning(1)     ! No counties listed 
   IF (TractList   == YES .AND. NumTracts  ==0)  CALL Warning(2)     ! No tracts listed 
   IF (AgeMax  == 0)                             CALL Warning(3)     ! Warn that MaxAge is 0
   IF (MissGender<0 .OR. MissGender>1)           CALL Fatal(23)      ! Must be between 0 and 1
   IF (MissEmpl  <0 .OR. MissEmpl  >1)           CALL Fatal(24)      ! Must be between 0 and 1
   IF (MissAge   <0 .OR. MissAge   >1)           CALL Fatal(25)      ! Must be between 0 and 1
   IF (MissOcc   <0 .OR. MissOcc   >1)           CALL Fatal(45)      ! Must be between 0 and 1
   IF (Age2Probab<0 .OR. Age2Probab>1)           CALL Fatal(26)      ! Must be between 0 and 1
   IF (ComProbab1<0 .OR. ComProbab1>1)           CALL Fatal(43)      ! Must be between 0 and 1
   IF (ComProbab2<0 .OR. ComProbab2>1)           CALL Fatal(43)      ! Must be between 0 and 1
   IF (Rollback==YES) THEN                                           ! Rollback air quality data 
      IF (RBMax<=RBBkgnd)                        CALL Fatal(27)      ! Cannot have Max<Background
      IF (RBTarget>RBMax)                        CALL Fatal(28)      ! Cannot have Target>Max 
      IF (RBTarget<=RBBkgnd)                     CALL Fatal(29)      ! Cannot have Target<Background
   ENDIF                                                             ! End of rollback logic   
   IF (DebugLevel<0 .OR. DebugLevel>3)           CALL Fatal(30)      ! Debuglevel must be 0-3                                             
   IF (HeavyEVR1   <= 0)                         CALL Fatal(31)      ! Must be greater than 0 
   IF (HeavyEVR8   <= 0)                         CALL Fatal(31)      ! Must be greater than 0 
   IF (ModEVR1   <= 0)                           CALL Fatal(31)      ! Must be greater than 0 
   IF (ModEVR8   <= 0)                           CALL Fatal(31)      ! Must be greater than 0 
   IF (ChildMin   < 0)                           CALL Fatal(32)      ! Must be non-negative 
   IF (ChildMax   < 0)                           CALL Fatal(32)      ! Must be non-negative 
   IF (ActivePAI  < 1)                           CALL Fatal(33)      ! Must be 1 or greater 
   IF (AutoC  < -1 .OR. AutoC > 1)               CALL Fatal(34)      ! abs(AutoC) <=1   
   IF (DiaryD < 0 .OR. DiaryD > 1)               CALL Fatal(35)      ! DiaryD between 0 an 1  
   IF (NumODistricts==0 .AND. SampleLocs==YES)   CALL Fatal(36)      ! Num dists must be defined 
   IF (ANY(MList(1:NumMicrosPr)>NumMicros))      CALL Fatal(37)      ! Bad micro num in list
   IF (TimeStepsPerDay>24.AND.MOD(TimeStepsPerDay,24) .NE. 0)       &
      CALL Fatal(38)                                                 ! Integer timesteps/hour req'd
   IF (TimeStepsPerDay<24.AND.MOD(24,TimeStepsPerDay) .NE. 0)       &
      CALL Fatal(38)                                                 ! tsteps must be multiple of hour
   IF (TimeStepsPerDay.NE.24 .AND. ModelAQVar==YES)                 &
      CALL Fatal(39)                                                 ! Must be 24 steps/day for AQ var
   IF (ANY(Pol(:)%NumSources <  0))              CALL Fatal(3)       ! # sources must not be negative 
   DO I=1, NumPollutants                                             ! Loop over pollutants
      IF (POL(I)%PM == YES .AND. POL(I)%Size == 0) CALL Fatal(40)    ! Each PM pollut must have a size             
      IF (POL(I)%PM == YES .AND. POL(I)%Density == 0) CALL Fatal(41) ! PM polluts must have a density             
      IF (POL(I)%PM == YES .AND. POL(I)%InUnits .NE. ugm3)          &
         CALL Warning(4)                                             ! PM polluts must be "ug/m3"            
      IF (POL(I)%PM == YES .AND. POL(I)%OutUnits .NE. ugm3)         &
         CALL Warning(4)                                             ! PM polluts must be "ug/m3"            
   ENDDO                                                             ! End loop over pollutants
   IF (LongitDiary==YES) THEN
      IF (CLUSTDIARYB==YES.OR.CLUSTDIARYA==YES) CALL Fatal(42)
   ENDIF
   IF (UseOccGroups==YES .AND. OccFactor==NO) CALL Fatal(46)         ! User wants matching, doesn't specify groups
   ProcLevel = ProcLevel-1                                           ! Decrement procedure level
   IF (DebugLevel>0) WRITE(LogU,ST_FMT) ST_MSG, ProcName(ProcLevel)  ! Message for debugging
END SUBROUTINE CheckParameters

SUBROUTINE OutputParameters                                          ! Called from main APEX program
   INTEGER (KIND=IK) :: I                                            ! Local integer loop index
   ! Global constants: AQU, ChEvU, ChMapU, ChQuU, CommU, DistU, EmpU, ProU, HDoseU, HExpU, IK,
   !                   LogU, MetsU, MicroU, MonU, MResU, MSumU, ParU, PhysU, PSumU, NUMINPUTS
   !                   SectU, SiteU, TabU, TempU, UNITTEXT, VentU, YES, YESNO, ZoneU     
  ! Global vars used:  ActivePAI, AgeCutPct, AgeMin, AgeMax. Age2Probab, AirRadius, Altitude, 
   !                   Center, ChildMin, ChildMax, CityRadius, CList, Clock, ClockSeed, CommuteFlag,
   !                   CountyList, DebugLevel, DoDose, DSTAdjust, EndDate, FName, HeavyEVR1, HeavyEVR8, HourList 
   !                   DailyOut, HourlyOut, InUnits, ISeed, KeepLeavers, LeaverAdd, LeaverMult, Location, LogDistrict,
   !                   LogPopulate, LogProfiles, LogSectors, LogTables, MissAge, MissEmpl, MissGender,
   !                   ModEVR1, ModEVR8, MResOut, MSumOut, NumCounties, NumMicros, NumPopTypes, 
   !                   NumProfiles, NumSources, OutUnits, Pollutant, PopCat, PSumList, RBBkgnd, RBFactor, 
   !                   Rollback, Scenario, SectU, StartDate, TempU, TList, VAoutput, ZoneRadius    
   ! Global vars set : Ios, ProcLevel, ProcName
   ! Intrinsic procs : TRIM, WRITE
   ProcLevel = ProcLevel+1                                           ! Increment procedure depth 
   ProcName(ProcLevel) = "OutputParameters"                          ! Procedure name for messages
   IF (DebugLevel>0) WRITE(LogU,ST_FMT) ST_MSG, ProcName(ProcLevel)  ! Message for debugging
   WRITE(LogU,*) "                              "                    ! --  Job information -- 
   WRITE(LogU,'(A16,A)') "  Location    = ", TRIM(Location)          ! Location (echoed from infile) 
   WRITE(LogU,'(A16,A)') "  Pollutant   = ", TRIM(PolList)           ! Pollutants simulated 
   WRITE(LogU,'(A16,A)') "  Scenario    = ", TRIM(Scenario)          ! Scenario (echoed from infile) 
   WRITE(LogU,*) "  Start date  = ", StartDate                       ! Start date of simulation 
   WRITE(LogU,*) "  End date    = ", EndDate                         ! End data of simulation  
   WRITE(LogU,*)                                                     ! Write blank line to log
   WRITE(LogU,*) "Files used for this run:"                          ! List of file names
   WRITE(LogU,*) "Input:"                                            ! Input files:
   WRITE(LogU,*) "  Simulation          = ",TRIM(FName(ParU))        ! Main job parameters input file
   WRITE(LogU,*) "  Pop. Sectors        = ",TRIM(FName(SectU))       ! Used in SpaceTimeModule
   WRITE(LogU,*) "  Air Districts       = ",TRIM(FName(MonU))        ! Used in SpaceTimeModule
   WRITE(LogU,*) "  Meteorology Zones   = ",TRIM(FName(ZoneU))       ! Used in SpaceTimeModule
   WRITE(LogU,*) "  Meteorology Data    = ",TRIM(FName(TempU))       ! Used in SpaceTimeModule
   DO I=1, NumPollutants                                             ! Loop over pollutants
      WRITE(LogU,*) "  Air Quality Data    = ",TRIM(Pol(I)%AQName)   ! Used in SpaceTimeModule
   ENDDO                                                             ! End loop over pollutants   
   WRITE(LogU,*) "  METS distributions  = ",TRIM(FName(MetsU))       ! Used in DiaryModule 
   WRITE(LogU,*) "  Physiology          = ",TRIM(FName(PhysU))       ! Used in PhysiologyModule
   WRITE(LogU,*) "  Ventilation         = ",TRIM(FName(VentU))       ! Used in PhysiologyModule
   WRITE(LogU,*) "  Employment          = ",TRIM(FName(EmpU))        ! Used in ProfileModule
   IF (UseProfileFactors == YES)                                    &
      WRITE(LogU,*) "  Profile Factors     = ",TRIM(FName(ProU))     ! Used in ProfileModule   
   WRITE(LogU,*) "  Commuting           = ",TRIM(FName(CommU))       ! Used in ProfileModule
   WRITE(LogU,*) "  Commuting Time      = ",TRIM(FName(CommTU))      ! Used in ProfileModule   
   WRITE(LogU,*) "  Distributions       = ",TRIM(FName(DistU))       ! Used in DistributionModule
   WRITE(LogU,*) "  Cluster L.Mapping   = ",TRIM(FName(ClMapU))      ! Used in DiaryModule    
   WRITE(LogU,*) "  Location Mapping    = ",TRIM(FName(ChMapU))      ! Used in DiaryModule 
   WRITE(LogU,*) "  Diary Events        = ",TRIM(FName(ChEvU))       ! Used in DiaryModule 
   WRITE(LogU,*) "  Diary Questionnaire = ",TRIM(FName(ChQuU))       ! Used in DiaryModule  
   WRITE(LogU,*) "  Diary Statistics    = ",TRIM(FName(StatU))       ! Used in DiaryModule  
   WRITE(LogU,*) "  Diary Employment    = ",TRIM(FName(DEmpU))       ! Used in DiaryModule  
   WRITE(LogU,*) "  Microenvironments   = ",TRIM(FName(MicroU))      ! Used in MicroEnvModule
   WRITE(LogU,*) "  Disease Prevalence  = ",TRIM(FName(PrevU))       ! Used in DiaryModule
   WRITE(LogU,*) "Output:"                                           ! Output files:
   WRITE(LogU,*) "  Log                 = ",TRIM(FName(LogU))        ! Used by all modules
   WRITE(LogU,*) "  Hourly File         = ",TRIM(FName(HourU))       ! Used in ExposureDoseModule
   WRITE(LogU,*) "  Daily File          = ",TRIM(FName(DayU))        ! Used in ExposureDoseModule
   WRITE(LogU,*) "  Profile Summary     = ",TRIM(FName(PSumU))       ! Used in ExposureDoseModule
   WRITE(LogU,*) "  Micro. Summary      = ",TRIM(FName(MSumU))       ! Used in ExposureDoseModule
   WRITE(LogU,*) "  Micro. Results      = ",TRIM(FName(MResU))       ! Used in MicroEnvModule
   WRITE(LogU,*) "  Summary tables      = ",TRIM(Fname(TabU))        ! Used in ExposureDoseModule
   WRITE(LogU,*) "  Site list           = ",TRIM(FName(SiteU))       ! Used in SpaceTimeModule
   WRITE(LogU,*) "  Events file         = ",TRIM(FName(EventU))      ! Used in ExposureDoseModule
   WRITE(LogU,*)                                                     ! Write blank line to log
   WRITE(LogU,*) "Default Values Assumed for Parameters:"            ! List of job parameter settings
   DO I=1, MAXINPUTS                                                 ! Loop ovr nonfile, nonlvl vars   
      IF (UseDef(I)==YES) Write(LogU,*) "  "//TRIM(VarNames(I))      ! If default used, write message
   ENDDO                                                             ! End loop of input vars       
   WRITE(LogU,*)
   WRITE(LogU,*) "Job Parameters:"                                   ! List of job parameter settings
   WRITE(LogU,*) "  # Personal profiles      = ", NumProfiles        ! Range: 1 - <no limit> 
   WRITE(LogU,*) "  # Microenvironments      = ", NumMicros          ! Range: 1 - 127
   WRITE(LogU,*) "  Age minimum              = ", AgeMin             ! Minimum age for simulations
   WRITE(LogU,*) "  Age maximum              = ", AgeMax             ! Maximum age for simulations
   WRITE(LogU,*) "  City center latitude     = ", Center%Lat         ! Latitude in decimal degrees
   WRITE(LogU,*) "  City center longitude    = ", Center%Lon         ! Longitude in decimal degrees
   WRITE(LogU,*) "  Study area radius (km)   = ", CityRadius         ! Study area radius in kilometers
   WRITE(LogU,*) "  List of counties         = ", YESNO(CountyList)  ! Sectors in specific counties
   WRITE(LogU,*) "  List of tracts           = ", YESNO(TractList)   ! Sectors in specific tracts
   WRITE(LogU,*) "  Commuting implemented    = ", YESNO(CommuteFlag) ! Commuting on or off  
   WRITE(LogU,*) "  Keep commuters who leave = ", YESNO(KeepLeavers) ! Model leavers or not
   WRITE(LogU,*) "  Mult. factor for leavers = ", LeaverMult         ! Multiply average AQ by this
   WRITE(LogU,*) "  Add. term for leavers    = ", LeaverAdd          ! Add this to ambient AQ 
   IF (AirRadius/=99999) THEN
      WRITE(LogU,*) "  Air district radius (km) = ", AirRadius       ! Radius of air districts in km
   ENDIF
   WRITE(LogU,*) "  Nearby radius (km)       = ", NearbyRadius       ! Radius of nearby sample in km
   WRITE(LogU,*) "  Resample nearby?         = ", YESNO(ResampleNear)! Resample nearby districts?      
   WRITE(LogU,*) "  Temp. zone radius (km)   = ", ZoneRadius         ! Radius of temp. zones in km
   WRITE(LogU,*) "  Daylight Savings adjust  = ", YESNO(DSTAdjust)   ! All districts adjusted or none
   WRITE(LogU,*) "  Use Profile Factor       = ", YESNO(UseProfileFactors) ! Using profile factors?
   WRITE(LogU,*) "  Factor tied to occupation= ", YESNO(OccFactor)   ! Profile factor tied to employment?   
   WRITE(LogU,*) "  Save hourly results      = ", YESNO(HourlyOut)   ! Keep hourly info
   WRITE(LogU,*) "  Save daily results       = ", YESNO(DailyOut)    ! Keep daily info
   WRITE(LogU,*) "  Save micro summary       = ", YESNO(MSumOut)     ! Keep micro summary information 
   WRITE(LogU,*) "  Save micro results       = ", YESNO(MResOut)     ! Keep micro results information 
   WRITE(LogU,*) "  Save event-level output  = ", YESNO(EventsOut)   ! 1.5 MB per person-year
   WRITE(LogU,*) "  Number of event diaries  = ", EventSample        ! Number of 1.5 MB diaries saved
   WRITE(LogU,*) "  Usr-defined evnt diaries = ", CustProfs          ! User-defined profiles 
   WRITE(LogU,*) "  Write VA to output file  = ", YESNO(VAoutput)    ! Put VA on events file
   WRITE(LogU,*) "  Rollback                 = ", YESNO(Rollback)    ! All districts rolled or none
   WRITE(LogU,*) "  Rollback factor          = ", RBFactor           ! One factor for all districts
   WRITE(LogU,*) "  Rollback background      = ", RBBkgnd            ! One background for all districts 
   WRITE(LogU,*) "  Missing gender probab.   = ", MissGender         ! Range: 0.0 - 1.0
   WRITE(LogU,*) "  Missing age probab.      = ", MissAge            ! Range: 0.0 - 1.0            
   WRITE(LogU,*) "  Missing employ. probab.  = ", MissEmpl           ! Range: 0.0 - 1.0
   WRITE(LogU,*) "  Missing occup. probab.   = ", MissOcc            ! Range: 0.0 - 1.0
   WRITE(LogU,*) "  Age window width, pct.   = ", AgeCutPct          ! Range: 0.0 - <no limit> 
   WRITE(LogU,*) "  Age shoulder probab.     = ", Age2Probab         ! Range: 0.0 - 1.0  
   WRITE(LogU,*) "  Com1 shoulder probab.    = ", ComProbab1         ! Range: 0.0 - 1.0  
   WRITE(LogU,*) "  Com2 shoulder probab.    = ", ComProbab2         ! Range: 0.0 - 1.0     
   WRITE(LogU,*) "  Com1 window width, pct.  = ", ComWindow1         ! Range: 0.0 - <no limit> 
   WRITE(LogU,*) "  Com2 window width, pct.  = ", ComWindow2         ! Range: 0.0 - <no limit> 
   WRITE(LogU,*) "  Altitude (feet)          = ", Altitude           ! Range: 0.0 - <no limit>
   WRITE(LogU,*) "  Heavy 1-Hour EVR         = ", HeavyEVR1          ! Range: 0.0 - <no limit>
   WRITE(LogU,*) "  Heavy 8-Hour EVR         = ", HeavyEVR8          ! Range: 0.0 - <no limit>
   WRITE(LogU,*) "  Moderate 1-Hour EVR      = ", ModEVR1            ! Range: 0.0 - <no limit>
   WRITE(LogU,*) "  Moderate 8-Hour EVR      = ", ModEVR8            ! Range: 0.0 - <no limit>
   WRITE(LogU,*) "  Active PAI level         = ", ActivePAI          ! Range: 1.0 - <no limit>
   WRITE(LogU,*) "  Min child age (tables)   = ", ChildMin           ! Range: 0 - <no limit>
   WRITE(LogU,*) "  Max child age (tables)   = ", ChildMax           ! Range: 0 - <no limit>
   WRITE(LogU,*) "  Longitudinal diaries?    = ", YESNO(LongitDiary) ! Perform logitudinal diary assembly 
   WRITE(LogU,*) "  Diary target diversity   = ", DiaryD             ! Target D statistic,0-1
   WRITE(LogU,*) "  Diary Autocorrelation    = ", AutoC              ! Lag-1 autocorelation,-1 to 1
   WRITE(LogU,*) "  Clustering Algorithm A?  = ", YESNO(CLUSTDIARYA) ! Cluster Algorithm A?
   WRITE(LogU,*) "  Clustering Algorithm B?  = ", YESNO(CLUSTDIARYB) ! Cluster Algorithm B?
   WRITE(LogU,*) "  Re-calculate Cluster?    = ", YESNO(RERUNCLUS)   ! Recalculate Cluster Values?
   WRITE(LogU,*) "  Debug Level              = ", DebugLevel         ! Range: 0 - 1
   WRITE(LogU,*) "  Note each 1% of profiles?= ", YESNO(OnePercent)         
   WRITE(LogU,*) "  Seed from clock          = ", YESNO(ClockSeed)   ! Run either random or preset
   WRITE(LogU,*) "  Random number seed       = ", ISeed(1)           ! Range: 0 - (2**31-1)
   WRITE(LogU,*) "  Print districts on log   = ", YESNO(LogDistrict) ! List district info on log
   WRITE(LogU,*) "  Print populations on log = ", YESNO(LogPopulate) ! List population table on log
   WRITE(LogU,*) "  Print profiles on log    = ", YESNO(LogProfiles) ! List profile status on log
   WRITE(LogU,*) "  Print sectors on log     = ", YESNO(LogSectors)  ! List sector info on log
   WRITE(LogU,*) "  Print all tables on log  = ", YESNO(LogTables)   ! Show output tables on log
   WRITE(LogU,*) "  Print zones on log       = ", YESNO(LogZones)    ! List temperature zones on log
   WRITE(LogU,*) "  Pollutant Parameters:      "                     ! List pollut-specific params
   DO I= 1, NumPollutants
      WRITE(LogU,*) "  Pollutant =    ", TRIM(ADJUSTL(Pol(I)%Name))  ! List pollut-specific params   
      WRITE(LogU,*) "     Calculate doses          = ",             &
         YESNO(Pol(I)%DoDose)                                        ! Do dose for this pollut
      WRITE(LogU,*) "     # Enumerated Sources     = ",             &
         Pol(I)%NumSources                                           ! Numbered sources per micro
      WRITE(LogU,*) "     Input Units              = ",             &
         UNITTEXT(Pol(I)%InUnits)                                    ! Input units for pollutant    
      WRITE(LogU,*) "     Output Units             = ",             &
         UNITTEXT(Pol(I)%OutUnits)                                   ! Output units for pollutant
      WRITE(LogU,*) "     Using roadway concs.     = ",             &
         YESNO(Pol(I)%UseRoadAQ)                                     ! Roadway concs on or off     
      WRITE(LogU,*) "     Use H/W roadway concs.   = ",             &
         YESNO(Pol(I)%RoadLast)                                      ! Roadway concs for H/W locs?                   
   ENDDO                                                             ! End pollutant logic
   WRITE(LogU,*)                                                     ! Write blank line to log
   IF (CountyList==YES) THEN                                         ! If study area includes counties
      WRITE(LogU,*) "List of Counties included in study area"        ! Put header on log file
      DO I=1,NumCounties                                             ! Loop over included counties
         WRITE(LogU,*) I, '   ',CList(I)                             ! Put county FIPS code on log
      ENDDO                                                          ! Continue with next county
      WRITE(LogU,*)                                                  ! Put a blank line on log
   ENDIF                                                             ! End of logic block for counties
   IF (TractList==YES) THEN                                          ! If study area includes tracts
      WRITE(LogU,*) "List of Census Tracts included in study area"   ! Put header on log file
      DO I=1,NumTracts                                               ! Loop over included census tracts
         WRITE(LogU,*) I, '   ',TList(I)                             ! Put tract ID on log
      ENDDO                                                          ! Continue with next tract
      WRITE(LogU,*)                                                  ! Put a blank line on log
   ENDIF                                                             ! End of logic block for tracts
   DO I=1,NumPopTypes                                                ! Loop over population type
      WRITE(LogU,*) 'Population type ',I,PopCat(I)%Gender,'  ',     &
         PopCat(I)%Race,'  ',TRIM(PopCat(I)%FName)                   ! Write description to log file
   ENDDO                                                             ! Continue with next pop type
   IF (DebugLevel>0) WRITE(LogU,LV_FMT) LV_MSG, ProcName(ProcLevel)  ! Message for debugging
   ProcLevel = ProcLevel-1                                           ! Decrement procedure depth 
END SUBROUTINE OutputParameters                                      ! Return to main APEX program

INTEGER FUNCTION CheckKey(Key)
   CHARACTER(LEN=LEN3) :: Key                                        ! Keyword found in input files
   INTEGER  (KIND=IK)  :: I                                          ! Temporary integer variables
   ! Intrinsic procs : INDEX
   ! Global vars set: UseDef   
   ! Global variables used: MAXINPUTS, VarKey
   CheckKey=0                                                        ! Initialize CheckKey   
   IF (INDEX(Key,"LOG ")>0)        CheckKey=1                        ! Log file keyword found
   IF (INDEX(Key," FILE")>0) THEN                                    ! For file names...
      IF (INDEX(Key,"HOUR")>0)     CheckKey=1                        ! Hourly output keyword found
      IF (INDEX(Key,"DAILY")>0)    CheckKey=1                        ! Daily output keyword found
      IF (INDEX(Key,"TIMESTEP")>0) CheckKey=1                        ! Timestep output keyword found
      IF (INDEX(Key,"TABLE")>0)    CheckKey=1                        ! Output tables keyword found
      IF (INDEX(Key,"PREV")>0)     CheckKey=1                        ! Prevalence file keyword found
      IF (INDEX(Key,"PERSON")>0)   CheckKey=1                        ! Personal data keyword found
      IF (INDEX(Key,"SITE")>0)     CheckKey=1                        ! Site list keyword found
      IF (INDEX(Key,"EVENT")>0.AND.INDEX(Key,"DIARY")==0) CheckKey=1 ! Events filename keyword
      IF (INDEX(Key,"MICROSUM")>0) CheckKey=1                        ! Micro. summary keyword found
      IF (INDEX(Key,"MICRORES")>0) CheckKey=1                        ! Micro. results keyword found
      IF (INDEX(Key,"MICROENV")>0) CheckKey=1                        ! Microenvironment keyword found
      IF (INDEX(Key,"DIARYQUEST")>0) CheckKey=1                      ! Diary quest. keyword found
      IF (INDEX(Key,"DIARYEVE")>0) CheckKey=1                        ! Diary event keyword found
      IF (INDEX(Key,"MEMAP")>0)    CheckKey=1                        ! Location mapping keyword found
      IF (INDEX(Key,"COMMTIME")>0) CheckKey=1                        ! Commuting time keyword found            
      IF (INDEX(Key,"DIARYCLU")>0) CheckKey=1                        ! Location mapping keyword found      
      IF (INDEX(Key,"DIARYSTAT")>0)CheckKey=1                        ! Diary statistics keyword found
      IF (INDEX(Key,"DIARYOCC")>0) CheckKey=1                        ! Diary empl keyword found
      IF (INDEX(Key,"FUNCTIONS")>0)CheckKey=1                        ! Functions keyword found 
      IF (INDEX(Key,"DISTRIB")>0)  CheckKey=1                        ! Distribution keyword found 
      IF (INDEX(Key,"PHYSIOL")>0)  CheckKey=1                        ! Physiology keyword found
      IF (INDEX(Key,"METABOL")>0)  CheckKey=1                        ! Metabolic keyword found
      IF (INDEX(Key,"COMMUT")>0)   CheckKey=1                        ! Commuting keyword found
      IF (INDEX(Key,"QUALITY")>0)  CheckKey=1                        ! Air quality keyword found
      IF (INDEX(Key,"METEOR")>0)   CheckKey=1                        ! Meteorology keyword found
      IF (INDEX(Key,"SECTOR")>0)   CheckKey=1                        ! Pop. sectors keyword found
      IF (INDEX(Key,"DISTRICT")>0) CheckKey=1                        ! Air districts keyword found
      IF (INDEX(Key,"ZONE")>0)     CheckKey=1                        ! Temp zones keyword found
      IF (INDEX(Key,"EMPLOY")>0)   CheckKey=1                        ! Employment keyword found
      IF (INDEX(Key,"PROFILE")>0)  CheckKey=1                        ! Profile factor keyword found      
      IF (INDEX(Key,"VENTIL")>0)   CheckKey=1                        ! Ventilation keyword found                
      IF (INDEX(Key,"POP")>0)      CheckKey=1                        ! Pop file keyword found
      IF (INDEX(Key,"METSMAP")>0)  CheckKey=1                        ! METS map file keyword found
   ENDIF                                                             ! End of file logic   
   IF (INDEX(Key,"COUNTY ")>0)     CheckKey=1                        ! County keyword found
   IF (INDEX(Key,"DM1HEXP")>0)     CheckKey=1                        ! Found DM1HExp levels keyword
   IF (INDEX(Key,"DM8HEXP")>0)     CheckKey=1                        ! Found DM8HExp levels keyword 
   IF (INDEX(Key,"DAVGEXP")>0)     CheckKey=1                        ! Found DAvgExp levels keyword
   IF (INDEX(Key,"SAVGEXP")>0)     CheckKey=1                        ! Found SAvgExp levels keyword 
   IF (INDEX(Key,"DM1HDOSE")>0)    CheckKey=1                        ! Found DM1HDose levels keyword 
   IF (INDEX(Key,"DM8HDOSE")>0)    CheckKey=1                        ! Found DM8HDose levels keyword 
   IF (INDEX(Key,"DAVGDOSE")>0)    CheckKey=1                        ! Found DAvgDose levels keyword 
   IF (INDEX(Key,"SAVGDOSE")>0)    CheckKey=1                        ! Found SAvgDose levels keyword 
   IF (INDEX(Key,"DMEHDOSE")>0)    CheckKey=1                        ! Found DMEHDose levels keyword 
   IF (INDEX(Key,"TIMEDOSE")>0)    CheckKey=1                        ! Found TimeDose levels keyword  
   IF (INDEX(Key,"H_EHDOSE")>0)    CheckKey=1                        ! Found H_EHDose levels keyword
   IF (INDEX(Key,"PERCENTILE")>0)  CheckKey=1                        ! Found keyword for percentiles  
   IF (INDEX(Key,"TIMEEXP")>0)     CheckKey=1                        ! Found TimeExp levels keyword   
   IF (INDEX(Key,"TSEXP")>0)       CheckKey=1                        ! Found TimeExp levels keyword   
   IF (INDEX(Key,"TSDOSE")>0)      CheckKey=1                        ! Found TimeExp levels keyword   
   IF (INDEX(Key,"DMTSEXP")>0)     CheckKey=1                        ! Found TimeExp levels keyword   
   IF (INDEX(Key,"DMTSDOSE")>0)    CheckKey=1                        ! Found TimeExp levels keyword
   IF (INDEX(Key,"FEV1RISK")>0)    CheckKey=1                        ! Found FEV1Risk keyword
   IF (INDEX(Key,"FEVRISK")>0)     CheckKey=1                        ! Found FEV1Risk keyword    
   DO I=1, MAXINPUTS                                                 ! Loop over nonfile, nonlevel keys
      IF (INDEX(TRIM(UP(VarNames(I))),TRIM(Key))>0) THEN             ! Key matches an input keyword found
         CheckKey=1                                                  ! Return 1 for funct, key found           
         UseDef(I)=NO                                                ! Default not used for this variable           
         CYCLE                                                       ! Next input  
      ENDIF                                                          ! End key found logic 
   ENDDO                                                             ! End loop over keys           
END FUNCTION CheckKey                                                ! Return to calling subroutine      

SUBROUTINE SetSeeds(P)                                               ! Called from Main APEX module
   INTEGER (KIND=IK), INTENT(IN) :: P                                ! Input variable (profile number)
   INTEGER (KIND=IK)   :: I,J                                        ! Local integer variable
   INTEGER (KIND=IK)   :: Seed(MAXK)                                 ! Local integer variable
   REAL  (KIND=LONG), ALLOCATABLE :: R(:)                            ! Array for seeds
   ! Global vars used: KSeed 
   ! Global vars set: MPSeeds 
   ProcLevel = ProcLevel+1                                           ! Increment procedure depth 
   ProcName(ProcLevel) = "SetSeeds"                                  ! Procedure name for messages
   IF (DebugLevel>1) WRITE(LogU,ST_FMT) ST_MSG, ProcName(ProcLevel)  ! Message for debugging
   Seed(1:KSEED)=Person(P)%Seed;                                     ! Set seed for each person   
   CALL RANDOM_SEED(Put=Seed(1:KSeed))                               ! Set random number seed
   ALLOCATE(R(6+NumMP+MaxCorrNum),STAT=AllocErr)                     ! Buffer for required seeds
   CALL CheckAllocation(AllocErr)                                    ! CheckAllocation
   CALL RANDOM_NUMBER(R)                                             ! Set the seeds
   ISdAct  = 2147483648*R(1)                                         ! Set each word in ISdAct
   ISdDay  = 2147483648*R(2)                                         ! Set each word in ISdDay 
   ISdMETS = 2147483648*R(3)                                         ! Set each word in ISdMETS 
   ISdMic  = 2147483648*R(4)                                         ! Set each word in ISdMic
   ISdVent = 2147483648*R(5)                                         ! Set each word in ISdVent 
   ISdPhys = 2147483648*R(6)                                         ! Set each word in ISdPhys 
   DO I = 1, NumMP                                                   ! Loop over micro-parameter combs
     MPSeeds(I) = 2147483648*R(6+I)                                  ! Seed for MP combination     
      DO J=1, MaxCorrNum                                             ! Loop over correlation subsets
        IF (MPar(I)%CorrNum==J) MPSeeds(I) = 2147483648*R(6+NumMP+J) ! Set seeds for correlated vars 
      ENDDO                                                          ! End loop over subsets
   ENDDO                                                             ! End micro-parameter logic   
   IF (DebugLevel>1) WRITE(LogU,LV_FMT) LV_MSG, ProcName(ProcLevel)  ! Message for debugging
   ProcLevel = ProcLevel-1                                           ! Decrement procedure depth                         
END SUBROUTINE SetSeeds                                              ! Return to calling module

END PROGRAM                                                          ! End of main APEX program code