MODULE DistributionModule                                            ! Compile after modules in USE
   ! Latest revisions by WGG on April 11, 2012
   USE GlobalModule                                                  ! Must compile GlobalModule first
   USE UtilityModule                                                 ! Must compile UtilityModule next
   IMPLICIT NONE                                                     ! Must declare all variables

   TYPE :: PTRS                                                      ! Stores pointer arrays 
      INTEGER   (KIND=IK),  POINTER :: I(:) => NULL()                ! Pointer to integer table values 
      REAL      (KIND=RK),  POINTER :: R(:) => NULL()                ! Pointer to real table values
      CHARACTER (LEN=LEN1), POINTER :: C(:) => NULL()                ! Pointer to character table vals
   END TYPE PTRS                                                     ! Used to declare PA in distrib

   TYPE :: CDIST                                                     ! Stores continuous distribs
      REAL    (KIND=LONG) :: LPerc     = 0.0                         ! Percentile of max truncation 
      REAL    (KIND=LONG) :: UPerc     = 1.0                         ! Percentile of min truncation 
      REAL    (KIND=LONG) :: Shift     = 0.0                         ! General shift parameter 
      REAL    (KIND=LONG) :: Scale     = 0.0                         ! General scale parameter 
      REAL    (KIND=LONG) :: ShapeP1   = 0.0                         ! General shape parameter 1  
      REAL    (KIND=LONG) :: ShapeP2   = 0.0                         ! General shape parameter 2
      INTEGER (KIND=IK)   :: Shape     = 0                           ! Distribution type # (shape)    
      INTEGER (KIND=IK)   :: ResampOut = YES                         ! Resample outside truncation?
      REAL    (KIND=LONG) :: P1      = -9999                         ! First parameter for dist
      REAL    (KIND=LONG) :: P2      = 0.0                           ! Second parameter for dist
      REAL    (KIND=LONG) :: P3      = 0.0                           ! Third parameter for dist  
      REAL    (KIND=LONG) :: P4      = 0.0                           ! Fourth parameter for dist  
      REAL    (KIND=LONG) :: LTrunc    = 0.0                         ! Lower truncation limit
      REAL    (KIND=LONG) :: UTrunc    = 0.0                         ! Upper truncation limit
      REAL    (KIND=LONG) :: D(100)    = 0.0                         ! Discrete values
   END TYPE CDIST                                                    ! Used for continuous dists

   TYPE :: PFUNCTION                                                 ! Profile functions
      INTEGER  (KIND=IK)        :: Index           = 0               ! Index # for function
      CHARACTER(LEN=LEN2)       :: Name            = ""              ! Name of function
      INTEGER  (KIND=IK)        :: DistType        = 0               ! Function type 
      INTEGER  (KIND=IK)        :: ResultType      = 0               ! Type of result (integer or real) 
      INTEGER  (KIND=IK)        :: NDim            = 0               ! # input dimensions for tables
      INTEGER  (KIND=IK)        :: TLen(MAXDIM)    = 0               ! Extents of input dimensions 
      INTEGER  (KIND=IK)        :: VarType(MAXDIM) = 0               ! Type of each input dimension
      TYPE(PTRS)                :: PA(MAXDIM)                        ! Pointer array for table inputs
      INTEGER (KIND=IK),POINTER :: IRes(:,:,:,:,:,:) => NULL()       ! Pointer to integer results 
      REAL    (KIND=RK),POINTER :: RRes(:,:,:,:,:,:) => NULL()       ! Pointer to real results         
   END TYPE PFUNCTION                                                ! Used to declare FunctionList

   INTEGER (KIND=IK), PARAMETER :: CONDITIONAL=1, PROBABILITY=2,    &  
      REALRANGE=3, INTRANGE=4, INTVALUE=5, INTINDEX=6, CHARVALUE=7, &
      REGIONINDEX=8                                                  ! Allowed variable types
   INTEGER (KIND=IK), PARAMETER :: INTEG=1, HISTO=2, REALS=3         ! Allowed result types
   INTEGER (KIND=IK), PARAMETER :: DAILY=1, HOURLY=2, TStep=3        ! Allowed cond var variation rates
   INTEGER (KIND=IK)            :: NumDistrib, NumRDistrib           ! # of user distributions defined
   TYPE(PFunction), TARGET   :: FunctionList(MAXDISTRIB)             ! List of profile functions 
   TYPE(PFunction), ALLOCATABLE, TARGET  :: RegionFList(:,:)         ! List of regional profile funcs 
   
CONTAINS                                                             ! Routines internal to this module 


SUBROUTINE SetCVList                                                 ! Called from main APEX program  
   INTEGER (KIND=IK) :: I                                            ! Local integer variable
   ! Global constants: IK, LV_FMT, LV_MSG, NO, ST_FMT, ST_MSG, YES 
   ! Global vars used: DebugLevel
   ! Global vars set : CVList, NumCV, NumCVFixed, ProcLevel, ProcName
   ! Intrinsic procs : TRIM, WRITE
   ! APEX procedures : Up
   ProcLevel = ProcLevel+1                                           ! Increment procedure depth
   ProcName(ProcLevel) = "SetCVList"                                 ! Procedure name for messages
   IF (DebugLevel>0) WRITE(LogU,ST_FMT) ST_MSG, ProcName(ProcLevel)  ! Message for debugging
   NumCV           = 36                                              ! # of potential CV variables
   NumCVFixed      = 13                                              ! # of CV that do not change daily
   CVList(1)%Name  = "Gender"                                        ! Name of first CV variable
   CVList(2)%Name  = "PopCat"                                        !    Note that the order of the
   CVList(3)%Name  = "Employed"                                      !    CV variables is arbitrary,
   CVList(4)%Name  = "HasGasStove"                                   !    except that the array 
   CVList(5)%Name  = "HasGasPilot"                                   !    positions indicated here
   CVList(6)%Name  = "AC_Home"                                       !    must agree with the ones  
   CVList(7)%Name  = "AC_Car"                                        !    in Function CVValue.
   CVList(8)%Name  = "ProfileConditional1"                           !     
   CVList(9)%Name  = "ProfileConditional2"                           !     
   CVList(10)%Name = "ProfileConditional3"                           !      
   CVList(11)%Name = "ProfileConditional4"                           !      
   CVList(12)%Name = "ProfileConditional5"                           !      
   CVList(13)%Name = "FactorGroup"                                   ! Profile factor conditional
   CVList(14)%Name = "WindowRes"                                     !    
   CVList(15)%Name = "WindowCar"                                     !     
   CVList(16)%Name = "MaxTempCat"                                    !    The profile-level vars are
   CVList(17)%Name = "AvgTempCat"                                    !    #1-10.
   CVList(18)%Name = "SpeedCat"                                      !    The daily varying variables
   CVList(19)%Name = "DailyConditional1"                             !    are #11-#20. 
   CVList(20)%Name = "DailyConditional2"                             !    The hourly variables are
   CVList(21)%Name = "DailyConditional3"                             !    #21-#25.   
   CVList(22)%Name = "TempCat"                                       !    The regionally-varying 
   CVList(23)%Name = "HumidCat"                                      !    profile vars are #26-#30.
   CVList(24)%Name = "PrecipCat"                                     !     
   CVList(25)%Name = "WindCat"                                       !
   CVList(26)%Name = "DirCat"                                        !     
   CVList(27)%Name = "RegionalConditional1"                          !     
   CVList(28)%Name = "RegionalConditional2"                          !     
   CVList(29)%Name = "RegionalConditional3"                          !        
   CVList(30)%Name = "RegionalConditional4"                          !     
   CVList(31)%Name = "RegionalConditional5"                          !        
   CVList(32)%Name = "AQConditional1"                                ! Event-varying AQ1 category
   CVList(33)%Name = "AQConditional2"                                ! Event-varying AQ2 category
   CVList(34)%Name = "AQConditional3"                                ! Event-varying AQ3 category
   CVList(35)%Name = "AQConditional4"                                ! Event-varying AQ4 category
   CVList(36)%Name = "AQConditional5"                                ! Event-varying AQ5 category             
   CVList(1:3)%Supplied   = YES                                      ! These ones are supplied by APEX
   CVList(22:26)%Rate     = HOURLY                                   ! These are hourly variables
   CVList(32:36)%Rate     = TStep                                    ! These are event variables
   DO I=1,NumCV                                                      ! Loop over the CV List
      CVList(I)%Index = I                                            ! Set the index position
      CVList(I)%Name  = TRIM(Up(CVList(I)%Name))                     ! Put name in uppercase 
   ENDDO                                                             ! Continue with next item 
   IF (DebugLevel>0) WRITE(LogU,LV_FMT) LV_MSG, ProcName(ProcLevel)  ! Message for debugging
   ProcLevel = ProcLevel-1                                           ! Decrement procedure depth 
END SUBROUTINE SetCVList                                             ! Return to main APEX program 


SUBROUTINE SetCVLevels                                               ! Called from main APEX program  
   INTEGER (KIND=IK) :: I                                            ! Local integer variable
   ! Global constants: IK, LV_FMT, LV_MSG, ST_FMT, ST_MSG
   ! Global vars used: DebugLevel, NumAC_Car, NumAC_Home, NumAvgTemp, NumMaxTemp, NumPopTypes, 
   !                   NumSpeeds, NumWinCar, NumWinRes
   ! Global vars set : CVList, ProcLevel, ProcName
   ! Intrinsic procs : ANY, WRITE
   ! APEX procedures : Warning
   ProcLevel = ProcLevel+1                                           ! Increment procedure depth
   ProcName(ProcLevel) = "SetCVLevels"                               ! Procedure name for messages
   IF (DebugLevel>0) WRITE(LogU,ST_FMT) ST_MSG, ProcName(ProcLevel)  ! Message for debugging
   CVList(1)%Levels       = 2                                        ! # of possible genders
   CVlist(2)%Levels       = NumPopTypes                              ! # of possible pop categories
   CVList(3:5)%Levels     = 2                                        ! # of possible response values
   CVList(6)%Levels       = NumAC_Home                               ! # of possible home AC categories
   CVList(7)%Levels       = NumAC_Car                                ! # of possible car AC categories
   CVList(8)%Levels       = NumPCond1                                ! # of profile cond1 categories
   CVList(9)%Levels       = NumPCond2                                ! # of profile cond2 categories
   CVList(10)%Levels      = NumPCond3                                ! # of profile cond3 categories
   CVList(11)%Levels      = NumPCond4                                ! # of profile cond4 categories
   CVList(12)%Levels      = NumPCond5                                ! # of profile cond5 categories
   CVList(13)%Levels      = NumLevels                                ! # of Profile factor conditional
   CVList(14)%Levels      = NumWinRes                                ! # of home window positions
   CVList(15)%Levels      = NumWinCar                                ! # of car window positions
   CVList(16)%Levels      = NumMaxTemp                               ! # of max temperature categories
   CVList(17)%Levels      = NumAvgTemp                               ! # of avg temperature categories
   CVList(18)%Levels      = NumSpeeds                                ! # of speed categories
   CVList(19)%Levels      = NumDCond1                                ! # of daily cond1 categories
   CVList(20)%Levels      = NumDCond2                                ! # of daily cond2 categories
   CVList(21)%Levels      = NumDCond3                                ! # of daily cond3 categories
   CVList(22)%Levels      = NumTemp                                  ! # of temp categories
   CVList(23)%Levels      = NumHumid                                 ! # of humidity categories
   CVList(24)%Levels      = NumPrecip                                ! # of precip categories
   CVList(25)%Levels      = NumWind                                  ! # of windspeed categories
   CVList(26)%Levels      = NumDir                                   ! # of wind dir categories
   CVList(27)%Levels      = NumRCond1                                ! # of reg cond1 categories
   CVList(28)%Levels      = NumRCond2                                ! # of reg cond2 categories
   CVList(29)%Levels      = NumRCond3                                ! # of reg cond3 categories
   CVList(30)%Levels      = NumRCond4                                ! # of reg cond4 categories
   CVList(31)%Levels      = NumRCond5                                ! # of reg cond5 categories
   CVList(32)%Levels      = NumAQCond1                               ! # of AQ1 categories
   CVList(33)%Levels      = NumAQCond2                               ! # of AQ2 categories
   CVList(34)%Levels      = NumAQCond3                               ! # of AQ3 categories
   CVList(35)%Levels      = NumAQCond4                               ! # of AQ4 categories
   CVList(36)%Levels      = NumAQCond5                               ! # of AQ5 categories              
   DO I=1,NumCV                                                      ! Loop over the CV List
      WRITE(LogU,*) I,CVList(I)%Name,CVList(I)%Levels                ! Put # levels on log
   ENDDO                                                             ! Continue with next item 
   IF (DebugLevel>0) WRITE(LogU,LV_FMT) LV_MSG, ProcName(ProcLevel)  ! Message for debugging
   ProcLevel = ProcLevel-1                                           ! Decrement procedure depth 
END SUBROUTINE SetCVLevels                                           ! Return to main APEX program 


INTEGER FUNCTION CVValue(P,I,C,PL,L)                                 ! Called from MicroEnvModule
   INTEGER (KIND=IK), INTENT(IN) :: P, I ,C, PL,L                    ! Input parameters are integers
   INTEGER (KIND=IK) :: Z, D                                         ! Local integer variable
   ! Global constants: IK, LV_MS3, SL_FM3, ST_MS3
   ! Global vars used: DebugLevel, NumCV, Person, PersonDay, SList, TemData
   ! Global vars set : ProcLevel, ProcName
   ! Intrinsic procs : WRITE
   ! APEX procedures : Fatal
   CVValue = 1                                                       ! Default value
   IF (C==0) RETURN                                                  ! C=Zero means no CV selected 
   ProcLevel = ProcLevel+1                                           ! Increment procedure depth
   ProcName(ProcLevel) = "CVValue"                                   ! Procedure name for messages
   IF (DebugLevel>1) WRITE(LogU,SL_FM3) ST_MS3, ProcName(ProcLevel)  ! Message for debugging
   IF (C<0.OR.C>NumCV) CALL Fatal(1)                                 ! C must be one of the CV List
   Z = SList(Person(P)%HomeSec)%NumZ                                 ! Zone # for person P
   IF (L==1) D = Person(P)%HomeD                                     ! District # for person P
   IF (L==2) D = Person(P)%WorkD                                     ! District # for person P
   IF (L==3) D = 0                                                   ! District # for person P
   IF (L==4) THEN                                                    ! District # for person P   
       IF (Pol(PL)%UseRoadAQ==YES) THEN                              ! If Roadway L, then 
          D = Person(P)%RoadD                                        ! Place #4 is either roadway district
       ELSE                                                          ! Or if it's not
          D = 0                                                      ! It's zero
       ENDIF                                                         ! End conditional
   ENDIF                                                             ! End conditional   
   IF (L==5) D = SList(NHLocs(I))%NumD                               ! District # for near home locs
   IF (L==6) D = SList(NWLocs(I))%NumD                               ! District # for near work locs
   IF (L==7) THEN                                                    ! District # for road work locs
       IF (Pol(PL)%UseRoadAQ==YES) THEN                              ! If Roadway L, then 
          D = Person(P)%RoadWD                                       ! Place #7 is either roadway district
       ELSE                                                          ! Or if it's not
          D = 0                                                      ! It's zero
       ENDIF                                                         ! End conditional
   ENDIF                                                             ! End conditional 
   IF (C==1) CVValue  = Person(P)%Gender                             ! As mentioned in SetCVList above,
   IF (C==2) CVValue  = Person(P)%PopCat                             !    the items in this function
   IF (C==3) CVValue  = Person(P)%Employed                           !    must be in the same order 
   IF (C==4) CVValue  = Person(P)%GasStove                           !    as the ones in SetCVList.
   IF (C==5) CVValue  = Person(P)%GasPilot                           !    
   IF (C==6) CVValue  = Person(P)%AC_Home                            !    
   IF (C==7) CVValue  = Person(P)%AC_Car                             !    
   IF (C==8) CVValue  = Person(P)%PCond1                             ! Profile Cond1 for day I
   IF (C==9) CVValue  = Person(P)%PCond2                             ! Profile Cond2 for day I
   IF (C==10) CVValue = Person(P)%PCond3                             ! Profile Cond3 for day I
   IF (C==11) CVValue = Person(P)%PCond4                             ! Profile Cond4 for day I
   IF (C==12) CVValue = Person(P)%PCond5                             ! Profile Cond5 for day I
   IF (C==13) CVValue = Person(P)%FactorGroup                        ! Group number for person
   IF (C==14) CVValue = PersonDay(I)%WindowRes                       ! Window residence result for day I   
   IF (C==15) CVValue = PersonDay(I)%WindowCar                       ! Window car result for day I
   IF (C==16) CVValue = TemData(I,Z)%TempCatM                        ! MaxTempCat for day I, person P  
   IF (C==17) CVValue = TemData(I,Z)%TempCatA                        ! AvgTempCat for day I, person P  
   IF (C==18) CVValue = PersonDay(I)%SpeedCat                        ! Vehicle speed category for day I
   IF (C==19) CVValue = PersonDay(I)%DCond1                          ! Daily Cond1 for day I
   IF (C==20) CVValue = PersonDay(I)%DCond2                          ! Daily Cond2 for day I
   IF (C==21) CVValue = PersonDay(I)%DCond3                          ! Daily Cond3 for day I
   IF (C==22) CVValue = HourMtr(I,Z)%TempCat                         ! TempCat for hour I, person P
   IF (C==23) CVValue = HourMtr(I,Z)%HumidCat                        ! HumidCat for hour I, person P
   IF (C==24) CVValue = HourMtr(I,Z)%PrecipCat                       ! PrecipCat for hour I, person P  
   IF (C==25) CVValue = HourMtr(I,Z)%WindCat                         ! WindCat for hour I, person P  
   IF (C==26) CVValue = HourMtr(I,Z)%DirCat                          ! DirCat for hour I, person P  
   IF (C==27) CVValue = Person(P)%RCond1                             ! Region Cond1 for person P
   IF (C==28) CVValue = Person(P)%RCond2                             ! Region Cond2 for person P
   IF (C==29) CVValue = Person(P)%RCond3                             ! Region Cond3 for person P
   IF (C==30) CVValue = Person(P)%RCond4                             ! Region Cond4 for person P
   IF (C==31) CVValue = Person(P)%RCond5                             ! Region Cond5 for person P
   IF (C==32) CVValue = AQCond(PL,I,D)%AQCond1                       ! AQCond1, TS I, district D, pol PL
   IF (C==33) CVValue = AQCond(PL,I,D)%AQCond2                       ! AQCond2, TS I, district D, pol PL
   IF (C==34) CVValue = AQCond(PL,I,D)%AQCond3                       ! AQCond3, TS I, district D, pol PL
   IF (C==35) CVValue = AQCond(PL,I,D)%AQCond4                       ! AQCond4, TS I, district D, pol PL
   IF (C==36) CVValue = AQCond(PL,I,D)%AQCond5                       ! AQCond5, TS I, district D, pol PL
   IF ((CVValue<1 .OR. CVValue>CVList(C)%Levels) .AND.              &
      (INDEX(ProcName(ProcLevel-1),"WriteMResFile")==0)) THEN        ! Can write bad vals to MRes file 
      WRITE(LogU,*) C,CVValue,CVList(C)%Name,CVList(C)%Levels        ! Else put CV info on log
      CALL Fatal(2)                                                  ! Level must be in range
   ENDIF                                                             ! End logic for error
   IF (DebugLevel>1) WRITE(LogU,SL_FM3) LV_MS3, ProcName(ProcLevel)  ! Message for debugging
   ProcLevel = ProcLevel-1                                           ! Decrement procedure depth
END FUNCTION CVValue                                                 ! Return to MicroConcs subroutine 

SUBROUTINE ReadProfileFunctions                                      ! Called from main APEX module
   INTEGER (KIND=IK)   :: J, K, L(MAXDIM), DN, DLen, CV, LType, N    ! Local integer variables
   INTEGER (KIND=IK)   :: CondIndex, CondSize, CR, By=0, NRegions=0  ! More local integer variables 
   INTEGER (KIND=IK)   :: MaxNRegions=0, R, RCV=0, RegionFound, I    ! More local integer variables 
   INTEGER (KIND=IK)   :: DPFlag=0, CNum                             ! More local integer variables
   REAL    (KIND=RK)   :: Norm                                       ! Local real variables
   CHARACTER(LEN=LEN4) :: InString, Str1, Str2, DLenStr              ! Buffers of length LEN4
   CHARACTER(LEN=LEN1) :: DType, ByType, Region, Buf                 ! Buffers of length LEN1
   CHARACTER(LEN=LEN2) :: Compare, IStr                              ! Buffers of length LEN2
   CHARACTER(LEN=1)    :: Letter                                     ! Local variable for one letter
   TYPE(PFunction), TARGET :: D                                      ! Buffer to hold the distribution
   ! Global constants: DistU, FMT2, IK, LEN1, LEN3, LEN4, LogU, LV_FMT, LV_MSG, 
   !                   MAXDIM, MAXDISTRIB, RK, ST_FMT, ST_MSG 
   ! Global vars used: CVList, DebugLevel, FName, NumCV
   ! Global vars set : Ios, NumAC_Car, NumAC_Home, NumAvgTemp, NumMaxTemp, NumPools, NumSpeeds, 
   !                   NumWinCar, NumWinRes, ProcLevel, ProcName
   ! Module constants: TABLE, POINT, UNIFORM, NORMAL, LOGNORMAL, TRIANGLE, EXPONENTIAL, OFFON,
   !                   CONDITIONAL, PROBABILITY, REALRANGE, INTRANGE, INVALUE, INTINDEX,
   !                   INTEG, HISTO, REALS
   ! Module vars set : FunctionList, NumDistrib
   ! Intrinsic procs : ADJUSTL, ALLOCATE, CLOSE, HUGE, LOG, MOD, OPEN, PRODUCT, READ, SQRT,
   !                   SUM, TRIM, WRITE
   ! APEX procedures : CheckAllocation, Fatal, GetLine, OpenInput, Up
   ProcLevel = ProcLevel+1                                           ! Increment procedure depth
   ProcName(ProcLevel) = "ReadProfileFunctions"                      ! Procedure name for messages
   IF (DebugLevel>0) WRITE(LogU,ST_FMT) ST_MSG, ProcName(ProcLevel)  ! Message for debugging
   NumDistrib =  0                                                   ! Start with no functions
   NumRDistrib =  0                                                  ! Start with no regional funcs
   CALL OpenInput(DistU,h=NO)                                        ! Open distributions input file 
   WRITE(LogU,'(/A35/)') " List of user-defined distributions"       ! Put header line on log file
   DO                                                                ! Pre-read the functions file
      CALL GetLine(DistU,InString,LType)                             ! Read next valid input line
      IF (Ios(DistU)<0) EXIT                                         ! Exit DO loop on end of file
      IF (InString(1:2)=="BY") THEN                                  ! If BY statement
         READ(InString,*,IOSTAT=Ios(7)) Buf, ByType, NRegions        ! Read county/sector choice         
         IF (NRegions > MaxNRegions) MaxNRegions=NRegions            ! Max number of regions used
      ENDIF                                                          ! End BY statement logic       
   ENDDO                                                             ! End pre-read of file
   IF (MaxNRegions > 0) THEN                                         ! If regions are used in file
     ALLOCATE (RegionFList(MaxNRegions,5),STAT=AllocErr)             ! Space for regional data
     CALL CheckAllocation(AllocErr)                                  ! Check the success of the alloc     
   ENDIF                                                             ! End max # regions logic
   REWIND(DistU)                                                     ! Rewind file for second pass
   DO                                                                ! Loop over distributions on file
      D%ResultType = REALS                                           ! Default result type is real
      D%DistType   = 0                                               ! No default distrib type
      CALL GetLine(DistU,InString,LType)                             ! Read next valid input line
      IF (Ios(DistU)<0) EXIT                                         ! Exit DO loop on end of file
      D%Index = NumDistrib + 1                                       ! Set distrib number  
      IF (D%Index>MAXDISTRIB) CALL Fatal(1)                          ! Stop if too many distributions 
      IF (LType/=0) CALL Fatal(2)                                    ! Stop if name is invalid
      D%Name  = Up(InString)                                         ! Distrib name is first line
      DO CV=1, NumCV                                                 ! Loop over CV distributions
         IF (CVList(CV)%Name==D%Name) EXIT                           ! If dist name matches dist
      ENDDO                                                          ! Try next distribution in list
      IF (CV<=NumCV) CVList(CV)%Supplied = YES                       ! Set flag for user supplied dist.
      IF (CV>NumCV)  CV=0                                            ! Set CV to 0       
      D%DistType   = TABLE                                           ! Set distribution type
      D%NDim       = 0                                               ! Start with zero dimensions
      D%VarType(:) = 0                                               ! Set all variable types to NONE
      D%TLen       = 1                                               ! Set default lengths to 1
      L            = 1                                               ! L are extents of result table
      CondIndex    = 0                                               ! No conditional indices yet      
      IF (INDEX(D%Name,"REGION")>0) THEN                             ! If regional variable
         RCV = RCV+1                                                 ! Update regional CV counter
         IF (RCV > 5) CALL Fatal(26)                                 ! Too many regional CVs     
         CALL GetLine(DistU,InString,LType)                          ! Read next valid input line
         InString=TRIM(ADJUSTL(InString))                            ! Remove leading blanks
         IF (Ios(DistU)/=0) CALL Fatal(4)                            ! Stop if end of file
         READ(InString,*,IOSTAT=Ios(7)) Buf, ByType, NRegions        ! Read county/sector choice         
         IF (NRegions==0)  Call Fatal(27)                            ! # of counties/sectors missing
         IF (INDEX(ByType, "COUNTY")>0) THEN                         ! If indicating county
            By=2                                                     ! set by COUNTY               
         ELSE                                                        ! Otherwise
            BY=1                                                     ! set by SECTOR
         ENDIF                                                       ! End county/sector logic  
         IF (By == 0) CALL Fatal(28)                                 ! Missing BY statement                   
         CALL GetLine(DistU,InString,LType)                          ! Read next valid input line
         IF (InString(1:2)=="TA")                                   &
         CALL GetLine(DistU,InString,LType)                          ! Read next valid input line
         READ(InString(6:),*,IOSTAT=Ios(7)) DN,DType,DLen            ! Read data for new variable
         IF (DN>MAXDIM) CAll Fatal(5)                                ! Array bounds exceed MAXDIM
         IF (Ios(7)/=0) CALL Fatal(6)                                ! Stop if data are unreadable 
         IF (DN/=D%NDim+1) CALL Fatal(7)                             ! Stop if variables out of order
         D%NDim     = DN                                             ! Assign input variable number 
         D%TLen(DN) = DLen                                           ! Assign input variable extent 
         L(DN)      = DLen                                           ! Assign result extent
         IF (Up(DType(1:8))=="PROBABIL") THEN                        ! If probability
            D%VarType(DN) = PROBABILITY                              ! Assign value
         ELSEIF (Up(DType(1:8))=="REGIONIN") THEN                    ! If regionalindex
            D%VarType(DN) = REGIONINDEX                              ! Assign value
         ELSE                                                        ! If neither...
            CALL Fatal(29)                                           ! DType must be prob or regindex
         ENDIF                                                       ! End conditional logic    
         DO R=1, NRegions                                            ! Read data for all regions   
            ALLOCATE(D%PA(DN)%R(DLen),D%PA(DN)%I(0),                &
               STAT=AllocErr)                                        ! Make space for reals
            CALL CheckAllocation(AllocErr)                           ! Check the success of the alloc
            CALL GetLine(DistU,InString,LType)                       ! Read next valid input line
            IF(INDEX("0123456789",InString(1:1))==0)                &      
               CALL Fatal(31)                                        ! Non-numeric data found             
            IF (D%VarType(DN)==REGIONINDEX) THEN                     ! If type REGIONINDEX
               READ (InString,*,IOSTAT=Ios(DistU)) Region, N         ! Read Region, Index
               IF (N < 1 .OR. N > NRegions) CALL Fatal(33)               
               Str1 = ""                                             ! Initializing Str1
               Str2 = ""                                             ! Initializing Str2
               DO I=1,N-1                                            ! Create first string of 0's
                  Str1 = "0 " // Str1                                ! Add 0's to string
               ENDDO                                                 ! Finished creating first string
               DO I=N,NRegions                                       ! Create second string of 0's
                  Str2 = "0 " // Str2                                ! Add 0's to string
               ENDDO                                                 ! Finished creating second string
               InString = TRIM(Region) // " " // TRIM(Str1) // " 1 " &
                  // TRIM(Str2)                                      ! Concatenate full string
            ENDIF                                                    ! End REGIONINDEX conditional
            READ (InString,*,IOSTAT=Ios(DistU)) Region, D%PA(DN)%R   ! Read probability data
            IF (Ios(DistU)/=0) CALL Fatal(12)                        ! Stop if problems reading data
            Norm = SUM(D%PA(DN)%R)                                   ! Sum of all probabilities 
            D%PA(DN)%R=D%PA(DN)%R/Norm                               ! Normalize probability to unity 
            DO J=2, DLen-1                                           ! Loop over probability values...
               D%PA(DN)%R(J)=D%PA(DN)%R(J)+D%PA(DN)%R(J-1)           ! Convert to cumulative probab.  
            ENDDO                                                    ! Do the next element
            D%PA(DN)%R(DLen)=1.0                                     ! Eliminate any roundoff error         
            RegionFound = NO                                         ! Initialize region found flag   
            DO I=1, NumSectors                                       ! Loop over sectors 
               IF (By==1) THEN                                       ! When using sectors 
                  Compare =  SList(I)%Name                           ! Compare whole sector name  
               ELSE                                                  ! Otherwise using counties
                  Compare =  SList(I)%Name(1:5)                      ! Just compare FIPS code  
               ENDIF                                                 ! End by logic
               IF(TRIM(ADJUSTL(Region))==TRIM(ADJUSTL(Compare)))    &
                  THEN                                               ! If sector matches region
                  RegionFound   = YES                                ! Region is in sector list             
                  SList(I)%CV   = YES                                ! Update flag for this sector
                  SList(I)%NumR = R                                  ! Region number for this sector
               ENDIF                                                 ! End matching logic          
            ENDDO                                                    ! End loop over sectors      
            RegionFList(R,RCV) = D                                   ! Set function for this region
            IF (RegionFound == NO) CALL Warning(1)                   ! Warn if region not used
         ENDDO                                                       ! End reading regional data       
         IF (ANY(SList(1:NumSectors)%CV==NO)) CALL Fatal(30)         ! Sectors not assigned a region     
         SList(1:NumSectors)%CV=NO                                   !Reseting all to NO for next RC
         CALL GetLine(DistU,InString,LType)                          ! Read next valid input line
         IF (D%VarType(DN) == REGIONINDEX) THEN                      ! If RegionIndex
            IF (InString(1:1)/='#') CALL Fatal(24)                   ! Distribution ends improperly
            WRITE(DLenStr, '(i10)' ) DLen                            ! Create DLen string
            InString = "RESULT INTEGER " // TRIM(DLenStr)            ! Create result string
         ENDIF                                                       ! End conditional fatality
         IF (Ios(DistU)/=0) CALL Fatal(4)                            ! Stop if end of file
         IF (D%VarType(DN) == PROBABILITY) THEN                      ! If PROBABILITY type
            IF (Instring(1:2) .NE. "RE") CALL Fatal(34)              ! Results should start here
         ENDIF                                                       ! End probability logic
         READ(InString(8:),*,IOSTAT=Ios(8)) DType, DLen              ! Read result type and extent
         IF (Ios(8)/=0) CALL Fatal(18)                               ! Stop if unreadable result data
         SELECT CASE (Up(DType(1:3)))                                ! Find result type  
         CASE ("REA")                                                ! For real number results...
            IF (CV>0) CALL Fatal(19)                                 ! CVs must have integer results
            IF (DLen/=PRODUCT(L)) CALL Fatal(20)                     ! Stop if tables do not conform
            D%ResultType=REALS                                       ! Assign result type
            ALLOCATE (D%RRes(L(1),L(2),L(3),L(4),L(5),L(6)),        &
               STAT=AllocErr)                                        ! Make space for real results
            CALL CheckAllocation(AllocErr)                           ! Check the success of the alloc
            ALLOCATE (D%IRes(0,0,0,0,0,0),STAT=AllocErr)             ! No space needed for integers
            CALL CheckAllocation(AllocErr)                           ! Check the success of the alloc
            READ (DistU,*,IOSTAT=Ios(DistU)) D%RRes                  ! Read real results from file
         CASE ("INT")                                                ! For integer results 
            IF (DLen/=PRODUCT(L)) CALL Fatal(20)                     ! Stop if tables do not conform
            D%ResultType=INTEG                                       ! Assign result type
            IF (CV>0) CVList(CV)%Levels = DLen                       ! Set # of CV levels         
            ALLOCATE (D%IRes(L(1),L(2),L(3),L(4),L(5),L(6)),        &
               STAT=AllocErr)                                        ! Make space for integer results
            CALL CheckAllocation(AllocErr)                           ! Check the success of the alloc
            ALLOCATE (D%RRes(0,0,0,0,0,0),STAT=AllocErr)             ! No space needed for reals 
            CALL CheckAllocation(AllocErr)                           ! Check the success of the alloc
            IF (D%VarType(DN) == PROBABILITY) THEN                   ! If PROBABILITY...
               READ (DistU,*,IOSTAT=Ios(DistU)) D%IRes               ! Read integer results from file
            ELSE                                                     ! Else if REGIONINDEX...
               InString = ""                                         ! Initialize InString
               DO I=1,DLen                                           ! Loop over all regions
                  WRITE(IStr, '(i10)' ) DLen - I + 1                 ! Create integer string
                  InString = TRIM(IStr) // " " // InString           ! Concat integer strings
               ENDDO                                                 ! End region loop
               READ (InString,*,IOSTAT=Ios(DistU)) D%IRes            ! Read into new array
            ENDIF                                                    ! End regionindex logic
         CASE ("HIS")                                                ! For histogram results
            IF (CV>0) CALL Fatal(19)                                 ! CVs must have integer results
            IF (DLen/=L(1)) CALL Fatal(20)                           ! #results must match #bins
            IF (ANY(L(2:6)>1)) CALL Fatal(21)                        ! Must have only one index
            D%ResultType=HISTO                                       ! Assign result type
            ALLOCATE (D%RRes(0:L(1),1,1,1,1,1),STAT=AllocErr)        ! Make space for histo bounds
            CALL CheckAllocation(AllocErr)                           ! Check the success of the alloc
            ALLOCATE (D%IRes(0,0,0,0,0,0),STAT=AllocErr)             ! No space needed for integers
            CALL CheckAllocation(AllocErr)                           ! Check the success of the alloc
            READ (DistU,*,IOSTAT=Ios(DistU)) D%RRes                  ! Read real results from file 
         CASE DEFAULT                                                ! If result type is unknown
            CALL Fatal(22)                                           ! Print error message 
         END SELECT                                                  ! End of result type logic block   
         DO I = 1, NRegions                                          ! Loop over regions
            RegionFList(I,RCV)%ResultType=D%ResultType               ! Set result type for region 
            IF (D%ResultType==REALS) THEN                            ! If results are real  
               ALLOCATE  (RegionFList(I,RCV)                        &
                  %RRes(L(1),L(2),L(3),L(4),L(5),L(6)),             &
                  STAT=AllocErr)                                     ! Make space for real results
               CALL CheckAllocation(AllocErr)                        ! Check the success of the alloc
               ALLOCATE (RegionFList(I,RCV)                         &
                 %IRes(0,0,0,0,0,0),STAT=AllocErr)                   ! No space needed for integs 
               CALL CheckAllocation(AllocErr)                        ! Check the success of the alloc       
               RegionFList(I,RCV)%RRes = D%RRes                      ! Set results for regional                           
            ENDIF                                                    ! End real logic  
            IF (D%ResultType==INTEG) THEN                            ! If  results are integer
               ALLOCATE  (RegionFList(I,RCV)                        &
                  %IRes(L(1),L(2),L(3),L(4),L(5),L(6)),             &
                  STAT=AllocErr)                                     ! Make space for integer results
               CALL CheckAllocation(AllocErr)                        ! Check the success of the alloc
               ALLOCATE (RegionFList(I,RCV)                         &
                 %RRes(0,0,0,0,0,0),STAT=AllocErr)                   ! No space needed for reals 
               CALL CheckAllocation(AllocErr)                        ! Check the success of the alloc
               RegionFList(I,RCV)%IRes = D%IRes                      ! Set results for regional   
            ENDIF                                                    ! End integer logic  
         ENDDO                                                       ! End loop over regions           
         IF (Ios(DistU)/=0) CALL Fatal(23)                           ! Stop if problem reading results
         IF (D%VarType(DN) == PROBABILITY) THEN                      ! If a regional probability
            READ (DistU,*,IOSTAT=Ios(DistU)) Letter                  ! Check for termination sign '#'           
            IF (Letter/='#') CALL Fatal(24)                          ! Distribution ends improperly
         ENDIF                                                       ! End regional probability 
         NumRDistrib=NumRDistrib+1                                   ! Update distribution #           
         IF (D%VarType(DN) == PROBABILITY) CNum = MAXVAL(D%IRes)     ! Probab uses maxval
         IF (D%VarType(DN) == REGIONINDEX) CNum = DLen               ! Region uses region Num
         IF (TRIM(D%Name)=="REGIONALCONDITIONAL1") NumRCond1 = CNum  ! Set # of R Cond 1 categories  
         IF (TRIM(D%Name)=="REGIONALCONDITIONAL2") NumRCond2 = CNum  ! Set # of R Cond 2 categories  
         IF (TRIM(D%Name)=="REGIONALCONDITIONAL3") NumRCond3 = CNum  ! Set # of R Cond 3 categories  
         IF (TRIM(D%Name)=="REGIONALCONDITIONAL4") NumRCond4 = CNum  ! Set # of R Cond 4 categories  
         IF (TRIM(D%Name)=="REGIONALCONDITIONAL5") NumRCond5 = CNum  ! Set # of R Cond 5 categories  
         WRITE(LogU,*) 'Name = ',RegionFList(1,RCV)%Name             ! Write distrib name to log   
         WRITE(LogU,*) 'Index=', RegionFList(1,RCV)%Index            ! Write distrib number to log   
         WRITE(LogU,*) 'RCV #=', RCV                                 ! Write CV position to log
         WRITE(LogU,*) 'Type =', RegionFList(1,RCV)%DistType         ! Write distrib type to log   
         WRITE(LogU,*) 'RType=', RegionFList(1,RCV)%ResultType       ! Write result type to log   
         WRITE(LogU,*) 'NDim =', RegionFList(1,RCV)%NDim             ! Write # indices to log
         WRITE(LogU,*) 'TLen =',                                    &
            RegionFList(1,RCV)%TLen(1:D%NDim)                        ! Write index lengths to log
         WRITE(LogU,*) 'VType=',                                    &
            RegionFList(1,RCV)%VarType(1:D%NDim)                     ! Write index types to log 
         WRITE(LogU,*)                                               ! Write blank line to log  
      ENDIF                                                          ! End if regional CV
      IF (INDEX(D%Name,"REGION")==0) THEN                            ! If NOT regional CV
         DO                                                          ! Loop over table information
            READ (DistU,FMT2,IOSTAT=Ios(DistU)) InString             ! Read next line of input
            IF (Ios(DistU)<0) CALL Fatal(3)                          ! Stop if file ends prematurely 
            IF (InString(1:5)=="TABLE") CYCLE                        ! TABLE identifier no longer used
            IF (Ios(DistU)/=0) CALL Fatal(4)                         ! Stop if end of file
            InString=TRIM(ADJUSTL(InString))                         ! Remove leading blanks
            IF (InString(1:1)=="!") CYCLE                            ! Drop comment lines
            IF (InString(1:5)=="INPUT") THEN                         ! If new input variable...
               READ(InString(6:),*,IOSTAT=Ios(7)) DN,DType,DLen      ! Read data for new variable
               IF (TRIM(D%Name)=="DIARYPOOLS") DPFlag=DPFlag+DN      ! Incrementing diary pools flag
               IF (DN>MAXDIM) CAll Fatal(5)                          ! Array bounds exceed MAXDIM    
               IF (Ios(7)/=0) CALL Fatal(6)                          ! Stop if data are unreadable 
               IF (DN/=D%NDim+1) CALL Fatal(7)                       ! Stop if variables out of order
               D%NDim     = DN                                       ! Assign input variable number 
               D%TLen(DN) = DLen                                     ! Assign input variable extent 
               L(DN)      = DLen                                     ! Assign result extent
               SELECT CASE (Up(DType(1:8)))                          ! Find indicated variable type
               CASE ("CONDITIO")                                     ! If var type is CONDITIONAL...
                  D%VarType(DN) = CONDITIONAL                        ! Assign type for input variable 
                  IF (DN==1) CALL Fatal(8)                           ! First var cannot be conditional
                  IF (CondIndex>0) CALL Fatal(9)                     ! Maximum is one conditional var 
                  CondIndex = DN                                     ! Assign index of conditional var
                  CondSize  = PRODUCT(L(1:DN-1))                     ! Extent of conditional input
                  IF (MOD(DLen,CondSize)/=0) CALL Fatal(10)          ! Stop if extents do not conform
                  CR        = DLen/CondSize                          ! Extent of conditional output
                  L(1:DN-1) = 1                                      ! Reassign extent of result table
                  L(DN)     = CR                                     ! Extent of conditional result
                  ALLOCATE (D%PA(DN)%R(DLen),D%PA(DN)%I(0),         & 
                     STAT=AllocErr)                                  ! Make space for reals
                  CALL CheckAllocation(AllocErr)                     ! Check the success of the alloc
                  READ (DistU,*,IOSTAT=Ios(DistU)) D%PA(DN)%R        ! Read cond. probability data
                  IF (Ios(DistU)/=0) CALL Fatal(11)                  ! Stop if problems reading data
                  DO K=0,DLen-CR, CR                                 ! Loop over rows of data
                     Norm = SUM(D%PA(DN)%R(K+1:K+CR))                ! Sum of each probability row 
                     D%PA(DN)%R(K+1:K+CR)=D%PA(DN)%R(K+1:K+CR)/Norm  ! Normalize probability to unity
                     DO J=K+2, K+CR-1                                ! For elements in each row...
                        D%PA(DN)%R(J)=D%PA(DN)%R(J)+D%PA(DN)%R(J-1)  ! Convert to cumulative probab.
                     ENDDO                                           ! Do the next element
                     D%PA(DN)%R(K+CR)=1.0                            ! Eliminate any roundoff error
                  ENDDO                                              ! Finished row of probabilities
               CASE ("PROBABIL")                                     ! If var type is PROBABILITY...
                  D%VarType(DN) = PROBABILITY                        ! Assign type for input variable
                  ALLOCATE(D%PA(DN)%R(DLen),D%PA(DN)%I(0),          &
                     STAT=AllocErr)                                  ! Make space for reals
                  CALL CheckAllocation(AllocErr)                     ! Check the success of the alloc
                  READ (DistU,*,IOSTAT=Ios(DistU)) D%PA(DN)%R        ! Read probability data
                  IF (Ios(DistU)/=0) CALL Fatal(12)                  ! Stop if problems reading data
                  Norm = SUM(D%PA(DN)%R)                             ! Sum of all probabilities 
                  D%PA(DN)%R=D%PA(DN)%R/Norm                         ! Normalize probability to unity 
                  DO J=2, DLen-1                                     ! Loop over probability values...
                    D%PA(DN)%R(J)=D%PA(DN)%R(J)+D%PA(DN)%R(J-1)      ! Convert to cumulative probab.  
                  ENDDO                                              ! Do the next element
                  D%PA(DN)%R(DLen)=1.0                               ! Eliminate any roundoff error
               CASE ("REALRANG")                                     ! If var type is REALRANGE...
                  D%VarType(DN) = REALRANGE                          ! Assign type for input variable
                  ALLOCATE (D%PA(DN)%R(DLen-1),D%PA(DN)%I(0),       &
                     STAT=AllocErr)                                  ! Make space for reals
                  CALL CheckAllocation(AllocErr)                     ! Check the success of the alloc
                  IF(DLen>1) READ (DistU,*,IOSTAT=Ios(DistU))       & 
                     D%PA(DN)%R(1:DLen-1)                            ! Read all but last cutpoint
                  IF (Ios(DistU)/=0) CALL Fatal(13)                  ! Stop if trouble reading data
               CASE ("INTRANGE")                                     ! If var type is INTRANGE...
                  D%VarType(DN) = INTRANGE                           ! Assign type for input variable
                  ALLOCATE (D%PA(DN)%I(DLen-1),D%PA(DN)%R(0),       & 
                     STAT=AllocErr)                                  ! Make space for integers
                  CALL CheckAllocation(AllocErr)                     ! Check the success of the alloc
                  IF(DLen>1) READ (DistU,*,IOSTAT=Ios(DistU))       & 
                     D%PA(DN)%I(1:DLen-1)                            ! Read all but last cutpoint 
                  IF (Ios(DistU)/=0) CALL Fatal(14)                  ! Stop if trouble reading data 
               CASE ("INTVALUE")                                     ! If var type is INTVALUE...
                  D%VarType(DN) = INTVALUE                           ! Assign type for input variable
                  ALLOCATE (D%PA(DN)%I(DLen),D%PA(DN)%R(0),         &
                     STAT=AllocErr)                                  ! Make space for integers
                  CALL CheckAllocation(AllocErr)                     ! Check the success of the alloc
                  READ (DistU,*,IOSTAT=Ios(DistU)) D%PA(DN)%I        ! Read all values
                  IF (Ios(DistU)/=0) CALL Fatal(15)                  ! Stop if trouble reading data 
               CASE ("CHARVALU")                                     ! If var type is CHARVALUE...
                  D%VarType(DN) = CHARVALUE                          ! Assign type for input variable
                  ALLOCATE (D%PA(DN)%C(DLen),STAT=AllocErr)          ! Make space for characters
                  CALL CheckAllocation(AllocErr)                     ! Check the success of the alloc
                  READ (DistU,*,IOSTAT=Ios(DistU)) D%PA(DN)%C        ! Read all values
                  IF (Ios(DistU)/=0) CALL Fatal(16)                  ! Stop if trouble reading data 
               CASE ("INTINDEX")                                     ! INTINDEX needs no values 
                  D%VarType(DN) = INTINDEX                           ! Assign type for input variable
               CASE DEFAULT                                          ! Unknown variable type
                  CALL Fatal(17)                                     ! Stop if unknown variable type
               END SELECT                                            ! End of SELECT CASE J block 
            ENDIF                                                    ! End input logic
            IF (InString(1:6)=="RESULT") THEN                        ! Start of TABLE RESULT block
               READ(InString(8:),*,IOSTAT=Ios(8)) DType, DLen        ! Read result type and extent
               IF (Ios(8)/=0) CALL Fatal(18)                         ! Stop if unreadable result data
               SELECT CASE (Up(DType(1:3)))                          ! Find result type  
               CASE ("REA")                                          ! For real number results...
                  IF (CV>0) CALL Fatal(19)                           ! CVs must have integer results
                  IF (DLen/=PRODUCT(L)) CALL Fatal(20)               ! Stop if tables do not conform
                  D%ResultType=REALS                                 ! Assign result type
                  ALLOCATE (D%RRes(L(1),L(2),L(3),L(4),L(5),L(6)),  &
                     STAT=AllocErr)                                  ! Make space for real results
                  CALL CheckAllocation(AllocErr)                     ! Check the success of the alloc
                  ALLOCATE (D%IRes(0,0,0,0,0,0),STAT=AllocErr)       ! No space needed for integers
                  CALL CheckAllocation(AllocErr)                     ! Check the success of the alloc
                  READ (DistU,*,IOSTAT=Ios(DistU)) D%RRes            ! Read real results from file
               CASE ("INT")                                          ! For integer results                   
                  IF (DLen/=PRODUCT(L)) CALL Fatal(20)               ! Stop if tables do not conform
                  D%ResultType=INTEG                                 ! Assign result type
                  IF (CV>0) CVList(CV)%Levels = DLen                 ! Set # of CV levels
                  ALLOCATE (D%IRes(L(1),L(2),L(3),L(4),L(5),L(6)),  &
                     STAT=AllocErr)                                  ! Make space for integer results
                  CALL CheckAllocation(AllocErr)                     ! Check the success of the alloc
                  ALLOCATE (D%RRes(0,0,0,0,0,0),STAT=AllocErr)       ! No space needed for reals 
                  CALL CheckAllocation(AllocErr)                     ! Check the success of the alloc
                  READ (DistU,*,IOSTAT=Ios(DistU)) D%IRes            ! Read integer results from file
               CASE ("HIS")                                          ! For histogram results
                  IF (CV>0) CALL Fatal(19)                           ! CVs must have integer results
                  IF (DLen/=L(1)) CALL Fatal(20)                     ! #results must match #bins
                  IF (ANY(L(2:6)>1)) CALL Fatal(21)                  ! Must have only one index
                  D%ResultType=HISTO                                 ! Assign result type
                  ALLOCATE (D%RRes(0:L(1),1,1,1,1,1),STAT=AllocErr)  ! Make space for histo bounds
                  CALL CheckAllocation(AllocErr)                     ! Check the success of the alloc
                  ALLOCATE (D%IRes(0,0,0,0,0,0),STAT=AllocErr)       ! No space needed for integers
                  CALL CheckAllocation(AllocErr)                     ! Check the success of the alloc
                  READ (DistU,*,IOSTAT=Ios(DistU)) D%RRes            ! Read real results from file 
               CASE DEFAULT                                          ! If result type is unknown
                  CALL Fatal(22)                                     ! Print error message 
               END SELECT                                            ! End of result type logic block
               IF (Ios(DistU)/=0) CALL Fatal(23)                     ! Stop if problem reading results
               READ (DistU,*,IOSTAT=Ios(DistU)) Letter               ! Check for termination sign '#'
               IF (Letter/='#') CALL Fatal(24)                       ! Distribution ends improperly
               EXIT                                                  ! Exit DO, finished distribution
            ENDIF                                                    ! End of TABLE RESULT block
         ENDDO                                                       ! End of DO loop in CASE TABLE   
         IF (TRIM(D%Name)=="DIARYPOOLS") THEN
            NumPools   = MAXVAL(D%IRes)                              ! Set # of diary pools
            IF (DPFlag /= 6) CALL Fatal(32)                          ! Calls error if INPUTS undefined
         ENDIF
         IF (TRIM(D%Name)=="AC_HOME")    NumAC_Home = MAXVAL(D%IRes) ! Set # of home AC categories
         IF (TRIM(D%Name)=="AC_CAR")     NumAC_Car  = MAXVAL(D%IRes) ! Set # of car AC categories
         IF (TRIM(D%Name)=="PROFILECONDITIONAL1")                   &
            NumPCond1 = MAXVAL(D%IRes)                               ! Set # of P Cond 1 categories
         IF (TRIM(D%Name)=="PROFILECONDITIONAL2")                   &
            NumPCond2 = MAXVAL(D%IRes)                               ! Set # of P Cond 2 categories
         IF (TRIM(D%Name)=="PROFILECONDITIONAL3")                   &
            NumPCond3 = MAXVAL(D%IRes)                               ! Set # of P Cond 3 categories
         IF (TRIM(D%Name)=="PROFILECONDITIONAL4")                   &
            NumPCond4 = MAXVAL(D%IRes)                               ! Set # of P Cond 4 categories
         IF (TRIM(D%Name)=="PROFILECONDITIONAL5")                   &
            NumPCond5 = MAXVAL(D%IRes)                               ! Set # of P Cond 5 categories 
         IF (TRIM(D%Name)=="WINDOWRES")  NumWinRes  = MAXVAL(D%IRes) ! Set # of window positions (home)
         IF (TRIM(D%Name)=="WINDOWCAR")  NumWinCar  = MAXVAL(D%IRes) ! Set # of window positions (car)
         IF (TRIM(D%Name)=="MAXTEMPCAT") NumMaxTemp = MAXVAL(D%IRes) ! Set # of max temp categories
         IF (TRIM(D%Name)=="AVGTEMPCAT") NumAvgTemp = MAXVAL(D%IRes) ! Set # of avg temp categories       
         IF (TRIM(D%Name)=="AQCONDITIONAL1") NumAQCond1 = MAXVAL(D%IRes) ! Set # of AQ1 categories   
         IF (TRIM(D%Name)=="AQCONDITIONAL2") NumAQCond2 = MAXVAL(D%IRes) ! Set # of AQ2 categories   
         IF (TRIM(D%Name)=="AQCONDITIONAL3") NumAQCond3 = MAXVAL(D%IRes) ! Set # of AQ3 categories   
         IF (TRIM(D%Name)=="AQCONDITIONAL4") NumAQCond4 = MAXVAL(D%IRes) ! Set # of AQ4 categories   
         IF (TRIM(D%Name)=="AQCONDITIONAL5") NumAQCond5 = MAXVAL(D%IRes) ! Set # of AQ5 categories
         IF (TRIM(D%Name)=="SPEEDCAT")   NumSpeeds  = MAXVAL(D%IRes) ! Set # of speed categories
         IF (TRIM(D%Name)=="DAILYCONDITIONAL1")                     &
            NumDCond1 = MAXVAL(D%IRes)                               ! Set # of D Cond 1 categories
         IF (TRIM(D%Name)=="DAILYCONDITIONAL2")                     &
            NumDCond2 = MAXVAL(D%IRes)                               ! Set # of D Cond 2 categories
         IF (TRIM(D%Name)=="DAILYCONDITIONAL3")                     &          
            NumDCond3 = MAXVAL(D%IRes)                               ! Set # of D Cond 3 categories
         IF (TRIM(D%Name)=="TEMPCAT")      NumTemp  = MAXVAL(D%IRes) ! Set # of temp categories
         IF (TRIM(D%Name)=="HUMIDCAT")     NumHumid = MAXVAL(D%IRes) ! Set # of humidity categories
         IF (TRIM(D%Name)=="PRECIPCAT")    NumPrecip= MAXVAL(D%IRes) ! Set # of precip categories
         IF (TRIM(D%Name)=="WINDCAT")      NumWind  = MAXVAL(D%IRes) ! Set # of wind categories
         IF (TRIM(D%Name)=="DIRCAT")       NumDir   = MAXVAL(D%IRes) ! Set # of dircat categories
         IF (Ios(DistU)>0) THEN                                      ! If problems occured...
            WRITE(LogU,*) 'Error ',Ios,' Reading Distribution ',    &
               D%Name                                                ! Note which distrib failed
            EXIT                                                     ! Get out of subroutine   
         ELSE                                                        ! Else if no problems...
            NumDistrib=NumDistrib+1                                  ! Update distribution #   
            FunctionList(NumDistrib) = D                             ! Entire D structure copied
            WRITE(LogU,*) 'Name = ', D%Name                          ! Write distrib name to log   
            WRITE(LogU,*) 'Index=', D%Index                          ! Write distrib number to log   
            WRITE(LogU,*) 'CV # =', CV                               ! Write CV position to log
            WRITE(LogU,*) 'Type =', D%DistType                       ! Write distrib type to log   
            WRITE(LogU,*) 'RType=', D%ResultType                     ! Write result type to log   
            WRITE(LogU,*) 'NDim =', D%NDim                           ! Write # indices to log
            WRITE(LogU,*) 'TLen =', D%TLen(1:D%NDim)                 ! Write index lengths to log
            WRITE(LogU,*) 'VType=', D%VarType(1:D%NDim)              ! Write index types to log 
            WRITE(LogU,*)                                            ! Write blank line to log 
         ENDIF                                                       ! Finished distribution
      ENDIF                                                          ! End non-regional CV logic
   ENDDO                                                             ! Loop to next distribution
   CLOSE(DistU)                                                      ! Close input file
   WRITE(LogU,*) "# of diary pools      = ", NumPools                ! Report # diary pools to log
   WRITE(LogU,*) "# of home AC cats.    = ", NumAC_Home              ! Report # AC_Home levels to log
   WRITE(LogU,*) "# of car AC cats.     = ", NumAC_Car               ! Report # AC_Car levels to log
   WRITE(LogU,*) "# of categories, Profile Cond1 = ", NumPCond1      ! Report # categories for PCond1
   WRITE(LogU,*) "# of categories, Profile Cond2 = ", NumPCond2      ! Report # categories for PCond2
   WRITE(LogU,*) "# of categories, Profile Cond3 = ", NumPCond3      ! Report # categories for PCond3
   WRITE(LogU,*) "# of categories, Profile Cond4 = ", NumPCond4      ! Report # categories for PCond3
   WRITE(LogU,*) "# of categories, Profile Cond5 = ", NumPCond5      ! Report # categories for PCond3
   WRITE(LogU,*) "# of home window pos. = ", NumWinRes               ! Report # home window levels
   WRITE(LogU,*) "# of car window pos.  = ", NumWinCar               ! Report # car window levels
   WRITE(LogU,*) "# of max temp cats.   = ", NumMaxTemp              ! Report # groups max temperature 
   WRITE(LogU,*) "# of avg temp cats.   = ", NumAvgTemp              ! Report # groups avg temperature  
   WRITE(LogU,*) "# of categories, AQ Cond1 = ", NumAQCond1          ! Report # categories for AQ1   
   WRITE(LogU,*) "# of categories, AQ Cond2 = ", NumAQCond2          ! Report # categories for AQ2 
   WRITE(LogU,*) "# of categories, AQ Cond3 = ", NumAQCond3          ! Report # categories for AQ3 
   WRITE(LogU,*) "# of categories, AQ Cond4 = ", NumAQCond4          ! Report # categories for AQ4 
   WRITE(LogU,*) "# of categories, AQ Cond5 = ", NumAQCond5          ! Report # categories for AQ5             
   WRITE(LogU,*) "# of vehicle speeds   = ", NumSpeeds               ! Report # vehicle speed levels
   WRITE(LogU,*) "# of categories, Daily Cond1 = ", NumDCond1        ! Report # categories for DCond1
   WRITE(LogU,*) "# of categories, Daily Cond2 = ", NumDCond2        ! Report # categories for DCond2
   WRITE(LogU,*) "# of categories, Daily Cond3 = ", NumDCond3        ! Report # categories for DCond3
   WRITE(LogU,*) "# of vehicle speeds   = ", NumSpeeds               ! Report # vehicle speed levels
   WRITE(LogU,*) "# of vehicle speeds   = ", NumSpeeds               ! Report # vehicle speed levels
   WRITE(LogU,*) "# of categories, Regional Cond1 = ", NumRCond1     ! Report # categories for RCond1
   WRITE(LogU,*) "# of categories, Regional Cond2 = ", NumRCond2     ! Report # categories for RCond2
   WRITE(LogU,*) "# of categories, Regional Cond3 = ", NumRCond3     ! Report # categories for RCond3
   WRITE(LogU,*) "# of categories, Regional Cond4 = ", NumRCond4     ! Report # categories for RCond4
   WRITE(LogU,*) "# of categories, Regional Cond5 = ", NumRCond5     ! Report # categories for RCond5
   IF (DebugLevel>0) WRITE(LogU,LV_FMT) LV_MSG, ProcName(ProcLevel)  ! Message for debugging
   ProcLevel = ProcLevel-1                                           ! Decrement procedure depth
END SUBROUTINE ReadProfileFunctions                                  ! Return to main APEX program 


SUBROUTINE RnNor(N,Harvest,Min,Max)                                  ! <Called by several modules>
   INTEGER (KIND=IK), INTENT(IN)  :: N                               ! This routine generates 
   REAL    (KIND=RK), INTENT(OUT) :: Harvest(:)                      !   a bivariate normal by 
   REAL    (KIND=RK), INTENT(IN)  :: Min, Max                        !   transforming a point in 
   INTEGER (KIND=IK)              :: ISet, I                         !   a uniformly sampled
   REAL    (KIND=RK)              :: V(0:1), R2                      !   unit circle.
   ! Global constants: IK, RK
   ! Intrinsic procs : LOG, RANDOM_NUMBER, SQRT
   ISet=0                                                            ! Initialize even-odd flag
   I=0                                                               ! Initialize loop counter
   DO WHILE (I<N)                                                    ! Loop until enough found
      IF (ISet.EQ.0) THEN                                            ! If even get two new ones
         R2=2.                                                       ! Start with bad point
         DO WHILE ((R2>=1.0).OR.(R2==0.))                            ! End if in unit circle
            CALL RANDOM_NUMBER(V)                                    ! Get two uniform (0-1) #s
            V=2.*V-1.                                                ! Map onto square 
            R2=V(0)**2+V(1)**2                                       ! Find distance from origin
         END DO                                                      ! Loop for new point
         V = V * SQRT(-2.*LOG(R2)/R2)                                ! V is bivariate normal 
      END IF                                                         ! End of even condition
      ISet = 1-ISet                                                  ! Flip even-odd condition 
      IF ((V(ISet)>=Min).AND.(V(ISet)<=Max)) THEN                    ! If value within bounds...
         I=I+1                                                       ! Increment counter
         Harvest(I)=V(ISet)                                          ! Add random value to list 
      ENDIF                                                          ! Done with this value
   END DO                                                            ! Do next random value
END SUBROUTINE RnNor                                                 ! Return to calling routine

SUBROUTINE EvalFunc(Name,Profile,Ptr,R1,R2,R3,R4,R5,R6,             &
   I1,I2,I3,I4,I5,I6,C1,C2,C3,RRes,IRes)                             ! <Called by several modules>
   CHARACTER(LEN=LEN2), OPTIONAL, INTENT(IN) :: Name                 ! Name of desired distribution
   INTEGER(KIND=IK),    OPTIONAL, INTENT(IN) :: Profile              ! Current profile number
   TYPE(PFunction),     OPTIONAL, POINTER :: Ptr                     ! Pointer to distribution
   INTEGER(KIND=IK),    OPTIONAL, POINTER :: I1(:), I2(:), I3(:)     ! First 3 input vectors (integer)
   INTEGER(KIND=IK),    OPTIONAL, POINTER :: I4(:), I5(:), I6(:)     ! Next 3 input vectors (integer)
   REAL   (KIND=RK),    OPTIONAL, POINTER :: R1(:), R2(:), R3(:)     ! First 3 input vectors (real)
   REAL   (KIND=RK),    OPTIONAL, POINTER :: R4(:), R5(:), R6(:)     ! Next 3 input vectors (real)
   CHARACTER(LEN=LEN1), OPTIONAL, POINTER :: C1(:), C2(:), C3(:)     ! Character input vectors
   INTEGER(KIND=IK),    OPTIONAL, INTENT(OUT) :: IRes(:)             ! Integer output vector
   REAL   (KIND=RK),    OPTIONAL, INTENT(OUT) :: RRes(:)             ! Real output vector
   TYPE(PFunction),     POINTER :: D                                 ! Pointer to profile function
   TYPE(PTRS) :: P(MAXDIM)                                           ! Pointers to I1-I6 and R1-R6
   INTEGER(KIND=IK) :: I, J, K, Indx, N(1), L(MAXDIM)                ! Local integer variables -local variable SHAPE never referenced
   REAL   (KIND=RK), ALLOCATABLE :: TProb(:), R(:)                   ! Dynamic arrays of probabilities
   INTEGER(KIND=IK), ALLOCATABLE :: TInd(:,:)                        ! Dynamic array of indices
   ! Global constants: IK, LEN1, LogU, MAXDIM, RK, YES
   ! Global vars set : ProcLevel, ProcName
   ! Module constants: TABLE, POINT, UNIFORM, NORMAL, LOGNORMAL, TRIANGLE, EXPONENTIAL, OFFON,
   !                   CONDITIONAL, PROBABILITY, REALRANGE, INTRANGE, INVALUE, INTINDEX,
   !                   INTEG, HISTO, REALS, DISTRIBUTION, PTRS, CDIST
   ! Module vars used: FunctionList
   ! Intrinsic procs : ALLOCATE, DEALLOCATE, EXP, LBOUND, LOG, PRESENT, 
   !                   RANDOM_NUMBER, SQRT, TRIM, UBOUND
   ! APEX procedures : CheckAllocation, FuncNum, Fatal, RnNor
   ProcLevel = ProcLevel+1                                           ! Increment procedure depth
   ProcName(ProcLevel) = "EvalFunc"                                  ! Procedure name for messages
   IF (PRESENT(IRes)) THEN                                           ! If the results are integer...
      N = UBOUND(IRes)-LBOUND(IRes)+1                                ! Find number of values needed
      IRes = 0                                                       ! Initialize results to zero
   ELSE                                                              ! Else if results are real...
      N = UBOUND(RRes)-LBOUND(RRes)+1                                ! Find number of values needed 
      RRes = 0.                                                      ! Initialize results to zero
   ENDIF                                                             ! End of result type logic block
   ALLOCATE(R(N(1)),STAT=AllocErr)                                   ! Make space for random numbers
   CALL CheckAllocation(AllocErr)                                    ! Check the success of the alloc
   IF (Index(Name,"REGION")==0) THEN                                 ! Non-regional variable 
      D => FunctionList(FuncNum(Name,YES))                           ! Locate Name in FunctionList   
   ELSE                                                              ! Otherwise regional variable
      D => RegionFList(SList(Person(Profile)%HomeSec)%NumR,         &
         RegionFuncNum(Name))                                        ! Locate function for this person    
   ENDIF                                                             ! End regional logic
   IF(PRESENT(I1)) P(1)%I => I1                                      ! Load I1 into P array
   IF(PRESENT(I2)) P(2)%I => I2                                      ! Load I2 into P array, etc.
   IF(PRESENT(I3)) P(3)%I => I3                                      ! Note: P is an array of pointers
   IF(PRESENT(I4)) P(4)%I => I4                                      !       to all input variables.
   IF(PRESENT(I5)) P(5)%I => I5                                      !       It holds from 1 to 6 
   IF(PRESENT(I6)) P(6)%I => I6                                      !       variables, each either 
   IF(PRESENT(R1)) P(1)%R => R1                                      !       integer, real, or char-
   IF(PRESENT(R2)) P(2)%R => R2                                      !       each of arbitrary extent.
   IF(PRESENT(R3)) P(3)%R => R3                                      !       This avoids the need for 
   IF(PRESENT(R4)) P(4)%R => R4                                      !       large numbers of IF 
   IF(PRESENT(R5)) P(5)%R => R5                                      !       statements later on.
   IF(PRESENT(R6)) P(6)%R => R6                                      !  
   IF(PRESENT(C1)) P(1)%C => C1                                      !  
   IF(PRESENT(C2)) P(2)%C => C2                                      !  
   IF(PRESENT(C3)) P(3)%C => C3                                      ! End of loading the P array
   ALLOCATE (TInd(N(1),MAXDIM),STAT=AllocErr)                        ! Make space for array of indices
   CALL CheckAllocation(AllocErr)                                    ! Check the success of the alloc
   TInd(:,:) = 1                                                     ! Initialize all indices to one
   DO I=1,D%NDim                                                     ! Loop over table dimensions
      SELECT CASE(D%VarType(I))                                      ! Find the variable type
      CASE(CONDITIONAL)                                              ! For CONDITIONAL variables...
         L(1) = D%Tlen(1)                                            ! Initialize extent of result
         DO K=2,I-1                                                  ! Loop over other variables
            L(K) = D%TLen(K)*L(K-1)                                  ! Extent is product of extents
         ENDDO                                                       ! End of loop over variables
         L(I) = D%Tlen(I)/L(I-1)                                     ! Result extent is #table/#input
         ALLOCATE (TProb(L(I)),STAT=AllocErr)                        ! Make space for results
         CALL CheckAllocation(AllocErr)                              ! Check the success of the alloc
         CALL RANDOM_NUMBER(R)                                       ! Get vector of random uniforms
         DO J=1,N(1)                                                 ! Loop over # results needed
            Indx = L(I)*(TInd(J,1)-1)                                ! Index for conditional lookup
            DO K=2,I-1                                               ! Loop over conditional inputs
               Indx = Indx+L(I)*L(K-1)*(TInd(J,K)-1)                 ! Find total index
            ENDDO                                                    ! Do next conditional input
            TProb = D%PA(I)%R(Indx+1:Indx+L(I))                      ! Vector of result probabilities
            DO K=1,L(I)-1                                            ! Search probability vector
               IF (R(J)<TProb(K)) EXIT                               ! Exit if random # < probability
            ENDDO                                                    ! Try next probab. vector element
            TInd(J,I) = K                                            ! Assign index for this result
         ENDDO                                                       ! Do next result
         DEALLOCATE (TProb)                                          ! Free conditional probab. table
         TInd(:,1:I-1) = 1                                           ! Set previous indices to unity
      CASE(PROBABILITY)                                              ! For PROBABILITY variables...
         CALL RANDOM_NUMBER(R)                                       ! Get vector of random uniforms
         DO J=1,N(1)                                                 ! Loop over # results needed
            DO K=1,D%TLen(I)-1                                       ! Loop over cumul. probabilties
               IF (R(J)<D%PA(I)%R(K)) EXIT                           ! Exit if random # < cum. probab.
            ENDDO                                                    ! Try next cumulative probability
            TInd(J,I) = K                                            ! Assign index for this result
         ENDDO                                                       ! Do next result
      CASE(REGIONINDEX)                                              ! REGIONINDEX does same as PROBAB
         CALL RANDOM_NUMBER(R)                                       ! Get vector of random uniforms
         DO J=1,N(1)                                                 ! Loop over # results needed
            DO K=1,D%TLen(I)-1                                       ! Loop over cumul. probabilties
               IF (R(J)<D%PA(I)%R(K)) EXIT                           ! Exit if random # < cum. probab.
            ENDDO                                                    ! Try next cumulative probability
            TInd(J,I) = K                                            ! Assign index for this result
         ENDDO                                                       ! Do next result
      CASE(REALRANGE)                                                ! For REALRANGE variables...
         DO J=1,N(1)                                                 ! Loop over # results needed
            DO K=1,D%TLen(I)-1                                       ! Loop over real cutpoints
               IF (P(I)%R(J)<D%PA(I)%R(K)) EXIT                      ! Exit if input value < cutpoint
            ENDDO                                                    ! Try next cutpoint
            TInd(J,I) = K                                            ! Assign index for this result
         ENDDO                                                       ! Do next result
      CASE(INTRANGE)                                                 ! For INTRANGE variables...
         DO J=1,N(1)                                                 ! Loop over # results needed
            DO K=1,D%TLen(I)-1                                       ! Loop over integer cutpoints
               IF (P(I)%I(J)<D%PA(I)%I(K)) EXIT                      ! Exit if input value < cutpoint
            ENDDO                                                    ! Try next cutpoint
            TInd(J,I) = K                                            ! Assign index for this result 
         ENDDO                                                       ! Do next result
      CASE(INTVALUE)                                                 ! For INTVALUE variables...
         DO J=1,N(1)                                                 ! Loop over # results needed
            DO K=1,D%TLen(I)                                         ! Loop over allowed values
               IF (P(I)%I(J)==D%PA(I)%I(K)) EXIT                     ! Exit if input matches value
            ENDDO                                                    ! Try next allowed value
            IF (K>D%TLen(I)) THEN                                    ! If no match is found...
               WRITE(LogU,*) D%Name                                  ! Put distribution name on Log
               CALL Fatal(2)                                         ! Stop with error message
            ENDIF                                                    ! End of no match condition 
            TInd(J,I) = K                                            ! Assign index for this result 
         ENDDO                                                       ! Do next result
      CASE(CHARVALUE)                                                ! For CHARVALUE variables...
         DO J=1,N(1)                                                 ! Loop over # results needed
            DO K=1,D%TLen(I)                                         ! Loop over allowed values
               IF (TRIM(P(I)%C(J))==TRIM(D%PA(I)%C(K))) EXIT         ! Exit if input matches value
            ENDDO                                                    ! Try next allowed value
            IF (K>D%TLen(I)) THEN                                    ! If no match is found...
               WRITE(LogU,*) D%Name                                  ! Put distribution name on Log
               CALL Fatal(3)                                         ! Stop with error message
            ENDIF                                                    ! End of no match condition 
            TInd(J,I) = K                                            ! Assign index for this result 
         ENDDO                                                       ! Do next result
      CASE(INTINDEX)                                                 ! For INTINDEX variables...
         TInd(1:N(1),I) = P(I)%I(1:N(1))                             ! Copy input indices to result
      CASE DEFAULT                                                   ! For unknown variable types...
         WRITE(LogU,*) D%VarType(I)                                  ! Put variable type on Log
         CALL Fatal(4)                                               ! Stop with error message
      END SELECT                                                     ! End of TABLE VARIABLE logic
   ENDDO                                                             ! Do next TABLE dimension
   SELECT CASE(D%ResultType)                                         ! Find the result type
   CASE (INTEG)                                                      ! For INTEGER results
      DO J=1,N(1)                                                    ! Loop over vector of results
          IF (TInd(J,1) > D%TLen(1)) CALL Fatal(6)                   ! If array out of bounds
          IF (TInd(J,2) > D%TLen(1)) CALL Fatal(7)                   ! If array out of bounds
          IRes(J) = D%IRes(TInd(J,1),TInd(J,2),TInd(J,3),           &
            TInd(J,4),TInd(J,5),TInd(J,6))                           ! Lookup result from table                
      ENDDO                                                          ! Do next result    
   CASE (REALS)                                                      ! For REAL results
      DO J=1,N(1)                                                    ! Loop over vector of results
         RRes(J) = D%RRes(TInd(J,1),TInd(J,2),TInd(J,3),            &
            TInd(J,4),TInd(J,5),TInd(J,6))                           ! Lookup result from table
      ENDDO                                                          ! Do next result 
   CASE (HISTO)                                                      ! For HISTOGRAM results
      CALL RANDOM_NUMBER(R)                                          ! Get vector of random uniforms
      DO J=1,N(1)                                                    ! Loop over vector of results
         RRes(J) = D%RRes(TInd(J,1)  ,1,1,1,1,1)*R(J) +             &
                   D%RRes(TInd(J,1)-1,1,1,1,1,1)*(1.0-R(J))          ! Interpolate within chosen bin
      ENDDO                                                          ! Do next result          
   CASE DEFAULT                                                      ! If unknown result type 
      WRITE(LogU,*) D%ResultType                                     ! Put result type on log file
      CALL Fatal(5)                                                  ! Print error message
   END SELECT                                                        ! End of result type logic
   DEALLOCATE(TInd)                                                  ! Free index array for results
   DEALLOCATE(R)                                                     ! Free space for random numbers
   ProcLevel = ProcLevel-1                                           ! Decrement procedure depth
END SUBROUTINE EvalFunc                                              ! Return to calling routine

SUBROUTINE SampleDist(C,S)                                           ! Called from many places
   TYPE(CDIST), INTENT(IN)          :: C                             ! Input distribution definition
   REAL   (KIND=RK), INTENT(OUT)    :: S(:)                          ! Output vector of samples  
   REAL   (KIND=LONG), ALLOCATABLE  :: U(:)                          ! Dynamic array of unit uniforms
   REAL   (KIND=LONG)  :: A, B, C1, C2                               ! Local copies of dist parameters
   INTEGER (KIND=IK)   :: I, N(1), Bin                               ! Local integer variables 
   N = UBOUND(S)-LBOUND(S)+1                                         ! Find number of values needed
   ALLOCATE(U(N(1)),STAT=AllocErr)                                   ! Make space for random uniforms
   CALL CheckAllocation(AllocErr)                                    ! Check the success of the alloc
   CALL RANDOM_NUMBER(U)                                             ! Get vector of random uni 0-1
   IF (C%ResampOut==YES) THEN                                        ! Resampling outside truncation
      U  = C%LPerc + (C%UPerc-C%LPerc)*U                             ! Scale uniform to correct %tiles 
   ELSE                                                              ! Otherwise do not resample
      DO I= 1,N(1)                                                   ! Loop over number of samples
         U(I) = MIN(U(I),C%UPerc)                                    ! Move high values to upper limit
         U(I) = MAX(U(I),C%LPerc)                                    ! Move low values to lower limit
      ENDDO                                                          ! End loop over number of samples
   ENDIF                                                             ! End truncation logic
   A  = C%Shift                                                      ! Local copy of shift parameter
   B  = C%Scale                                                      ! Local copy of scale parameter
   C1 = C%ShapeP1                                                    ! Local copy of shape parameter 1            
   C2 = C%ShapeP2                                                    ! Local copy of shape parameter 2
   SELECT CASE (C%Shape)                                             ! Select distribution type 
   CASE (BETA)                                                       ! If type is BETA...
      DO I=1,N(1)                                                    ! Loop over number of samples
         S(I)=BetaInv(REAL(U(I),LONG),REAL(C1,LONG),REAL(C2,LONG))   ! Get general beta
      ENDDO                                                          ! End loop over samples
      S = A+B*S                                                      ! Shift and scale to final dist      
   CASE (BURR)                                                       ! If type is BURR...  
      S = A+B*((1-U)**(-1/C1)-1)**(1/C2)                             ! Transform to final dist     
   CASE (CAUCHY)                                                     ! If type is CAUCHY...
      S = A+B*TAN(PI*(U-0.5))                                        ! Transform to final dist     
   CASE (DISCRETE)                                                   ! If type is Discrete...
      DO I=1,N(1)                                                    ! Loop over number of samples
         Bin=FLOOR(C1*U(I))+1;                                       ! Bin number for this sample 
         S(I)=C%D(Bin)                                               ! Value for this sample   
      ENDDO                                                          ! End loop over samples
   CASE (EXPONENTIAL)                                                ! If type is EXPONENTIAL...
      S = A-B*LOG(1-U)                                               ! Transform to final dist     
   CASE (EVALUE)                                                     ! If type is EVALUE...
      S = A-B*LOG(-1*LOG(U))                                         ! Transform to final dist     
   CASE(GAMMA)                                                       ! If type is GAMMA... 
      DO I=1,N(1)                                                    ! Loop over number of samples
         S(I) = A+B*GammaInv(U(I),C1)                                ! Transform to final dist 
      ENDDO                                                          ! End loop over samples
   CASE(LGT)                                                         ! If type is LGT...
      S = A-B*LOG(1/(1-u)-1)                                         ! Transform to final dist 
   CASE(LOGNORMAL)                                                   ! If type is LOGNORMAL...
      DO I=1,N(1)                                                    ! Loop over number of samples
         S(I) = A+B*EXP(C1*NormInv(U(I)))                            ! Transform to final dist 
      ENDDO                                                          ! End loop over samples
   CASE(LUNIFORM)                                                    ! If type is LUNIFORM...
      S = A+B*EXP(C1*U)                                              ! Transform to final dist 
   CASE(NORMAL)                                                      ! If type is NORMAL...
      DO I=1,N(1)                                                    ! Loop over number of samples
         S(I) = A+B*NormInv(U(I))                                    ! Transform to final dist 
      ENDDO                                                          ! End loop over samples
   CASE(OFFON)                                                       ! If type is OFFON...
      DO I=1,N(1)                                                    ! Loop over number of samples
         IF (U(I)<C%P1) THEN                                         ! For points below prob of zero
            S(I) = 0.                                                ! Set these points to zero (off) 
          ELSE                                                       ! For the remaining points...
            S(I) = 1.                                                ! Set them to one (on)
          ENDIF                                                      ! End probability logic
      ENDDO                                                          ! End loop over samples
   CASE(PARETO)                                                      ! If type is PARETO... 
      S = A+B*(1-U)**(-1/C1)                                         ! Transform to final dist   
   CASE(POINT)                                                       ! If type is POINT... 
      S = C%P1                                                       ! Results = minimum value
   CASE(TRIANGLE)                                                    ! If type is TRIANGLE... 
      S = A+B*C1+B*(ABS(U-C1)/(U-C1))*MAX(ABS(C1-SQRT(C1*U)),       & 
        ABS(1-C1-SQRT((1-C1)*(1-U))))                                ! Transform to final dist                    
   CASE(UNIFORM)                                                     ! If type is UNIFORM...
      S = A+B*U                                                      ! Transform to final dist 
   CASE(WEIBULL)                                                     ! If type is WEIBULL...
      S = A+B*(-1*LOG(1-U))**(1/C1)                                  ! Transform to final dist 
   END SELECT                                                        ! End select dist type
END SUBROUTINE SampleDist                                            ! Return to calling routine


SUBROUTINE Percentiles(Y,N,Pctl,Npctl,Results)                       ! Called from WriteTables
   ! This is a modified version of the HeapSort algorithm from Numerical Recipes
   INTEGER (KIND=IK),  INTENT(In)    :: N, Npctl                     ! # data points, # percentiles
   REAL    (KIND=LONG),INTENT(InOut) :: Y(N)                         ! Input values (sorted in place)
   REAL    (KIND=RK),  INTENT(In)    :: Pctl(Npctl)                  ! List of percentiles needed
   REAL    (KIND=RK),  INTENT(Out)   :: Results(Npctl)               ! Output value for each percentile
   REAL    (KIND=LONG) :: Z, G                                       ! Local real variables
   INTEGER (KIND=IK)   :: I, J, L, IR, NJ                            ! Local integer variables
   ! Global constants: IK, LONG, RK
   ! Intrinsic procs : INT, REAL
   IF (N>1) THEN                                                     ! Only sort if more than one
      L  = N/2+1                                                     ! Index for first new hire 
      IR = N                                                         ! Index for first retiree
      DO                                                             ! Start of Heapsort algorithm
         IF (L>1) THEN                                               ! If still in hiring phase...
            L = L-1                                                  ! Decrement index of new hire
            Z = Y(L)                                                 ! Value for the next new hire
         ELSE                                                        ! Else in retirement phase
            Z     = Y(IR)                                            ! Treat this one as a new hire
            Y(IR) = Y(1)                                             ! Retiring chairman goes to IR
            IR    = IR-1                                             ! Company shrinks by one
            IF (IR==1) EXIT                                          ! Done, if down to one employee
         ENDIF                                                       ! End of phase selection logic
         I = L                                                       ! Initial index for new hire
         J = L+L                                                     ! Index for first subordinate
         DO WHILE (J<=IR)                                            ! Do until no more subordinates
            IF (J<IR) THEN                                           ! While J<IR
              IF (Y(J)<Y(J+1)) J = J+1                               ! Pick the higher subordinate
            ENDIF                                                    ! End J logic
            IF (Z>=Y(J)) EXIT                                        ! If new hire > both, then done
            Y(I) = Y(J)                                              ! Else promote highest subord.
            I    = J                                                 ! New hire shifts to this index
            J    = J+J                                               ! Index of next subordinate
         ENDDO                                                       ! Continue with next subordinate
         Y(I) = Z                                                    ! New hire belongs here
      ENDDO                                                          ! Continue hiring or retiring
      Y(1) = Z                                                       ! Final employee is lowest of all
   ENDIF                                                             ! End of sorting phase 
   DO I=1,Npctl                                                      ! Loop over list of percentiles
      Z  = REAL(N+1)*Pctl(I)/100.                                    ! Find location in sorted list
      NJ = INT(Z)                                                    ! Integer part of location
      G  = Z-REAL(NJ)                                                ! Fractional part of location
      IF (NJ==0) THEN                                                ! If integer part is zero...
         Results(I) = Y(1)                                           ! Percentile = minimum value
      ELSEIF (NJ==N) THEN                                            ! If integer part is largest #...
         Results(I) = Y(N)                                           ! Percentile = maximum value
      ELSE                                                           ! Otherwise, just the usual case
         Results(I) = (1.-G)*Y(NJ)+G*Y(NJ+1)                         ! Percentile is interpolated
      ENDIF                                                          ! End of NJ logic block
   ENDDO                                                             ! Continue with next percentile
END SUBROUTINE Percentiles                                           ! Return to WriteTables

INTEGER (KIND=IK) FUNCTION FuncNum(FN,Required)                      ! From Distrib, Distribution
   CHARACTER (LEN=LEN2), INTENT(IN) :: FN                            ! Input is a character string
   INTEGER(KIND=IK),     INTENT(IN) :: Required                      ! Flag for handling missing dists.
   INTEGER   (KIND=IK) :: I                                          ! Local integer variable 
   ! Global constants: IK, LEN1, YES
   ! Module vars used: FunctionList, NumDistrib
   ! Intrinsic procs : PRINT, WRITE
   ! APEX procedures : Fatal
   ProcLevel = ProcLevel+1                                           ! Increment procedure depth
   ProcName(ProcLevel) = "FuncNum"                                   ! Procedure name for messages
   DO I=1, NumDistrib                                                ! Loop over known distributions
      IF (FunctionList(I)%Name==FN) EXIT                             ! Exit loop when name matches
   ENDDO                                                             ! Try next distribution in list
   IF (I>NumDistrib) I=0                                             ! If missing, set position to zero
   IF(Required==YES .AND. I==0) THEN                                 ! If distribution is required 
      PRINT*, 'Missing input function ',FN                           ! Put message on screen
      WRITE(LogU,*) 'Missing input function ',FN                     ! Put message on log file
      IF(Required==YES) CALL Fatal(1)                                ! Stop job execution
   ENDIF                                                             ! End of missing function logic 
   FuncNum = I                                                       ! Return value is # of match
   ProcLevel = ProcLevel-1                                           ! Decrement procedure depth
END FUNCTION FuncNum                                                 ! Return to calling routine 

INTEGER (KIND=IK) FUNCTION RegionFuncNum(FN)                         ! From Distrib, Distribution
   CHARACTER (LEN=LEN2), INTENT(IN) :: FN                            ! Input is a character string
   INTEGER   (KIND=IK) :: I                                          ! Local integer variable 
   ! Global constants: IK, LEN1, YES
   ! Module vars used: FunctionList, NumDistrib
   ! Intrinsic procs : PRINT, WRITE
   ! APEX procedures : Fatal
   ProcLevel = ProcLevel+1                                           ! Increment procedure depth
   ProcName(ProcLevel) = "RegionFuncNum"                             ! Procedure name for messages
   DO I=1, NumRDistrib                                               ! Loop over known distributions       
      IF (RegionFList(1,I)%Name==FN) EXIT                            ! Exit loop when name matches
   ENDDO                                                             ! Try next distribution in list
   IF (I>NumRDistrib) I=0                                            ! If missing, set position to zero
   RegionFuncNum = I                                                 ! Return value is # of match
   ProcLevel = ProcLevel-1                                           ! Decrement procedure depth
END FUNCTION RegionFuncNum                                           ! Return to calling routine 

FUNCTION ReadDist(String)                                            ! Called from multiple places
   TYPE(CDist) :: ReadDist                                           ! Output distribution definition
   TYPE(CDist) :: D                                                  ! Local copy of distribution
   CHARACTER(LEN=LEN3), INTENT(IN) :: String                         ! Character input variable
   CHARACTER(LEN=LEN3)  :: Line                                      ! Buffer for holding input string
   CHARACTER(LEN=2)     :: Letter                                    ! Two letters extracted from input
   CHARACTER(LEN=1)     :: Letter1                                   ! One letter extracted from input   
   CHARACTER(LEN=2)     :: C(7)                                      ! Two letters extracted from input
   INTEGER  (KIND=IK)   :: I                                         ! Local integer variable
   REAL     (KIND=RK)   :: R(7)                                      ! Local real variables 
   REAL     (KIND=LONG) :: x                                         ! Local real variables 
   ! Global constants: IK, LEN2, LEN3, LogU, LV_FMT, LV_MSG, RK, ST_FMT, ST_MSG
   ! Global vars used: DebugLevel
   ! Global vars set : Ios, ProcLevel, ProcName
   ! Intrinsic procs : ADJUSTL, HUGE, IACHAR, INDEX, LOG, READ, SQRT, WRITE
   ! APEX procedures : CountNums, FuncNum, Fatal, Up
   ProcLevel = ProcLevel+1                                           ! Increment procedure depth
   ProcName(ProcLevel) = "ReadDist"                                  ! Procedure name for messages
   IF (DebugLevel>2) WRITE(LogU,ST_FMT) ST_MSG, ProcName(ProcLevel)  ! Message for debugging
   Line = ADJUSTL(Up(String))                                        ! Uppercase, remove leading blanks
   DO                                                                ! Loop over characters in input
      READ (Line,'(A1)',IOSTAT=Ios(MicroU)) Letter1                  ! Read one character
      I = IACHAR(Letter1)                                            ! Get ASCII code for character
      IF (I>=65.AND.I<=90) EXIT                                      ! Exit loop if a letter is found
      Line=ADJUSTL(Line(2:))                                         ! Remove first character and...
   ENDDO                                                             ! Continue with next character
   C(:) = '. '                                                       ! Default values for C
   I = CountNums(Line)                                               ! Count numbers in input line 
   READ(Line,*,IOSTAT=Ios(MicroU)) Letter                            ! Read first part of dist type
   Letter = Up(Letter)                                               ! Convert letter to uppercase
   R = -999.                                                         ! Undefined parameters flag
   IF (Letter/='DI') THEN                                            ! For all types except discrete
      IF (I>7) CALL Fatal(1)                                         ! Error if > 7 number or . inputs
      IF (I<6) CALL Fatal(2)                                         ! Error if < 6 number or . inputs 
      READ(Line,*,IOSTAT=Ios(MicroU)) Letter, C(1:7)                 ! Read line as character
      DO I=1,6                                                       ! Loop over parameters
         Line = ADJUSTL(Line(INDEX(Line," "):))                      ! Delete first remaining item
         IF (C(I)/='. ') READ(Line,*) R(I)                           ! Read if item is not a period
      ENDDO                                                          ! Continue with next parameter
      D%P1   = R(1)                                                  ! Par1 is R1
      D%P2   = R(2)                                                  ! Par2 is R2
      D%P3   = R(3)                                                  ! Par3 is R3
      D%P4   = R(4)                                                  ! Par4 is R4
      D%LTrunc = R(5)                                                ! Lower trunc point is R5
      D%UTrunc = R(6)                                                ! Upper trunc point is R6
      IF (INDEX(C(7),"N")>0) D%ResampOut = NO                        ! Resample outside truncation
   ELSE                                                              ! For type DISCRETE
      IF (I>100) CALL FATAL(3)                                       ! Exceeding max # values   
      IF (I<1)   CALL FATAL(4)                                       ! Too few values for discrete   
      READ(Line,*,IOSTAT=Ios(MicroU)) Letter, D%D(1:I)               ! Read discrete values            
   ENDIF                                                             ! End dist type logic   
   SELECT CASE (Letter)                                              ! Select distrib based on letter
   CASE ('BE')                                                       ! If word is 'Beta'
      D%Shape = BETA                                                 ! Set shape to BETA
      IF (ANY(C(1:4) == ". "))     CALL Fatal(10+BETA)               ! Stop if required values missing  
      IF (D%P1 <= 0. .OR. D%P2<=0.)CALL Fatal(10+BETA)               ! Stop if shape par <= 0
      D%ShapeP1 = D%P3                                               ! For Beta shape par1 is P3
      D%ShapeP2 = D%P4                                               ! For Beta shape par1 is P4
      D%Scale = D%P2-D%P1                                            ! For Beta scale is max-min
      D%Shift = D%P1                                                 ! For Beta shift is minimum 
      IF (D%LTrunc /= -999 .AND. D%LTrunc > D%Shift ) THEN           ! If lower truncation needed
         x = (D%LTrunc-D%Shift)/D%Scale                              ! Truncation scaled to 0-1       
         D%LPerc = BetaCDF(x, D%ShapeP1,D%ShapeP2)                   ! Get %ile of truncation from CDF  
      ENDIF                                                          ! End truncation logic      
      IF (D%UTrunc /= -999 .AND. D%UTrunc < D%Shift + D%Scale ) THEN ! If upper truncation defined
         x = (D%UTrunc-D%Shift)/D%Scale                              ! Truncation scaled to 0-1       
         D%UPerc = BetaCDF(x, D%ShapeP1,D%ShapeP2)                   ! Get %ile of truncation from CDF 
      ENDIF                                                          ! End truncation logic      
   CASE ('BU')                                                       ! If Word is 'Burr'...
      D%Shape = BURR                                                 ! Set Shape to BURR
      IF (ANY(C(1:3) == ". ")) CALL Fatal(10+BURR)                   ! Stop if required values missing  
      IF (D%P1<0. .OR. D%P2<0. .OR. D%P3<0.) CALL Fatal(10+BURR)     ! Stop if user sets negative pars.
      IF (C(4) == ". ") D%P4=0.                                      ! Set shift P4 to zero if missing
      D%Scale   = D%P1                                               ! For Burr scale  (theta) is P1
      D%ShapeP1 = D%P2                                               ! For Burr shape1 (alpha) is P2
      D%ShapeP2 = D%P3                                               ! For Burr shape2 (gamma) is P3 
      D%Shift   = D%P4                                               ! For Burr, optional shift is P4
      IF (D%LTrunc /= -999 .AND. D%LTrunc > D%Shift) THEN            ! If lower truncation needed
         D%LPerc = 1-(1+((D%LTrunc-D%Shift)/D%P1)**D%P3)**(-D%P2)    ! Get %ile of truncation from CDF
      ENDIF                                                          ! End truncation logic
      IF (D%UTrunc /= -999 .AND. D%UTrunc > D%Shift) THEN            ! If upper truncation defined
         D%UPerc = 1-(1+((D%LTrunc-D%Shift)/D%P1)**D%P3)**(-D%P2)    ! Get %ile of truncation from CDF 
      ENDIF                                                          ! End truncation logic    
      WRITE(LogU,*) "Burr ",D%P1,D%P2,D%P3,D%P4,D%LPerc,D%UPerc
   CASE ('CA')                                                       ! If word is 'Cauchy'
      D%Shape = CAUCHY                                               ! Set shape to CAUCHY
      IF (ANY(C(1:2) == ". ")) CALL Fatal(10+CAUCHY)                 ! Stop if required values missing  
      IF (ANY(C(3:4) /= ". ")) CALL Warning(10+CAUCHY)               ! Warn if extra parameter defined  
      D%Shift = D%P1                                                 ! For Cauchy shift is P1 (median) 
      D%Scale = D%P2                                                 ! For Cauchy scale is P2
      IF (D%LTrunc /= -999) THEN                                     ! If lower truncation defined
         x = (D%LTrunc-D%Shift)/D%Scale                              ! Truncation scaled to 0-1       
         D%LPerc = 0.5 + (atan(x)/PI)                                ! Get %ile of truncation from CDF 
      ENDIF                                                          ! End truncation logic      
      IF (D%UTrunc /= -999) THEN                                     ! If upper truncation defined
         x = (D%UTrunc-D%Shift)/D%Scale                              ! Truncation scaled to 0-1       
         D%UPerc = 0.5 + (atan(x)/PI)                                ! Get %ile of truncation from CDF 
      ENDIF                                                          ! End truncation logic      
   CASE ('DI')                                                       ! If Word is 'Discrete'...
      D%Shape = DISCRETE                                             ! Set Shape to DISCRETE     
      D%ResampOut = NO                                               ! Ignore for discrete dists
      D%ShapeP1 = I                                                  ! Var shapeP1 holds # of vals
      D%P1      = I                                                  ! As does P1 (for error checks) 
   CASE ('EX')                                                       ! If Word is 'Exponential'...
      D%Shape = EXPONENTIAL                                          ! Set Shape to EXPONENTIAL
      IF (ANY(C(1:2) == ". ")) CALL Fatal(10+EXPONENTIAL)            ! Stop if required values missing  
      IF (ANY(C(3:4) /= ". ")) CALL Warning(10+EXPONENTIAL)          ! Warn if extra parameter defined  
      IF (D%P1 <= 0.)          CALL Fatal(10+EXPONENTIAL)            ! Stop if K not > 0 
      D%Scale = 1/D%P1                                               ! For Exp scale is 1/K
      D%Shift = D%P2                                                 ! For Exp shift is P2 
      IF (D%LTrunc /= -999 .AND. D%LTrunc > D%Shift ) THEN           ! If lower truncation needed
         x = (D%LTrunc-D%Shift)/D%Scale                              ! Truncation scaled to 0-1       
         D%LPerc= 1-exp(-1*x)                                        ! Get %ile of truncation from CDF 
      ENDIF                                                          ! End truncation logic      
      IF (D%UTrunc /= -999) THEN                                     ! If upper truncation defined
         x = (D%UTrunc-D%Shift)/D%Scale                              ! Truncation scaled to 0-1       
         D%UPerc= 1-exp(-1*x)                                        ! Get %ile of truncation from CDF 
      ENDIF                                                          ! End truncation logic      
   CASE ('EV')                                                       ! If Word is 'EValue'...
      D%Shape = EVALUE                                               ! Set Shape to EVALUE
      IF (ANY(C(1:2) == ". ")) CALL Fatal(10+EVALUE)                 ! Stop if required values missing  
      IF (ANY(C(3:4) /= ". ")) CALL Warning(10+EVALUE)               ! Warn if extra parameter defined  
      D%Scale = D%P1                                                 ! For EValue scale is P1
      D%Shift = D%P2                                                 ! For EValue shift is P2 
      IF (D%LTrunc /= -999) THEN                                     ! If lower truncation defined
         x = (D%LTrunc-D%Shift)/D%Scale                              ! Truncation scaled to 0-1       
         D%LPerc= exp(-1*exp(-1*x))                                  ! Get %ile of truncation from CDF 
      ENDIF                                                          ! End truncation logic      
      IF (D%UTrunc /= -999) THEN                                     ! If upper truncation defined
         x = (D%UTrunc-D%Shift)/D%Scale                              ! Truncation scaled to 0-1       
         D%UPerc= exp(-1*exp(-1*x))                                  ! Get %ile of truncation from CDF 
      ENDIF                                                          ! End truncation logic      
   CASE ('GA')                                                       ! If word is 'Gamma'
      D%Shape = GAMMA                                                ! Set Shape to GAMMA
      IF (ANY(C(1:3) == ". ")) CALL Fatal(10+GAMMA)                  ! Stop if required values missing  
      IF (ANY(C(4:4) /= ". ")) CALL Warning(10+GAMMA)                ! Warn if extra parameter defined  
      IF (D%P1<=0.)     CALL Fatal(10+GAMMA)                         ! Stop if shape not > 0
      D%ShapeP1 = D%P1                                               ! For Gamma shape par is P1
      D%Scale   = D%P2                                               ! For Gamma scale is P2
      D%Shift   = D%P3                                               ! For Gamma shift is P3 
      IF (D%LTrunc /= -999 .AND. D%LTrunc > D%Shift) THEN            ! If lower truncation needed
         x = (D%LTrunc-D%Shift)/D%Scale                              ! Truncation scaled to 0-1       
         D%LPerc = GammaCDF(x, D%ShapeP1)                            ! Get %ile of truncation from CDF  
      ENDIF                                                          ! End truncation logic      
      IF (D%UTrunc /= -999) THEN                                     ! If upper truncation defined
         x = (D%UTrunc-D%Shift)/D%Scale                              ! Truncation scaled to 0-1       
         D%UPerc = GammaCDF(x, D%ShapeP1)                            ! Get %ile of truncation from CDF 
      ENDIF                                                          ! End truncation logic      
   CASE ('LG')                                                       ! If Word is 'LGT'...
      D%Shape = LGT                                                  ! Set Shape to LGT
      IF (ANY(C(1:2) == ". ")) CALL Fatal(10+LGT)                    ! Stop if required values missing  
      IF (ANY(C(3:4) /= ". ")) CALL Warning(10+LGT)                  ! Warn if extra parameter defined  
      D%Scale   = D%P2                                               ! For Logistic scale is P2
      D%Shift   = D%P1                                               ! For Logistic shift is P1 (mean) 
      IF (D%LTrunc /= -999 ) THEN                                    ! If lower truncation defined
         x = (D%LTrunc-D%Shift)/D%Scale                              ! Truncation scaled to 0-1       
         D%LPerc = (1+exp(-1*x))**(-1)                               ! Get %ile of truncation from CDF  
      ENDIF                                                          ! End truncation logic      
      IF (D%UTrunc /= -999) THEN                                     ! If upper truncation defined
         x = (D%UTrunc-D%Shift)/D%Scale                              ! Truncation scaled to 0-1       
         D%UPerc = (1+exp(-1*x))**(-1)                               ! Get %ile of truncation from CDF 
      ENDIF                                                          ! End truncation logic      
   CASE ('LO')                                                       ! If Word is 'Lognormal'...
      D%Shape = LOGNORMAL                                            ! Set Shape to LOGNORMAL
      IF (ANY(C(1:3) == ". ")) CALL Fatal(10+LOGNORMAL)              ! Stop if required values missing  
      IF (ANY(C(4:4) /= ". ")) CALL Warning(10+LOGNORMAL)            ! Warn if extra parameter defined  
      IF (D%P1<0.)             CALL Fatal(10+LOGNORMAL)              ! Stop if user sets negative mean
      IF (D%P2<=1.)            CALL Fatal(10+LOGNORMAL)              ! Stop if GSD is not over one
      D%ShapeP1 = LOG(D%P2)                                          ! For Lognormal shape is LOG(GSD)
      D%Scale   = D%P1                                               ! For Lognormal scale is GM
      D%Shift   = D%P3                                               ! For Lognormal shift is P3 
      IF (D%LTrunc /= -999 .AND. D%LTrunc > D%Shift) THEN            ! If lower truncation needed
         x = (D%LTrunc-D%Shift)/D%Scale                              ! Truncation scaled to 0-1       
         D%LPerc = NormCDF(log(x)/D%ShapeP1)                         ! Get %ile of truncation from CDF  
      ENDIF                                                          ! End truncation logic      
      IF (D%UTrunc /= -999) THEN                                     ! If upper truncation defined
         x = (D%UTrunc-D%Shift)/D%Scale                              ! Truncation scaled to 0-1       
         D%UPerc = NormCDF(log(x)/D%ShapeP1)                         ! Get %ile of truncation from CDF 
      ENDIF                                                          ! End truncation logic      
   CASE ('LU')                                                       ! If Word is 'LUniform'...
      D%Shape = LUNIFORM                                             ! Set Shape to LUNIFORM
      IF (ANY(C(1:2) == ". "))    CALL Fatal(10+LUNIFORM)            ! Stop if required values missing  
      IF (ANY(C(3:4) /= ". ")) CALL Warning(10+LUNIFORM)             ! Warn if extra parameter defined  
      IF (D%P1<=0. .OR. D%P2<=0.) CALL Fatal(10+LUNIFORM)            ! Stop if min, max are not > 0
      D%ShapeP1 = LOG(D%P2/D%P1)                                     ! For LUniform shape=LOG(max/min)
      D%Scale   = D%P1                                               ! For LUniform scale is minimum
      D%Shift   = 0                                                  ! For LUniform shift is 0 
      IF (D%LTrunc /= -999 .AND. D%LTrunc > D%P1) THEN               ! If lower truncation needed
         x = (D%LTrunc-D%Shift)/D%Scale                              ! Truncation scaled to 0-1       
         D%LPerc = Log(x)/D%ShapeP1                                  ! Get %ile of truncation from CDF  
      ENDIF                                                          ! End truncation logic      
      IF (D%UTrunc /= -999 .AND.  D%UTrunc < D%P2) THEN              ! If upper truncation needed
         x = (D%UTrunc-D%Shift)/D%Scale                              ! Truncation scaled to 0-1       
         D%UPerc = Log(x)/D%ShapeP1                                  ! Get %ile of truncation from CDF 
      ENDIF                                                          ! End truncation logic      
   CASE ('NO')                                                       ! If Word is 'Normal'...
      D%Shape = NORMAL                                               ! Set Shape to NORMAL
      IF (ANY(C(1:2) == ". ")) CALL Fatal(10+NORMAL)                 ! Stop if required values missing  
      IF (ANY(C(3:4) /= ". ")) CALL Warning(10+NORMAL)               ! Warn if extra parameter defined  
      IF (D%P2<=0.)            CALL Fatal(10+NORMAL)                 ! Stop if Std. Dev. not positive
      D%Scale   = D%P2                                               ! For Normal scale is SD
      D%Shift   = D%P1                                               ! For Normal shift is mean 
      IF (D%LTrunc /= -999) THEN                                     ! If lower truncation defined
         x = (D%LTrunc-D%Shift)/D%Scale                              ! Truncation scaled to 0-1       
         D%LPerc = NormCDF(x)                                        ! Get %ile of truncation from CDF  
      ENDIF                                                          ! End truncation logic      
      IF (D%UTrunc /= -999) THEN                                     ! If upper truncation defined
         x = (D%UTrunc-D%Shift)/D%Scale                              ! Truncation scaled to 0-1       
         D%UPerc = NormCDF(x)                                        ! Get %ile of truncation from CDF 
      ENDIF                                                          ! End truncation logic      
    CASE ('OF')                                                      ! If word is 'OffOn'
      D%Shape = OFFON                                                ! Set shape to OFFON
      IF (C(1)==". ")          CALL Fatal(10+OFFON)                  ! Stop if required param missing
      IF (ANY(C(2:6) /= ". ")) CALL Warning(10+OFFON)                ! Warn if extra parameter defined  
      IF (D%P1<0.)             CALL Fatal(10+OFFON)                  ! Stop if probability is negative
      IF (D%P1>1.)             CALL Fatal(10+OFFON)                  ! Stop if probability exceeds one
   CASE ('PA')                                                       ! If Word is 'Pareto'...
      D%Shape = PARETO                                               ! Set Shape to PARETO
      IF (ANY(C(1:3) == ". ")) CALL Fatal(10+PARETO)                 ! Stop if required values missing  
      IF (ANY(C(4:4) /= ". ")) CALL Warning(10+PARETO)               ! Warn if extra parameter defined  
      IF (D%P1<=0.)            CALL Fatal(10+PARETO)                 ! Stop if shape not > 0
      D%ShapeP1 = D%P1                                               ! For Pareto shape is P1
      D%Scale   = D%P2                                               ! For Pareto scale is P2
      D%Shift   = D%P3                                               ! For Pareto shift is P3 
      IF (D%LTrunc /= -999 .AND. D%LTrunc > D%Shift+D%Scale) THEN    ! If lower truncation needed
         x = (D%LTrunc-D%Shift)/D%Scale                              ! Truncation scaled to 0-1       
         D%LPerc = 1-((x)**(-1))**(D%ShapeP1)                        ! Get %ile of truncation from CDF  
      ENDIF                                                          ! End truncation logic      
      IF (D%UTrunc /= -999) THEN                                     ! If upper truncation defined
         x = (D%UTrunc-D%Shift)/D%Scale                              ! Truncation scaled to 0-1       
         D%UPerc = 1-((x)**(-1))**(D%ShapeP1)                        ! Get %ile of truncation from CDF 
      ENDIF                                                          ! End truncation logic      
   CASE ('PO','FI')                                                  ! If Word is 'fixed' or 'point'
      D%Shape = POINT                                                ! Set Shape to POINT
      IF (C(1)==". ")          CALL Fatal(10+POINT)                  ! Stop if required param missing
      IF (ANY(C(2:6) /= ". ")) CALL Warning(10+POINT)                ! Warn if extra parameter defined  
   CASE ('TR')                                                       ! If Word is 'Triangle'...
      D%Shape = TRIANGLE                                             ! Set Shape to TRIANGLE
      IF (ANY(C(1:3) == ". "))       CALL Fatal(10+TRIANGLE)         ! Stop if required values missing  
      IF (ANY(C(4:4) /= ". "))       CALL Warning(10+TRIANGLE)       ! Warn if extra parameter defined  
      IF (D%P2<=D%P1)                CALL Fatal(10+TRIANGLE)         ! Stop if max not above min 
      IF (D%P3<D%P1 .OR. D%P3>D%P2)  CALL Fatal(10+TRIANGLE)         ! Stop if peak outide range
      D%ShapeP1 = (D%P3-D%P1)/(D%P2-D%P1)                            ! For Tri. shape=(p-min)/max-min)
      D%Scale   = D%P2-D%P1                                          ! For Triangle scale is max-min
      D%Shift   = D%P1                                               ! For Triangle shift is min 
      IF (D%LTrunc /= -999 .AND. D%LTrunc > D%P1) THEN               ! If lower truncation needed
         IF (D%LTrunc < D%Shift+D%Scale*D%ShapeP1) THEN              ! On left side of triangle
            D%LPerc=((D%LTrunc-D%Shift)**2)/((D%Scale**2)*D%ShapeP1) ! Get %ile of truncation from CDF
         ELSE                                                        ! On right side of triangle
            D%LPerc = (D%ShapeP1-2*((D%LTrunc-D%Shift)/D%Scale)+    &
               ((D%LTrunc-D%Shift)**2)/D%Scale**2)/(D%ShapeP1-1)     ! Get %ile of truncation from CDF 
         ENDIF                                                       ! End left/right logic
      ENDIF                                                          ! End truncation logic      
      IF (D%UTrunc /= -999 .AND. D%UTrunc < D%P2) THEN               ! If upper truncation needed
         IF (D%UTrunc < D%Shift+D%Scale*D%ShapeP1) THEN              ! On left side of triangle
            D%UPerc=((D%UTrunc-D%Shift)**2)/((D%Scale**2)*D%ShapeP1) ! Get %ile of truncation from CDF
         ELSE                                                        ! On right side of triangle
            D%UPerc = (D%ShapeP1-2*((D%UTrunc-D%Shift)/D%Scale)+    &
               ((D%UTrunc-D%Shift)**2)/D%Scale**2)/(D%ShapeP1-1)     ! Get %ile of truncation from CDF 
         ENDIF                                                       ! End left/right logic
      ENDIF                                                          ! End truncation logic      
   CASE ('UN')                                                       ! If Word is 'Uniform'...
      D%Shape = UNIFORM                                              ! Set Shape to UNIFORM
      IF (ANY(C(1:2) == ". ")) CALL Fatal(10+UNIFORM)                ! Stop if required values missing  
      IF (ANY(C(3:4) /= ". ")) CALL Warning(10+UNIFORM)              ! Warn if extra parameter defined  
      IF (D%P2 <=D%P1)         CALL Fatal(10+UNIFORM)                ! Stop if maximum not above min 
      D%Scale   = D%P2-D%P1                                          ! For Uniform scale is max-min
      D%Shift   = D%P1                                               ! For Uniform shift is min 
      IF (D%LTrunc /= -999 .AND. D%LTrunc > D%P1) THEN               ! If lower truncation needed
         x = (D%LTrunc-D%Shift)/D%Scale                              ! Truncation scaled to 0-1       
         D%LPerc = x                                                 ! Get %ile of truncation from CDF  
      ENDIF                                                          ! End truncation logic      
      IF (D%UTrunc /= -999 .AND. D%UTrunc < D%P2) THEN               ! If upper truncation needed
         x = (D%UTrunc-D%Shift)/D%Scale                              ! Truncation scaled to 0-1       
         D%UPerc = x                                                 ! Get %ile of truncation from CDF 
      ENDIF                                                          ! End truncation logic      
   CASE ('WE')                                                       ! If Word is 'Weibull'...
      D%Shape = WEIBULL                                              ! Set Shape to WEIBULL
      IF (ANY(C(1:3) == ". ")) CALL Fatal(10+WEIBULL)                ! Stop if required values missing  
      IF (ANY(C(4:4) /= ". ")) CALL Warning(10+WEIBULL)              ! Warn if extra parameter defined  
      IF (D%P1<=0.)            CALL Fatal(10+WEIBULL)                ! Stop if shape not > 0
      D%ShapeP1 = D%P1                                               ! For Weibull shape is P1
      D%Scale   = D%P2                                               ! For Weibull scale is P2
      D%Shift   = D%P3                                               ! For Weibull shift is P3 
      IF (D%LTrunc /= -999 .AND. D%LTrunc > D%P1) THEN               ! If lower truncation needed
         x = (D%LTrunc-D%Shift)/D%Scale                              ! Truncation scaled to 0-1       
         D%LPerc = 1-exp(-1*(x)**D%ShapeP1)                          ! Get %ile of truncation from CDF  
      ENDIF                                                          ! End truncation logic      
      IF (D%UTrunc /= -999) THEN                                     ! If upper truncation defined
         x = (D%UTrunc-D%Shift)/D%Scale                              ! Truncation scaled to 0-1       
         D%UPerc = 1-exp(-1*(x)**D%ShapeP1)                          ! Get %ile of truncation from CDF 
      ENDIF                                                          ! End truncation logic      
   CASE DEFAULT                                                      ! If Word not in above list...
      CALL Fatal(21)                                                 ! Stop if shape is unknown
   END SELECT                                                        ! End of Word selection (shape)
   ReadDist = D                                                      ! Final output distribution
   IF (DebugLevel>2) WRITE(LogU,LV_FMT) LV_MSG, ProcName(ProcLevel)  ! Message for debugging
   ProcLevel = ProcLevel-1                                           ! Decrement procedure depth
END FUNCTION ReadDist                                                ! Return to calling routine

FUNCTION NormInv(u)                                                  ! Inverse CDF for unit normal
   ! Uses Applied Statistics algorithm 241, tested accurate to relative error 8E-17 everywhere
   ! Implementation in Real*8 loses a little precision 
   REAL(KIND=LONG) :: NormInv, u, v, q, r                            ! Double precision variables
   REAL(KIND=LONG),PARAMETER :: zero=0D0, half=0.5D0, one=1D0        ! Double precision constants 
   REAL(KIND=LONG),PARAMETER :: split1=0.425D0,    split2=5D0        ! Double precision constants 
   REAL(KIND=LONG),PARAMETER :: const1=0.180625D0, const2=1.6D0      ! Double precision constants 
   REAL(KIND=LONG),PARAMETER ::     A0=3.3871328727963666080D+0,    &
      A1=1.3314166789178437745D+2,  A2=1.9715909503065514427D+3,    &
      A3=1.3731693765509461125D+4,  A4=4.5921953931549871457D+4,    &
      A5=6.7265770927008700853D+4,  A6=3.3430575583588128105D+4,    &
      A7=2.5090809287301226727D+3,  B1=4.2313330701600911252D+1,    &
      B2=6.8718700749205790830D+2,  B3=5.3941960214247511077D+3,    &
      B4=2.1213794301586595867D+4,  B5=3.9307895800092710610D+4,    &
      B6=2.8729085735721942674D+4,  B7=5.2264952788528545610D+3,    &
      C0=1.42343711074968357734D+0, C1=4.63033784615654529590D+0,   & 
      C2=5.76949722146069140550D+0, C3=3.64784832476320460504D+0,   &
      C4=1.27045825245236838258D+0, C5=2.41780725177450611770D-1,   &
      C6=2.27238449892691845833D-2, C7=7.74545014278341407640D-4,   &
      D1=2.05319162663775882187D+0, D2=1.67638483018380384940D+0,   &
      D3=6.89767334985100004550D-1, D4=1.48103976427480074590D-1,   &
      D5=1.51986665636164571966D-2, D6=5.47593808499534494600D-4,   &
      D7=1.05075007164441684324D-9, E0=6.65790464350110377720D+0,   &
      E1=5.46378491116411436990D+0, E2=1.78482653991729133580D+0,   &
      E3=2.96560571828504891230D-1, E4=2.65321895265761230930D-2,   &
      E5=1.24266094738807843860D-3, E6=2.71155556874348757815D-5,   &
      E7=2.01033439929228813265D-7, F1=5.99832206555887937690D-1,   &
      F2=1.36929880922735805310D-1, F3=1.48753612908506148525D-2,   &
      F4=7.86869131145613259100D-4, F5=1.84631831751005468180D-5,   &
      F6=1.42151175831644588870D-7, F7=2.04426310338993978564D-15    ! Coeffs in continued fractions
   ProcLevel = ProcLevel+1                                           ! Increment procedure depth
   ProcName(ProcLevel) = "NormInv"                                   ! Procedure name for messages
   v = u                                                             ! Standard function
   IF (u<zero .AND. u>-one) v = -u                                   ! Use complementary function
   IF (v<zero .OR. v>one) CALL Fatal(1)                              ! If argument out of range, exit
   q = v-half                                                        ! Symmetrize the argument
   IF (v==zero) THEN                                                 ! Special case at lower endpoint
      NormInv = -HUGE(v)                                             ! Set to smallest possible value 
   ELSEIF (v==one) THEN                                              ! Special case at upper endpoint
      NormInv = HUGE(v)                                              ! Set to largest possible value
   ELSEIF (ABS(q)<split1) THEN                                       ! If away from ends of range 
      r = const1 - q*q                                               ! Argument in continued fraction 
      NormInv=q*(((((((A7*r+A6)*r+A5)*r+A4)*r+A3)*r+A2)*r+A1)*r+A0) &
         / (((((((B7*r+B6)*r+B5)*r+B4)*r+B3)*r+B2)*r+B1)*r+one)      ! Eighth convergent for 1st c.f.
   ELSE                                                              ! If close to ends...  
      r = SQRT(-LOG(MIN(v,one-v)))                                   ! Transform argument
      IF (r<=split2) THEN                                            ! If not too near ends...
        r = r-const2                                                 ! Use second continued fraction 
        NormInv=(((((((C7*r+C6)*r+C5)*r+C4)*r+C3)*r+C2)*r+C1)*r+C0) &
           / (((((((D7*r+D6)*r+D5)*r+D4)*r+D3)*r+D2)*r+D1)*r+one)    ! Eighth convergent for 2nd c.f.
      ELSE                                                           ! Within 1.38E-11 of ends...
        r = r-split2                                                 ! Use third continued fraction 
        NormInv=(((((((E7*r+E6)*r+E5)*r+E4)*r+E3)*r+E2)*r+E1)*r+E0) &
           / (((((((F7*r+F6)*r+F5)*r+F4)*r+F3)*r+F2)*r+F1)*r+one)    ! Eighth convergent for 3rd c.f.
      ENDIF                                                          ! End of splitting logic
      IF (v<half) NormInv = -NormInv                                 ! Change sign if at lower end
   ENDIF                                                             ! Done with all cases 
   IF (u<zero) NormInv = -NormInv                                    ! Flip to other tail
   ProcLevel = ProcLevel-1                                           ! Decrement procedure depth
END FUNCTION                                                         ! End of NormInv code  

FUNCTION NormCDF(z)                                                  ! CDF for unit normal
   REAL(KIND=LONG) :: NormCDF, z, y, ser, mult                       ! Double precision variables
   REAL(KIND=LONG),PARAMETER :: zero=0D0,half=0.5D0,one=1D0,two=2D0  ! Double precision constants
   REAL(KIND=LONG),PARAMETER :: ispi=0.564189583547756287D0,four=4D0 ! 1/Sqrt[Pi] and 4
   INTEGER(KIND=4),PARAMETER :: nterms=12                            ! Half # of convergents for c.f.
   INTEGER(KIND=IK) :: I                                             ! Local integer variables
   ! Intrinsic procs: ABS, EXP, REAL, SIGN, SQRT
   ! APEX procedures: GammaCDF
   IF (ABS(z)<=four) THEN                                            ! For all within 4 sigma of mean  
      NormCDF = (one+GammaCDF(z*z/two,half)*SIGN(one,z))/two         ! Call CDF for Gamma distrib.
   ELSE                                                              ! In tails of normal
      y = ABS(z)/SQRT(two)                                           ! Argument for error function
      ser = zero                                                     ! Start continued fraction
      mult = two*REAL(nterms,LONG)+two                               ! Numerator for final term
      DO I=nterms,1,-1                                               ! Backward summation of terms  
         mult = mult-two                                             ! Adjust numerator
         ser  = (mult-one)/(y+mult/(two*y+ser))                      ! Add two terms at once 
      ENDDO                                                          ! Continue with next two terms  
      NormCDF=ispi*EXP(-y*y)/(two*y+ser)                             ! Include other terms  
      IF(z>zero) NormCDF = one-NormCDF                               ! Take complement for upper tail 
   ENDIF                                                             ! End of logic for tails 
END FUNCTION NormCDF                                                 ! Return to calling routine

FUNCTION BetaInv (z,p,q)                                             ! Inverse CDF for beta
   REAL (KIND=LONG) :: BetaInv,z,p,q                                 ! Real variables -local variable X never referenced
   REAL (KIND=LONG) :: a,b,c,d,e,fa,fb,fc,lim,r,s,u,v,xm             ! More real variables
   REAL (KIND=LONG), PARAMETER :: eps = 2D-15, tol = 2D-15           ! Convergence criteria
   REAL (KIND=LONG), PARAMETER :: three=3D0, two=2D0, one=1D0        ! Numeric constants
   REAL (KIND=LONG), PARAMETER :: half=0.5D0, zero=0D0               ! More numeric constants
   INTEGER(KIND=IK), PARAMETER :: itmax = 100                        ! Iteration limit
   INTEGER(KIND=IK) :: iter                                          ! Loop counter for iterations
   ! Intrinsic procs : SIGN, ABS, MIN
   ProcLevel = ProcLevel+1                                           ! Increment procedure depth
   ProcName(ProcLevel) = "BetaInv"                                   ! Procedure name for messages
   IF (p<=0..OR.q<=0..OR.z<0..OR.z>1.) CALL Fatal(1)                 ! Exit if invalid parms to beta
   a  = zero                                                         ! Start 'a' at lower endpoint
   b  = one                                                          ! Start 'b' at upper endpoint
   fa = -z                                                           ! Function at 'a'
   fb = one-z                                                        ! Function at 'b'
   fc = fb                                                           ! 'c' starts at 'b'
   DO iter = 1, itmax                                                ! Loop over iterations
      IF (fb*fc > zero) THEN                                         ! If b and c on same side of root
         c  = a                                                      ! Move c to a
         fc = fa                                                     ! New function value at c
         d  = b-a                                                    ! Interval between b and a
         e  = d                                                      ! Interval between b and c
      ENDIF                                                          ! End of logic block
      IF (ABS(fc) < ABS(fb)) THEN                                    ! If c closer to root than b
         a  = b                                                      ! Move a to b
         b  = c                                                      ! Move b to c
         c  = a                                                      ! Move c to a
         fa = fb                                                     ! Function at new a
         fb = fc                                                     ! Function at new b
         fc = fa                                                     ! Function at new c
      ENDIF                                                          ! End of logic block
      lim = two*eps*ABS(b)+half*tol                                  ! Tolerance limit
      xm  = half*(c-b)                                               ! Increment for x
      IF (ABS(xm)<=lim .OR. fb==zero) EXIT                           ! Test of convergence
      IF (ABS(e) >=lim .AND. ABS(fa)>ABS(fb)) THEN                   ! Condition for interpolation
         s = fb/fa                                                   ! Distance ratio to root (linear)
         IF (a==c) THEN                                              ! Case with a and c coincident
            u = two*xm*s                                             ! Use linear approx for root
            v = one-s                                                ! V for this case
         ELSE                                                        ! If a and c different
            v = fa/fc                                                ! Use quadratic interpolation 
            r = fb/fc                                                ! Linear distance ratios
            u = s*(two*xm*v*(v-r)-(b-a)*(r-one))                     ! Increment for x
            v = (v-one)*(r-one)*(s-one)                              ! V for this case
         ENDIF                                                       ! End a/c logic
         IF (u > zero) v = -v                                        ! When u > 0
         u = ABS(u)                                                  ! Positive u
         IF (two*u < MIN(three*xm*v-ABS(lim*v),ABS(e*v))) THEN       ! When 2*u fulfills this inequal
            e = d                                                    ! E for this case
            d = u/v                                                  ! D for this case
         ELSE                                                        ! In all other cases
            d = xm                                                   ! D for this case
            e = d                                                    ! E for this case
         ENDIF                                                       ! End
      ELSE                                                           ! If not ready for fancy interp.
         d = xm                                                      ! Delta x is half of interval
         e = d                                                       ! E for this case
      ENDIF                                                          ! End of logic block
      a  = b                                                         ! Move a to b
      fa = fb                                                        ! New function value at a
      IF (ABS(d) > lim) THEN                                         ! If delta x is large enough
         b = b+d                                                     ! Move the estimated root by d
      ELSE                                                           ! Otherwise 
         b = b+SIGN(lim,xm)                                          ! Move estimate by lim
      ENDIF                                                          ! End of logic block
      fb = BetaCDF(b,p,q)-z                                          ! Find function at new estimate
   ENDDO                                                             ! End iteration loop
   IF (iter>itmax) CALL Warning(1)                                   ! Warn if itmax reached
   BetaInv = b                                                       ! Root is estimated to be at b
   ProcLevel = ProcLevel-1                                           ! Decrement procedure depth
END FUNCTION                                                         ! Return to calling routine

FUNCTION BetaCDF(x,a,b)                                              ! CDF for Beta distribution 
   REAL (KIND=LONG) :: BetaCDF, x, a, b, c                           ! Double precision variables
   REAL (KIND=LONG),PARAMETER :: zero = 0D0, one = 1D0, two = 2D0    ! Double precision constants
   ! Intrinsic procs : EXP, LOG
   ! APEX procedures : BetaFrac
   IF(zero<x .AND. x<one) THEN                                       ! Test for x inside range
      c=EXP(GmL(a+b)-GmL(a)-GmL(b)+a*LOG(x)+b*LOG(one-x))            ! Overall common factor
   ELSE                                                              ! Else x or edge or out of range   
      c = zero                                                       ! Factor is zero for x<=0 or x>=1 
   ENDIF                                                             ! End of logic for factor
   IF (x < (a+one)/(a+b+two)) THEN                                   ! For small enough x
      BetaCDF = c*BetaFrac(x,a,b)/a                                  ! Evaluate from lower end 
   ELSE                                                              ! Else if x is large
      BetaCDF = one-c*BetaFrac(one-x,b,a)/b                          ! Evaluate from upper end
   ENDIF                                                             ! End of logic for direction
END FUNCTION BetaCDF                                                 ! Return to calling routine

FUNCTION BetaFrac(x,a,b)                                             ! Continued fraction for Beta 
   ! Slightly modified version of 'Betacf' from Numerical Recipes
   REAL (KIND=LONG) :: BetaFrac, x, a, b, d, aold, em, atem          ! Double precision variables
   REAL (KIND=LONG) :: qab, am, bm, az, bz, ap, bp, app, bpp         ! Double precision variables
   REAL (KIND=LONG), PARAMETER :: eps = 5D-16, one = 1D0             ! Double precision constants
   INTEGER (KIND=IK),PARAMETER :: itmax = 100                        ! Maximum number of iterations
   INTEGER (KIND=IK) :: m                                            ! Loop index variable
   ! Intrinsic procs : ABS, PRINT, REAL
   ProcLevel = ProcLevel+1                                           ! Increment procedure depth
   ProcName(ProcLevel) = "BetaFrac"                                  ! Procedure name for messages
   qab = a+b                                                         ! Sum of Beta parameters
   am  = one                                                         ! Value of A(0) for Beta c.f.
   bm  = one                                                         ! Value of B(0) for Beta c.f.
   az  = one                                                         ! Value of A(1) for Beta c.f.
   bz  = one-qab*x/(a+one)                                           ! Value of B(1) for Beta c.f.  
   DO m=1, itmax                                                     ! Loop over iterations
      aold = az                                                      ! Record previous approximation
      em   = REAL(m,KIND=LONG)                                       ! Loop index as a real number
      atem = a+em+em                                                 ! Twice the loop index plus a
      d    = em*(b-em)*x/((atem-one)*atem)                           ! Even term in continued fraction
      ap   = az+d*am                                                 ! Add even step to numerator
      bp   = bz+d*bm                                                 ! Add even step to denominator
      d    = -(a+em)*(qab+em)*x/(atem*(atem+one))                    ! Odd term in continued fraction
      app  = ap+d*az                                                 ! Add odd term to numerator
      bpp  = bp+d*bz                                                 ! Add odd term to denominator
      am   = ap/bpp                                                  ! Renormalize numerator (-1 term)
      bm   = bp/bpp                                                  ! Renormalize denominator (-1)
      az   = app/bpp                                                 ! Renormalize numerator (0 term)
      bz   = one                                                     ! Renormalize denominator (0)
      IF (ABS(az-aold) < eps*ABS(az)) EXIT                           ! Exit loop if within tolerance
   ENDDO                                                             ! Continue with next iteration
   IF (m>itmax) Call Fatal(1)                                        ! Quit if maxiter reached
   BetaFrac = az                                                     ! Set return value
   ProcLevel = ProcLevel-1                                           ! Decrement procedure depth
END FUNCTION BetaFrac                                                ! Return to calling routine

FUNCTION GammaInv(u,q)                                               ! Find root of (GammaCDF-u==0)
   REAL (KIND=LONG) :: GammaInv,u,q,xlo,flo,xhi,fhi,val              ! Double precision variables
   REAL (KIND=LONG), PARAMETER :: N0=0D0, N1=1D0, N2=2D0, eps=2D-15  ! Constants and convergence limit
   INTEGER (KIND=IK) :: iter, itmax=100                              ! Integer variables -local variable NRMAX never referenced
   ! Intrinsic procs : ABS,EXP,LOG,MAX,MIN,MOD,NINT,SQRT
   ! APEX procedures : GammaCDF,GmL, Fatal
   ProcLevel = ProcLevel+1                                           ! Increment procedure depth
   ProcName(ProcLevel) = "GammaInv"                                  ! Procedure name for messages
   IF(q<=N0 .OR. u<N0 .OR. u>N1) CALL Fatal(1)                       ! Bad input to GammaInv   
   IF(u==N0 .OR. u==N1 .OR. q==N1) THEN                              ! Special conditions   
      IF (u==N0) GammaInv = 0D0                                      ! Value is zero  
      IF (u==N1) GammaInv = HUGE(GammaInv)                           ! Set to largest possible value  
      IF (q==N1 .AND. u<N1) GammaInv = -LOG(N1-u)                    ! Analytical result for q=1  
      ProcLevel=ProcLevel-1                                          ! Decrement procedure depth
      RETURN                                                         ! Return to calling program 
   ENDIF                                                             ! End of special conditions logic
   IF (q>N1) THEN                                                    ! If shape greater than 1
      GammaInv = q+(N2*u-N1)*SQRT(q)                                 ! Use this initial value
   ELSEIF (u<=0.5D0) THEN                                            ! If uniform variate is <0.5
      GammaInv = 1.38629D0*q*u                                       ! Use this initial guess
   ELSE                                                              ! In all other cases    
      GammaInv = -q*LOG(N1-u)                                        ! Start at good guess  
   ENDIF                                                             ! End of special conditions logic         
   IF (GammaInv<=N0) CALL Fatal(2)                                   ! Error if initial guess <0
   val = GammaCDF(GammaInv,q)-u                                      ! Test CFD of guess versus orig u   
   IF (val<0) THEN                                                   ! When difference is < 0
      xhi = GammaInv                                                 ! Initialize xhi
      DO                                                             ! Do until diff > 0
         xlo = xhi                                                   ! The guess was too low   
         flo = val                                                   ! F(guess) is flo
         xhi = xlo*N2                                                ! Higher guess is 2*guess
         fhi = GammaCDF(xhi,q)-u                                     ! Recalcuate f(guess)       
         IF (fhi>0) EXIT                                             ! Exit if diff > 0       
      ENDDO	                                                         ! End unbounded guess loop
   ELSE                                                              ! Otherwise if diff was > 0
      xlo = GammaInv                                                 ! Initialize xlo
      DO                                                             ! Do until diff < 0
         xhi = xlo                                                   ! The guess was too high   
         fhi = val                                                   ! F(guess) is fhi   
         xlo = xhi/N2                                                ! Lower guess is 1/2*guess
         flo = GammaCDF(xlo,q)-u                                     ! Recalculate f(guess)   
         IF (flo<0) EXIT                                             ! Exit if diff < 0
      ENDDO                                                          ! End unbounded guess loop 
   ENDIF                                                             ! End guess logic
   DO iter=1,itmax                                                   ! Iterate until limit 
      IF (MOD(iter,2)==0) THEN                                       ! Even numbered iterations
         GammaInv = (xlo+xhi)/N2                                     ! X is mean of hi/low
      ELSE                                                           ! Otherwise on odd interations
         GammaInv = (xlo*fhi-xhi*flo)/(fhi-flo)                      ! Find new x using linear interp. 
      ENDIF                                                          ! End iteration # logic
      val = GammaCDF(GammaInv,q)-u                                   ! Function value at GammaInv
      IF (MIN(ABS(val),xhi-xlo,N1-xlo/xhi) < eps) THEN               ! If converged
         ProcLevel=ProcLevel-1                                       ! Decrement procedure depth
         RETURN                                                      ! Exit subroutine
      ENDIF                                                          ! End convergence logic
      IF (val<N0) THEN                                               ! Test sign at new location
         xlo = GammaInv                                              ! If negative, adjust lower bound
         flo = val                                                   ! Adjust lower function value
      ELSE                                                           ! Else sign is positive    
         xhi = GammaInv                                              ! Adjust upper bound on root
         fhi = val                                                   ! Adjust upper function value 
      ENDIF                                                          ! End of sign logic    
   ENDDO                                                             ! Continue with next iteration 
   CALL Fatal(3)                                                     ! Convergence failed
   ProcLevel = ProcLevel-1                                           ! Decrement procedure depth
END FUNCTION GammaInv                                                ! Return to calling routine

FUNCTION GammaCDF(x,c)                                               ! CDF for Gamma distribution
   REAL (KIND=LONG) :: GammaCDF , c, x                               ! Double precision arguments    
   REAL (KIND=LONG), PARAMETER :: zero=0D0, one=1D0, tol=1D-15       ! Constants and convergence limit
   REAL (KIND=LONG) :: a0,a1,b0,b1,fac,ir,denom,term,ser,prev        ! Double precision variables  
   INTEGER (KIND=4) :: I, itmax=200                                  ! Integer index for loop counting
   ! Intrinsic procs: ABS, EXP, LOG, PRINT
   ProcLevel = ProcLevel+1                                           ! Increment procedure depth
   ProcName(ProcLevel) = "GammaCDF"                                  ! Procedure name for messages
   IF (c<=zero .OR. x<zero) CALL Fatal(1)                            ! Test if arguments are valid
   IF (x==zero) THEN                                                 ! If at lower bound...
      GammaCDF = zero                                                ! P(c,0) = 0
   ELSEIF (x < c+one) THEN                                           ! For small x use series 
      denom = c                                                      ! Initialize series denominator     
      term  = one/c                                                  ! First term in series
      ser   = term                                                   ! Equate to running total  
      DO I=1,itmax                                                   ! Iteration loop 
         denom = denom+one                                           ! Increment denominator
         term  = term*x/denom                                        ! Next term in series
         ser   = ser+term                                            ! New partial sum for series    
         IF (ABS(term/ser)<tol) EXIT                                 ! Exit when sums not changing
      ENDDO                                                          ! End of iteration loop
      GammaCDF = Ser*EXP(-x+c*LOG(x)-GmL(c))                         ! P(c,x) = g(c,x)/G(c)  
   ELSE                                                              ! For x large use continued frac.
      prev = zero                                                    ! Last value for comparison
      a0   = zero                                                    ! Value of A(0) in Gamma c.f. 
      a1   = one                                                     ! Value of A(1) in Gamma c.f. 
      b0   = one                                                     ! Value of B(0) in Gamma c.f.
      b1   = x                                                       ! Value of B(1) in Gamma c.f.
      fac  = one                                                     ! Renormalization factor 
      ir   = zero                                                    ! Real version of loop counter
      DO I=1,itmax                                                   ! Iterate until limit
         ir = ir+one                                                 ! Increment real counter
         a0 = (a1+(ir-c)*a0)*fac                                     ! Recurrence for even A
         b0 = (b1+(ir-c)*b0)*fac                                     ! Recurrence for even B
         a1 = x*a0+ir*a1*fac                                         ! Recurrence for odd A
         b1 = x*b0+ir*b1*fac                                         ! Recurrence for odd B 
         IF (b1 .NE. zero) THEN                                      ! Renormalize if denom. not zero 
            fac = one/b1                                             ! Renormalization factor
            ser = a1*fac                                             ! Current estimate for result 
            IF (ABS(one-prev/ser) < tol) EXIT                        ! Check if different from prev
            prev = ser                                               ! Update previous estimate
         ENDIF                                                       ! End of renormalization logic
      ENDDO                                                          ! Iterate next two terms
      GammaCDF = one-Ser*EXP(-x+c*LOG(x)-GmL(c))                     ! P(c,x) = 1-G(c,x)/G(c)  
   ENDIF                                                             ! End of large x logic
   ProcLevel = ProcLevel-1                                           ! Decrement procedure depth
END FUNCTION GammaCDF                                                ! Return to calling routine

FUNCTION GmL(z)                                                      ! Log of the Gamma function
   REAL(KIND=LONG) :: GmL,z,s                                        ! Local real variables
   REAL(KIND=LONG),PARAMETER :: c0= 1.1423741895793849D-2,          &
      c1=2.321852708508292D0,   c2=-3.3849376523679164D0,           &
      c3=1.432849298393438D0,   c4=-1.7538556222327772D-1,          &
      c5=3.369657871067317D-3,  c6=-3.4976520256994604D-7            ! Constants in Lanczos approx.
   ! Intrinsic procs : LOG
   s=c6/(z+5D0)+c5/(z+4D0)+c4/(z+3D0)+c3/(z+2D0)+c2/(z+1D0)+c1/z+c0  ! Intermediate step
   GmL  = LOG(s)+(z-0.5D0)*LOG(z+5.391D0)-z                          ! Set the function value
END FUNCTION GmL                                                     ! Return to calling routine

END MODULE DistributionModule                                        ! Last source code line in module
