MODULE OutputModule                                                  ! Compile after the modules below
   ! Latest revisions by WGG on August 16, 2012
   USE GlobalModule                                                  ! Compile this module first
   USE UtilityModule                                                 ! Compile this module second
   USE DistributionModule                                            ! Compile this module third
   USE MicroEnvModule                                                ! Compile this module third

   IMPLICIT NONE                                                     ! Must declare all variables

   CHARACTER*200     :: Header, Header5                              ! Buffer for table title
   CHARACTER*85      :: Header2                                      ! Buffer for table title
   CHARACTER*100     :: Header3, Header4                             ! Buffer for table identifier
   REAL  (KIND=LONG), ALLOCATABLE :: TAccExp(:,:)                    ! Acc. Minutes per exposure level 
   REAL  (KIND=LONG), ALLOCATABLE :: Array(:,:)                      ! Buffer for table data
   REAL  (KIND=LONG), ALLOCATABLE :: TableLev(:)                     ! Buffer for table levels
   REAL    (KIND=RK), ALLOCATABLE :: Percs(:,:)                      ! Buffer for desired percentiles
    
   CHARACTER (LEN=20), ALLOCATABLE :: PHeader(:)                     ! PSum file var header
   CHARACTER (LEN=20), ALLOCATABLE::  PVals(:)                       ! PSum file var values   
   CHARACTER (LEN=20), ALLOCATABLE :: HHeader(:)                     ! Hour file var header
   CHARACTER (LEN=20), ALLOCATABLE::  HVals(:)                       ! Hour file var values   
   CHARACTER (LEN=20), ALLOCATABLE :: EHeader(:)                     ! Events file var header
   CHARACTER (LEN=20), ALLOCATABLE::  EVals(:)                       ! Events file var values   
   CHARACTER (LEN=20), ALLOCATABLE :: DHeader(:)                     ! Daily file var header
   CHARACTER (LEN=20), ALLOCATABLE :: DVals(:)                       ! Daily file var values
   CHARACTER (LEN=20), ALLOCATABLE :: TSHeader(:)                    ! Timestep file var header
   CHARACTER (LEN=20), ALLOCATABLE :: TSVals(:)                      ! Timestep file var values

   LOGICAL   (KIND=1), ALLOCATABLE :: F(:)                           ! Short form of hourly flags
   CHARACTER(LEN=85) :: GrpHeader(49)                                ! Buffer for group table headers     
   CHARACTER(LEN=18):: PopGroup(7) = (/"ALL,        ",              &
      "CHILD,      ", "ACTIVE,     ",  "ACTIVECHILD,",              &
      "            ", "            ",  "ALL,        "/)              ! Table IDs: Population Group   
CONTAINS                                                             ! Routines internal to this module

SUBROUTINE SetupOutput                                               ! Called from main APEX program
   INTEGER (KIND=IK) :: K                                            ! Local integer variable
   ! Global constants: IK, LogU, LV_FMT, LV_MSG, ST_FMT, ST_MSG, YES
   ! Global vars used: DAvgDoseLev, DAvgExpLev, DebugLevel, DM1HDoseLev, DM1HExpLev, DM8HDoseLev,  
   !                   DM8HExpLev, DMEHDoseLev, DoDose, H_EHDoseLev, LogProfiles, NumProfiles,
   !                   Pctlm, SAvgDoseLev, SAvgExpLev, TimeDoseLev, TimeExpLev
   ! Global vars set : DailyDose, DailyExp, ProcLevel, ProcName
   ! Module vars set : Array, DAvgDose, DAvgExp, DM1HDose, DM1HExp, DM8HDose, DM8HExp, 
   !                   DMEHDose, H_EHDose, MSum, NLev, Percs, SAvgDose, SAvgExp, 
   !                   TableLev, TAccExp, TimeDose, TimeExp
   ! Intrinsic procs : ALLOCATE, ALLOCATED, MAXVAL, NINT, WRITE
   ! Apex procs      : CheckAllocation
   ProcLevel = ProcLevel+1                                           ! Increment procedure depth
   ProcName(ProcLevel) = "SetupOutput"                               ! Procedure name for messages
   IF (DebugLevel>0) WRITE(LogU,ST_FMT) ST_MSG, ProcName(ProcLevel)  ! Message for debugging
   ALLOCATE (DailyExp(NumDays),STAT=AllocErr)                        ! Daily exposure and dose stats
   CALL CheckAllocation(AllocErr)                                    ! Check the success of the alloc
   ALLOCATE (DailyDose(NumDays),STAT=AllocErr)                       ! Daily exposure and dose stats
   CALL CheckAllocation(AllocErr)                                    ! Check the success of the alloc
   ALLOCATE (MSum(0:NumMicros),STAT=AllocErr)                        ! Micro summary table
   CALL CheckAllocation(AllocErr)                                    ! Check the success of the alloc
   K = MAXVAL(Pol(:)%N_TimeExpLev)                                   ! Find largest # of table levels 
   ALLOCATE (TAccExp(0:NumMicros,K),STAT=AllocErr)                   ! Accum. minutes for above table
   CALL CheckAllocation(AllocErr)                                    ! Check the success of the alloc
   K = MAXVAL(Pol(:)%N_DAvgExpLev)                                   ! Find largest # of table levels 
   ALLOCATE (DAvgExp(NumProfiles,K, NumPollutants),STAT=AllocErr)                   ! Daily average exposures
   CALL CheckAllocation(AllocErr)                                    ! Check the success of the alloc
   K = MAXVAL(Pol(:)%N_DM1HExpLev)                                   ! Find largest # of table levels 
   ALLOCATE (DM1HExp(NumProfiles,K,21,NumPollutants),STAT=AllocErr)  ! Daily max 1-hour exposures
   CALL CheckAllocation(AllocErr)                                    ! Check the success of the alloc
   K = MAXVAL(Pol(:)%N_DM8HExpLev)                                   ! Find largest # of table levels 
   ALLOCATE (DM8HExp(NumProfiles,K,21,NumPollutants),STAT=AllocErr)  ! Daily max 8-hour exposures
   CALL CheckAllocation(AllocErr)                                    ! Check the success of the alloc
   K = MAXVAL(Pol(:)%N_DMTSExpLev)                                   ! Find largest # of table levels 
   ALLOCATE (DMTSExp(NumProfiles,K,21,NumPollutants),STAT=AllocErr)  ! Daily max timestep exposures
   CALL CheckAllocation(AllocErr)                                    ! Check the success of the alloc
   K = MAXVAL(Pol(:)%N_TSExpLev)                                     ! Find largest # of table levels 
   ALLOCATE (ALLTSExp(NumProfiles,K,21,NumPollutants),STAT=AllocErr) ! All-sim timestep exposures
   CALL CheckAllocation(AllocErr)                                    ! Check the success of the alloc
   K = MAXVAL(Pol(:)%N_SAvgExpLev)                                   ! Find largest # of table levels 
   ALLOCATE (SAvgExp(NumProfiles,K, NumPollutants), STAT=AllocErr)   ! Simulation average exposures
   CALL CheckAllocation(AllocErr)                                    ! Check the success of the alloc
   K = MAXVAL(Pol(:)%N_TimeExpLev)                                   ! Find largest # of table levels 
   ALLOCATE (TimeExp(0:NumMicros,K, NumPollutants),STAT=AllocErr)    ! Minutes at each exposure level
   CALL CheckAllocation(AllocErr)                                    ! Check the success of the alloc
   K = MAXVAL(Pol(:)%N_DAvgDoseLev)                                  ! Find largest # of table levels 
   ALLOCATE (DAvgDose(NumProfiles,K, NumPollutants),STAT=AllocErr)   ! Daily average doses
   CALL CheckAllocation(AllocErr)                                    ! Check the success of the alloc
   K = MAXVAL(Pol(:)%N_DM1HDoseLev)                                  ! Find largest # of table levels 
   ALLOCATE (DM1HDose(NumProfiles,K,21,NumPollutants),STAT=AllocErr) ! Daily max 1-hour doses
   CALL CheckAllocation(AllocErr)                                    ! Check the success of the alloc
   K = MAXVAL(Pol(:)%N_DM8HDoseLev)                                  ! Find largest # of table levels 
   ALLOCATE (DM8HDose(NumProfiles,K,21,NumPollutants),STAT=AllocErr) ! Daily max 8-hour doses
   CALL CheckAllocation(AllocErr)                                    ! Check the success of the alloc
   K = MAXVAL(Pol(:)%N_DMTSDoseLev)                                  ! Find largest # of table levels 
   ALLOCATE (DMTSDose(NumProfiles,K,21,NumPollutants),STAT=AllocErr) ! Daily max timestep doses
   CALL CheckAllocation(AllocErr)                                    ! Check the success of the alloc
   K = MAXVAL(Pol(:)%N_TSDoseLev)                                    ! Find largest # of table levels 
   ALLOCATE (ALLTSDose(NumProfiles,K,21,NumPollutants),STAT=AllocErr)! All-sim timestep doses
   CALL CheckAllocation(AllocErr)                                    ! Check the success of the alloc
   K = MAXVAL(Pol(:)%N_DMEHDoseLev)                                  ! Find largest # of table levels 
   ALLOCATE (DMEHDose(NumProfiles,K,21,NumPollutants),STAT=AllocErr) ! Daily max end-of-hour doses
   CALL CheckAllocation(AllocErr)                                    ! Check the success of the alloc
   K = MAXVAL(Pol(:)%N_H_EHDoseLev)                                  ! Find largest # of table levels 
   ALLOCATE (H_EHDose(NumProfiles,K, NumPollutants),STAT=AllocErr)   ! Hours at end-of-hour doses
   CALL CheckAllocation(AllocErr)                                    ! Check the success of the alloc
   K = MAXVAL(Pol(:)%N_SAvgDoseLev)                                  ! Find largest # of table levels 
   ALLOCATE (SAvgDose(NumProfiles,K, NumPollutants),STAT=AllocErr)   ! Simulation average doses
   CALL CheckAllocation(AllocErr)                                    ! Check the success of the alloc
   K = MAXVAL(Pol(:)%N_TimeDoseLev)                                  ! Find largest # of table levels 
   ALLOCATE (TimeDose(NumProfiles,K, NumPollutants),STAT=AllocErr)   ! Minutes at each dose level
   CALL CheckAllocation(AllocErr)                                    ! Check the success of the alloc
   K = MAXVAL(Pol(:)%N_Pctl)                                         ! Find largest # of table levels 
   ALLOCATE (Percs(K,MaxLevels),STAT=AllocErr)                       ! Percentiles for output tables
   CALL CheckAllocation(AllocErr)                                    ! Check the success of the alloc
   ALLOCATE (Array(NumProfiles,MaxLevels),STAT=AllocErr)             ! Buffer for values in tables
   CALL CheckAllocation(AllocErr)                                    ! Check the success of the alloc
   ALLOCATE (TableLev(MaxLevels),STAT=AllocErr)                      ! Buffer for table levels
   CALL CheckAllocation(AllocErr)                                    ! Check the success of the alloc
   ALLOCATE (AvgExp(NumProfiles, NumPollutants),STAT=AllocErr)       ! Buffer for table levels
   ALLOCATE (MaxExp(NumProfiles, NumPollutants),STAT=AllocErr)       ! Buffer for table levels
   ALLOCATE (AvgDose(NumProfiles, NumPollutants),STAT=AllocErr)      ! Buffer for table levels
   ALLOCATE (MaxDose(NumProfiles, NumPollutants),STAT=AllocErr)      ! Buffer for table levels
   AvgExp   = 0.                                                     ! Initialize average exps
   MaxExp   = 0.                                                     ! Initialize max exps
   AvgDose  = 0.                                                     ! Initialize average dose
   MaxDose  = 0.                                                     ! Initialize max dose
   DAvgExp  = 0                                                      ! Initialize DAvgExp table
   DM1HExp  = 0                                                      ! Initialize DM1HExp table
   DM8HExp  = 0                                                      ! Initialize DM8HExp table
   DMTSExp  = 0                                                      ! Initialize DMTSExp table
   SAvgExp  = 0                                                      ! Initialize SAvgExp table 
   TimeExp  = 0.                                                     ! Initialize TimeExp table
   TAccExp  = 0.                                                     ! Initialize TAccExp table
   DAvgDose = 0                                                      ! Initialize DAvgDose table
   DM1HDose = 0                                                      ! Initialize DM1HDose table
   DM8HDose = 0                                                      ! Initialize DM8HDose table
   DMEHDose = 0                                                      ! Initialize DMEHDose table 
   DMTSDose = 0                                                      ! Initialize DMTSDose table 
   H_EHDose = 0                                                      ! Initialize H_EHDose table
   SAvgDose = 0                                                      ! Initialize SAvgDose table
   TimeDose = 0.                                                     ! Initialize TimeDose table 
   AllTSExp = 0                                                      ! Initialize mult exceedances       
   AllTSDose= 0                                                      ! Initialize mult exceedances       
   IF (LogProfiles==NO .AND. DebugLevel>1) DebugLevel=1              ! Reduce printing in profile loop
   IF (DebugLevel>0) WRITE(LogU,LV_FMT) LV_MSG, ProcName(ProcLevel)  ! Message for debugging
   ProcLevel = ProcLevel-1                                           ! Decrement procedure depth
END SUBROUTINE SetupOutput                                           ! Return to main APEX program 

SUBROUTINE Output(P,PL)                                              ! Called from main APEX program
   INTEGER (KIND=IK), INTENT(IN) :: P, PL                            ! Input variable (profile number)
   INTEGER (KIND=IK) :: I, C=1                                       ! Local integer variables
   ! Global constants: DAYOFWEEK, EMPTEXT, EventU, GASTEXT, HDoseU, HExpU, HOURSDAY, IK, LogU,
   !                   LV_FMT, LV_MSG, MSumU, PILOTTEXT, PSumU, SEXTEXT, ST_FMT, ST_MSG, YES
   ! Global vars used: Calen, CustProfs, DebugLevel, DoCustProfs, DoDose, EventSample, EventSeq,  
   !                   EventsOut, EventStart, FirstProfile, HourlyDose, HourlyExp, HourlyVe, HourlyVa,  
   !                   HourlyEE, HourlyEVR, HourSeq, MicroName, MResOut, MSumOut, NumCustProfs,  
   !                   NumDays, NumEvents, NumHours, NumMicros, NumProfiles, Person, Phys, PopCat,  
   !                   SList, TemData, VAoutp
   ! Global vars set : EventsPerson, EventStop, Ios, ProcLevel, ProcName
   ! Module vars used: MSum
   ! Intrinsic procs : CLOSE, MAXVAL, OPEN, PRINT, REAL, SUM, WRITE
   ! APEX procedures : WriteJobTime
   ProcLevel = ProcLevel+1                                           ! Increment procedure depth
   ProcName(ProcLevel) = "Output"                                    ! Procedure name for messages
   IF (DebugLevel>1) WRITE(LogU,ST_FMT) ST_MSG, ProcName(ProcLevel)  ! Message for debugging
   IF (MOD(P, 100000) == 0 .and. PL==1) CALL WriteJobTime(P)         ! Write job time to log 
   IF (MSumOut==YES) THEN                                            ! If user wants micro summary 
      DO I=0, NumMicros                                              ! Loop over micros
      WRITE(Pol(PL)%MSumU,'(I9,I4,I8,2ES12.3)') P,I,                &  
         MSum(I)%Minutes, MSum(I)%MeanConc, MSum(I)%MaxConc          ! Write micro data
      ENDDO                                                          ! Continue with next micro
   ENDIF                                                             ! End MSumOut logic block
   IF (PL==NumPollutants) THEN                                       ! If this is the last pollutant 
      IF (HourlyOut==YES .AND. TimeStepsPerDay>=24)                 &    
         CALL WriteHourlyFile(P)                                     ! Write data to hourly file
      IF (TimeStepOut==YES .AND. DoTimesteps==YES)                  &        
         CALL WriteTimeStepFile(P)                                   ! Write data to timestep file
      IF (DailyOut==YES)  CALL WriteDailyFile(P)                     ! Write data to daily file
      IF (DebugLevel>1) WRITE(LogU,*) ' ',P
      CALL WritePSumFile(P)                                          ! Write the personal summary file
      IF (DoCustProfs==YES) THEN                                     ! If doing custom profiles
         DO I=1,NumCustProfs                                         ! Loop through custom profiles   
            IF (P==CustProfs(I)) EventsPerson=YES                    ! Prof matches cust, wrt evt
         ENDDO                                                       ! End custom prof loop   
      ENDIF                                                          ! End custom profile logic   
      IF (EventSample*P >= NumProfiles*C) THEN                       ! Test for printing event data 
         EventsPerson = YES                                          ! This person gets event output
         C = C+1                                                     ! Reset threshold for next output
      ENDIF                                                          ! End of event-level logic block
      IF (EventsOut==YES .AND. EventsPerson==YES)                   &
         CALL WriteEventsFile(P)                                     ! Write events file 
      PrintCounter = PrintCounter+100   
      IF (P==1 .OR. OnePercent==NO .OR. PrintCounter>=NumProfiles) THEN
        PrintCounter = MOD(PrintCounter,NumProfiles)         
        PRINT*,'Finished output for profile #',P,' of ',NumProfiles  ! Put person counter on screen
      ENDIF  
   ENDIF                                                             ! End of last pollutant logic
   IF (DebugLevel>1) WRITE(LogU,LV_FMT) LV_MSG, ProcName(ProcLevel)  ! Message for debugging
   ProcLevel = ProcLevel-1                                           ! Decrement procedure depth
END SUBROUTINE Output                                                ! Return to main APEX program

SUBROUTINE WriteTablesFile(PL)
   INTEGER (KIND=IK), INTENT(IN) :: PL                               ! Input variable (pollut number)
   ProcLevel = ProcLevel+1                                           ! Increment procedure depth
   ProcName(ProcLevel) = "WriteTablesFile"                           ! Procedure name for messages 
   ! Global vars set : ProcLevel, ProcName
   ! Module vars set : PopGroup, GrpHeader, H
   ! Intrinsic procs : WRITE, TRIM
   ! APEX procs      : WriteExposureTables, WriteDoseTables
   IF (Disease /= "")  THEN                                          ! If modeling disease   
     PopGroup(5)=Up(Trim(Disease)//",")                              ! Identifier for ill population
     PopGroup(6)=Up(Trim(Disease)//"CHILD,")                         ! Identifier for ill children
   ENDIF                                                             ! End disease logic    
   WRITE(GrpHeader(1),'(A)')  "Group: All People"                    ! Header for Group 1 
   WRITE(GrpHeader(2),'(A,I2,A,I2,A)') "Group: Children"            &  
      //" (ages ",ChildMin,"-",ChildMax,")"                          ! Header for Group 2  
   WRITE(GrpHeader(3),'(A,F5.3,A)')  "Group: All Ages, Active"      &   
      //" (PAI>",ActivePAI,")"                                       ! Header for Group 3
   WRITE(GrpHeader(4),'(A,I2,A,I2,A,F5.3,A)')  "Group:"             &   
      //" Children (ages ",ChildMin,"-",ChildMax,"),"               & 
      //" Active (PAI>",ActivePAI,")"                                ! Header for Group 4
   WRITE(GrpHeader(5),'(A,A)')  "Group: All Ages, ", Trim(Disease)   ! Header for Group 5 
   WRITE(GrpHeader(6),'(A,I2,A,I2,A,A)') "Group: Children"          &  
      //" (ages ",ChildMin,"-",ChildMax,"), ", Trim(Disease)         ! Header for Group 6  
   WRITE(GrpHeader(7),'(A)')  "Group: All Employed People"           ! Header for Group 7
   WRITE(GrpHeader(8),'(A,F5.2,A,F5.2,A )') "Group: All"            &     
      //" Ages, Moderate Exertion (EVR ",ModEVR1,"-",HeavyEVR1,")"   ! Header for Group 8, 1-Hour
   WRITE(GrpHeader(9),'(A,I2,A,I2,A,F5.2,A,F5.2,A)')                & 
      "Group: Children (ages ",ChildMin,"-",ChildMax,"),"           &
       //" Moderate Exertion (EVR ",ModEVR1,"-",HeavyEVR1,")"        ! Header for Group 9, 1-Hour
   WRITE(GrpHeader(10),'(A,F5.3,A,F5.2,A,F5.2,A)')                   &
      "Group: All Ages, Active (PAI>",ActivePAI,"),"                &
       //" Moderate Exertion (EVR ",ModEVR1,"-",HeavyEVR1,")"        ! Header for Group 10, 1-Hour 
   WRITE(GrpHeader(11),'(A,I2,A,I2,A,F5.3,A,F5.2,A,F5.2,A)')        &
    "Group: Children (ages ",                                       & 
      ChildMin,"-",ChildMax,"), Active (PAI>",ActivePAI,"),"        & 
      //" Moderate Exertion (EVR ",ModEVR1,"-",HeavyEVR1,")"         ! Header for Group 11, 1-Hour
   WRITE(GrpHeader(12),'(A,A,A,F5.2,A,F5.2,A)')                     &
     "Group: All Ages, ",Trim(Disease),                             &  
     ", Moderate Exertion (EVR ",ModEVR1,"-",HeavyEVR1,")"           ! Header for Group 12, 1-Hour
   WRITE(GrpHeader(13),'(A,I2,A,I2,A,A,A,F5.2,A,F5.2,A)')           &
     "Group: Children (ages ",ChildMin,"-",ChildMax,"), ",          &
     Trim(Disease), ", Moderate Exertion (EVR ",ModEVR1,            &
     "-",HeavyEVR1,")"                                               ! Header for Group 13, 1-Hour  
   WRITE(GrpHeader(14),'(A,F5.2,A,F5.2,A )') "Group: All"            &     
      //" Employed, Moderate Exertion (EVR ",ModEVR1,"-",HeavyEVR1,")"   ! Header for Group 14, 1-Hour
   WRITE(GrpHeader(15),'(A,F5.2,A)') "Group: All Ages,"             &
      //" Heavy Exertion (EVR>",HeavyEVR1,")"                        ! Header for Group 15, 1-Hour
   WRITE(GrpHeader(16),'(A,I2,A,I2,A,F5.2,A)')                      &
      "Group: Children (ages ",ChildMin,"-",ChildMax,"),"           &
       //" Heavy Exertion (EVR>",HeavyEVR1,")"                       ! Header for Group 16, 1-Hour
   WRITE(GrpHeader(17),'(A,F5.3,A,F5.2,A)') "Group: All Ages,"      &
      //" Active (PAI>",ActivePAI,"), Heavy Exertion (EVR>",        &
      HeavyEVR1,")"                                                  ! Header for Group 17, 1-Hour
   WRITE(GrpHeader(18),'(A,I2,A,I2,A,F5.3,A,F5.2,A)')               &
      "Group: Children (ages ",ChildMin,"-",ChildMax,"),"           &
      //" Active (PAI>",ActivePAI,"), Heavy Exertion (EVR>",        &
       HeavyEVR1,")"                                                 ! Header for Group 18, 1-Hour  
   WRITE(GrpHeader(19),'(A,A,A,F5.2,A )') "Group: All Ages, ",      &
     Trim(Disease),", Heavy Exertion (EVR>",HeavyEVR1,")"            ! Header for Group 19, 1-Hour
   WRITE(GrpHeader(20),'(A,I2,A,I2,A,A,A,F5.2,A)')                  &
      "Group: Children (ages ",ChildMin,"-",ChildMax,"), ",         &
      Trim(Disease),", Heavy Exertion (EVR>",HeavyEVR1,")"           ! Header for Group 20, 1-Hour
   WRITE(GrpHeader(21),'(A,F5.2,A)') "Group: All Employed,"         &
      //" Heavy Exertion (EVR>",HeavyEVR1,")"                        ! Header for Group 21, 1-Hour      
   WRITE(GrpHeader(22),'(A,F5.2,A,F5.2,A)') "Group: All Ages,"      &
      //" Moderate Exertion (EVR ",ModEVR8,"-",HeavyEVR8,")"         ! Header for Group 22, 8-Hour
   WRITE(GrpHeader(23),'(A,I2,A,I2,A,F5.2,A,F5.2,A)')               &
      "Group: Children (ages ",ChildMin,"-",ChildMax,"),"           &
      //" Moderate Exertion (EVR ",ModEVR8,"-",HeavyEVR8,")"         ! Header for Group 23, 8-Hour
   WRITE(GrpHeader(24),'(A,F5.3,A,F5.2,A,F5.2,A)')"Group:"          &
      //" All Ages, Active (PAI>",ActivePAI,"), Moderate Exertion"  &
      //" (EVR ",ModEVR8,"-",HeavyEVR8,")"                           ! Header for Group 24, 8-Hour   
   WRITE(GrpHeader(25),'(A,I2,A,I2,A,F5.3,A,F5.2,A,F5.2,A)')        &
      "Group: Children (ages ",ChildMin,"-",ChildMax,               &  
      "), Active (PAI>",ActivePAI,"), Moderate Exertion (EVR ",     &  
      ModEVR8,"-",HeavyEVR8,")"                                      ! Header for Group 25, 8-Hour  
   WRITE(GrpHeader(26),'(A,A,A,F5.2,A,F5.2,A)')                     &
     "Group: All Ages, ", Trim(Disease),                            &  
     ", Moderate Exertion (EVR ",ModEVR8,"-",HeavyEVR8,")"           ! Header for Group 26, 8-Hour
   WRITE(GrpHeader(27),'(A,I2,A,I2,A,A,A,F5.2,A,F5.2,A)')           &
     "Group: Children (ages ",ChildMin,"-",ChildMax,"), ",          &
     Trim(Disease), ", Moderate Exertion (EVR ",ModEVR8,"-",        &
     HeavyEVR8,")"                                                   ! Header for Group 27, 8-Hour  
   WRITE(GrpHeader(28),'(A,F5.2,A,F5.2,A)') "Group: All Employed,"  &
      //" Moderate Exertion (EVR ",ModEVR8,"-",HeavyEVR8,")"         ! Header for Group 28, 8-Hour     
   WRITE(GrpHeader(29),'(A,F5.2,A)') "Group: All Ages,"             &  
      //" Heavy Exertion (EVR>",HeavyEVR8,")"                        ! Header for Group 29, 8-Hour 
   WRITE(GrpHeader(30),'(A,I2,A,I2,A,F5.2,A)') "Group:"             &
      //" Children (ages ",ChildMin,"-",ChildMax,"), Heavy Exertion"&
      //" (EVR>",HeavyEVR8,")"                                       ! Header for Group 30, 8-Hour 
   WRITE(GrpHeader(31),'(A,F5.3,A,F5.2,A)') "Group: All Ages,"      &
      //" Active (PAI>",ActivePAI,"), Heavy Exertion (EVR>",        &
      HeavyEVR8,")"                                                  ! Header for Group 31, 8-Hour 
   WRITE(GrpHeader(32),'(A,I2,A,I2,A,F5.3,A,F5.2,A)')               &
      "Group: Children (ages ",ChildMin,"-",ChildMax,"), Active"    &
      //" (PAI>",ActivePAI,"), Heavy Exertion (EVR>",HeavyEVR8,")"   ! Header for Group 32, 8-Hour 
   WRITE(GrpHeader(33),'(A,A,A,F5.2,A )') "Group: All Ages,",       &
     Trim(Disease),", Heavy Exertion (EVR>",HeavyEVR8,")"            ! Header for Group 33, 8-Hour
   WRITE(GrpHeader(34),'(A,I2,A,I2,A,A,A,F5.2,A)')                  &
     "Group: Children (ages ",ChildMin,"-",ChildMax,"), ",          &
     Trim(Disease), ", Heavy Exertion (EVR>",HeavyEVR8,")"           ! Header for Group 34, 8-Hour
   WRITE(GrpHeader(35),'(A,F5.2,A)') "Group: All Employed,"         &  
      //" Heavy Exertion (EVR>",HeavyEVR8,")"                        ! Header for Group 35, 8-Hour 
   WRITE(GrpHeader(36),'(A,F5.2,A,F5.2,A)') "Group: All Ages,"      &
      //" Moderate Exertion (EVR ",ModEVRTS,"-",HeavyEVRTS,")"       ! Header for Group 36, TS
   WRITE(GrpHeader(37),'(A,I2,A,I2,A,F5.2,A,F5.2,A)')               &
      "Group: Children (ages ",ChildMin,"-",ChildMax,"),"           &
      //" Moderate Exertion (EVR ",ModEVRTS,"-",HeavyEVRTS,")"       ! Header for Group 37, TS
   WRITE(GrpHeader(38),'(A,F5.3,A,F5.2,A,F5.2,A)')"Group:"          &
      //" All Ages, Active (PAI>",ActivePAI,"), Moderate Exertion"  &
      //" (EVR ",ModEVRTS,"-",HeavyEVRTS,")"                         ! Header for Group 38, TS
   WRITE(GrpHeader(39),'(A,I2,A,I2,A,F5.3,A,F5.2,A,F5.2,A)')        &
      "Group: Children (ages ",ChildMin,"-",ChildMax,               &  
      "), Active (PAI>",ActivePAI,"), Moderate Exertion (EVR ",     &  
      ModEVRTS,"-",HeavyEVRTS,")"                                    ! Header for Group 39, TS
   WRITE(GrpHeader(40),'(A,A,A,F5.2,A,F5.2,A)')                     &
     "Group: All Ages, ", Trim(Disease),                            &  
     ", Moderate Exertion (EVR ",ModEVRTS,"-",HeavyEVRTS,")"         ! Header for Group 40, TS
   WRITE(GrpHeader(41),'(A,I2,A,I2,A,A,A,F5.2,A,F5.2,A)')           &
     "Group: Children (ages ",ChildMin,"-",ChildMax,"), ",          &
     Trim(Disease), ", Moderate Exertion (EVR ",ModEVRTS,"-",        &
     HeavyEVRTS,")"                                                  ! Header for Group 41, TS  
   WRITE(GrpHeader(42),'(A,F5.2,A,F5.2,A)') "Group: All Employed,"  &
      //" Moderate Exertion (EVR ",ModEVRTS,"-",HeavyEVRTS,")"       ! Header for Group 42, TS     
   WRITE(GrpHeader(43),'(A,F5.2,A)') "Group: All Ages,"             &  
      //" Heavy Exertion (EVR>",HeavyEVRTS,")"                       ! Header for Group 43, TS
   WRITE(GrpHeader(44),'(A,I2,A,I2,A,F5.2,A)') "Group:"             &
      //" Children (ages ",ChildMin,"-",ChildMax,"), Heavy Exertion"&
      //" (EVR>",HeavyEVRTS,")"                                      ! Header for Group 44, TS
   WRITE(GrpHeader(45),'(A,F5.3,A,F5.2,A)') "Group: All Ages,"      &
      //" Active (PAI>",ActivePAI,"), Heavy Exertion (EVR>",        &
      HeavyEVRTS,")"                                                 ! Header for Group 45, TS
   WRITE(GrpHeader(46),'(A,I2,A,I2,A,F5.3,A,F5.2,A)')               &
      "Group: Children (ages ",ChildMin,"-",ChildMax,"), Active"    &
      //" (PAI>",ActivePAI,"), Heavy Exertion (EVR>",HeavyEVRTS,")"  ! Header for Group 46, TS 
   WRITE(GrpHeader(47),'(A,A,A,F5.2,A )') "Group: All Ages,",       &
     Trim(Disease),", Heavy Exertion (EVR>",HeavyEVRTS,")"           ! Header for Group 47, TS
   WRITE(GrpHeader(48),'(A,I2,A,I2,A,A,A,F5.2,A)')                  &
     "Group: Children (ages ",ChildMin,"-",ChildMax,"), ",          &
     Trim(Disease), ", Heavy Exertion (EVR>",HeavyEVRTS,")"          ! Header for Group 48, TS
   WRITE(GrpHeader(49),'(A,F5.2,A)') "Group: All Employed,"             &  
      //" Heavy Exertion (EVR>",HeavyEVRTS,")"                       ! Header for Group 49, TS     
  CALL WriteExposureTables(PL)                                       ! Write the tables for exposure
  IF (Pol(PL)%DoDose==YES) CALL WriteDoseTables(PL)                  ! Write the tables for dose
  IF (DebugLevel>1) WRITE(LogU,LV_FMT) LV_MSG, ProcName(ProcLevel)   ! Message for debugging
  ProcLevel = ProcLevel-1    
END SUBROUTINE WriteTablesFile

SUBROUTINE WriteExposureTables(PL)
   INTEGER (KIND=IK), INTENT(IN) :: PL                               ! Input variable (pollut number)
   INTEGER (KIND=IK) :: I, J, K, M, N, U, EFlag, Width               ! Local integer variables
   INTEGER (KIND=IK) :: PopIndex, NGroup, OutUnits, OZRisk           ! Local integer variables
   REAL    (KIND=RK) :: Cumul, D1, D2                                ! Local real variables
   CHARACTER (LEN=1),ALLOCATABLE :: C(:)                             ! Dynamic array of dashes   
   CHARACTER*40      :: TFI, TFR, TFH, CFMT, H3FMT                   ! Formats for writing table rows
   CHARACTER(LEN=11) :: Exps(3)  =(/"TIME,      ","PERSONDAYS,",    & 
      "MULTIPLE,  "/)                                                ! Table IDs: Exposures 
   CHARACTER(LEN=10) :: Types(8) = (/"WITHIN,   ","ATABOVE,  ",     &
      "DM1H,     ","DM8H,     ","DAVG,     ","SAVG,     ",          &
      "DMTS,     ","TIMESTEPS,"/)                                    ! Table IDs: Types  
   CHARACTER(LEN=6) :: EVRGroup(3) = (/"ALL,  ","MOD,  ","HEAVY,"/)  ! Table IDs: EVR Levels 
   CHARACTER(LEN=8) :: EMPGroup(2) = (/"ALL     ","EMPLOYED"/)       ! Table IDs: Employment Groups
   ! Global constants: IK, LogU, LONG, LV_FMT, LV_MSG, NO, RK, TabU, ST_FMT, ST_MSG, YES
   !                   PILOTTEXT, RK, SEXTEXT, TabU, UNITTEXT, ST_FMT, ST_MSG, YES
   ! Global vars used: AgeMin, AgeMax, DAvgDoseLev, DAvgExpLev, DebugLevel, DM1HDoseLev, DM1HExpLev,
   !                   DM8HDoseLev, DM8HExpLev, DMEHDoseLev, DoDose, FName, LogTables, NumDays, 
   !                   NumDistricts, NumMicros, NumPopTypes, NumProfiles, Person, TimeDoseLev, 
   !                   TimeExpLev, TotPop
   ! Global vars set : Ios, ProcLevel, ProcName
   ! Module constants: DAVGD, DAVGE, DM1HD, DM1HE, DM8HD, DM8HE, DMEHD,
   !                   H_EHD, SAVGD, SAVGE, TIMED, TIMEE
   ! Module vars used: DAvgDose, DAvgExp, DM1HDose, DM1HExp, DM8HDose, DM8HExp, DMEHDose, 
   !                   H_EHDose, MSum, NLev, SAvgDose, SAvgExp, TAccExp, TimeDose, TimeExp
   ! Module vars set : Array, Header, Header2, TableLev
   ! Intrinsic procs : ALLOCATE, CLOSE, COUNT, REAL, SIZE, SUM, SYSTEM_CLOCK, WRITE
   ! APEX procedures : CheckAllocation, WriteTables, WriteTablesTypeII
   ProcLevel = ProcLevel+1                                           ! Increment procedure depth
   ProcName(ProcLevel) = "WriteExposureTables"                       ! Procedure name for messages
   H3FMT="(5A)"                                                      ! Header3 format
   OutUnits=Pol(PL)%OutUnits                                         ! Output units for current pollut
   K = Pol(PL)%N_TimeExpLev                                          ! # of levels in next table
   N = NumProfiles                                                   ! # of profiles to sum across
   WRITE(TSText,'(I3, A4)')  TSMinutes, " min"                       ! Make text label for tables           
   IF (K>0 .AND. TableFlags(15)==1) THEN                             ! If TimeExp table is defined 
      J = MAX(1,K-4)                                                 ! # of extra columns for format
      WRITE(TFI,'(A22,I3,A6)') '(I4,A10,F16.0,3F14.0,',J,'F12.0)'    ! Integer format for TimeExp table
      WRITE(TFR,'(A22,I3,A6)') '(I4,A10,F16.4,3F14.4,',J,'F12.4)'    ! Real format for TimeExp table
      WRITE(TFH,'(A22,I3,A6)') '(4X,A10,F16.4,3F14.4,',J,'F12.4)'    ! Level format for TimeExp table
      Width = 12*K + 17                                              ! Width of data part of table
      ALLOCATE (C(Width),STAT=AllocErr)                              ! Create buffer for dashes
      CALL CheckAllocation(AllocErr)                                 ! Check the success of the alloc
      C = "-"                                                        ! Fill buffer with dashes
      WRITE(CFMT,'(A4,I4,A3)') '(A7,',Width,'A1)'                    ! Format for row of dashes
      DO M=0,NumMicros                                               ! Loop over microenvironments 
         Cumul = 0.                                                  ! Initialize cumulative total
         DO J=K,1,-1                                                 ! Loop downwards over table levels
            Cumul        = Cumul + TimeExp(M,J,PL)                   ! Accumulate data from old table
            TAccExp(M,J) = Cumul                                     ! Record accum data in new table 
         ENDDO                                                       ! Continue with next table level
      ENDDO                                                          ! Continue with next micro 
      DO I=1,2                                                       ! Loop over output units
         IF (I==1) U=Pol(PL)%TabU                                    ! First loop writes to table file 
         IF (I==2) U=LogU                                            ! Second loop writes to log file 
         IF (I==2 .AND. LogTables==NO) CYCLE                         ! Skip loop if no log output
         WRITE(Header3, H3FMT) TRIM(Exps(1)), TRIM(Types(1)),       &
            TRIM(PopGroup(1)), TRIM(EVRGroup(1)), TRIM(EmpGroup(1))  ! Write the table ID 
         WRITE(U,'(///,A39)') Header3                                ! Write the table ID 
         WRITE(U,'(A81,I8,A9,/)') "Exposure: Minutes in each"       &
            //" Exposure interval ("//UNITTEXT(OutUnits)//"), by "  &
            //"microenvironment, for N =",NumProfiles," Profiles"    ! Header for exposure time table
         WRITE(U,TFH) " Level:",(TimeExpLev(PL,J),J=1,K)             ! Write table levels
         WRITE(U,CFMT) "Micro  ",C                                   ! Write line of dashes
         D1 = SUM(TimeExp)/100.                                      ! Total time for % calculations
         DO M=0,NumMicros                                            ! Loop over microenvironments 
            D2 = TAccExp(M,1)/100.                                   ! Total time in this micro,as 1/%
            WRITE(U,TFI) M,"   Minutes",(TimeExp(M,J,PL),J=1,K)      ! Minutes at this level & micro
            IF (D2 == 0) THEN                                        ! If no time spent in micro           
               WRITE(U,TFR) M,"     Row_%",(0.0, J=1,K)              ! Percentages are 0.0
            ELSE                                                     ! Otherwise if time is not 0    
               WRITE(U,TFR) M,"     Row_%",(TimeExp(M,J,PL)/D2,J=1,K)! Percentage of this row
            ENDIF                                                    ! End time-in-micro logic
            WRITE(U,TFR) M,"     Tot_%",(TimeExp(M,J,PL)/D1,J=1,K)   ! Percentage of total across rows
         ENDDO                                                       ! Continue with next micro
         WRITE(U,CFMT) '-------',C                                   ! Write line of dashes
         WRITE(Header3, H3FMT) TRIM(Exps(1)), TRIM(Types(2)),       &
         TRIM(PopGroup(1)), TRIM(EVRGroup(1)), TRIM(EmpGroup(1))     ! Write the table ID 
         WRITE(U,'(///,A39)') Header3                                ! Write the table ID     
         WRITE(U,'(A87,I8,A9,/)') "Exposure: Minutes at or "        &
            //"above each Exposure Level ("//UNITTEXT(OutUnits)     &
            //"), by microenvironment, for N =",NumProfiles,        &
            " Profiles"                                              ! Header for cumul. TimeExp table 
         WRITE(U,TFH) " Level:",(TimeExpLev(PL,J),J=1,K)             ! Write table levels
         WRITE(U,CFMT) "Micro  ",C                                   ! Write line of dashes
         DO M=0,NumMicros                                            ! Loop over microenvironments 
            D2 = TAccExp(M,1)/100.                                   ! Total time in this micro,as 1/%
            WRITE(U,TFI) M,"   Minutes",(TAccExp(M,J),J=1,K)         ! Minutes at this level & micro
            IF (D2 == 0) THEN                                        ! If no time spent in micro           
               WRITE(U,TFR) M,"     Row_%",(0.0, J=1,K)              ! Percentages are 0.0
            ELSE                                                     ! Otherwise if time is not 0    
               WRITE(U,TFR) M,"     Row_%",(TAccExp(M,J)/D2,J=1,K)   ! Percentage of this row
            ENDIF                                                    ! End time-in-micro logic
            WRITE(U,TFR) M,"     Tot_%",(TAccExp(M,J)/D1,J=1,K)      ! Percentage of total across rows
         ENDDO                                                       ! Continue with next micro 
         WRITE(U,CFMT) '-------',C                                   ! Write line of dashes
      ENDDO                                                          ! Continue with next output unit
   ENDIF                                                             ! End of TimeExp logic block
   DO I=1,21                                                         ! Loop over groups   
      EFlag = 1                                                      ! Setting Employed Flag
      IF (TimeStepsPerDay<24) EXIT                                   ! Don't write if timestep>1hr 
      IF (Disease == "") THEN                                        ! If not modeling disease
        IF (I==5 .OR. I==12 .OR. I==19 .OR.                        &
            I==6 .OR. I==13 .OR. I==20) CYCLE                        ! Don't write disease tables
      ENDIF                                                          ! End no disease logic
      IF ((I==2 .OR. I==9 .OR.  I==16) .AND. TableFlags(14)/=1) CYCLE ! If no CHILD, cycle
      IF ((I==3 .OR. I==10 .OR. I==17) .AND.  TableFlags(8)/=1) CYCLE ! If no ACTIVE, cycle
      IF ((I==4 .OR. I==11 .OR. I==18) .AND.  TableFlags(9)/=1) CYCLE ! If no ACTIVECHILD, cycle
      IF ((I==5 .OR. I==12 .OR. I==19) .AND. TableFlags(10)/=1) CYCLE ! If no ILL, cycle
      IF ((I==6 .OR. I==13 .OR. I==20) .AND. TableFlags(11)/=1) CYCLE ! If no ILLCHILD, cycle 
      IF ((I==7 .OR. I==14 .OR. I==21) .AND. TableFlags(19)/=1) CYCLE ! If no EMPLOYMENT, cycle       
      IF (I>=8 .AND. I<=14 .AND. TableFlags(12)/=1) CYCLE            ! If no MOD, cycle
      IF (I>=15 .AND. I<=21 .AND. TableFlags(13)/=1) CYCLE           ! If no HEAVY, cycle      
      PopIndex=I-(CEILING(I/7.0)-1)*7                                ! Population index for group
      NGroup = NumProfiles                                           ! Initialize group pop to all
      IF (I==2 .OR. I==9 .OR.  I==16) NGroup=NChild                  ! Child groups   
      IF (I==3 .OR. I==10 .OR. I==17) NGroup=NActive                 ! Active groups   
      IF (I==4 .OR. I==11 .OR. I==18) NGroup=NActiveChild            ! Active child groups   
      IF (I==5 .OR. I==12 .OR. I==19) NGroup=NIll                    ! Ill groups   
      IF (I==6 .OR. I==13 .OR. I==20) NGroup=NIllChild               ! Ill child groups   
      IF (I==7 .OR. I==14 .OR. I==21) NGroup=NEmployed               ! Employed groups   
      IF (I==7 .OR. I==14 .OR. I==21) EFlag=2                        ! Setting Employed flag
      IF (TableFlags(1)) THEN                                        ! If table specified
        WRITE(Header,'(A91,I8,A29,I10)') "Exposure: Person-Days at "  &
            //"or above each Daily Maximum 1-Hour Exposure Level ("   & 
            //UNITTEXT(OutUnits)//"), for N =",NumProfiles,           &
            " Profiles.  Area Population =" , TotPop                 ! Header for DM1HExp table
        WRITE(Header2,'(A85)') GrpHeader(I)                          ! Write the group header   
        WRITE(Header3, H3FMT) TRIM(Exps(2)), TRIM(Types(3)),          &
            TRIM(PopGroup(PopIndex)), TRIM(EVRGroup(CEILING(I/7.0))), &
            TRIM(EmpGroup(EFlag))                                    ! Write the table ID 
        K = Pol(PL)%N_DM1HExpLev                                     ! # of levels in DM1HExp table
        IF (K>0 .AND. NGroup>0) THEN                                 ! If DM1HExp table is defined 
            Array(1:N,1:K) = REAL(DM1HExp(:,:,I,PL),LONG)            ! Fill table buffer with data
            TableLev(1:K)  = REAL(DM1HExpLev(PL,1:K),LONG)           ! Fill level buffer with data
            CALL WriteTables(N,NGroup,K,NumDays,YES, PL,NO)          ! Code is in OutputModule
        ENDIF                                                        ! End of DM1HExp table logic
      ENDIF                                                          ! End of write table logic
      IF (TableFlags(2)) THEN                                        ! If table specified      
        WRITE(Header,'(A91,I8,A29,I10)') "Exposure: Person-Days at "  &
            //"or above each Daily Maximum 8-Hour Exposure Level ("   &
            //UNITTEXT(OutUnits)//"), for N =",NumProfiles,           &
            " Profiles.  Area Population =", TotPop                  ! Header for DM8HExp table
        WRITE(Header3, H3FMT) TRIM(Exps(2)), TRIM(Types(4)),          &
            TRIM(PopGroup(PopIndex)), TRIM(EVRGroup(CEILING(I/7.0))), &
            TRIM(EmpGroup(EFlag))                                        ! Write the table ID 
        IF (I<8) WRITE(Header2,'(A85)') GrpHeader(I)                 ! Write the group header 
        IF (I>=8) WRITE(Header2,'(A85)') GrpHeader(I+14)             ! Write the EVR grp headers
        K = Pol(PL)%N_DM8HExpLev                                     ! # of levels in DM8HExp table
        IF (K>0 .AND. NGroup>0) THEN                                 ! If DM8HExp table is defined 
            Array(1:N,1:K) = REAL(DM8HExp(:,:,I,PL),LONG)            ! Fill table buffer with data
            TableLev(1:K)  = REAL(DM8HExpLev(PL,1:K),LONG)           ! Fill Level buffer with data 
            OZRisk = NO
            IF (Up(Pol(PL)%Name)=="OZONE".AND.I>=8.AND.I<+14) THEN
              OZRisk=YES
              WRITE(Header5,'(A71,I8,A29,I10)') "FEV1 Decrement: "    &
              //"Number of persons affected in each exposure bin, "   &
              //"for N=",NumProfiles," profiles.  Area population =", &
              TotPop
            ENDIF   
            CALL WriteTables(N,NGroup,K,NumDays,YES, PL,OZRisk)      ! Code is in OutputModule
        ENDIF                                                        ! End table level logic   
      ENDIF                                                          ! End of write table logic 
      IF (TableFlags(1)) THEN                                        ! If table specified                    
        WRITE(Header,'(A131,I8,A10)') "Exposure: Number of "          &
            //"Simulated Persons with Multiple Exposures at or "       &
            //"above each Daily Maximum 1-Hour Exposure Level ("       &
            //UNITTEXT(OutUnits)//"), for N =",NumProfiles,            &
            " Profiles."                                             ! Header for 1H mult exposures
        WRITE(Header2,'(A85)') GrpHeader(I)                          ! Write the group header
        WRITE(Header3, H3FMT) TRIM(Exps(3)), TRIM(Types(3)),          &
            TRIM(PopGroup(PopIndex)), TRIM(EVRGroup(CEILING(I/7.0))), &
            TRIM(EmpGroup(EFlag))                                        ! Write the table ID 
        K = Pol(PL)%N_DM1HExpLev                                     ! # of levels in DM1HExp table
        IF (K>0) THEN                                                ! If DM8HExp table is defined 
            Array(1:N,1:K) = REAL(DM1HExp(:,:,I,PL),LONG)            ! Fill table buffer with data
            TableLev(1:K)  = REAL(DM1HExpLev(PL,1:K),LONG)           ! Fill level buffer with data 
            CALL WriteTablesTypeII(N,K,PL)                           ! Code is in OutputModule
        ENDIF                                                        ! End of table level logic 
      ENDIF                                                          ! End of write table logic    
      IF (TableFlags(2)) THEN                                        ! If table specified                  
        WRITE(Header,'(A131,I8,A10)') "Exposure: Number of "           &
            //"Simulated Persons with Multiple Exposures at or "       &
            //"above each Daily Maximum 8-Hour Exposure Level ("       &
            //UNITTEXT(OutUnits)//"), for N =",NumProfiles,            &
            " Profiles."                                             ! Header for 8H mult exposures
        IF (I<8) WRITE(Header2,'(A85)') GrpHeader(I)                 ! Write the group header
        IF (I>=8) WRITE(Header2,'(A85)')GrpHeader(I+14)              ! Write the EVR grp headers
        WRITE(Header3, H3FMT) TRIM(Exps(3)), TRIM(Types(4)),          &
            TRIM(PopGroup(PopIndex)), TRIM(EVRGroup(CEILING(I/7.0))), &
            TRIM(EmpGroup(EFlag))                                        ! Write the table ID 
        K = Pol(PL)%N_DM8HExpLev                                     ! # of levels in DM8HExp table
        IF (K>0) THEN                                                ! If DM8HExp table is defined 
            N = SIZE(DM8HExp,1)                                      ! # of levels in this table
            Array(1:N,1:K) = REAL(DM8HExp(:,:,I,PL),LONG)            ! Fill table buffer with data
            TableLev(1:K)  = REAL(DM8HExpLev(PL,1:K),LONG)           ! Fill level buffer with data 
            CALL WriteTablesTypeII(N,K,PL)                           ! Code is in OutputModule
        ENDIF                                                        ! End table level logic  
      ENDIF                                                          ! End of write table logic             
   ENDDO                                                             ! End loop over groups
   DO I=1,21                                                         ! Loop over groups  
      EFlag = 1                                                      ! Setting Employed Flag    
      IF (DoTimeSteps==NO) EXIT                                      ! Only write if need timesteps 
      IF (Disease == "") THEN                                        ! If not modeling disease
        IF (I==5 .OR. I==12 .OR. I==19 .OR.                        &
            I==6 .OR. I==13 .OR. I==20) CYCLE                        ! Don't write disease tables
      ENDIF                                                          ! End no disease logic
      IF ((I==2 .OR. I==9 .OR.  I==16) .AND. TableFlags(14)/=1) CYCLE ! If no CHILD, cycle
      IF ((I==3 .OR. I==10 .OR. I==17) .AND.  TableFlags(8)/=1) CYCLE ! If no ACTIVE, cycle
      IF ((I==4 .OR. I==11 .OR. I==18) .AND.  TableFlags(9)/=1) CYCLE ! If no ACTIVECHILD, cycle
      IF ((I==5 .OR. I==12 .OR. I==19) .AND. TableFlags(10)/=1) CYCLE ! If no ILL, cycle
      IF ((I==6 .OR. I==13 .OR. I==20) .AND. TableFlags(11)/=1) CYCLE ! If no ILLCHILD, cycle 
      IF ((I==7 .OR. I==14 .OR. I==21) .AND. TableFlags(19)/=1) CYCLE ! If no EMPLOYMENT, cycle       
      IF (I>=8 .AND. I<=14 .AND. TableFlags(12)/=1) CYCLE            ! If no MOD, cycle
      IF (I>=15 .AND. I<=21 .AND. TableFlags(13)/=1) CYCLE           ! If no HEAVY, cycle      
      PopIndex=I-(CEILING(I/7.0)-1)*7                                ! Population index for group            
      NGroup = NumProfiles                                           ! Initialize group pop to all
      IF (I==2 .OR. I==9 .OR.  I==16) NGroup=NChild                  ! Child groups   
      IF (I==3 .OR. I==10 .OR. I==17) NGroup=NActive                 ! Active groups   
      IF (I==4 .OR. I==11 .OR. I==18) NGroup=NActiveChild            ! Active child groups   
      IF (I==5 .OR. I==12 .OR. I==19) NGroup=NIll                    ! Ill groups   
      IF (I==6 .OR. I==13 .OR. I==20) NGroup=NIllChild               ! Ill child groups   
      IF (I==7 .OR. I==14 .OR. I==21) NGroup=NEmployed               ! Employed groups   
      IF (I==7 .OR. I==14 .OR. I==21) EFlag=2                        ! Setting Employed flag
      IF (TableFlags(3)) THEN                                        ! If table specified      
        WRITE(Header,'(A92,I8,A29,I10)') "Exposure: Person-Days at "  &
            //"or above each Daily Maximum "//TSText                   &
            //" Exposure Level ("                                      & 
            //UNITTEXT(OutUnits)//"), for N =",NumProfiles,            &
            " Profiles.  Area Population =" , TotPop                 ! Header for DMTSExp table
        IF (I<8) WRITE(Header2,'(A85)') GrpHeader(I)                 ! Write the group header 
        IF (I>=8) WRITE(Header2,'(A85)') GrpHeader(I+28)             ! Write the EVR grp headers         
        WRITE(Header3, H3FMT) TRIM(Exps(2)), TRIM(Types(7)),          &
            TRIM(PopGroup(PopIndex)), TRIM(EVRGroup(CEILING(I/7.0))), &
            TRIM(EmpGroup(EFlag))                                        ! Write the table ID 
        K = Pol(PL)%N_DMTSExpLev                                     ! # of levels in DMTSExp table
        IF (K>0 .AND. NGroup>0) THEN                                 ! If DMTSExp table is defined 
            N = SIZE(DMTSExp,1)                                      ! # of levels in this table  
            Array(1:N,1:K) = REAL(DMTSExp(:,:,I,PL),LONG)            ! Fill table buffer with data
            TableLev(1:K)  = REAL(DMTSExpLev(PL,1:K),LONG)           ! Fill level buffer with data
            CALL WriteTables(N,NGroup,K,NumDays,YES, PL,NO)          ! Code is in OutputModule
        ENDIF                                                        ! End of DMTSExp table logic            
        WRITE(Header,'(A132,I8,A10)') "Exposure: Number of "           &
            //"Simulated Persons with Multiple Exposures at or "       &
            //"above each Daily Maximum "//TSText                      &
            //" Exposure Level ("                                      &
            //UNITTEXT(OutUnits)//"), for N =",NumProfiles,            &
            " Profiles."                                             ! Header for ts mult exposures
        IF (I<8) WRITE(Header2,'(A85)') GrpHeader(I)                 ! Write the group header 
        IF (I>=8) WRITE(Header2,'(A85)') GrpHeader(I+28)             ! Write the EVR grp headers   
        WRITE(Header3, H3FMT) TRIM(Exps(3)), TRIM(Types(7)),          &
            TRIM(PopGroup(PopIndex)), TRIM(EVRGroup(CEILING(I/7.0))), &
            TRIM(EmpGroup(EFlag))                                        ! Write the table ID 
        K = Pol(PL)%N_DMTSExpLev                                     ! # of levels in DMTSExp table
        IF (K>0) THEN                                                ! If DMTSExp table is defined 
            N = SIZE(DMTSExp,1)                                      ! # of levels in this table  
            Array(1:N,1:K) = REAL(DMTSExp(:,:,I,PL),LONG)            ! Fill table buffer with data
            TableLev(1:K)  = REAL(DMTSExpLev(PL,1:K),LONG)           ! Fill level buffer with data 
            CALL WriteTablesTypeII(N,K,PL)                           ! Code is in OutputModule
        ENDIF                                                        ! End of table level logic           
        WRITE(Header,'(A128,I8,A10)') "Exposure: Number of "          &
            //"Simulated Persons with Multiple Exceedances in"         &
            //" Simulation above each "//TSTEXT                        & 
            //" Exposure Level ("                                      &
            //UNITTEXT(OutUnits)//"), for N =",NumProfiles,            &
            " Profiles."                                             ! Header for mult exceedances
        IF (I<8) WRITE(Header2,'(A85)') GrpHeader(I)                 ! Write the group header (TS?)
        IF (I>=8) WRITE(Header2,'(A85)')GrpHeader(I+28)              ! Write the EVR grp headers
        WRITE(Header3, H3FMT) TRIM(Exps(3)), TRIM(Types(8)),          &
            TRIM(PopGroup(PopIndex)), TRIM(EVRGroup(CEILING(I/7.0))), &
            TRIM(EmpGroup(EFlag))                                        ! Write the table ID 
        K = Pol(PL)%N_TSExpLev                                       ! # of levels in DM8HExp table
        IF (K>0) THEN                                                ! If table is defined 
            N = SIZE(ALLTSExp,1)                                     ! # profiles
            Array(1:N,1:K) = REAL(ALLTSExp(:,:,I,PL),LONG)           ! Fill table buffer with data
            TableLev(1:K)  = REAL(TSExpLev(PL,1:K),LONG)             ! Fill level buffer with data 
            CALL WriteTablesTypeIII(N,K,PL)                          ! Code is in OutputModule
        ENDIF                                                        ! End table level logic  
      ENDIF                                                          ! End of write table logic           
   ENDDO                                                             ! End loop over groups
   IF (TableFlags(16)) THEN                                          ! If table specified      
    WRITE(Header,'(A84,I8,A29,I10)') "Exposure: Person-Days at or "  &
        //"above each Daily Average Exposure Level ("                &
        //UNITTEXT(OutUnits)//"), for N =",NumProfiles,              &
        " Profiles.  Area Population =", TotPop                      ! Header for DAvgExp table
    WRITE(Header2,'(A85)') ""                                        ! Group header empty  
    WRITE(Header3, H3FMT) TRIM(Exps(2)), TRIM(Types(5)),             &
        TRIM(PopGroup(1)), TRIM(EVRGroup(1)), TRIM(EmpGroup(EFlag))  ! Write the table ID 
    K = Pol(PL)%N_DAvgExpLev                                         ! # of levels in DAvgExp table
    IF (K>0) THEN                                                    ! If DAvgExp table is defined 
        Array(1:N,1:K) = REAL(DAvgExp(:,:,PL),LONG)                  ! Fill table buffer with data
        TableLev(1:K)  = REAL(DAvgExpLev(PL,1:K),LONG)               ! Fill level buffer with data 
        CALL WriteTables(N,NumProfiles,K,NumDays,YES,PL,NO)          ! Code is in OutputModule
    ENDIF                                                            ! End of DAvgExp table logic
    WRITE(Header,'(A82,I8,A29,I10)') "Exposure: Persons at or "      &
        //"above each Overall Average Exposure Level ("              &
        //UNITTEXT(OutUnits)//"), for N =",NumProfiles,              &
        " Profiles.  Area Population =", TotPop                      ! Header for SAvgExp table
    WRITE(Header2,'(A85)') ""                                        ! Group header empty  
    WRITE(Header3, H3FMT) TRIM(Exps(2)), TRIM(Types(6)),             &
        TRIM(PopGroup(1)),TRIM(EVRGroup(1)), TRIM(EmpGroup(EFlag))   ! Write the table ID 
    K = Pol(PL)%N_SAvgExpLev                                         ! # of levels in SAvgExp table
    IF (K>0) THEN                                                    ! If SAvgExp table is defined 
        Array(1:N,1:K) = REAL(SAvgExp(:,:,PL),LONG)                  ! Fill table buffer with data
        TableLev(1:K)  = REAL(SAvgExpLev(PL,1:K),LONG)               ! Fill level buffer with data 
        CALL WriteTables(N,NumProfiles,K,1,NO,PL,NO)                 ! Code is in OutputModule
    ENDIF                                                            ! End of SAvgExp table logic
   ENDIF                                                             ! End write table logic
   IF (DebugLevel>1) WRITE(LogU,LV_FMT) LV_MSG, ProcName(ProcLevel)  ! Message for debugging
   ProcLevel = ProcLevel-1                                           ! Decrement procedure depth
END SUBROUTINE WriteExposureTables

SUBROUTINE WriteDoseTables(PL)   
   INTEGER (KIND=IK), INTENT(IN) :: PL                               ! Input variable (pollut number)
   INTEGER (KIND=IK) :: I, J, K, N, EFlag                            ! Local integer variables
   INTEGER (KIND=IK) :: PopIndex, NGroup, OutUnits                   ! Local integer variables
   REAL    (KIND=RK) :: Cumul                                        ! Local real variables
   CHARACTER*40      :: H3FMT                                        ! Formats for writing table rows
   CHARACTER(LEN=12) :: Doses(5) =(/"PERSONDAYS, ","PERSONS,    ", & 
      "PERSONHOURS,","MINUTES,    ","MULTIPLE,   "/)                 ! Table IDs: Doses 
   CHARACTER(LEN=12) :: Types(9) = (/"DOSEDMEH,   ","DOSEDM1H,   ", &  
      "DOSEDM8H,   ","DOSEDAVG,   ","DOSESAVG,   ","DOSEWITHIN, ",  &
      "DOSEATABOVE,","DOSEDMTS,   ","TIMESTEPS,  "/)                 ! Table IDs: Types  
   CHARACTER(LEN=6) :: EVRGroup(3) = (/"ALL,  ","MOD,  ","HEAVY,"/)     ! Table IDs: EVR Levels 
   CHARACTER(LEN=8) :: EmpGRoup(2) = (/"ALL     ","EMPLOYED"/)       ! Table IDs: Employed                     
   ! Global constants: IK, LogU, LONG, LV_FMT, LV_MSG, NO, RK, TabU, ST_FMT, ST_MSG, YES
   ! Global vars used: AgeMin, AgeMax, DAvgDoseLev, DAvgExpLev, DebugLevel, DM1HDoseLev, DM1HExpLev,
   !                   DM8HDoseLev, DM8HExpLev, DMEHDoseLev, DoDose, FName, LogTables, NumDays, 
   !                   NumDistricts, NumMicros, NumPopTypes, NumProfiles, Person, TimeDoseLev, 
   !                   TimeExpLev, TotPop
   ! Global vars set : Ios, ProcLevel, ProcName
   ! Module constants: DAVGD, DAVGE, DM1HD, DM1HE, DM8HD, DM8HE, DMEHD,
   !                   H_EHD, SAVGD, SAVGE, TIMED, TIMEE
   ! Module vars used: DAvgDose, DAvgExp, DM1HDose, DM1HExp, DM8HDose, DM8HExp, DMEHDose, 
   !                   H_EHDose, MSum, NLev, SAvgDose, SAvgExp, TAccExp, TimeDose, TimeExp
   ! Module vars set : Array, Header, Header2, TableLev
   ! Intrinsic procs : ALLOCATE, CLOSE, COUNT, REAL, SIZE, SUM, SYSTEM_CLOCK, WRITE
   ! APEX procedures : CheckAllocation, WriteTables, WriteTablesTypeII
   ProcLevel = ProcLevel+1                                           ! Increment procedure depth
   ProcName(ProcLevel) = "WriteDoseTables"                           ! Procedure name for messages
   H3FMT="(5A)"                                                      ! Header3 format
   N = NumProfiles                                                   ! # of profiles to sum across
   OutUnits=Pol(PL)%OutUnits                                         ! Output units for current pollut
   DO I=1,21                                                         ! Loop over groups
      EFlag = 1                                                      ! Setting Employed Flag   
      IF (DoHours ==NO) EXIT                                         ! Don't write if timestep>1hr
      IF (Disease == "") THEN                                        ! If not modeling disease
        IF (I==5 .OR. I==12 .OR. I==19 .OR.                        &
            I==6 .OR. I==13 .OR. I==20) CYCLE                        ! Don't write disease tables
      ENDIF                                                          ! End no disease logic
      IF ((I==2 .OR. I==9 .OR.  I==16) .AND. TableFlags(14)/=1) CYCLE ! If no CHILD, cycle
      IF ((I==3 .OR. I==10 .OR. I==17) .AND.  TableFlags(8)/=1) CYCLE ! If no ACTIVE, cycle
      IF ((I==4 .OR. I==11 .OR. I==18) .AND.  TableFlags(9)/=1) CYCLE ! If no ACTIVECHILD, cycle
      IF ((I==5 .OR. I==12 .OR. I==19) .AND. TableFlags(10)/=1) CYCLE ! If no ILL, cycle
      IF ((I==6 .OR. I==13 .OR. I==20) .AND. TableFlags(11)/=1) CYCLE ! If no ILLCHILD, cycle 
      IF ((I==7 .OR. I==14 .OR. I==21) .AND. TableFlags(19)/=1) CYCLE ! If no EMPLOYMENT, cycle       
      IF (I>=8 .AND. I<=14 .AND. TableFlags(12)/=1) CYCLE            ! If no MOD, cycle
      IF (I>=15 .AND. I<=21 .AND. TableFlags(13)/=1) CYCLE           ! If no HEAVY, cycle      
      PopIndex=I-(CEILING(I/7.0)-1)*7                                ! Population index for group            
      NGroup = NumProfiles                                           ! Initialize group pop to all
      IF (I==2 .OR. I==9 .OR.  I==16) NGroup=NChild                  ! Child groups   
      IF (I==3 .OR. I==10 .OR. I==17) NGroup=NActive                 ! Active groups   
      IF (I==4 .OR. I==11 .OR. I==18) NGroup=NActiveChild            ! Active child groups   
      IF (I==5 .OR. I==12 .OR. I==19) NGroup=NIll                    ! Ill groups   
      IF (I==6 .OR. I==13 .OR. I==20) NGroup=NIllChild               ! Ill child groups   
      IF (I==7 .OR. I==14 .OR. I==21) NGroup=NEmployed               ! Employed groups   
      IF (I==7 .OR. I==14 .OR. I==21) EFlag=2                        ! Setting Employed flag
      IF (TableFlags(6)) THEN                                        ! If table specified           
        WRITE(Header,'(A77,I8,A29,I10)') "Dose: Person-Days at or "   &
            //"above each Daily Max End-of-Hour Dose Level, "   &
            //" for N =", NumProfiles," Profiles.  Area Population ="  &
            , TotPop                                                 ! Header for DMEHDose table
        WRITE(Header2,'(A85)') GrpHeader(I)                          ! Write the group header   
        WRITE(Header3, H3FMT) TRIM(Doses(1)), TRIM(Types(1)),         &
            TRIM(PopGroup(PopIndex)), TRIM(EVRGroup(CEILING(I/7.0))), &
            TRIM(EmpGroup(EFlag))                                    ! Write the table ID 
        K = Pol(PL)%N_DMEHDoseLev                                    ! # of levels in DMEHDose table
        IF (K>0 .AND. NGroup>0) THEN                                 ! If DMEHDose table is defined 
            Array(1:N,1:K) = REAL(DMEHDose(:,:,I,PL),LONG)           ! Fill table buffer with data
            TableLev(1:K)  = REAL(DMEHDoseLev(PL,1:K),LONG)          ! Fill level buffer with data  
            CALL WriteTables(N,NGroup,K,NumDays,YES, PL,NO)          ! Code is in OutputModule
        ENDIF                                                        ! End of DMEHDose table logic
      ENDIF                                                          ! End write table logic
      IF (TableFlags(4)) THEN                                        ! If table specified             
        WRITE(Header,'(A71,I8,A29,I10)') "Dose: Person-Days at or "   &
            //"above each Daily Max 1-Hour Dose Level,"                &
            //" for N =", NumProfiles," Profiles.  Area Population ="  &
            , TotPop                                                 ! Header for DM1HDose table
        K = Pol(PL)%N_DM1HDoseLev                                    ! # of levels in DM1HDose table
        WRITE(Header2,'(A85)') GrpHeader(I)                          ! Write the group header   
        WRITE(Header3, H3FMT) TRIM(Doses(1)), TRIM(Types(2)),         &
            TRIM(PopGroup(PopIndex)), TRIM(EVRGroup(CEILING(I/7.0))), &
            TRIM(EmpGroup(EFlag))                                    ! Write the table ID 
        IF (K>0 .AND. NGroup>0) THEN                                 ! If DM1HDose table is defined 
            Array(1:N,1:K) = REAL(DM1HDose(:,:,I,PL),LONG)           ! Fill table buffer with data
            TableLev(1:K)  = REAL(DM1HDoseLev(PL,1:K),LONG)          ! Fill level buffer with data 
            CALL WriteTables(N,NGroup,K,NumDays,YES, PL,NO)          ! Code is in OutputModule
        ENDIF                                                        ! End of DM1HDose table logic 
      ENDIF                                                          ! End write table logic   
      IF (TableFlags(5)) THEN                                        ! If table specified                   
        WRITE(Header,'(A71,I8,A29,I10)') "Dose: Person-Days at or "   &
            //"above each Daily Max 8-Hour Dose Level,"                &
            //" for N =", NumProfiles," Profiles.  Area Population ="  &
            , TotPop                                                 ! Header for DM8HDose table
        IF (I<8) WRITE(Header2,'(A85)') GrpHeader(I)                 ! Write the group header
        IF (I>=8) WRITE(Header2,'(A85)')GrpHeader(I+14)              ! Write the EVR grp headers
        WRITE(Header3, H3FMT) TRIM(Doses(1)), TRIM(Types(3)),         &
            TRIM(PopGroup(PopIndex)), TRIM(EVRGroup(CEILING(I/7.0))), &
            TRIM(EmpGroup(EFlag))                                    ! Write the table ID 
        K = Pol(PL)%N_DM8HDoseLev                                    ! # of levels in DM8HDose table
        IF (K>0 .AND. NGroup>0) THEN                                 ! If DM8HDose table is defined 
            Array(1:N,1:K) = REAL(DM8HDose(:,:,I,PL),LONG)           ! Fill table buffer with data
            TableLev(1:K)  = DM8HDoseLev(PL,1:K)                     ! Fill level buffer with data 
            CALL WriteTables(N,NGroup,K,NumDays,YES, PL,NO)          ! Code is in OutputModule
        ENDIF                                                        ! End of DM8HDose table logic
      ENDIF                                                          ! End write table logic 
      IF (TableFlags(6)) THEN                                        ! If table specified                     
        WRITE(Header,'(A121,I8,A10)') "Dose: Number of "          &
            //"Simulated Persons with Multiple Doses at or "       &
            //"above each Daily Max End-of-Hour Dose Level, ("      &
            //UNITTEXT(OutUnits)//"), for N =",NumProfiles,        &
            " Profiles."                                             ! Header for EH mult exposures
        WRITE(Header2,'(A85)') GrpHeader(I)                          ! Write the group header
        WRITE(Header3, H3FMT) TRIM(Doses(5)), TRIM(Types(1)),         &
            TRIM(PopGroup(PopIndex)), TRIM(EVRGroup(CEILING(I/7.0))), &
            TRIM(EmpGroup(EFlag))                                    ! Write the table ID 
        K = Pol(PL)%N_DMEHDoseLev                                    ! # of levels in DMEHDose table
        IF (K>0) THEN                                                ! If DMEHDose table is defined 
            Array(1:N,1:K) = REAL(DMEHDose(:,:,I,PL),LONG)           ! Fill table buffer with data
            TableLev(1:K)  = REAL(DMEHDoseLev(PL,1:K),LONG)          ! Fill level buffer with data 
            CALL WriteTablesTypeIV(N,K,PL)                           ! Code is in OutputModule
        ENDIF                                                        ! End of table level logic  
      ENDIF                                                          ! End write table logic    
      IF (TableFlags(4)) THEN                                        ! If table specified                 
        WRITE(Header,'(A119,I8,A10)') "Dose: Number of "          &
            //"Simulated Persons with Multiple Doses at or "       &
            //"above each Daily Maximum 1-Hour Dose Level ("       &
            //UNITTEXT(OutUnits)//"), for N =",NumProfiles,        &
            " Profiles."                                             ! Header for 1H mult exposures
        WRITE(Header2,'(A85)') GrpHeader(I)                          ! Write the group header
        WRITE(Header3, H3FMT) TRIM(Doses(5)), TRIM(Types(2)),          &
            TRIM(PopGroup(PopIndex)), TRIM(EVRGroup(CEILING(I/7.0))),  &
            TRIM(EmpGroup(EFlag))                                    ! Write the table ID 
        K = Pol(PL)%N_DM1HDoseLev                                    ! # of levels in DM1HDose table
        IF (K>0) THEN                                                ! If DM1HDose table is defined 
            Array(1:N,1:K) = REAL(DM1HDose(:,:,I,PL),LONG)           ! Fill table buffer with data
            TableLev(1:K)  = REAL(DM1HDoseLev(PL,1:K),LONG)          ! Fill level buffer with data 
            CALL WriteTablesTypeIV(N,K,PL)                           ! Code is in OutputModule
        ENDIF                                                        ! End of table level logic 
      ENDIF                                                          ! End write table logic     
      IF (TableFlags(5)) THEN                                        ! If table specified                  
        WRITE(Header,'(A119,I8,A10)') "Dose: Number of "          &
            //"Simulated Persons with Multiple Doses at or "       &
            //"above each Daily Maximum 8-Hour Dose Level ("       &
            //UNITTEXT(OutUnits)//"), for N =",NumProfiles,            &
            " Profiles."                                             ! Header for 8H mult doses
        IF (I<8) WRITE(Header2,'(A85)') GrpHeader(I)                 ! Write the group header
        IF (I>=8) WRITE(Header2,'(A85)')GrpHeader(I+14)              ! Write the EVR grp headers
        WRITE(Header3, H3FMT) TRIM(Doses(5)), TRIM(Types(3)),          &
            TRIM(PopGroup(PopIndex)), TRIM(EVRGroup(CEILING(I/7.0))),  &
            TRIM(EmpGroup(EFlag))                                    ! Write the table ID 
        K = Pol(PL)%N_DM8HDoseLev                                    ! # of levels in DM8HDose table
        IF (K>0) THEN                                                ! If DM8HDose table is defined 
            N = SIZE(DM8HDose,1)                                     ! # of levels in this table
            Array(1:N,1:K) = REAL(DM8HDose(:,:,I,PL),LONG)           ! Fill table buffer with data
            TableLev(1:K)  = REAL(DM8HDoseLev(PL,1:K),LONG)          ! Fill level buffer with data 
            CALL WriteTablesTypeIV(N,K,PL)                           ! Code is in OutputModule
        ENDIF                                                        ! End table level logic  
      ENDIF                                                          ! End write table logic            
   ENDDO                                                             ! End loop over groups
   DO I=1,21                                                         ! Loop over groups   
      EFlag = 1                                                      ! Setting Employed Flag   
      IF (DoTimeSteps==NO) EXIT                                      ! Only write if need timesteps 
      IF (Disease == "") THEN                                        ! If not modeling disease
        IF (I==5 .OR. I==12 .OR. I==19 .OR.                        &
            I==6 .OR. I==13 .OR. I==20) CYCLE                        ! Don't write disease tables
      ENDIF                                                          ! End no disease logic
      IF ((I==2 .OR. I==9 .OR.  I==16) .AND. TableFlags(14)/=1) CYCLE ! If no CHILD, cycle
      IF ((I==3 .OR. I==10 .OR. I==17) .AND.  TableFlags(8)/=1) CYCLE ! If no ACTIVE, cycle
      IF ((I==4 .OR. I==11 .OR. I==18) .AND.  TableFlags(9)/=1) CYCLE ! If no ACTIVECHILD, cycle
      IF ((I==5 .OR. I==12 .OR. I==19) .AND. TableFlags(10)/=1) CYCLE ! If no ILL, cycle
      IF ((I==6 .OR. I==13 .OR. I==20) .AND. TableFlags(11)/=1) CYCLE ! If no ILLCHILD, cycle 
      IF ((I==7 .OR. I==14 .OR. I==21) .AND. TableFlags(19)/=1) CYCLE ! If no EMPLOYMENT, cycle       
      IF (I>=8 .AND. I<=14 .AND. TableFlags(12)/=1) CYCLE            ! If no MOD, cycle
      IF (I>=15 .AND. I<=21 .AND. TableFlags(13)/=1) CYCLE           ! If no HEAVY, cycle      
      PopIndex=I-(CEILING(I/7.0)-1)*7                                ! Population index for group            
      NGroup = NumProfiles                                           ! Initialize group pop to all
      IF (I==2 .OR. I==9 .OR.  I==16) NGroup=NChild                  ! Child groups   
      IF (I==3 .OR. I==10 .OR. I==17) NGroup=NActive                 ! Active groups   
      IF (I==4 .OR. I==11 .OR. I==18) NGroup=NActiveChild            ! Active child groups   
      IF (I==5 .OR. I==12 .OR. I==19) NGroup=NIll                    ! Ill groups   
      IF (I==6 .OR. I==13 .OR. I==20) NGroup=NIllChild               ! Ill child groups   
      IF (I==7 .OR. I==14 .OR. I==21) NGroup=NEmployed               ! Employed groups   
      IF (I==7 .OR. I==14 .OR. I==21) EFlag=2                        ! Setting Employed flag
      IF (TableFlags(7)) THEN                                        ! If table specified          
        WRITE(Header,'(A76,I8,A29,I10)') "Dose: Person-Days at "      &
            //"or above each Daily Maximum "//TSText                   &
            //" Dose Level, for N =",NumProfiles,                      &
            " Profiles.  Area Population =" , TotPop                 ! Header for DMTSDose table
        IF (I<8) WRITE(Header2,'(A85)') GrpHeader(I)                 ! Write the group header 
        IF (I>=8) WRITE(Header2,'(A85)') GrpHeader(I+28)             ! Write the EVR grp headers             
        WRITE(Header3, H3FMT) TRIM(Doses(5)), TRIM(Types(8)),         &
            TRIM(PopGroup(PopIndex)), TRIM(EVRGroup(CEILING(I/7.0))), &
            TRIM(EmpGroup(EFlag))                                    ! Write the table ID 
        K = Pol(PL)%N_DMTSDoseLev                                    ! # of levels in DMTSDose table
        IF (K>0 .AND. NGroup>0) THEN                                 ! If DMTSDose table is defined 
            Array(1:N,1:K) = REAL(DMTSDose(:,:,I,PL),LONG)           ! Fill table buffer with data
            TableLev(1:K)  = REAL(DMTSDoseLev(PL,1:K),LONG)          ! Fill level buffer with data
            CALL WriteTables(N,NGroup,K,NumDays,YES, PL,NO)          ! Code is in OutputModule
        ENDIF                                                        ! End of DMTSDose table logic
        WRITE(Header,'(A111,I8,A10)') "Dose: Number of "              &
            //"Simulated Persons with Multiple Doses at or "           &
            //"above each Daily Maximum "//TSText                      & 
            //" Dose Level, for N =",NumProfiles,                      &
            " Profiles."                                             ! Header for TS mult doses
        IF (I<8) WRITE(Header2,'(A85)') GrpHeader(I)                 ! Write the group header 
        IF (I>=8) WRITE(Header2,'(A85)') GrpHeader(I+28)             ! Write the EVR grp headers            
        WRITE(Header3, H3FMT) TRIM(Doses(5)), TRIM(Types(8)),        &
            TRIM(PopGroup(PopIndex)), TRIM(EVRGroup(CEILING(I/7.0))), &
            TRIM(EmpGroup(EFlag))                                    ! Write the table ID 
        K = Pol(PL)%N_DMTSDoseLev                                    ! # of levels in table
        IF (K>0) THEN                                                ! If DMTSDose table is defined 
            Array(1:N,1:K) = REAL(DMTSDose(:,:,I,PL),LONG)           ! Fill table buffer with data
            TableLev(1:K)  = REAL(DMTSDoseLev(PL,1:K),LONG)          ! Fill level buffer with data 
            CALL WriteTablesTypeIV(N,K,PL)                           ! Code is in OutputModule
        ENDIF                                                        ! End of table level logic 
        WRITE(Header,'(A108,I8,A10)') "Dose: Number of "              &
            //"Simulated Persons with Multiple Exceedances in "        &
            //"Simulation above each "//TSText                         &
            //" Dose Level, for N =",NumProfiles," Profiles."        ! Header for mult exceedances
        IF (I<8) WRITE(Header2,'(A85)') GrpHeader(I)                 ! Write the group header
        IF (I>=8) WRITE(Header2,'(A85)')GrpHeader(I+28)              ! Write the EVR grp headers
        WRITE(Header3, H3FMT) TRIM(Doses(5)), TRIM(Types(9)),         &
            TRIM(PopGroup(PopIndex)), TRIM(EVRGroup(CEILING(I/7.0))), &
            TRIM(EmpGroup(EFlag))                                    ! Write the table ID 
        K = Pol(PL)%N_TSDoseLev                                      ! # of levels in table
        IF (K>0) THEN                                                ! If table is defined 
            N = SIZE(DM8HExp,1)                                      ! # of levels in this table
            Array(1:N,1:K) = REAL(ALLTSDose(:,:,I,PL),LONG)          ! Fill table buffer with data
            TableLev(1:K)  = REAL(TSDoseLev(PL,1:K),LONG)            ! Fill level buffer with data 
            CALL WriteTablesTypeIII(N,K,PL)                          ! Code is in OutputModule
        ENDIF                                                        ! End table level logic  
      ENDIF                                                          ! End write table logic            
   ENDDO                                                             ! End loop over groups
   IF (TableFlags(17)) THEN                                          ! If table specified        
    WRITE(Header,'(A68,I8,A29,I10)') "Dose: Person-Days at or "      &
        //"above each Daily Average Dose Level,         "             &
        //" for N =", NumProfiles," Profiles.  Area Population ="     &
        , TotPop                                                     ! Header for DAvgDose table
    WRITE(Header2,'(A85)') ""                                        ! Group header empty  
    WRITE(Header3, H3FMT) TRIM(Doses(1)), TRIM(Types(4)),            &
        TRIM(PopGroup(1)), TRIM(EVRGroup(1)), TRIM(EmpGroup(EFlag))  ! Write the table ID 
    K = Pol(PL)%N_DAvgDoseLev                                        ! # of levels in DAvgDose table
    IF (K>0) THEN                                                    ! If DAvgDose table is defined 
        Array(1:N,1:K) = REAL(DAvgDose(:,:,PL),LONG)                 ! Fill table buffer with data
        TableLev(1:K)  = DAvgDoseLev(PL,1:K)                         ! Fill level buffer with data 
        CALL WriteTables(N,NumProfiles,K,NumDays,YES, PL,NO)         ! Code is in OutputModule
    ENDIF                                                            ! End of DAvgDose table logic
    WRITE(Header,'(A66,I8,A29,I10)') "Dose: Persons at or "          &
        //"above each Overall Average Dose Level,         "           &
        //" for N =", NumProfiles," Profiles.  Area Population ="     &
        , TotPop                                                     ! Header for SAvgDose table
    WRITE(Header2,'(A85)') ""                                        ! Group header empty  
    WRITE(Header3, H3FMT) TRIM(Doses(2)), TRIM(Types(5)),            &
        TRIM(PopGroup(1)), TRIM(EVRGroup(1)), TRIM(EmpGroup(EFlag))  ! Write the table ID 
    K = Pol(PL)%N_SAvgDoseLev                                        ! # of levels in SAvgDose table
    IF (K>0) THEN                                                    ! If AnnDose table is defined 
        Array(1:N,1:K) = REAL(SAvgDose(:,:,PL),LONG)                 ! Fill table buffer with data
        TableLev(1:K)  = REAL(SAvgDoseLev(PL,1:K),LONG)              ! Fill level buffer with data 
        CALL WriteTables(N,NumProfiles,K,1,NO,PL,NO)                 ! Code is in OutputModule
    ENDIF                                                            ! End of SAvgDose table logic
    WRITE(Header,'(A67,I8,A29,I10)') "Dose: Person-hours at or "     &
        //"above each End-of-Hour Dose Level,         "               &
        //" for N =", NumProfiles," Profiles.  Area Population ="     &
        , TotPop                                                     ! Header for H_EHDose table
    WRITE(Header2,'(A85)') ""                                        ! Group header empty  
    WRITE(Header3, H3FMT) TRIM(Doses(3)), TRIM(Types(1)),            &
        TRIM(PopGroup(1)), TRIM(EVRGroup(1)), TRIM(EmpGroup(EFlag))  ! Write the table ID 
    K = Pol(PL)%N_H_EHDoseLev                                        ! # of levels in H_EHDose table
    IF (K>0) THEN                                                    ! If H_EHDose table is defined 
        Array(1:N,1:K) = REAL(H_EHDose(:,:,PL),LONG)                 ! Fill table buffer with data
        TableLev(1:K)  = REAL(H_EHDoseLev(PL,1:K),LONG)              ! Fill level buffer with data
        CALL WriteTables(N,NumProfiles,K,NumHours,YES, PL, NO)       ! Code is in OutputModule
    ENDIF                                                            ! End of H_EHDose table logic   
   ENDIF                                                             ! End of write table logic  
   IF (TableFlags(18)) THEN                                          ! If table specified
    WRITE(Header,'(A52,I8,A29,I10)') "Dose: Minutes in each "        &
        //"Dose interval, for N =",NumProfiles,                       &
        " Profiles.  Area Population =",TotPop                       ! Header for TimeDose table
    WRITE(Header2,'(A85)') ""                                        ! Group header empty  
    WRITE(Header3, H3FMT) TRIM(Doses(4)), TRIM(Types(6)),            &
        TRIM(PopGroup(1)), TRIM(EVRGroup(1)), TRIM(EmpGroup(EFlag))  ! Write the table ID 
    K = Pol(PL)%N_TimeDoseLev                                        ! # of levels in TimeDose table
    IF (K>0) THEN                                                    ! If TimeDose table is defined 
        Array(1:N,1:K) = REAL(TimeDose(:,:,PL),LONG)                 ! Fill table buffer with data
        TableLev(1:K)  = REAL(TimeDoseLev(PL,1:K),LONG)              ! Fill level buffer with data 
        CALL WriteTables(N,NumProfiles,K,1440*NumDays,YES,PL,NO)     ! Code is in OutputModule
        WRITE(Header,'(A58,I8,A29,I10)') "Dose: Minutes at or "       &
            //"above each Dose Level, for N =",NumProfiles,            &
            " Profiles.  Area Population =",TotPop                   ! Header for cumul. TimeDose table
    WRITE(Header3, H3FMT) TRIM(Doses(4)), TRIM(Types(7)),            &
        TRIM(PopGroup(1)), TRIM(EVRGroup(1))                         ! Write the table ID 
        DO I=1,N                                                     ! Loop over statistics
            Cumul = 0.                                               ! Initialize cumulative total
            DO J=K,1,-1                                              ! Loop downwards over table levels
                Cumul      = Cumul + Array(I,J)                      ! Accumulate data in table
                Array(I,J) = Cumul                                   ! Replace data in old table 
            ENDDO                                                    ! Continue with next table level
        ENDDO                                                        ! Continue with next statistic
        CALL WriteTables(N,NumProfiles,K,1440*NumDays,YES,PL,NO)     ! Code is in OutputModule
    ENDIF                                                            ! End of TimeDose table logic 
   ENDIF                                                             ! End write table logic 
   IF (DebugLevel>1) WRITE(LogU,LV_FMT) LV_MSG,ProcName(ProcLevel)   ! Message for debugging
   ProcLevel = ProcLevel-1                                           ! Decrement procedure depth
END SUBROUTINE WriteDoseTables

SUBROUTINE FinalSummary                                              ! Called from main APEX program
   INTEGER (KIND=IK) :: ClockRate, ClockMax, I, J, K, PL             ! Local integer variables
   REAL    (KIND=RK) :: JobTime                                      ! Local real variables              
   CHARACTER*40      :: TFI
   ! Global constants: EMPTEXT, GASTEXT, IK, LENTAB, LogU, LONG, LV_FMT, LV_MSG, NO, 
   !                   PILOTTEXT, RK, SEXTEXT, TabU, UNITTEXT, ST_FMT, ST_MSG, YES
   ! Global vars used: AgeMin, AgeMax, DAvgDoseLev, DAvgExpLev, DebugLevel, DM1HDoseLev, DM1HExpLev,
   !                   DM8HDoseLev, DM8HExpLev, DMEHDoseLev, DoDose, FName, LogTables, NumDays, 
   !                   NumDistricts, NumMicros, NumPopTypes, NumProfiles, Person, TimeDoseLev, 
   !                   TimeExpLev, TotPop
   ! Global vars set : Clock, Ios, ProcLevel, ProcName
   ! Module constants: DAVGD, DAVGE, DM1HD, DM1HE, DM8HD, DM8HE, DMEHD,
   !                   H_EHD, SAVGD, SAVGE, TIMED, TIMEE
   ! Module vars used: DAvgDose, DAvgExp, DM1HDose, DM1HExp, DM8HDose, DM8HExp, DMEHDose, 
   !                   H_EHDose, MSum, NLev, SAvgDose, SAvgExp, TAccExp, TimeDose, TimeExp
   ! Module vars set : Array, Header, Header2, TableLev
   ! Intrinsic procs : ALLOCATE, CLOSE, COUNT, REAL, SIZE, SUM, SYSTEM_CLOCK, WRITE
   ! APEX procedures : CheckAllocation, WriteTables, WriteTablesTypeII
   IF (FirstProfile>1) RETURN                                        ! No summary for partial runs
   ProcLevel = ProcLevel+1                                           ! Increment procedure depth
   ProcName(ProcLevel) = "FinalSummary"                              ! Procedure name for messages
   IF (DebugLevel>0) WRITE(LogU,ST_FMT) ST_MSG, ProcName(ProcLevel)  ! Message for debugging
   WRITE(LogU,*)                                                     ! Put blank line on log file
   WRITE(LogU,*) 'Home District counts'                              ! # persons per home district
   WRITE(LogU,'(I8,I10,4X,A40)')                                    &
      (I,COUNT(Person%HomeD==I),DList(I)%Name,I=1,NumDistricts)      ! Write # persons living here 
   WRITE(LogU,*) 'Work District counts'                              ! # persons per work district
   WRITE(LogU,'(I8,I10,4X,A40)')                                    &
      (I,COUNT(Person%WorkD==I),DList(I)%Name,I=1,NumDistricts)      ! Write # persons working here  
   WRITE(LogU,*) '    Age       counts'                              ! # persons of each year of age
   WRITE(LogU,'(I8,I10)') (I,COUNT(Person%Age==I),I=AgeMin,AgeMax)   ! Write # persons of that age
   WRITE(LogU,*) '   Gender     counts'                              ! # persons of each gender
   WRITE(LogU,'(I8,I10,4X,A6)')                                     &
      (I,COUNT(Person%Gender==I),SEXTEXT(I),I=0,2)                   ! Write # persons of each gender
   WRITE(LogU,*) '   PopCat     counts'                              ! # persons of each pop category
   WRITE(LogU,'(I8,I10,4X,A6,4X,A5)')(I,COUNT(Person%PopCat==I),    & 
      PopCat(I)%Gender,PopCat(I)%Race,I=1,NumPopTypes)               ! Write # of that category
   WRITE(LogU,*) 'Employment   counts'                               ! # persons employed/unemployed
   WRITE(LogU,'(I8,I10,4X,A5)')                                     &  
      (I,COUNT(Person%Employed==I),EMPTEXT(I),I=0,2)                 ! # persons of that status
   WRITE(LogU,*) '   Disease   counts'                               ! # persons with disease
   WRITE(LogU,'(I8,I10,4X,A5)')                                     &  
      (I,COUNT(Phys%Ill==I),DISTEXT(I),I=0,2)                        ! # persons of that status
   WRITE(LogU,*) ' GasStove    counts'                               ! # persons with/without gas stove
   WRITE(LogU,'(I8,I10,4X,A5)')                                     &
      (I,COUNT(Person%GasStove==I),GASTEXT(I),I=0,2)                 ! Write # of that status
   WRITE(LogU,*) ' GasPilot    counts'                               ! # persons with/without gas pilot
   WRITE(LogU,'(I8,I10,4X,A5)')                                     &
      (I,COUNT(Person%GasPilot==I),PILOTTEXT(I),I=0,2)               ! Write # of that status     
   WRITE(LogU,*) '  Home AC    counts'                               ! # persons with/without home A/C
   WRITE(LogU,'(I8,I10,4X,A6)')                                     &
      (I,COUNT(Person%AC_Home==I),ACTEXT(I),I=1,NumAC_Home)          ! Write # of that status
   WRITE(LogU,*) '  Car AC     counts'                               ! # persons with/without car A/C
   WRITE(LogU,'(I8,I10,4X,A6)')                                     &
      (I,COUNT(Person%AC_Car==I),ACTEXT(I),I=1,NumAC_Car)            ! Write # of that status
   WRITE(LogU,*) '  Profile PCond1 Counts'                           ! # persons with this status
   WRITE(LogU,'(I8,I10,4X,A9,I7)')                                  &
      (I,COUNT(Person%PCond1==I),"Category ",I,I=1,NumPCond1)        ! Write # of that status
   WRITE(LogU,*) '  Profile PCond2 Counts'                           ! # persons with this status
   WRITE(LogU,'(I8,I10,4X,A9,I7)')                                  &
      (I,COUNT(Person%PCond2==I),"Category ",I,I=1,NumPCond2)        ! Write # of that status
   WRITE(LogU,*) '  Profile PCond3 Counts'                           ! # persons with this status
   WRITE(LogU,'(I8,I10,4X,A9,I7)')                                  &
      (I,COUNT(Person%PCond3==I),"Category ",I,I=1,NumPCond3)        ! Write # of that status
   WRITE(LogU,*) '  Profile PCond4 Counts'                           ! # persons with this status
   WRITE(LogU,'(I8,I10,4X,A9,I7)')                                  &
      (I,COUNT(Person%PCond4==I),"Category ",I,I=1,NumPCond4)        ! Write # of that status
   WRITE(LogU,*) '  Profile PCond5 Counts'                           ! # persons with this status
   WRITE(LogU,'(I8,I10,4X,A9,I7)')                                  &
      (I,COUNT(Person%PCond5==I),"Category ",I,I=1,NumPCond5)        ! Write # of that status
   WRITE(LogU,*) '  Profile RCond1 Counts'                           ! # persons with this status
   WRITE(LogU,'(I8,I10,4X,A9,I7)')                                  &
      (I,COUNT(Person%RCond1==I),"Category ",I,I=1,NumRCond1)        ! Write # of that status
   WRITE(LogU,*) '  Profile RCond2 Counts'                           ! # persons with this status
   WRITE(LogU,'(I8,I10,4X,A9,I7)')                                  &
      (I,COUNT(Person%RCond2==I),"Category ",I,I=1,NumRCond2)        ! Write # of that status
   WRITE(LogU,*) '  Profile RCond3 Counts'                           ! # persons with this status
   WRITE(LogU,'(I8,I10,4X,A9,I7)')                                  &
      (I,COUNT(Person%RCond3==I),"Category ",I,I=1,NumRCond3)        ! Write # of that status
   WRITE(LogU,*) '  Profile RCond4 Counts'                           ! # persons with this status
   WRITE(LogU,'(I8,I10,4X,A9,I7)')                                  &
      (I,COUNT(Person%RCond4==I),"Category ",I,I=1,NumRCond4)        ! Write # of that status
   WRITE(LogU,*) '  Profile RCond5 Counts'                           ! # persons with this status
   WRITE(LogU,'(I8,I10,4X,A9,I7)')                                  &
      (I,COUNT(Person%RCond5==I),"Category ",I,I=1,NumRCond5)        ! Write # of that status
   DO PL= 1,NumPollutants                                            ! Loop over pollutants
      K = Pol(PL)%N_DM1HExpLev                                       ! # of levels in current table
      IF (K>0.AND.K<=8.AND.NumProfiles<=20) THEN                     ! Write data only if <=20 profiles
         WRITE(TFI,'(A1,I3,A3)') '(',K+1,'I8)'                       ! Create format for writing table
         WRITE(LogU,*)                                               ! Put blank line on log       
         WRITE(LogU,*)"Pollutant=", POL(PL)%Name                     ! Put pollutant name on log
         WRITE(LogU,*)"# Days at or above each DM1H level, by person"! Write table header to log 
         WRITE(LogU,'(A8,8F8.2)') " Person ",(DM1HExpLev(PL,J),J=1,K)! Write data for given level
         DO I=1, NumProfiles                                         ! Loop over profiles
            WRITE(LogU,TFI) I,(DM1HExp(I,J,1,PL),J=1,K)              ! Write # days for each level
         ENDDO                                                       ! Continue with next profile
      ENDIF                                                          ! End of logic for <=20 profile  
   ENDDO                                                             ! End loop over pollutants
   CALL SYSTEM_CLOCK(Clock(2),ClockRate, ClockMax)                   ! Find job stop time
   JobTime = REAL(Clock(2)-Clock(1))/REAL(ClockRate)                 ! Elapsed time in seconds
   IF (Jobtime<0) JobTime=JobTime+REAL(ClockMax)/REAL(ClockRate)     ! Fix if clock was reset
   IF (DebugLevel>0) WRITE(LogU,LV_FMT) LV_MSG, ProcName(ProcLevel)  ! Message for debugging
   CALL ManageFiles(Action=CLSE)                                     ! Close all output files
   WRITE(LogU,*) ' Job Stop Time = ',Clock(2)                        ! Put final job time on log
   WRITE(LogU,*) ' Job Duration  = ',JobTime, ' seconds'             ! Put elapsed time on log
   WRITE(LogU,'(16X,A2,F8.2,A10)') '= ',JobTime/3600.,'     hours'   ! Job time in hours (to .01 hour)
   WRITE(LogU,*) ' APEX finished with completion code 0'             ! Successful completion message
   CLOSE(LogU)                                                       ! Close the log file
   ProcLevel = ProcLevel-1                                           ! Decrement procedure depth
END SUBROUTINE FinalSummary                                          ! Return to main APEX module

SUBROUTINE WriteTables(N,NGrp,K,Values,ByPerson,PL,OZrisk)           ! Called from FinalSummary 
   INTEGER (KIND=IK), INTENT(In) :: N,NGrp,K,Values,ByPerson,PL,OZRisk ! Input parameters
   INTEGER (KIND=IK) :: I, J , NP, NU, U                             ! Local integer variables
   REAL    (KIND=RK) :: D1, D2, D3, Den, Pop,GrpMin(1),MinPctl(1)    ! Local real variables
   CHARACTER*40      :: TFI, TFR, TFE, TFP, CFMT, CL, TFL, TFO, TFLR ! Local character variables
   CHARACTER (LEN=1) :: C(10*K+25)                                   ! Automatic array for dashes
   REAL  (KIND=LONG) :: SR(5,K)                                      ! Automatic array for real sums
   INTEGER (KIND=IK) :: SI(K)                                        ! Automatic array for integer sums 
   REAL  (KIND=LONG) :: RiskVec(K+3)                   
   REAL  (KIND=LONG), ALLOCATABLE :: X(:)                            ! Automatic array for one row
   REAL  (KIND=LONG), PARAMETER :: ONE=1.0                           ! LONG value of 1 for calculations
   ! Global constants: IK, LONG, LogU, LV_MS2, RK, SL_FM2, ST_MS2, TabU, YES
   ! Global vars used: DebugLevel, LogTables, Pctl, TotPop              
   ! Global vars set : ProcLevel, ProcName
   ! Module vars used: Array, Header, Header2, TableLev
   ! Intrinsic procs : COUNT, MAX, MAXVAL, MINVAL, NINT, REAL, SQRT, SUM, WRITE
   ! APEX procedures : Percentiles
   ProcLevel = ProcLevel+1                                           ! Increment procedure depth
   ProcName(ProcLevel) = "WriteTables"                               ! Procedure name for messages
   IF (DebugLevel>1) WRITE(LogU,SL_FM2) ST_MS2, ProcName(ProcLevel)  ! Message for debugging
   ALLOCATE (X(N),STAT=AllocErr)                                     ! Allocate space for 1 row
   CALL CheckAllocation(AllocErr)                                    ! Check the success of the alloc
   C   = "-"                                                         ! Fill buffer with dashes
   WRITE(CFMT,'(A1,I4,A3)') '(',10*K+25,'A1)'                        ! Write dashes format
   WRITE(CL,'(A1,I4,A3)')   '(',14*K+65,'A1)'
   D1  = REAL(NGrp)                                                  ! Sample size
   D2  = REAL(Values)/100.0                                          ! Sum of elements for one profile
   D3  = D1*D2                                                       ! Sum of all elements 
   Den = MAX(1.,D1-1.)                                               ! Denominator for std. dev.  
   Pop = REAL(TotPop)                                                ! Population of study area
   NP  = Pol(PL)%N_Pctl                                              ! Number of percentiles to print
   ALLOCATE (PctlGrp(1:NP),STAT=AllocErr)                            ! Make space for grp prcntls
   CALL CheckAllocation(AllocErr)                                    ! Check success of allocation
   DO J=1,NP                                                         ! Loop over # percentiles
      PctlGrp(J)=(100./NumProfiles)*                                &
         ((NumProfiles-NGrp)+(Pctl(PL,J)/100.)*NGrp)                 ! Calculate group percentiles
   ENDDO                                                             ! End loop over percentiles
   MinPctl(1)=  100.0*REAL(N-NGrp+1)/REAL(N+1)                       ! Effective %ile of group min 
   DO J=1,K                                                          ! Loop over table levels
      X       = Array(1:N,J)                                         ! Copy table row into buffer 
      SR(1,J) = SUM(X)                                               ! Sum over profiles
      SR(2,J) = SQRT((SUM(X**2)-SR(1,J)**2/D1)/Den)                  ! Std. dev. across profiles
      SR(3,J) = D1*SR(2,J)/MAX(SR(1,J),ONE)                          ! CV across profiles
      CALL Percentiles(X,N,MinPctl,1,GrpMin)                         ! Code is in DistributionModule 
      SR(4,J) = GrpMin(1)                                            ! Minimum value across profiles
      SR(5,J) = MAXVAL(X)                                            ! Maximum value across profiles
      SI(J)   = COUNT(X>0.)                                          ! Count of profiles above zero 
      CALL Percentiles(X,N,PctlGrp(1:),NP,Percs(:,J))                ! Code is in DistributionModule 
   ENDDO                                                             ! Continue with next table level
   WRITE(TFI,'(A23,I3,A6)') '(A13,2I14  ,2I12  ,',K-4,'I10)  '       ! Format for integer data
   WRITE(TFR,'(A23,I3,A6)') '(A13,2F14.3,2F12.3,',K-4,'F10.3)'       ! Format for real decimal data
   WRITE(TFL,'(A23,I3,A6)') '(A13,2F14.5,2F12.5,',K-4,'F10.5)'       ! Format for table levels
   WRITE(TFE,'(A25,I3,A7)') '(A13,2ES14.3,2ES12.3,',K-4,'ES10.3)'    ! Format for exponential data  
   WRITE(TFP,'(A23,I3,A6)') '(F5.1,A8,2F14.3,2F12.3,',K-4,'F10.3)'   ! Format for percentile data
   WRITE(TFO,'(A5,I3,A6)') '(A20,',K+3,'G14.4)'
   WRITE(TFLR,'(A5,I3,A11)') '(A20,',K,'F14.4,3A14)'
   IF (Values > NumDays) THEN                                        ! If vals are minutes (not days)
   WRITE(TFI,'(A23,I3,A6)') '(A13,2I14  ,2I12  ,',K-4,'I12)  '       ! More room for integer data
   WRITE(TFR,'(A23,I3,A6)') '(A13,2F14.3,2F12.3,',K-4,'F12.3)'       ! More room for decimal data 
   WRITE(TFE,'(A25,I3,A7)') '(A13,2ES14.3,2ES12.3,',K-4,'ES12.3)'    ! More room for exponential data  
   WRITE(TFP,'(A23,I3,A6)') '(F5.1,A8,2F14.3,2F12.3,',K-4,'F12.3)'   ! More room for percentile data
   ENDIF                                                             ! End minutes/days logic
   DO NU=1,2                                                         ! Loop over output units
      IF (NU==1) U=Pol(PL)%TabU                                      ! First loop writes to table file
      IF (NU==2) U=LogU                                              ! Second loop writes to log file
      IF (NU==2 .AND. LogTables==NO) CYCLE                           ! Skip loop if no tables on log
      WRITE(U,'(///, A)') Header3                                    ! Write the table ID      
      WRITE(U,'(A138)') Header                                       ! Write the table header
      WRITE(U,'(A85,/)') Header2                                     ! Write the table header if needed
      WRITE(U,TFL) " Level:    ",(TableLev(J),J=1,K)                 ! Write the table levels
      WRITE(U,CFMT) C                                                ! Write a line of dashes
      WRITE(U,TFE)                                                  & 
         " Counts(Pop):",((REAL(NGrp))/(REAL(NumProfiles))*         &
         Pop*SR(1,J)/D1,J=1,K)                                       ! Write the sum for the population
      WRITE(U,TFI)                                                  &
         " #Meet (Pop):",(NINT((REAL(NGrp))/(REAL(NumProfiles))     &        
         *Pop*SI(J)/D1),J=1,K)                                       ! Write the # > 0 in population
      WRITE(U,TFR) " %Meet (Pop):",(100.*SI(J)/D1,J=1,K)             ! Percentage of population > 0
      IF (ByPerson==YES) THEN                                        ! If table shows stats by person
         WRITE(U,TFR) " Mean       :",(SR(1,J)/D1,J=1,K)             ! Mean across population
         WRITE(U,TFR) " Std.Dev.   :",(SR(2,J),J=1,K)                ! Standard deviation across pop.
         WRITE(U,TFR) " CV         :",(SR(3,J),J=1,K)                ! Coefficient of variation for pop
         WRITE(U,TFR) " Minimum    :",(SR(4,J),J=1,K)                ! Minimum for this sample
         DO I=1,Pol(PL)%N_Pctl                                       ! Loop over chosen percentiles
            WRITE(U,TFP) Pctl(PL,I)," %ile  :",(Percs(I,J),J=1,K)    ! Write percentile for each level
         ENDDO                                                       ! Continue with next percentile
         WRITE(U,TFR) " Maximum    :",(SR(5,J),J=1,K)                ! Maximum for this sample
         WRITE(U,TFR) " Mean  (%)  :",(SR(1,J)/D3,J=1,K)             ! Mean as % of possible values
         WRITE(U,TFR) " Min   (%)  :",(SR(4,J)/D2,J=1,K)             ! Minimum as % of possible values 
         WRITE(U,TFR) " Max   (%)  :",(SR(5,J)/D2,J=1,K)             ! Maximum as % of possible values
      ENDIF                                                          ! End of stats by person
      WRITE(U,TFE) " Counts(Sim):",(SR(1,J),J=1,K)                   ! Sum across simulated individuals
      WRITE(U,TFI) " #Meet (Sim):",(SI(J),J=1,K)                     ! # of simulated persons > 0
      WRITE(U,CFMT) C                                                ! Write line of dashes
      IF (OZRisk==YES) THEN
         Header4 = "FEV1Risk"//Header3(11:100)
         WRITE(U,'(///, A)') Header4                                 ! Write the table ID      
         WRITE(U,'(A118)') Header5                                   ! Write the table header
         WRITE(U,'(A85,/)') Header2                                  ! Write the table header if needed
         WRITE(U,TFLR) " Level:",(TableLev(J),J=1,K),"#Sim",        &
            "Frac. Sim","#Pop"                                       ! Write the table levels
         WRITE(U,CL) C                                               ! Write a line of dashes  
         DO I=1,NumFEV
            DO J=1,K-1
               RiskVec(J) = FEVProbList(I,J)*(SI(J)-SI(J+1))
            ENDDO
            RiskVec(K)   = FEVProbList(I,K)*SI(K)
            RiskVec(K+1) = SUM(RiskVec(1:K))
            RiskVec(K+2) = RiskVec(K+1)/NumProfiles
            RiskVec(K+3) = RiskVec(K+2)*TotPop
            WRITE(U,TFO) FEVDescList(I),(RiskVec(J),J=1,K+3) 
         ENDDO     
         WRITE(U,CL) C                                               ! Write line of dashes
      ENDIF 
   ENDDO                                                             ! Continue with next output unit 
   DEALLOCATE (X, STAT = AllocErr)                                   ! Deallocate the temporary array
   DEALLOCATE (PctlGrp, STAT = AllocErr)                             ! Deallocate the temporary array
   IF (DebugLevel>1) WRITE(LogU,SL_FM2) LV_MS2, ProcName(ProcLevel)  ! Message for debugging
   ProcLevel = ProcLevel-1                                           ! Decrement procedure depth
END SUBROUTINE WriteTables                                           ! Return to FinalSummary

SUBROUTINE WriteTablesTypeII(N,K,PL)                                 ! Called from FinalSummary
   INTEGER (KIND=IK), INTENT(In) :: N, K, PL                         ! Input parameters
   INTEGER (KIND=IK) :: J , L, NU, U                                 ! Local integer variables
   CHARACTER (LEN=1) :: C(76)                                        ! Automatic array for dashes
   INTEGER (KIND=IK) :: NExp(K,6)                                    ! Table array of exposures counts 
   REAL  (KIND=LONG) :: X                                            ! Number of exposures
   ! Global constants: IK, LONG, LogU, LV_MS2, RK, SL_FM2, ST_MS2, TabU, YES
   ! Global vars used: DebugLevel, LogTables, Pctl, TotPop              
   ! Global vars set : ProcLevel, ProcName
   ! Module vars used: Array, Header, Header2, TableLev
   ! Intrinsic procs :  WRITE
   ProcLevel = ProcLevel+1                                           ! Increment procedure depth
   ProcName(ProcLevel) = "WriteTablesTypeII"                         ! Procedure name for messages
   IF (DebugLevel>1) WRITE(LogU,SL_FM2) ST_MS2, ProcName(ProcLevel)  ! Message for debugging
   NExp(:,:) = 0                                                     ! Initialize array of # expos.
   C   = "-"                                                         ! Fill buffer with dashes
   DO L=1,K                                                          ! Loop over table levels
      DO J=1,N                                                       ! Loop over people
         X  = Array(J,L)                                             ! Copy # exposures into buffer 
         IF (X>=1) NExp(L,1)=NExp(L,1)+1                             ! Increment 1 exps table value
         IF (X>=2) NExp(L,2)=NExp(L,2)+1                             ! Increment 2 exps table value
         IF (X>=3) NExp(L,3)=NExp(L,3)+1                             ! Increment 3 exps table value
         IF (X>=4) NExp(L,4)=NExp(L,4)+1                             ! Increment 4 exps table value
         IF (X>=5) NExp(L,5)=NExp(L,5)+1                             ! Increment 5 exps table value
         IF (X>=6) NExp(L,6)=NExp(L,6)+1                             ! Increment 6 exps table value
       ENDDO                                                         ! End loop over people
   ENDDO                                                             ! Continue with next table level
   DO NU=1,2                                                         ! Loop over output units
      IF (NU==1) U=Pol(PL)%TabU                                      ! First loop writes to table file
      IF (NU==2) U=LogU                                              ! Second loop writes to log file
      IF (NU==2 .AND. LogTables==NO) CYCLE                           ! Skip loop if no tables on log
      WRITE(U,'(///, A)') Header3                                    ! Write the table ID      
      WRITE(U,'(A150)') Header                                       ! Write the table header
      WRITE(U,'(A85,/)') Header2                                     ! Write the table hdr if needed
      WRITE(U, '(A82)')"       At least 1   At least 2   At least 3"& 
         //"   At least 4   At least 5   At least 6"                 ! Write 1st line of column label
      WRITE(U, '(A82)')"         Exposure    Exposures    Exposures"&
         //"    Exposures    Exposures    Exposures"                 ! Second line of label
      WRITE(U,'(A7, 76A1)') "  Level  ", C                          ! Write a line of dashes
      DO L=1,K                                                       ! Loop over exposure levels   
         WRITE(U, '(F8.3, I10, 5I13)') TableLev(L), (NExp(L,J),J=1,6)! Write the levels and counts
      ENDDO                                                          ! End loop over levels
      WRITE(U,'(A7, 76A1)') "-------",  C                           ! Write line of dashes
   ENDDO                                                             ! Continue with next output unit 
   IF (DebugLevel>1) WRITE(LogU,SL_FM2) LV_MS2, ProcName(ProcLevel)  ! Message for debugging
   ProcLevel = ProcLevel-1                                           ! Decrement procedure depth
END SUBROUTINE WriteTablesTypeII                                     ! Return to FinalSummary

SUBROUTINE WriteTablesTypeIII(N,K,PL)                                ! Called from FinalSummary
   INTEGER (KIND=IK), INTENT(In) :: N, K, PL                         ! Input parameters
   INTEGER (KIND=IK) :: J , L, NU, U, Cut, NCut                      ! Local integer variables
   CHARACTER (LEN=1) :: C(76)                                        ! Automatic array for dashes
   CHARACTER*40      :: TFI, TFR                                     ! Formats for writing table rows
   INTEGER (KIND=IK), ALLOCATABLE :: NExp(:,:)                       ! Table array of exposures counts 
   REAL  (KIND=LONG) :: X                                            ! Number of exposures
   ! Global constants: IK, LONG, LogU, LV_MS2, RK, SL_FM2, ST_MS2, TabU, YES
   ! Global vars used: DebugLevel, LogTables, Pctl, TotPop              
   ! Global vars set : ProcLevel, ProcName
   ! Module vars used: Array, Header, Header2, TableLev
   ! Intrinsic procs :  WRITE
   ProcLevel = ProcLevel+1                                           ! Increment procedure depth
   ProcName(ProcLevel) = "WriteTablesTypeIII"                        ! Procedure name for messages
   IF (DebugLevel>1) WRITE(LogU,SL_FM2) ST_MS2, ProcName(ProcLevel)  ! Message for debugging
   ALLOCATE(NExp(K,POL(PL)%N_TSMultiLev ), STAT=AllocErr)            ! Make room for data
   CALL CheckAllocation(AllocErr)                                    ! Check the success of the alloc  
   NCut = POL(PL)%N_TSMultiLev                                       ! Number of exceedance cutpoints   
   NExp(:,:) = 0                                                     ! Initialize array of # expos.
   C   = "-"                                                         ! Fill buffer with dashes
   DO L=1,K                                                          ! Loop over table levels
      DO J=1,N                                                       ! Loop over people
         X  = Array(J,L)                                             ! Copy # exp/doses into buffer 
         DO Cut=2, NCut                                              ! Loop over # "multiple" cutpoint 
            IF (X>=TSMultiLev(PL,Cut)) NExp(L,Cut)=NExp(L,Cut)+1     ! Increment exp/dose table value
         ENDDO                                                       ! End loop over cutpoints 
      ENDDO                                                          ! End loop over people
   ENDDO                                                             ! Continue with next table level
   DO NU=1,2                                                         ! Loop over output units
      IF (NU==1) U=Pol(PL)%TabU                                      ! First loop writes to table file
      IF (NU==2) U=LogU                                              ! Second loop writes to log file
      IF (NU==2 .AND. LogTables==NO) CYCLE                           ! Skip loop if no tables on log
      WRITE(U,'(///, A)') Header3                                    ! Write the table ID      
      WRITE(U,'(A150)') Header                                       ! Write the table header
      WRITE(U,'(A85,/)') Header2                                     ! Write the table hdr if needed
      WRITE(TFI,'(A6,I3,A4)') '(A18,',NCut-1,'I10) '                 ! Format for cutpoint header
      WRITE(TFR,'(A7,I3,A6)') '(F18.4,',NCut-1,'I10)'                ! Format for cutpoint counts
      WRITE(U,TFI) " #Exceedances:    ",(TSMultiLev(PL,J),J=2,NCut)  ! Write the table cutpoints
      WRITE(U,'(A7, 76A1)') "  Level  ", C                           ! Write a line of dashes
      DO L=1,K                                                       ! Loop over exposure levels   
         WRITE(U,TFR)TableLev(L),(NExp(L,J),J=2,NCut)                ! Write the levels and counts
      ENDDO                                                          ! End loop over levels
      WRITE(U,'(A7, 76A1)') "-------",  C                            ! Write line of dashes
   ENDDO                                                             ! Continue with next output unit 
   IF (DebugLevel>1) WRITE(LogU,SL_FM2) LV_MS2, ProcName(ProcLevel)  ! Message for debugging
   ProcLevel = ProcLevel-1                                           ! Decrement procedure depth
END SUBROUTINE WriteTablesTypeIII                                    ! Return to FinalSummary

SUBROUTINE WriteTablesTypeIV(N,K,PL)                                 ! Called from FinalSummary
   INTEGER (KIND=IK), INTENT(In) :: N, K, PL                         ! Input parameters
   INTEGER (KIND=IK) :: J , L, NU, U                                 ! Local integer variables
   CHARACTER (LEN=1) :: C(76)                                        ! Automatic array for dashes
   INTEGER (KIND=IK) :: NExp(K,6)                                    ! Table array of exposures counts 
   REAL  (KIND=LONG) :: X                                            ! Number of exposures
   ! Global constants: IK, LONG, LogU, LV_MS2, RK, SL_FM2, ST_MS2, TabU, YES
   ! Global vars used: DebugLevel, LogTables, Pctl, TotPop              
   ! Global vars set : ProcLevel, ProcName
   ! Module vars used: Array, Header, Header2, TableLev
   ! Intrinsic procs :  WRITE
   ProcLevel = ProcLevel+1                                           ! Increment procedure depth
   ProcName(ProcLevel) = "WriteTablesTypeIV"                         ! Procedure name for messages
   IF (DebugLevel>1) WRITE(LogU,SL_FM2) ST_MS2, ProcName(ProcLevel)  ! Message for debugging
   NExp(:,:) = 0                                                     ! Initialize array of # expos.
   C   = "-"                                                         ! Fill buffer with dashes
   DO L=1,K                                                          ! Loop over table levels
      DO J=1,N                                                       ! Loop over people
         X  = Array(J,L)                                             ! Copy # exposures into buffer 
         IF (X>=1) NExp(L,1)=NExp(L,1)+1                             ! Increment 1 exps table value
         IF (X>=2) NExp(L,2)=NExp(L,2)+1                             ! Increment 2 exps table value
         IF (X>=3) NExp(L,3)=NExp(L,3)+1                             ! Increment 3 exps table value
         IF (X>=4) NExp(L,4)=NExp(L,4)+1                             ! Increment 4 exps table value
         IF (X>=5) NExp(L,5)=NExp(L,5)+1                             ! Increment 5 exps table value
         IF (X>=6) NExp(L,6)=NExp(L,6)+1                             ! Increment 6 exps table value
       ENDDO                                                         ! End loop over people
   ENDDO                                                             ! Continue with next table level
   DO NU=1,2                                                         ! Loop over output units
      IF (NU==1) U=Pol(PL)%TabU                                      ! First loop writes to table file
      IF (NU==2) U=LogU                                              ! Second loop writes to log file
      IF (NU==2 .AND. LogTables==NO) CYCLE                           ! Skip loop if no tables on log
      WRITE(U,'(///, A)') Header3                                    ! Write the table ID      
      WRITE(U,'(A150)') Header                                       ! Write the table header
      WRITE(U,'(A85,/)') Header2                                     ! Write the table hdr if needed
      WRITE(U, '(A82)')"       At least 1   At least 2   At least 3"& 
         //"   At least 4   At least 5   At least 6"                 ! Write 1st line of column label
      WRITE(U, '(A82)')"         Dose        Doses        Doses    "&
         //"    Doses        Doses        Doses    "                 ! Second line of label
      WRITE(U,'(A7, 76A1)') "  Level  ", C                           ! Write a line of dashes
      DO L=1,K                                                       ! Loop over exposure levels   
         WRITE(U, '(F8.3, I10, 5I13)') TableLev(L), (NExp(L,J),J=1,6)! Write the levels and counts
      ENDDO                                                          ! End loop over levels
      WRITE(U,'(A7, 76A1)') "-------",  C                            ! Write line of dashes
   ENDDO                                                             ! Continue with next output unit 
   IF (DebugLevel>1) WRITE(LogU,SL_FM2) LV_MS2, ProcName(ProcLevel)  ! Message for debugging
   ProcLevel = ProcLevel-1                                           ! Decrement procedure depth
END SUBROUTINE WriteTablesTypeIV                                     ! Return to FinalSummary


SUBROUTINE WritePSumFile(P)                                          ! Called from Output
   INTEGER (KIND=IK), INTENT(IN) :: P                                ! Input parameters
   INTEGER (KIND=IK) :: I, N, NVars                                  ! Local integer variables
   LOGICAL (KIND=1)  :: F(NUMPSUMVARS+4)                             ! Short form of psum flags
   ! Global constants: PSumU
   ! Global vars used: F, NumPollutants, Person          
   ! Global vars set : ProcLevel, ProcName    
   ! Module vars set : PHeader, PVals    
   ! Intrinsic procs : WRITE, ALLOCATE, TRIM, ADJUSTL, ADJUSTR
   ! APEX procs      : CheckAllocation
   ProcLevel = ProcLevel+1                                           ! Increment procedure depth
   ProcName(ProcLevel) = "WritePSumFile"                             ! Procedure name for messages
   NVars = NUMPSUMVARS+(NumPollutants)*4                             ! Number of vars to write   
   F=(PSumFlags==YES)                                                ! Set F array to True/False
   IF (P==FirstProfile) THEN                                         ! If first person   
      ALLOCATE  (PHeader(NVars), STAT=AllocErr)                      ! Make space for header array
      CALL CheckAllocation(AllocErr)                                 ! Check the success of the alloc  
      ALLOCATE  (PVals(NVars), STAT=AllocErr)                        ! Make space for values array
      CALL CheckAllocation(AllocErr)                                 ! Check the success of the alloc   
      PHeader= ""                                                    ! Initialize the header array
      PVals  = ""                                                    ! Initialize the values array
      WRITE(PHeader(1),'(A2)')  'P'                                  ! Write label to header array  
      WRITE(PHeader(2),'(A6)')  'HSect'                              ! Write label to header array
      WRITE(PHeader(3),'(A6)')  'WSect'                              ! Write label to header array
      WRITE(PHeader(4),'(A5)')  'HDis'                               ! Write label to header array
      WRITE(PHeader(5),'(A5)')  'WDis'                               ! Write label to header array
      WRITE(PHeader(6),'(A5)')  'Zone'                               ! Write label to header array
      WRITE(PHeader(7),'(A4)')  'Age'                                ! Write label to header array
      WRITE(PHeader(8),'(A7)')  'Gender'                             ! Write label to header array
      WRITE(PHeader(9),'(A6)')  'Race'                               ! Write label to header array
      WRITE(PHeader(10),'(A5)') 'Empl'                               ! Write label to header array
      WRITE(PHeader(11),'(A7)') 'Height'                             ! Write label to header array
      WRITE(PHeader(12),'(A7)') 'Weight'                             ! Write label to header array
      IF(F(13)) WRITE(PHeader(13),'(A6)') 'ACCar'                    ! Write label to header array
      IF(F(14)) WRITE(PHeader(14),'(A6)') 'ACHom'                    ! Write label to header array
      IF(F(15)) WRITE(PHeader(15),'(A8)') 'Disease'                  ! Write label to header array
      IF(F(16)) WRITE(PHeader(16),'(A6)') 'Pilot'                    ! Write label to header array
      IF(F(17)) WRITE(PHeader(17),'(A6)') 'Stove'                    ! Write label to header array
      IF(F(18)) WRITE(PHeader(18),'(A10)') '#Events'                 ! Write label to header array
      IF(F(19)) WRITE(PHeader(19),'(A9)') 'BloodVol'                 ! Write label to header array 
      IF(F(20)) WRITE(PHeader(20),'(A4)') 'BSA'                      ! Write label to header array
      IF(F(21)) WRITE(PHeader(21),'(A4)') 'ECF'                      ! Write label to header array
      IF(F(22)) WRITE(PHeader(22),'(A7)') 'Diffus'                   ! Write label to header array
      IF(F(23)) WRITE(PHeader(23),'(A7)') 'Endgn1'                   ! Write label to header array
      IF(F(24)) WRITE(PHeader(24),'(A7)') 'Endgn2'                   ! Write label to header array
      IF(F(25)) WRITE(PHeader(25),'(A9)') 'Hemoglob'                 ! Write label to header array
      IF(F(26)) WRITE(PHeader(26),'(A8)') 'MetsMax'                  ! Write label to header array
      IF(F(27)) WRITE(PHeader(27),'(A7)') 'VO2Max'                   ! Write label to header array
      IF(F(28)) WRITE(PHeader(28),'(A5)') 'MOXD'                     ! Write label to header array
      IF(F(29)) WRITE(PHeader(29),'(A4)') 'PAI'                      ! Write label to header array
      IF(F(30)) WRITE(PHeader(30),'(A5)') 'RecT'                     ! Write label to header array
      IF(F(31)) WRITE(PHeader(31),'(A4)') 'RMR'                      ! Write label to header array
      IF(F(32)) WRITE(PHeader(32),'(A8)') 'VEInter'                  ! Write label to header array
      IF(F(33)) WRITE(PHeader(33),'(A8)') 'VESlope'                  ! Write label to header array
      IF(F(34)) WRITE(PHeader(34),'(A8)') 'VEResid'                  ! Write label to header array
      IF(F(35)) WRITE(PHeader(35),'(A8)') 'BMI'                      ! Write label to header array
      IF(F(36)) WRITE(PHeader(36),'(A8)') 'Beta1'                    ! Write label to header array      
      IF(F(37)) WRITE(PHeader(37),'(A8)') 'Beta2'                    ! Write label to header array 
      IF(F(38)) WRITE(PHeader(38),'(A8)') 'Beta3'                    ! Write label to header array 
      IF(F(39)) WRITE(PHeader(39),'(A8)') 'Beta4'                    ! Write label to header array 
      IF(F(40)) WRITE(PHeader(40),'(A8)') 'Beta5'                    ! Write label to header array 
      IF(F(41)) WRITE(PHeader(41),'(A8)') 'Beta6'                    ! Write label to header array 
      IF(F(42)) WRITE(PHeader(42),'(A8)') 'Beta7'                    ! Write label to header array 
      IF(F(43)) WRITE(PHeader(43),'(A8)') 'FEVU'                     ! Write label to header array
      IF(F(44)) WRITE(PHeader(44),'(A8)') 'FEVSLP'                   ! Write label to header array
      IF(F(45)) WRITE(PHeader(45),'(A8)') 'FEVINT'                   ! Write label to header array
      IF(F(46)) WRITE(PHeader(46),'(A9)') '#Diaries'                 ! Write label to header array
      IF(F(47)) WRITE(PHeader(47),'(A7)') 'PCond1'                   ! Write label to header array
      IF(F(48)) WRITE(PHeader(48),'(A7)') 'PCond2'                   ! Write label to header array
      IF(F(49)) WRITE(PHeader(49),'(A7)') 'PCond3'                   ! Write label to header array
      IF(F(50)) WRITE(PHeader(50),'(A7)') 'PCond4'                   ! Write label to header array
      IF(F(51)) WRITE(PHeader(51),'(A7)') 'PCond5'                   ! Write label to header array
      IF(F(52)) WRITE(PHeader(52),'(A7)') 'RCond1'                   ! Write label to header array
      IF(F(53)) WRITE(PHeader(53),'(A7)') 'RCond2'                   ! Write label to header array
      IF(F(54)) WRITE(PHeader(54),'(A7)') 'RCond3'                   ! Write label to header array
      IF(F(55)) WRITE(PHeader(55),'(A7)') 'RCond4'                   ! Write label to header array
      IF(F(56)) WRITE(PHeader(56),'(A7)') 'RCond5'                   ! Write label to header array
      IF(F(57)) WRITE(PHeader(57),'(A9)') 'CommDist'                 ! Write label to header array      
      IF(F(58)) WRITE(PHeader(58),'(A9)') 'CommTime'                 ! Write label to header array        
      IF(F(59)) WRITE(PHeader(59),'(A5)') 'RDis'                     ! Write label to header array        
      IF(F(60)) WRITE(PHeader(60),'(A6)') 'RWDis'                    ! Write label to header array        
      IF(F(61)) WRITE(PHeader(61),'(A7)') 'FGroup'                   ! Write label to header array 
      IF(F(62)) WRITE(PHeader(62),'(A10)') 'GroupName'               ! Write label to header array 
      IF(F(63)) WRITE(PHeader(63),'(A9)') 'MaxdFEV1'                 ! Write label to header array 
      N = NUMPSUMVARS                                                ! Short name for # Psum vars 
      DO I=1,NumPollutants                                           ! Loop over pollutants       
         IF (F(N+1)) WRITE(PHeader(NUMPSUMVARS+4*(I-1)+1),'(A13)')  &  
             "AvgExp-"//TRIM(ADJUSTL(Pol(I)%Name(1:5)))              ! Write label to header array
         IF (F(N+2)) WRITE(PHeader(NUMPSUMVARS+4*(I-1)+2),'(A13)')  &
            "MaxExp-"//TRIM(ADJUSTL(Pol(I)%Name(1:5)))               ! Write label to header array
         IF (F(N+3).AND. Pol(I)%DoDose==YES)                        & 
            WRITE(PHeader(NUMPSUMVARS+4*(I-1)+3),'(A14)')           &
            "AvgDose-"//TRIM(ADJUSTL(Pol(I)%Name(1:5)))              ! Write label to header array
         IF (F(N+4).AND. Pol(I)%DoDose==YES)                        & 
             WRITE(PHeader(NUMPSUMVARS+4*(I-1)+4),'(A14)')          &
            "MaxDose-"//TRIM(ADJUSTL(Pol(I)%Name(1:5)))              ! Write label to header array
      ENDDO                                                          ! End loop over pollutants
      WRITE(PSumU,'(100A)') (TRIM(PHeader(I)), I=1,NVars)            ! Write final header to file
   ENDIF   
   N = NUMPSUMVARS                                                 
   WRITE(PVals(1),'(I8)')  P                                         ! Write var to val array
   WRITE(PVals(2),'(I8)')  Person(P)%HomeSec                         ! Write var to val array              
   WRITE(PVals(3),'(I6)')  Person(P)%WorkSec                         ! Write var to val array
   WRITE(PVals(4),'(I4)')  Person(P)%HomeD                           ! Write var to val array
   WRITE(PVals(5),'(I4)')  Person(P)%WorkD                           ! Write var to val array
   WRITE(PVals(6),'(I4)')  Person(P)%Zone                            ! Write var to val array
   WRITE(PVals(7),'(I4)')  Person(P)%Age                             ! Write var to val array     
   WRITE(PVals(8),'(A2)')  ADJUSTR(ST(Person(P)%Gender))             ! Write var to val array
   WRITE(PVals(9),'(A6)')  ADJUSTR(PopCat(Person(P)%PopCat)%Race)    ! Write var to val array
   WRITE(PVals(10),'(A2)') ADJUSTR(YN(Person(P)%Employed))           ! Write var to val array
   WRITE(PVals(11),'(F7.3)') Phys(P)%Height                          ! Write var to val array
   WRITE(PVals(12),'(F8.3)') Phys(P)%Weight                          ! Write var to val array
   IF(F(13)) WRITE(PVals(13),'(A2)')                                &
      ADJUSTR(YN(Person(P)%AC_Car))                                  ! Write var to val array
   IF(F(14)) WRITE(PVals(14),'(I2)')                                &
      Person(P)%AC_Home                                              ! Write var to val array
   IF(F(15)) WRITE(PVals(15),'(A2)')                                &
      ADJUSTR(YN(Phys(P)%Ill))                                       ! Write var to val array
   IF(F(16)) WRITE(PVals(16),'(A2)')                                &
      ADJUSTR(YN(Person(P)%GasPilot))                                ! Write var to val array
   IF(F(17)) WRITE(PVals(17),'(A2)')                                &
      ADJUSTR(YN(Person(P)%GasStove))                                ! Write var to val array
   IF(F(18)) WRITE(PVals(18),'(I10)') NumEvents                      ! Write var to val array
   IF(F(19)) WRITE(PVals(19),'(F9.2)') Phys(P)%Blood                 ! Write var to val array
   IF(F(20)) WRITE(PVals(20),'(F5.2)') Phys(P)%BSA                   ! Write var to val array
   IF(F(21)) WRITE(PVals(21),'(F7.4)') Phys(P)%ECF                   ! Write var to val array
   IF(F(22)) WRITE(PVals(22),'(F7.3)') Phys(P)%Diff                  ! Write var to val array
   IF(F(23)) WRITE(PVals(23),'(F9.6)') Phys(P)%Endgn1                ! Write var to val array
   IF(F(24)) WRITE(PVals(24),'(F9.6)') Phys(P)%Endgn2                ! Write var to val array
   IF(F(25)) WRITE(PVals(25),'(F7.3)') Phys(P)%Hmglb                 ! Write var to val array
   IF(F(26)) WRITE(PVals(26),'(F7.3)') Phys(P)%MetsMax               ! Write var to val array
   IF(F(27)) WRITE(PVals(27),'(F7.3)') Phys(P)%Vo2max                ! Write var to val array
   IF(F(28)) WRITE(PVals(28),'(F7.3)') Phys(P)%MOXD                  ! Write var to val array
   IF(F(29)) WRITE(PVals(29),'(F5.2)') Person(P)%PAI                 ! Write var to val array
   IF(F(30)) WRITE(PVals(30),'(F7.3)') Phys(P)%RecTime               ! Write var to val array
   IF(F(31)) WRITE(PVals(31),'(F6.3)') Phys(P)%RMR                   ! Write var to val array
   IF(F(32)) WRITE(PVals(32),'(F9.6)') Phys(P)%VEinter               ! Write var to val array
   IF(F(33)) WRITE(PVals(33),'(F9.6)') Phys(P)%VEslope               ! Write var to val array
   IF(F(34)) WRITE(PVals(34),'(F9.6)') Phys(P)%VEresid               ! Write var to val array 
   IF(F(35)) WRITE(PVals(35),'(F9.3)') Phys(P)%BMI                   ! Write var to val array
   IF(F(36)) WRITE(PVals(36),'(F9.4)') Phys(P)%Beta1                 ! Write var to val array 
   IF(F(37)) WRITE(PVals(37),'(F9.4)') Phys(P)%Beta2                 ! Write var to val array 
   IF(F(38)) WRITE(PVals(38),'(F9.4)') Phys(P)%Beta3                 ! Write var to val array 
   IF(F(39)) WRITE(PVals(39),'(F9.4)') Phys(P)%Beta4                 ! Write var to val array 
   IF(F(40)) WRITE(PVals(40),'(F9.4)') Phys(P)%Beta5                 ! Write var to val array 
   IF(F(41)) WRITE(PVals(41),'(F9.4)') Phys(P)%Beta6                 ! Write var to val array 
   IF(F(42)) WRITE(PVals(42),'(F9.4)') Phys(P)%Beta7                 ! Write var to val array     
   IF(F(43)) WRITE(PVals(43),'(F9.4)') Phys(P)%FEVU                  ! Write var to val array
   IF(F(44)) WRITE(PVals(44),'(F9.4)') Phys(P)%FEVSLP                ! Write var to val array
   IF(F(45)) WRITE(PVals(45),'(F9.4)') Phys(P)%FEVINT                ! Write var to val array
   IF(F(46)) WRITE(PVals(46),'(I4)')   Person(P)%NumDiaries          ! Write var to val array 
   IF(F(47)) WRITE(PVals(47),'(I4)')   Person(P)%PCond1              ! Write var to val array
   IF(F(48)) WRITE(PVals(48),'(I4)')   Person(P)%PCond2              ! Write var to val array
   IF(F(49)) WRITE(PVals(49),'(I4)')   Person(P)%PCond3              ! Write var to val array
   IF(F(50)) WRITE(PVals(50),'(I4)')   Person(P)%PCond4              ! Write var to val array
   IF(F(51)) WRITE(PVals(51),'(I4)')   Person(P)%PCond5              ! Write var to val array
   IF(F(52)) WRITE(PVals(52),'(I4)')   Person(P)%RCond1              ! Write var to val array
   IF(F(53)) WRITE(PVals(53),'(I4)')   Person(P)%RCond2              ! Write var to val array
   IF(F(54)) WRITE(PVals(54),'(I4)')   Person(P)%RCond3              ! Write var to val array
   IF(F(55)) WRITE(PVals(55),'(I4)')   Person(P)%RCond4              ! Write var to val array
   IF(F(56)) WRITE(PVals(56),'(I4)')   Person(P)%RCond5              ! Write var to val array
   IF(F(57)) WRITE(PVals(57),'(F7.1)') Person(P)%CommDist            ! Write var to val array
   IF(F(58)) WRITE(PVals(58),'(F7.1)') Person(P)%CommTime            ! Write var to val array
   IF(F(59)) WRITE(PVals(59),'(I4)')   Person(P)%RoadD               ! Write var to val array
   IF(F(60)) WRITE(PVals(60),'(I4)')   Person(P)%RoadWD              ! Write var to val array
   IF(F(61)) WRITE(PVals(61),'(I4)')   Person(P)%FactorGroup         ! Write var to val array
   IF(F(62)) WRITE(PVals(62),'(A14)')  ADJUSTR(Person(P)%GroupName)  ! Write var to val array
   IF(F(63)) WRITE(Pvals(63),'(F9.4)') MAXVAL(DailyExp(1:Numdays)%dFEV1max)
   DO I=1,NumPollutants                                              ! Loop over pollutants
      IF(F(N+1))                                                    &
         WRITE(PVals(NUMPSUMVARS+4*(I-1)+1),'(ES10.3)')AvgExp(P,I)   ! Write var to val array
      IF(F(N+2))                                                    &
         WRITE(PVals(NUMPSUMVARS+4*(I-1)+2),'(ES10.3)')MaxExp(P,I)   ! Write var to val array
      IF(F(N+3) .AND. Pol(I)%DoDose==YES)                           & 
         WRITE(PVals(NUMPSUMVARS+4*(I-1)+3),'(ES10.3)')AvgDose(P,I)  ! Write var to val array
      IF(F(N+4) .AND. Pol(I)%DoDose==YES)                           & 
         WRITE(PVals(NUMPSUMVARS+4*(I-1)+4),'(ES10.3)')MaxDose(P,I)  ! Write var to val array
   ENDDO                                                             ! End loop over pollutants
   WRITE(PSumU,'(100A)') (TRIM(  PVals(I)), I=1,NVars)               ! Write final value array to file
   IF (DebugLevel>1) WRITE(LogU,SL_FM2) LV_MS2, ProcName(ProcLevel)  ! Message for debugging
   ProcLevel = ProcLevel-1                                           ! Decrement procedure depth
END SUBROUTINE WritePSumFile                                         ! Return to Output subroutine

SUBROUTINE WriteEventsFile(P)                                        ! Called from Output
   INTEGER (KIND=IK), INTENT(IN) :: P                                ! Input parameters
   INTEGER (KIND=IK) :: I, J, NVars,D, H, A                          ! Local integer variables
   REAL    (KIND=RK) :: VA                                           ! Local copy of VA 
   LOGICAL (KIND=1)  :: F(NUMEVENTVARS)                              ! Short form of event flags   
   ! Global constants: EventU
   ! Global vars used: NumPollutants, EventStart, NumEvents, EventSeq, EventExp,
   !                   EventDose, EventFDose, EventInDose, EventDepDose         
   ! Global vars set : ProcLevel, ProcName   
   ! Module vars set : EHeader, EVals  
   ! Intrinsic procs : WRITE, ALLOCATE,TRIM, ADJUSTL, ADJUSTR, ALLOCATED
   ! APEX procs      : CheckAllocation
   ProcLevel = ProcLevel+1                                           ! Increment procedure depth
   ProcName(ProcLevel) = "WriteEventsFile"                           ! Procedure name for messages
   NVars = NUMEVENTVARS +(NumPollutants)*7                           ! Number of vars to write
   F=(EventFlags==YES)                                               ! Set F array to True/False
   IF (ALLOCATED(EHeader)==.FALSE.) THEN                             ! Only on first person written
      ALLOCATE  (EHeader(NVars), STAT=AllocErr)                      ! Make space for header array
      CALL CheckAllocation(AllocErr)                                 ! Check the success of the alloc  
      ALLOCATE  (EVals(NVars), STAT=AllocErr)                        ! Make space for values array
      CALL CheckAllocation(AllocErr)                                 ! Check the success of the alloc   
      EHeader= ""                                                    ! Initialize the header array
      EVals  = ""                                                    ! Initialize the value array
      WRITE(EHeader(1),'(A8)')  'Person'                             ! Write label to header array  
      WRITE(EHeader(2),'(A6)')  'Seq'                                ! Write label to header array
      WRITE(EHeader(3),'(A5)')  'Day'                                ! Write label to header array
      WRITE(EHeader(4),'(A5)')  'Year'                               ! Write label to header array
      WRITE(EHeader(5),'(A3)')  'Mn'                                 ! Write label to header array
      WRITE(EHeader(6),'(A3)')  'Dy'                                 ! Write label to header array
      WRITE(EHeader(7),'(A3)')  'Hr'                                 ! Write label to header array
      WRITE(EHeader(8),'(A4)')  'Dur'                                ! Write label to header array
      WRITE(EHeader(9),'(A4)')  'Act'                                ! Write label to header array
      WRITE(EHeader(10),'(A4)') 'Mic'                                ! Write label to header array
      WRITE(EHeader(11),'(A3)') 'HW'                                 ! Write label to header array
      IF (F(12)) WRITE(EHeader(12),'(A7)') 'UMET'                    ! Write label to header array
      IF (F(13)) WRITE(EHeader(13),'(A7)') 'METS'                    ! Write label to header array
      IF (F(14)) WRITE(EHeader(14),'(A9)') 'VA'                      ! Write label to header array
      IF (F(15)) WRITE(EHeader(15),'(A9)') 'VE'                      ! Write label to header array
      IF (F(16)) WRITE(EHeader(16),'(A9)') 'Deficit'                 ! Write label to header array
      IF (F(17)) WRITE(EHeader(17),'(A8)') 'EVR'                     ! Write label to header array
      IF (F(18)) WRITE(EHeader(18),'(A8)') 'dFEV1%'                  ! Write label to header array
      IF (F(19)) WRITE(EHeader(19),'(A8)') 'X(FEV)'                  ! Write label to header array
      IF (F(20)) WRITE(EHeader(20),'(A8)') 'FEV-E1'                  ! Write label to header array
      IF (F(21)) WRITE(EHeader(21),'(A8)') 'FEV-E2'                  ! Write label to header array
      DO I=1,NumPollutants                                           ! Loop over pollutants       
         WRITE(EHeader(NUMEVENTVARS+7*(I-1)+1),'(A13)')             &  
             "Exp-"//TRIM(ADJUSTL(Pol(I)%Name(1:5)))                 ! Write label to header array
         IF  (Pol(I)%DoDose==YES)                                   & 
             WRITE(EHeader(NUMEVENTVARS+7*(I-1)+2),'(A14)')         &
            "Dose-"//TRIM(ADJUSTL(Pol(I)%Name(1:5)))                 ! Write label to header array
         IF  (Pol(I)%DoDose==YES .AND. Pol(I)%PM==NO)               & 
            WRITE(EHeader(NUMEVENTVARS+7*(I-1)+3),'(A16)')          &
            "EndDose-"//TRIM(ADJUSTL(Pol(I)%Name(1:5)))              ! Write label to header array
         IF  (Pol(I)%DoDose==YES.AND.Pol(I)%PM==YES)                & 
            WRITE(EHeader(NUMEVENTVARS+7*(I-1)+4),'(A16)')          &
            "IntakeDose-"//TRIM(ADJUSTL(Pol(I)%Name(1:5)))           ! Write label to header array
         IF  (Pol(I)%DoDose==YES.AND.Pol(I)%PM==YES)                & 
            WRITE(EHeader(NUMEVENTVARS+7*(I-1)+5),'(A13)')          &
            "DepDose-"//TRIM(ADJUSTL(Pol(I)%Name(1:5)))              ! Write label to header array
         IF (F(22)) WRITE(EHeader(NUMEVENTVARS+7*(I-1)+6),'(A15)')  &
            "Amb-"//TRIM(ADJUSTL(Pol(I)%Name(1:5)))                  ! Write label to header array
         IF (F(23)) WRITE(EHeader(NUMEVENTVARS+7*(I-1)+7),'(A15)')  &
            "HomeAmb-"//TRIM(ADJUSTL(Pol(I)%Name(1:5)))              ! Write label to header array
      ENDDO                                                          ! End loop over pollutants
      WRITE(EventU,'(100A)') (TRIM(EHeader(I)), I=1,NVars)           ! Write final header to file
   ENDIF                                                             ! End first person logic
   DO J=EventStart, NumEvents                                        ! Loop over events  
      D = EventSeq(J)%Day                                            ! Set day number
      H = 1+MOD(EventSeq(J)%Hour-1,24)                               ! Set hour of day  
      A = EventSeq(J)%Activity                                       ! Set activity (METS) number 
      IF (A<0) A = A+256                                             ! Shift negative categories
      VA = 0.                                                        ! Block VA output by default
      IF (VAoutput==YES) VA = EventSeq(I)%VA                         ! Set VA if user requested it      
      WRITE(EVals(1),'(I8)')    P                                    ! Write var to val array  
      WRITE(EVals(2),'(I6)')    J                                    ! Write var to val array 
      WRITE(EVals(3),'(I5)')    D                                    ! Write var to val array 
      WRITE(EVals(4),'(I5)')    Calen(D)%Year                        ! Write var to val array 
      WRITE(EVals(5),'(I3)')    Calen(D)%Month                       ! Write var to val array 
      WRITE(EVals(6),'(I3)')    Calen(D)%Day                         ! Write var to val array 
      WRITE(EVals(7),'(I3)')    H                                    ! Write var to val array 
      WRITE(EVals(8),'(I4)')    EventSeq(J)%Duration                 ! Write var to val array 
      WRITE(EVals(9),'(I4)')    A                                    ! Write var to val array 
      WRITE(EVals(10),'(I4)')   EventSeq(J)%Location                 ! Write var to val array 
      WRITE(EVals(11),'(I3)')   EventSeq(J)%HomeWork                 ! Write var to val array 
      IF (F(12)) WRITE(EVals(12),'(F7.3)') EventSeq(J)%UMET          ! Write var to val array 
      IF (F(13)) WRITE(EVals(13),'(F7.3)') EventSeq(J)%METS          ! Write var to val array
      IF (F(14)) WRITE(EVals(14),'(F9.0)') EventSeq(J)%VA            ! Write var to val array
      IF (F(15)) WRITE(EVals(15),'(F9.0)') EventSeq(J)%VE            ! Write var to val array
      IF (F(16)) WRITE(EVals(16),'(F9.5)') EventSeq(J)%Deficit       ! Write var to val array
      IF (F(17)) WRITE(EVals(17),'(F8.3)') EventSeq(J)%EVR           ! Write var to val array
      IF (F(18)) WRITE(EVals(18),'(F8.3)') EventSeq(J)%FinalFEV      ! Write var to val array
      IF (F(19)) WRITE(EVals(19),'(F8.3)') EventSeq(J)%FinalX        ! Write var to val array
      IF (F(20)) WRITE(EVals(20),'(F8.3)') EventSeq(J)%FEVE1         ! Write var to val array
      IF (F(21)) WRITE(EVals(21),'(F8.3)') EventSeq(J)%FEVE2         ! Write var to val array      
      DO I=1,NumPollutants                                           ! Loop over pollutants       
         WRITE(EVals(NUMEVENTVARS+7*(I-1)+1),'(ES10.3)')            &
            EventExp(J,I)                                            ! Write var to val array  
         IF(Pol(I)%DoDose==YES)                                     &
         WRITE(EVals(NUMEVENTVARS+7*(I-1)+2),'(ES10.3)')            &
            EventDose(J,I)                                           ! Write var to val array  
         IF(Pol(I)%DoDose==YES .AND. Pol(I)%PM==NO)                 &
         WRITE(EVals(NUMEVENTVARS+7*(I-1)+3),'(ES10.3)')            &
            EventFDose(J,I)                                          ! Write var to val array  
         IF(Pol(I)%DoDose==YES.AND.Pol(I)%PM==YES)                  &
         WRITE(EVals(NUMEVENTVARS+7*(I-1)+4),'(ES10.3)')            &
            EventInDose(J,I)                                         ! Write var to val array 
         IF(Pol(I)%DoDose==YES.AND.Pol(I)%PM==YES)                  &
         WRITE(EVals(NUMEVENTVARS+7*(I-1)+5),'(ES10.3)')            &
            EventDepDose(J,I)                                        ! Write var to val array
         IF (F(22)) WRITE(EVals(NUMEVENTVARS+7*(I-1)+6),'(ES10.3)') &
            EventAmb(J,I)                                            ! Write var to val array   
         IF (F(23)) WRITE(EVals(NUMEVENTVARS+7*(I-1)+7),'(ES10.3)') &
            EventAmbHome(J,I)                                        ! Write var to val array
      ENDDO                                                          ! End loop over pollutants
   WRITE(EventU,'(100A)') (TRIM(EVals(I)), I=1,NVars)                ! Write final value array to file
   ENDDO                                                             ! End loop over events   
   IF (DebugLevel>1) WRITE(LogU,SL_FM2) LV_MS2, ProcName(ProcLevel)  ! Message for debugging
   ProcLevel = ProcLevel-1                                           ! Decrement procedure depth
END SUBROUTINE WriteEventsFile                                       ! Return to Output subroutine

SUBROUTINE WriteHourlyFile(P)                                        ! Called from Output
   INTEGER (KIND=IK), INTENT(IN) :: P                                ! Input parameters
   INTEGER (KIND=IK) :: I, J, K, NVars, Flag, Index, NMic            ! Local integer variables
   CHARACTER(LEN=LEN1) :: MicNum                                     ! Buffer for micro number
   ! Global constants: HourU
   ! Global vars used: NumPollutants, HourTime, HourDose, HourConc, HourExp,
   !                   HourAmb, HourEF, HourSeq,          
   ! Global vars set : ProcLevel, ProcName   
   ! Module vars set : HHeader, HVals  
   ! Intrinsic procs : WRITE, SIZE, ALLOCATE, TRIM, ADJUSTL, ADJUSTR, ALLOCATED
   ! APEX procs      : CheckAllocation
   ProcLevel = ProcLevel+1                                           ! Increment procedure depth
   ProcName(ProcLevel) = "WriteHourlyFile"                           ! Procedure name for messages
   NMic = NumMicros+1                                                ! NumMicros (+ 1 for Micro 0)
   NVars = 10+8*(NumPollutants)+NMic*(1+NumPollutants)               ! Number of vars to write   
   IF (ALLOCATED(PHeader)==.FALSE.) THEN                             ! Only on first person,pollut
      ALLOCATE  (F(SIZE(HourFlags,1)), STAT=AllocErr)                ! Make space for flag array
      CALL CheckAllocation(AllocErr)                                 ! Check the success of the alloc  
      F=(HourFlags==YES)                                             ! Set F array to True/False
      ALLOCATE  (HHeader(NVars), STAT=AllocErr)                      ! Make space for header array
      CALL CheckAllocation(AllocErr)                                 ! Check the success of the alloc  
      ALLOCATE  (HVals(NVars), STAT=AllocErr)                        ! Make space for values array
      CALL CheckAllocation(AllocErr)                                 ! Check the success of the alloc   
      HHeader= ""                                                    ! Initialize the header array
      HVals  = ""                                                    ! Initialize the values array
      WRITE(HHeader(1),'(A8)')  'P'                                  ! Write label to header array  
      WRITE(HHeader(2),'(A8)')  'Hour'                               ! Write label to header array
      IF(F(3))  WRITE(HHeader(3), '(A8)') 'Ve'                       ! Write label to header array
      IF(F(4))  WRITE(HHeader(4), '(A8)') 'Va'                       ! Write label to header array
      IF(F(5))  WRITE(HHeader(5), '(A6)') 'EVR'                      ! Write label to header array
      IF(F(6))  WRITE(HHeader(6), '(A7)') 'METS'                     ! Write label to header array
      IF(F(7))  WRITE(HHeader(7), '(A6)') 'EE'                       ! Write label to header array
      IF(F(8))  WRITE(HHeader(8), '(A11)') '%dFEV1max'               ! Write label to header array      
      IF(F(9))  WRITE(HHeader(9), '(A11)') 'FEV-E1'                  ! Write label to header array      
      IF(F(10))  WRITE(HHeader(10), '(A11)') 'FEV-E2'                ! Write label to header array            
      Flag = 11                                                      ! Initialize flag index
      Index = 11                                                     ! Initialize header flag index
      DO I=1, NMic                                                   ! Loop over micros       
         IF (F(Flag+I)) THEN                                         ! If writing var to output
            Index = Index+1                                          ! Update header flag index
            WRITE(MicNum,*) I-1                                      ! Write the mic num to string
            WRITE(HHeader(Index),'(A8)')                            & 
               "Time_"//TRIM(ADJUSTL(MicNum))                        ! Write label to header array
         ENDIF                                                       ! End output logic
      ENDDO                                                          ! End loop over micros
      DO I=1,NumPollutants                                           ! Loop over pollutants       
         Flag = 11+NMic                                              ! Update flag index
         DO J=1, NMic                                                ! Loop over micros
            IF (F(Flag+J)) THEN                                      ! If writing var to output
               Index = Index+1                                       ! Update header index
               WRITE(MicNum,*) J-1                                   ! Write the mic num to string
               WRITE(HHeader(Index),'(A13)')                        &
                 "Exp_"//TRIM(ADJUSTL(MicNum))//"-"//               &
                  TRIM((Pol(I)%Name(1:5)))                           ! Write label to header array
            ENDIF                                                    ! End output logic   
         ENDDO                                                       ! End loop over micros 
         Flag = 11+2*NMic                                            ! Reinitialize flag
         IF (F(Flag+1)) THEN                                         ! If writing var to output
            Index = Index+1                                          ! Update header index
            WRITE(HHeader(Index),'(A11)')                           & 
               "Amb-"//TRIM((Pol(I)%Name(1:5)))                      ! Write label to header array
         ENDIF                                                       ! End output logic
         IF (F(Flag+2)) THEN                                         ! If writing var to output
            Index = Index+1                                          ! Update header index
            WRITE(HHeader(Index),'(A11)')                           & 
               "Exp-"//TRIM((Pol(I)%Name(1:5)))                      ! Write label to header array
         ENDIF                                                       ! End output logic
         IF (F(Flag+3) .AND. Pol(I)%DoDose==YES ) THEN               ! If writing var to output
            Index = Index+1                                          ! Update header index 
            WRITE(HHeader(Index),'(A11)')                           & 
               "Dose-"//TRIM((Pol(I)%Name(1:5)))                     ! Write label to header array
         ENDIF                                                       ! End output logic
         IF (F(Flag+4)) THEN                                         ! If writing var to output
            Index = Index+1                                          ! Update header index 
            WRITE(HHeader(Index),'(A10)')                           & 
               "EF-"//TRIM((Pol(I)%Name(1:5)))                       ! Write label to header array
          ENDIF                                                      ! End output logic
         IF (Pol(I)%PM==YES.AND. Pol(I)%DoDose==YES ) THEN           ! If this is PM
            IF (F(Flag+5)) THEN                                      ! If writing var to output
               Index = Index+1                                       ! Update header index 
               WRITE(HHeader(Index),'(A16)')                        & 
                  "IntakeDose-"//TRIM((Pol(I)%Name(1:5)))            ! Write label to header array
            ENDIF                                                    ! End output logic
            IF (F(Flag+6)) THEN                                      ! If writing var to output
               Index = Index+1                                       ! Update header index 
               WRITE(HHeader(Index),'(A14)')                        & 
                  "DepDose-"//TRIM((Pol(I)%Name(1:5)))               ! Write label to header array
            ENDIF                                                    ! End output logic
         ENDIF                                                       ! End PM logic
         IF (F(Flag+7)) THEN                                         ! If writing var to output
            Index=Index+1                                            ! Update header index
            WRITE(HHeader(Index),'(A15)')                           & 
                "HomeAmb-"//TRIM((Pol(I)%Name(1:5)))                 ! Write label to header array
         ENDIF                                                       ! End output logic
         IF (F(Flag+8)) THEN                                         ! If writing var to output
            Index = Index+1                                          ! Update header index
           WRITE(HHeader(Index),'(A15)')                           & 
                "HomeEF-"//TRIM((Pol(I)%Name(1:5)))                  ! Write label to header array
         ENDIF                                                       ! End output logic
      ENDDO                                                          ! End loop over pollutants
      WRITE(HourU,'(1000A)') (TRIM(HHeader(I)), I=1,NVars)           ! Write final header to file
   ENDIF                                                             ! End first person logic
   DO K=1, NumHours                                                  ! Loop over hours in simulation           
      WRITE(HVals(1),'(I8)')  P                                      ! Write var to val array   
      WRITE(HVals(2),'(I8)')  K                                      ! Write var to val array 
      IF(F(3))  WRITE(HVals(3), '(F8.0)') HourSeq(K)%Ve              ! Write var to val array 
      IF(F(4))  WRITE(HVals(4), '(F8.0)') HourSeq(K)%Va              ! Write var to val array 
      IF(F(5))  WRITE(HVals(5), '(F6.2)') HourSeq(K)%EVR             ! Write var to val array 
      IF(F(6))  WRITE(HVals(6), '(F7.3)') HourSeq(K)%METS            ! Write var to val array 
      IF(F(7))  WRITE(HVals(7), '(F6.2)') HourSeq(K)%EE              ! Write var to val array   
      IF(F(8))  WRITE(HVals(8), '(F8.3)') HourSeq(K)%dFEV1max        ! Write var to val array        
      IF(F(9))  WRITE(HVals(9), '(F8.3)') HourSeq(K)%FEVE1           ! Write var to val array        
      IF(F(10))  WRITE(HVals(10), '(F8.3)') HourSeq(K)%FEVE2         ! Write var to val array        
      Flag = 11                                                       ! Initialize flag index
      Index = 11                                                      ! Initialize header flag index
      DO I=1, NMic                                                   ! Loop over micros       
         IF (F(Flag+I)) THEN                                         ! If writing var to output
            Index = Index+1                                          ! Update val array index
            WRITE(HVals(Index),'(I8)') HourTime(K,I-1)               ! Write var to val array
         ENDIF                                                       ! End output logic 
      ENDDO                                                          ! End loop over micros
      DO I=1,NumPollutants                                           ! Loop over pollutants       
         Flag = 11+NMic                                              ! Reinitialize flag
         DO J=1, NMic                                                ! Loop over micros
            IF (F(Flag+J)) THEN                                      ! If writing var to output
               Index = Index+1                                       ! Update val array index 
               WRITE(HVals(Index),'(ES10.3)') HourConc(K,I,J-1)      ! Write var to val array 
            ENDIF                                                    ! End output logic 
         ENDDO                                                       ! End loop over micros       
         Flag = 11+2*NMic                                             ! Reinitialize flag
         IF (F(Flag+1)) THEN                                         ! If writing var to output
            Index = Index+1                                          ! Update val array index 
            WRITE(HVals(Index),'(ES10.3)') HourAmb(K,I)              ! Write var to val array 
         ENDIF                                                       ! End output logic 
         IF (F(Flag+2)) THEN                                         ! If writing var to output
            Index = Index+1                                          ! Update val array index
            WRITE(HVals(Index),'(ES10.3)') HourExp(K,I)              ! Write var to val array 
         ENDIF                                                       ! End output logic 
         IF (F(Flag+3) .AND. Pol(I)%DoDose==YES ) THEN               ! If writing var to output
            Index = Index+1                                          ! Update val array index
            WRITE(HVals(Index),'(ES10.3)') HourDose(K,I)             ! Write var to val array                   
         ENDIF                                                       ! End output logic 
         IF (F(Flag+4)) THEN                                         ! If writing var to output
            Index = Index+1                                          ! Update val array index
            WRITE(HVals(Index),'(ES10.3)')  HourEF(K,I)              ! Write var to val array             
         ENDIF                                                       ! End output logic 
         IF (Pol(I)%PM==YES.AND. Pol(I)%DoDose==YES ) THEN           ! If this is PM
            IF (F(Flag+5)) THEN                                      ! If writing var to output
               Index = Index+1                                       ! Update val array index
               WRITE(HVals(Index),'(ES10.3)')  HourIntake(K,I)       ! Write var to val array             
            ENDIF                                                    ! End output logic 
            IF (F(Flag+6)) THEN                                      ! If writing var to output
               Index = Index+1                                       ! Update val array index
               WRITE(HVals(Index),'(ES10.3)')  HourDep(K,I)          ! Write var to val array             
            ENDIF                                                    ! End output logic   
         ENDIF                                                       ! End PM logic
         IF (F(Flag+7)) THEN                                         ! If writing var to output
            Index = Index+1                                          ! Update val array index 
            WRITE(HVals(Index),'(ES10.3)') HourHomeAmb(K,I)          ! Write var to val array 
         ENDIF                                                       ! End output logic 
         IF (F(Flag+8)) THEN                                         ! If writing var to output
            Index = Index+1                                          ! Update val array index
            WRITE(HVals(Index),'(ES10.3)') HourHomeEF(K,I)           ! Write var to val array 
         ENDIF                                                       ! End output logic
      ENDDO                                                          ! End loop over pollutants     
      I=1                                                            ! Initialize I for FORTRAN quirk
      WRITE(HourU,'(100A)') (TRIM(HVals(I)), I=1,NVars)              ! Write final value array to file
   ENDDO                                                             ! End loop over hours
   IF (DebugLevel>1) WRITE(LogU,SL_FM2) LV_MS2, ProcName(ProcLevel)  ! Message for debugging
   ProcLevel = ProcLevel-1                                           ! Decrement procedure depth
 END SUBROUTINE WriteHourlyFile                                      ! Return to Output subroutine

SUBROUTINE WriteTimeStepFile(P)                                      ! Called from Output
   INTEGER (KIND=IK), INTENT(IN) :: P                                ! Input parameters
   INTEGER (KIND=IK) :: I, K, NVars, Flag, Index                     ! Local integer variables
   LOGICAL (KIND=1)  :: F(NUMTSVARS)                                 ! Short form of ts flags
   ! Global constants: TSU
   ! Global vars used: NumPollutants, HourTime, HourDose, HourConc, HourExp,
   !                   HourAmb, HourEF, HourSeq,          
   ! Global vars set : ProcLevel, ProcName   
   ! Module vars set : TSHeader, TSVals  
   ! Intrinsic procs : WRITE, SIZE, ALLOCATE, TRIM, ADJUSTL, ADJUSTR, ALLOCATED
   ! APEX procs      : CheckAllocation
   ProcLevel = ProcLevel+1                                           ! Increment procedure depth
   ProcName(ProcLevel) = "WriteTimeStepFile"                         ! Procedure name for messages
   NVars = 8+6*(NumPollutants)                                       ! Number of vars to write   
   F=(TSFlags==YES)                                                  ! Set F array to True/False
   IF (P==FirstProfile) THEN                                         ! Only on first person
      ALLOCATE  (TSHeader(NVars), STAT=AllocErr)                     ! Make space for header array
      CALL CheckAllocation(AllocErr)                                 ! Check the success of the alloc  
      ALLOCATE  (TSVals(NVars), STAT=AllocErr)                       ! Make space for values array
      CALL CheckAllocation(AllocErr)                                 ! Check the success of the alloc   
      TSHeader= ""                                                   ! Initialize the header array
      TSVals  = ""                                                   ! Initialize the values array
      WRITE(TSHeader(1),'(A8)')  'P'                                 ! Write label to header array  
      WRITE(TSHeader(2),'(A8)')  'Hour'                              ! Write label to header array
      WRITE(TSHeader(3),'(A9)')  'Timestep'                          ! Write label to header array
      IF(F(4))  WRITE(TSHeader(4), '(A8)') 'Ve'                      ! Write label to header array
      IF(F(5))  WRITE(TSHeader(5), '(A8)') 'Va'                      ! Write label to header array
      IF(F(6))  WRITE(TSHeader(6), '(A6)') 'EVR'                     ! Write label to header array
      IF(F(7))  WRITE(TSHeader(7), '(A7)') 'METS'                    ! Write label to header array
      IF(F(8))  WRITE(TSHeader(8), '(A6)') 'EE'                      ! Write label to header array
      IF(F(9))  WRITE(TSHeader(9), '(A11)') '%dFEV1max'               ! Write label to header array      
      Flag = 9                                                       ! Initialize flag index
      Index = 9                                                      ! Initialize header flag index
      DO I=1,NumPollutants                                           ! Loop over pollutants       
         IF (F(Flag+1)) THEN                                         ! If writing var to output
            Index = Index+1                                          ! Update header index
            WRITE(TSHeader(Index),'(A11)')                          & 
               "Amb-"//TRIM((Pol(I)%Name(1:5)))                      ! Write label to header array
         ENDIF                                                       ! End output logic
         IF (F(Flag+2)) THEN                                         ! If writing var to output
            Index = Index+1                                          ! Update header index
            WRITE(TSHeader(Index),'(A11)')                          & 
               "Exp-"//TRIM((Pol(I)%Name(1:5)))                      ! Write label to header array
         ENDIF                                                       ! End output logic
         IF (F(Flag+3)) THEN                                         ! If writing var to output
            Index = Index+1                                          ! Update header index 
            WRITE(TSHeader(Index),'(A11)')                          & 
               "Dose-"//TRIM((Pol(I)%Name(1:5)))                     ! Write label to header array
         ENDIF                                                       ! End output logic
         IF (F(Flag+4)) THEN                                         ! If writing var to output
            Index = Index+1                                          ! Update header index 
            WRITE(TSHeader(Index),'(A10)')                          & 
               "EF-"//TRIM((Pol(I)%Name(1:5)))                       ! Write label to header array
         ENDIF                                                       ! End output logic
         IF (Pol(I)%PM==YES.AND. Pol(I)%DoDose==YES ) THEN           ! If this is PM
            IF (F(Flag+5)) THEN                                      ! If writing var to output
               Index = Index+1                                       ! Update header index 
               WRITE(TSHeader(Index),'(A16)')                       & 
                  "IntakeDose-"//TRIM((Pol(I)%Name(1:5)))            ! Write label to header array
            ENDIF                                                    ! End output logic
            IF (F(Flag+6)) THEN                                      ! If writing var to output
               Index = Index+1                                       ! Update header index 
               WRITE(TSHeader(Index),'(A13)')                       & 
                  "DepDose-"//TRIM((Pol(I)%Name(1:5)))               ! Write label to header array
            ENDIF                                                    ! End output logic
         ENDIF                                                       ! End PM logic
      ENDDO                                                          ! End loop over pollutants
      WRITE(TSU,'(1000A)') (TRIM(TSHeader(I)), I=1,NVars)            ! Write final header to file
   ENDIF                                                             ! End first person logic
   DO K=1, NumTS                                                     ! Loop over ts in simulation           
      WRITE(TSVals(1),'(I8)')  P                                     ! Write var to val array   
      WRITE(TSVals(2),'(I8)')  TSSeq(K)%Hour                         ! Write var to val array   
      WRITE(TSVals(3),'(I9)')  K                                     ! Write var to val array 
      IF(F(4))  WRITE(TSVals(4), '(F8.0)') TSSeq(K)%Ve               ! Write var to val array 
      IF(F(5))  WRITE(TSVals(5), '(F8.0)') TSSeq(K)%Va               ! Write var to val array 
      IF(F(6))  WRITE(TSVals(6), '(F6.2)') TSSeq(K)%EVR              ! Write var to val array 
      IF(F(7))  WRITE(TSVals(7), '(F7.3)') TSSeq(K)%METS             ! Write var to val array 
      IF(F(8))  WRITE(TSVals(8), '(F6.2)') TSSeq(K)%EE               ! Write var to val array 
      IF(F(9))  WRITE(TSVals(9), '(F8.3)') TSSeq(K)%dFEV1max         ! Write var to val array       
      Flag = 9                                                       ! Initialize flag index
      Index = 9                                                      ! Initialize header flag index
      DO I=1,NumPollutants                                           ! Loop over pollutants       
         IF (F(Flag+1)) THEN                                         ! If writing var to output
            Index = Index+1                                          ! Update val array index 
            WRITE(TSVals(Index),'(ES10.3)') TSAmb(K,I)               ! Write var to val array 
         ENDIF                                                       ! End output logic 
         IF (F(Flag+2)) THEN                                         ! If writing var to output
            Index = Index+1                                          ! Update val array index
            WRITE(TSVals(Index),'(ES10.3)') TSExp(K,I)               ! Write var to val array 
         ENDIF                                                       ! End output logic 
         IF (F(Flag+3) .AND. Pol(I)%DoDose==YES ) THEN               ! If writing var to output
            Index = Index+1                                          ! Update val array index
            WRITE(TSVals(Index),'(ES10.3)') TSDose(K,I)              ! Write var to val array
         ENDIF                                                       ! End output logic 
         IF (F(Flag+4)) THEN                                         ! If writing var to output
            Index = Index+1                                          ! Update val array index
            WRITE(TSVals(Index),'(ES10.3)')  TSEF(K,I)               ! Write var to val array             
         ENDIF                                                       ! End output logic 
         IF (Pol(I)%PM==YES.AND. Pol(I)%DoDose==YES ) THEN           ! If this is PM
            IF (F(Flag+5)) THEN                                      ! If writing var to output
               Index = Index+1                                       ! Update val array index
               WRITE(TSVals(Index),'(ES10.3)')  TSInDose(K,I)        ! Write var to val array             
            ENDIF                                                    ! End output logic 
            IF (F(Flag+6)) THEN                                      ! If writing var to output
               Index = Index+1                                       ! Update val array index
               WRITE(TSVals(Index),'(ES10.3)')  TSDepDose(K,I)       ! Write var to val array             
            ENDIF                                                    ! End output logic 
         ENDIF                                                       ! End PM Logic   
      ENDDO                                                          ! End loop over pollutants     
      I=1                                                            ! Initialize I for FORTRAN quirk
      WRITE(TSU,'(100A)') (TRIM(TSVals(I)), I=1,NVars)               ! Write final value array to file
   ENDDO                                                             ! End loop over hours
   IF (DebugLevel>1) WRITE(LogU,SL_FM2) LV_MS2, ProcName(ProcLevel)  ! Message for debugging
   ProcLevel = ProcLevel-1                                           ! Decrement procedure depth
 END SUBROUTINE WriteTimeStepFile                                    ! Return to Output subroutine

SUBROUTINE WriteDailyFile(P)                                         ! Called from Output
   INTEGER (KIND=IK), INTENT(IN) :: P                                ! Input parameters
   INTEGER (KIND=IK) :: I, J, NVars                                  ! Local integer variables
   LOGICAL (KIND=1)  :: F(NUMDAYVARS+10)                             ! Short form of psum flags
   ! Global constants: PSumU
   ! Global vars used: F, NumPollutants, Person          
   ! Global vars set : ProcLevel, ProcName    
   ! Module vars set : DHeader, DVals    
   ! Intrinsic procs : WRITE, ALLOCATE, TRIM, ADJUSTL, ADJUSTR
   ! APEX procs      : CheckAllocation
   ProcLevel = ProcLevel+1                                           ! Increment procedure depth
   ProcName(ProcLevel) = "WriteDailyFile"                            ! Procedure name for messages
   NVars = NUMDAYVARS+(NumPollutants)*10                             ! Number of vars to write   
   F=(DayFlags==YES)                                                 ! Set F array to True/False
   IF (ALLOCATED(DHeader)==.FALSE.) THEN                             ! Only on first person written
      ALLOCATE  (DHeader(NVars), STAT=AllocErr)                      ! Make space for header array
      CALL CheckAllocation(AllocErr)                                 ! Check the success of the alloc  
      ALLOCATE  (DVals(NVars), STAT=AllocErr)                        ! Make space for values array
      CALL CheckAllocation(AllocErr)                                 ! Check the success of the alloc   
      DHeader= ""                                                    ! Initialize the header array
      DVals  = ""                                                    ! Initialize the values array
      WRITE(DHeader(1),'(A7)')  'P'                                  ! Write label to header array  
      WRITE(DHeader(2),'(A4)')  'Day'                                ! Write label to header array
      IF(F(3))  WRITE(DHeader(3),'(A7)')'CHADID'                     ! Write label to header array
      IF(F(4))  WRITE(DHeader(4),'(A8)') 'CHADAge'                   ! Write label to header array
      IF(F(5))  WRITE(DHeader(5),'(A8)') 'CHADEmp'                   ! Write label to header array
      IF(F(6))  WRITE(DHeader(6),'(A10)')'DiaryPool'                 ! Write label to header array
      IF(F(7))  WRITE(DHeader(7),'(A4)') 'PAI'                       ! Write label to header array
      IF(F(8))  WRITE(DHeader(8),'(A7)') 'KeyVar'                    ! Write label to header array
      IF(F(9))  WRITE(DHeader(9),'(A10)')'WindowRes'                 ! Write label to header array 
      IF(F(10)) WRITE(DHeader(10),'(A10)')'WindowCar'                ! Write label to header array
      IF(F(11)) WRITE(DHeader(11),'(A9)') 'SpeedCat'                 ! Write label to header array
      IF(F(12)) WRITE(DHeader(12),'(A7)') 'DCond1'                   ! Write label to header array
      IF(F(13)) WRITE(DHeader(13),'(A7)') 'DCond2'                   ! Write label to header array
      IF(F(14)) WRITE(DHeader(14),'(A7)') 'DCond3'                   ! Write label to header array
      IF(F(15)) WRITE(DHeader(15),'(A11)')'MaxTempCat'               ! Write label to header array
      IF(F(16)) WRITE(DHeader(16),'(A11)')'AvgTempCat'               ! Write label to header array
      IF(F(17)) WRITE(DHeader(17),'(A8)') 'AvgTemp'                  ! Write label to header array
      IF(F(18)) WRITE(DHeader(18),'(A8)') 'MaxTemp'                  ! Write label to header array
      IF(F(19)) WRITE(DHeader(19),'(A10)') '%dFEV1max'               ! Write label to header array
      IF(F(20)) WRITE(DHeader(20),'(A8)') 'CHADOcc'                  ! Write label to header array
      DO J=1,NumPollutants                                           ! Loop over pollutants       
         IF (F(21)) WRITE(DHeader(NUMDAYVARS+10*(J-1)+1),'(A13)')    &
            "AvgExp-"//TRIM(ADJUSTL(Pol(J)%Name(1:5)))               ! Write label to header array
         IF (F(22)) WRITE(DHeader(NUMDAYVARS+10*(J-1)+2),'(A14)')    &  
            "Max1Exp-"//TRIM(ADJUSTL(Pol(J)%Name(1:5)))              ! Write label to header array
         IF (F(23)) WRITE(DHeader(NUMDAYVARS+10*(J-1)+3),'(A14)')    &
            "Max8Exp-"//TRIM(ADJUSTL(Pol(J)%Name(1:5)))              ! Write label to header array
         IF (F(24).AND. Pol(J)%DoDose==YES)                         & 
            WRITE(DHeader(NUMDAYVARS+10*(J-1)+4),'(A14)')            &
            "AvgDose-"//TRIM(ADJUSTL(Pol(J)%Name(1:5)))              ! Write label to header array
         IF (F(25).AND. Pol(J)%DoDose==YES)                         & 
             WRITE(DHeader(NUMDAYVARS+10*(J-1)+5),'(A15)')           &
            "Max1Dose-"//TRIM(ADJUSTL(Pol(J)%Name(1:5)))             ! Write label to header array
         IF (F(26).AND. Pol(J)%DoDose==YES)                         & 
             WRITE(DHeader(NUMDAYVARS+10*(J-1)+6),'(A15)')           &
            "Max8Dose-"//TRIM(ADJUSTL(Pol(J)%Name(1:5)))             ! Write label to header array
         IF (F(27).AND. Pol(J)%DoDose==YES)                         & 
             WRITE(DHeader(NUMDAYVARS+10*(J-1)+7),'(A16)')           &
            "Max1FDose-"//TRIM(ADJUSTL(Pol(J)%Name(1:5)))            ! Write label to header array
         IF (F(28).AND.Pol(J)%DoDose==YES.AND.Pol(J)%PM==YES)       & 
             WRITE(DHeader(NUMDAYVARS+10*(J-1)+9),'(A17)')           &
            "IntakeDose-"//TRIM(ADJUSTL(Pol(J)%Name(1:5)))           ! Write label to header array   
         IF (F(29).AND.Pol(J)%DoDose==YES.AND.Pol(J)%PM==YES)       & 
             WRITE(DHeader(NUMDAYVARS+10*(J-1)+8),'(A15)')           &
            "DepDose-"//TRIM(ADJUSTL(Pol(J)%Name(1:5)))              ! Write label to header array
         IF (F(30)) WRITE(DHeader(NUMDAYVARS+10*(J-1)+10),'(A14)')    &
            "Max8E/C-"//TRIM(ADJUSTL(Pol(J)%Name(1:5)))              ! Write label to header array
      ENDDO                                                          ! End loop over pollutants
      WRITE(DayU,'(100A)') (TRIM(DHeader(J)), J=1,NVars)             ! Write final header to file
   ENDIF                                                             ! End first person logic
   DO I=1, NumDays                                                   ! Loop over the number of days
      WRITE(DVals(1),'(I7)') P                                       ! Write var to val array
      WRITE(DVals(2),'(I4)') I                                       ! Write var to val array
      IF(F(3)) WRITE(DVals(3),'(A10)')PersonDay(I)%DiaryID           ! Write var to val array
      IF(F(4)) WRITE(DVals(4),'(I4)') PersonDay(I)%DiaryAge          ! Write var to val array
      IF(F(5)) WRITE(DVals(5),'(A2)')                               &        
         ADJUSTR(YN(PersonDay(I)%DiaryEmp))                          ! Write var to val array
      IF(F(6)) WRITE(DVals(6),'(I3)')                               &
         TemData(I,SList(Person(P)%HomeSec)%NumZ)%Pool               ! Write var to val array
      IF(F(7)) WRITE(DVals(7),'(F5.2)') PersonDay(I)%PAI             ! Write var to val array
      IF(F(8)) WRITE(DVals(8),'(F8.2)') PersonDay(I)%Stat            ! Write var to val array
      IF(F(9)) WRITE(DVals(9),'(I2)')  PersonDay(I)%WindowRes        ! Write var to val array
      IF(F(10)) WRITE(DVals(10),'(I2)') PersonDay(I)%WindowCar       ! Write var to val array
      IF(F(11)) WRITE(DVals(11),'(I2)') PersonDay(I)%SpeedCat        ! Write var to val array
      IF(F(12)) WRITE(DVals(12),'(I2)') PersonDay(I)%DCond1          ! Write var to val array     
      IF(F(13)) WRITE(DVals(13),'(I2)') PersonDay(I)%DCond2          ! Write var to val array     
      IF(F(14)) WRITE(DVals(14),'(I2)') PersonDay(I)%DCond3          ! Write var to val array     
      IF(F(15)) WRITE(DVals(15),'(I2)')                             &   
         TemData(I,SList(Person(P)%HomeSec)%NumZ)%TempCatM           ! Write var to val array
      IF(F(16)) WRITE(DVals(16),'(I2)')                             &  
         TemData(I,SList(Person(P)%HomeSec)%NumZ)%TempCatA           ! Write var to val array
      IF(F(17))  WRITE(DVals(17),'(I4)')                            &            
         TemData(I,SList(Person(P)%HomeSec)%NumZ)%AvgTemp            ! Write var to val array
      IF(F(18)) WRITE(DVals(18),'(I4)')                             &
         TemData(I,SList(Person(P)%HomeSec)%NumZ)%MaxTemp            ! Write var to val array
      IF(F(19)) WRITE(DVals(19),'(F8.3)') DailyExp(I)%dFEV1max       ! Write var to val array
      IF(F(20)) WRITE(DVals(20),'(A14)') ADJUSTR(PersonDay(I)%DiaryOcc) ! Write var to val array
      DO J=1,NumPollutants                                           ! Loop over pollutants       
         IF(F(21)) WRITE(DVals(NUMDAYVARS+10*(J-1)+1),'(ES10.3)')    &
            DayAvgExp(I,J)                                           ! Write var to val array  
         IF(F(22)) WRITE(DVals(NUMDAYVARS+10*(J-1)+2),'(ES10.3)')    &
            DayMax1Exp(I,J)                                          ! Write var to val array  
         IF(F(23)) WRITE(DVals(NUMDAYVARS+10*(J-1)+3),'(ES10.3)')    &  
            DayMax8Exp(I,J)                                          ! Write var to val array 
         IF(F(24) .AND. Pol(J)%DoDose==YES)                         & 
            WRITE(DVals(NUMDAYVARS+10*(J-1)+4),'(ES10.3)')          &
            DayAvgDose(I,J)                                          ! Write var to val array   
         IF (F(25) .AND. Pol(J)%DoDose==YES)                        &   
            WRITE(DVals(NUMDAYVARS+10*(J-1)+5),'(ES10.3)')           &
            DayMax1Dose(I,J)                                         ! Write var to val array   
         IF (F(26) .AND. Pol(J)%DoDose==YES)                        & 
            WRITE(DVals(NUMDAYVARS+10*(J-1)+6),'(ES10.3)')           &     
            DayMax8Dose(I,J)                                         ! Write var to val array   
         IF (F(27) .AND. Pol(J)%DoDose==YES)                        & 
            WRITE(DVals(NUMDAYVARS+10*(J-1)+7),'(ES10.3)')           &   
            DayMax1FDose(I,J)                                        ! Write var to val array   
         IF (F(28) .AND. Pol(J)%DoDose==YES.AND.Pol(J)%PM==YES)     & 
            WRITE(DVals(NUMDAYVARS+10*(J-1)+8),'(ES10.3)')           &   
            DayIntake(I,J)                                           ! Write var to val array   
         IF (F(29) .AND. Pol(J)%DoDose==YES.AND.Pol(J)%PM==YES)     & 
            WRITE(DVals(NUMDAYVARS+10*(J-1)+9),'(ES10.3)')           &   
            DayDep(I,J)                                              ! Write var to val array   
         IF(F(30)) WRITE(DVals(NUMDAYVARS+10*(J-1)+10),'(ES10.3)')    & 
            DayMax8EC(I,J)                                           ! Write var to val array
      ENDDO                                                          ! End loop over pollutants  
      WRITE(DayU,'(100A)') (TRIM(  DVals(J)), J=1,NVars)             ! Write final value array to file
   ENDDO                                                             ! End loop over days
   IF (DebugLevel>1) WRITE(LogU,SL_FM2) LV_MS2, ProcName(ProcLevel)  ! Message for debugging
   ProcLevel = ProcLevel-1                                           ! Decrement procedure depth
END SUBROUTINE WriteDailyFile                                        ! Return to Output subroutine

END MODULE OutputModule                                              ! End of source code for module