MODULE ProfileModule                                                 ! Compile after the modules below
   ! Latest revisions by MCN on November 9, 2011
   USE GlobalModule                                                  ! Compile this module first
   USE UtilityModule                                                 ! Compile this module second
   USE DistributionModule                                            ! Compile this module third
   IMPLICIT NONE                                                     ! Must declare all variables
   INTEGER (KIND=IK),ALLOCATABLE :: PopCounts(:,:,:)                 ! Sector-age populations
   REAL    (KIND=RK),ALLOCATABLE :: PopProbs(:,:,:)                  ! Sector-age cumul. probabilities
   INTEGER (KIND=IK),ALLOCATABLE :: PopAges(:,:)                     ! Ages for population groups
   INTEGER (KIND=IK),ALLOCATABLE :: PopGroup(:)                      ! Population group by age 
   INTEGER (KIND=IK),ALLOCATABLE :: CommWork(:)                      ! CommProb - Work sector mapping
   INTEGER (KIND=IK),ALLOCATABLE :: CommHome(:)                      ! Home sector - CommProb mapping
   REAL    (KIND=RK),ALLOCATABLE :: CommProb(:)                      ! Work sector probabilities
   REAL    (KIND=RK),ALLOCATABLE :: CommTimeBins(:)                  ! Commuting Time Bins
   REAL    (KIND=RK),ALLOCATABLE :: CommDists(:)                     ! Commuting Distances
   REAL    (KIND=RK),ALLOCATABLE :: CumDistances(:,:)                ! Cumulative probs for comm distances   
   REAL    (KIND=RK),ALLOCATABLE :: CumDistProbs(:,:)                ! Cumulative probs for comm distances      
   REAL    (KIND=RK),ALLOCATABLE :: CumTimes(:,:)                    ! Cumulative probs for comm times      
   REAL    (KIND=RK),ALLOCATABLE :: DestFracIn(:)                    ! Worker fractions in study area 
   INTEGER (KIND=IK),ALLOCATABLE :: EmpGroup(:,:)                    ! Employment group by age, gender
   REAL    (KIND=RK),ALLOCATABLE :: EmpProbs(:,:)                    ! Employment probabilities
   INTEGER (KIND=IK),ALLOCATABLE :: DisGroup(:,:)                    ! Disease prevalence by age, gender
   REAL    (KIND=RK),ALLOCATABLE :: DisProbs(:,:)                    ! Disease probabilities
   INTEGER (KIND=IK),ALLOCATABLE :: ProGroup(:,:)                    ! Profile factor group by age, gender
   REAL    (KIND=RK),ALLOCATABLE :: ProProbs(:,:,:)                  ! Profile factor probabilities
   INTEGER (KIND=IK)             :: NumPopAges = 0                   ! Number of population age groups
   TYPE (CDist),        TARGET,ALLOCATABLE :: VE1(:,:)               ! FEV E1-term model parameter
   TYPE (CDist),        TARGET,ALLOCATABLE :: VE2(:,:)               ! FEV E2-term model parameter       
CONTAINS                                                             ! Internal subroutines  

SUBROUTINE ReadPopulation                                            ! Called from main APEX program
   INTEGER  (KIND=IK)   :: I, J, K, LCount, SCount, Ltype, Ages      ! Local integer variables
   INTEGER  (KIND=IK)   :: RangeCount = 0                            ! Count of persons in age bounds
   INTEGER  (KIND=IK)   :: PrevIndex = 0                             ! Loc of sector in pop file
   INTEGER  (KIND=IK)   :: OutofOrder = 0                            ! Sectors "out-of-order" flag
   INTEGER  (KIND=IK), ALLOCATABLE :: Buff(:,:)                      ! Buffer for age inputs
   CHARACTER(LEN=LEN2)  :: Name                                      ! Variable for sectors IDs 
   CHARACTER(LEN=LEN4)  :: String                                    ! Buffer for line from input file
   CHARACTER(LEN=1)     :: Gender                                    ! Buffer for gender on file header
   CHARACTER(LEN=5)     :: Race                                      ! Buffer for race on file header
   REAL     (KIND=LONG) :: RPop                                      ! Local real variable
   ! Global constants: IK, LEN2, LEN3, LEN4, LogU, LONG, LV_FMT, LV_MSG, NO, PopU, ST_FMT, ST_MSG, YES
   ! Global vars used: AgeMin, AgeMax, CommuteFlag, DebugLevel, NumSectors, NumPopTypes, PopCat, SList
   ! Global vars set : Ios, ProcLevel, ProcName, TotPop
   ! Module vars set : DestFracIn, NumPopAges, PopAges, PopCounts, PopGroup, PopProbs
   ! Intrinsic procs : ADJUSTL, ALLOCATE, ANY, CLOSE, DEALLOCATE, INDEX, MAX, PRINT, 
   !                   READ, REAL, SCAN, SUM, TRIM, WRITE 
   ! APEX procedures : CheckAllocation, Fatal, GetLine, OpenInput, ReadCommute, Up, Warning
   ProcLevel = ProcLevel+1                                           ! Increment procedure depth
   ProcName(ProcLevel) = "ReadPopulation"                            ! Procedure name for messages
   IF (DebugLevel>0) WRITE(LogU,ST_FMT) ST_MSG, ProcName(ProcLevel)  ! Message for debugging
   ALLOCATE (DestFracIn(NumSectors),STAT=AllocErr)                   ! Space for worker fractions
   CALL CheckAllocation(AllocErr)                                    ! Check the success of the alloc
   DestFracIn(:) = 1                                                 ! Default is work at home
   IF (CommuteFlag==YES) CALL ReadCommute                            ! Read commuting data if needed
   IF (CommuteFlag==YES) CALL ReadCommuteTimes                       ! Read commuting time data if needed
   DO I=1,NumPopTypes                                                ! Loop over gender-race combs.
      CALL OpenInput(PopU,LEN4,PopCat(I)%FName,preblank=YES,h=NO)    ! Open population input file
      CALL GetLine(PopU,String,Ltype)                                ! Get next data line from input
      String = String(INDEX(String,"=")+1:)                          ! Drop the keywords 
      READ(String,*,IOSTAT=Ios(PopU)) Gender, Race, Ages             ! Read data values
      IF (I==1) THEN                                                 ! For first population file only 
         NumPopAges = Ages                                           ! Set # of age groups
         IF (NumPopAges<=0) CALL Fatal(1)                            ! Must have at least one group
         ALLOCATE(PopCounts(NumPopTypes,0:NumSectors,-1:NumPopAges),&
            STAT=AllocErr)                                           ! Declare list of populations 
         CALL CheckAllocation(AllocErr)                              ! Check the success of the alloc
         ALLOCATE(PopProbs(NumPopTypes,0:NumSectors,-1:NumPopAges), &
            STAT=AllocErr)                                           ! Declare list of probabilities
         CALL CheckAllocation(AllocErr)                              ! Check the success of the alloc
         ALLOCATE(PopAges(NumPopAges,2), Buff(NumPopAges,2),        &
            STAT=AllocErr)                                           ! Min and max ages for each group
         CALL CheckAllocation(AllocErr)                              ! Check the success of the alloc
         PopCounts = 0                                               ! Initialize populations to zero
         PopProbs  = 0.                                              ! Initialize probabilities to zero
      ENDIF                                                          ! End of first file logic block
      IF (Up(Gender)/=Up(PopCat(I)%Gender(1:1))) CALL Fatal(2)       ! Gender must match params file
      IF (len(race)  <5   ) CALL Fatal(13)                           ! Race must have at lest 5 letters
      IF (Up(Race)  /=Up(PopCat(I)%Race  )) CALL Fatal(3)            ! Race must match params file
      IF (Ages      /=NumPopAges          ) CALL Fatal(4)            ! # age groups must match 
      CALL GetLine(PopU,String,Ltype)                                ! Read next data line from input 
      String = String(INDEX(String,"=")+1:)                          ! Drop the keywords 
      READ(String,FMT3,IOSTAT=Ios(PopU)) PopCat(I)%RaceDesc          ! Read race description
      WRITE(LogU,'(I3,2A10,5X,A60)')I,Gender,Race,PopCat(I)%RaceDesc ! Write gender and race to log 
      CALL GetLine(PopU,String,Ltype)                                ! Read next data line from input 
      String = String(INDEX(String,"=")+1:)                          ! Drop the keywords   
      READ(String,*,IOSTAT=Ios(PopU)) Buff(:,1)                      ! Read data for minimum ages
      CALL GetLine(PopU,String,Ltype)                                ! Read next data line from input 
      String = String(INDEX(String,"=")+1:)                          ! Drop the keywords    
      READ(String,*,IOSTAT=Ios(PopU)) Buff(:,2)                      ! Read data for maximum ages
      IF (I==1) THEN                                                 ! For first population file only 
         PopAges(:,:) = Buff(:,:)                                    ! Copy buffer to module array 
         IF (PopAges(1,1)   >AgeMin) CALL Warning(1)                 ! If lower age > AgeMin, warn
         IF (PopAges(Ages,2)<AgeMax) CALL Warning(2)                 ! If top age   < AgeMax, warn
         WRITE(LogU,*)                                               ! Put blank line on log
         WRITE(LogU,*)'Min Age on Population File =',PopAges(1,1)    ! Minimum age with pop data
         WRITE(LogU,*)'Max Age on Population File =',PopAges(Ages,2) ! Maximum age with pop data
         WRITE(LogU,*)'Min Age for Profiles       =',AgeMin          ! Minimum profile age
         WRITE(LogU,*)'Max Age for Profiles       =',AgeMax          ! Maximum profile age
         ALLOCATE (PopGroup(0:PopAges(Ages,2)),STAT=AllocErr)        ! Population group for each age
         CALL CheckAllocation(AllocErr)                              ! Check the success of the alloc
         PopGroup = -1                                               ! Ages without data have zero pop
         WRITE(LogU,'(/A31)') ' Age Groups on Population Files'      ! Header for table on log file
         WRITE(LogU,*) '      Group#      Min Age     Max Age'       ! Column descriptors for table
         DO J=1, Ages                                                ! Loop over population groups
            PopGroup(PopAges(J,1):PopAges(J,2)) = J                  ! Assign group # to ages in range
            WRITE(LogU,*) J, PopAges(J,1), PopAges(J,2)              ! Population age group ranges
         ENDDO                                                       ! Continue with next pop group 
         IF(ANY(PopAges(:,1)>PopAges(:,2))) CALL Fatal(5)            ! Stop if max less than min
         IF(ANY(PopAges(2:,1)<PopAges(:Ages-1,2)+1)) CALL Fatal(6)   ! Stop if groups overlap
         IF(ANY(PopAges(2:,1)>PopAges(:Ages-1,2)+1)) CALL Warning(3) ! Warn if groups have gaps
      ELSE                                                           ! Else do if not first pop file
         IF (ANY(PopAges(:,:)/=Buff(:,:))) CALL Fatal(7)             ! Check if ages match first file
      ENDIF                                                          ! End of logic block
      LCount    = 0                                                  ! Initialize line counter
      SCount    = 0                                                  ! Initialize sector counter
      PrevIndex = 0                                                  ! Intlz sector location in file       
      OutOfOrder= 0                                                  ! Initialize out-of-order flag   
      DO                                                             ! Loop over input records
         CALL GetLine(PopU,String,Ltype)                             ! Get one line from input file
         IF (Ios(PopU)<0) CALL Fatal(100+I)                          ! EOF but not all sectors found
         LCount = LCount+1                                           ! Increment line counter 
         READ(String,*,IOSTAT=Ios(PopU)) Name                        ! Read sector ID from input file
         DO J=1,NumSectors                                           ! Loop over sectors in study area
            IF (SList(J)%Name==Name) THEN                            ! If sector names match
               IF (J<PrevIndex) OutofOrder = 1                       ! Sectors out of order in file  
               SCount = SCount+1                                     ! Increment sector count
               PrevIndex = J                                         ! Reset the location index 
               EXIT                                                  ! If name matches, continue
            ENDIF                                                    ! End matching sector logic   
         ENDDO                                                       ! END loop over sectors            
         IF (J<=NumSectors) THEN                                     ! If sector name is found
            READ(String,*,IOSTAT=Ios(PopU))                         &    
               Name,PopCounts(I,SCount,1:)                           ! Get ID and counts from input
            IF (Ios(PopU)/=0) CALL Fatal(9)                          ! If not enough numbers on line
            IF (ANY(PopCounts(I,SCount,1:)<0)) CALL Fatal(10)        ! Stop if negative population 
            PopCounts(I,SCount,0) = SUM(PopCounts(I,SCount,1:))      ! Calculate sums over ages
            IF (SCount==NumSectors) EXIT                             ! Exit DO when last sector found
         ENDIF 
      ENDDO                                                          ! Otherwise continue reading lines
      CLOSE(PopU)                                                    ! Close current pop input file
      If (OutofOrder==1) Call Warning(100+I)                         ! Call sec out-of-order warning  
      PopCounts(I,0,0) = SUM(PopCounts(I,1:,0))                      ! Sum over sectors and ages
      DO J=1,NumPopAges                                              ! For each agegroup
         PopCounts(I,0,J) = SUM(PopCounts(I,1:,J))                   ! Calculate total across sectors
         IF (AgeMin<=PopAges(J,2) .AND. AgeMax>=PopAges(J,1))       &
            RangeCount = RangeCount + PopCounts(I,0,J)               ! Count people who might qualify 
      ENDDO                                                          ! Continue with next age group
   ENDDO                                                             ! Continue with next pop type
   TotPop = SUM(PopCounts(:,0,0))                                    ! Population universe
   WRITE(LogU,'(/A16,I10)') "Area population=",TotPop                ! Write pop total to log file
   IF (RangeCount == 0) CALL Fatal(12)                               ! No persons to be modeled
   RPop = REAL(MAX(TotPop,1))                                        ! Ensure denominator >=1 
   PopProbs(:,0,0) = REAL(PopCounts(:,0,0))/RPop                     ! Convert type-totals to fractions
   DO I=2,NumPopTypes                                                ! Loop over type-totals vector
      PopProbs(I,0,0) = PopProbs(I-1,0,0)+PopProbs(I,0,0)            ! Convert to cumulative probab.
   ENDDO                                                             ! Do next element in pop vector
   DO I=1,NumPopTypes                                                ! Loop over population types
      RPop = REAL(MAX(PopCounts(I,0,0),1))                           ! Ensure denominator >=1 
      PopProbs(I,1:,0) = REAL(PopCounts(I,1:,0))/RPop                ! Normalize sector totals by type
      DO J=2,NumSectors                                              ! Loop over sectors
         PopProbs(I,J,0) = PopProbs(I,J-1,0)+PopProbs(I,J,0)         ! Convert to cumulative probabs
      ENDDO                                                          ! Continue with next sector
      DO J=1,NumSectors                                              ! Loop over sectors
         RPop = REAL(MAX(PopCounts(I,J,0),1))                        ! Ensure denominator >=1 
         PopProbs(I,J,1:) = REAL(PopCounts(I,J,1:))/RPop             ! Normalize age totals by type
         DO K=2,NumPopAges                                           ! Loop over agegroups
            PopProbs(I,J,K) = PopProbs(I,J,K)+PopProbs(I,J,K-1)      ! Convert to cumulative probabs
         ENDDO                                                       ! Continue with next agegroup
      ENDDO                                                          ! Continue with next sector
   ENDDO                                                             ! Continue with next pop type
   DEALLOCATE (Buff)                                                 ! Release space for local buffer
   PRINT*,'Finished ReadPopulation'                                  ! Status message for screen
   IF (DebugLevel>0) WRITE(LogU,LV_FMT) LV_MSG, ProcName(ProcLevel)  ! Message for debugging
   ProcLevel = ProcLevel-1                                           ! Decrement procedure depth
END SUBROUTINE ReadPopulation                                        ! Return to main APEX program


SUBROUTINE ReadCommute                                               ! Called from ReadPopulation
   INTEGER   (KIND=IK),PARAMETER   :: MEANDESTS=3000                 ! Mean # of dests per home sector
   INTEGER   (KIND=IK)             :: I, Keep=0, DNum=0, Home=0, N, V   ! Local integer variables 
   REAL      (KIND=RK)             :: Frac, DFrac, Dist              ! Local real variables
   CHARACTER (LEN=LEN2)            :: ID                             ! Local variable for sector name
   INTEGER   (KIND=IK),ALLOCATABLE :: WBuf(:)                        ! Temporary buffer for Work IDs
   REAL      (KIND=RK),ALLOCATABLE :: PBuf(:)                        ! Temporary buffer for probabs
   REAL      (KIND=RK),ALLOCATABLE :: DBuf(:)                        ! Temporary buffer for work distances
   REAL      (KIND=RK),ALLOCATABLE :: SortedDBuf(:)                  ! Sorted buffer for work distances 
   REAL      (KIND=RK),ALLOCATABLE :: SortedPBuf(:)                  ! Sorted buffer for probabs     
   REAL      (KIND=RK)             :: TmpDist, TmpProb, Last         ! Temporary variables for sorting
   INTEGER   (KIND=IK)             :: DistLoc, Count, CCount         ! Temporary variable for sorting
   ! Global constants: CommU, IK, LEN2, LV_FMT, LV_MSG, NO, RK, ST_FMT, ST_MSG
   ! Global vars used: DebugLevel, NumSectors, SList
   ! Global vars set : Ios, ProcLevel, ProcName
   ! Module vars set : CommHome, CommProb, CommWork, DestFracIn 
   ! Intrinsic procs : ALLOCATE, CLOSE, DEALLOCATE, PRINT, READ, WRITE
   ! APEX procedures : CheckAllocation, OpenInput, Fatal
   ProcLevel = ProcLevel+1                                           ! Increment procedure depth
   ProcName(ProcLevel) = "ReadCommute"                               ! Procedure name for messages
   IF (DebugLevel>0) WRITE(LogU,ST_FMT) ST_MSG, ProcName(ProcLevel)  ! Message for debugging
   PRINT*, 'Starting ReadCommute'                                    ! Status message for screen
   ALLOCATE (CommHome(NumSectors),STAT=AllocErr)                     ! Space for home indexing vector
   CALL CheckAllocation(AllocErr)                                    ! Check the success of the alloc
   ALLOCATE (PBuf(NumSectors*MEANDESTS),STAT=AllocErr)               ! Space for temporary buffers 
   CALL CheckAllocation(AllocErr)                                    ! Check the success of the alloc
   ALLOCATE (WBuf(NumSectors*MEANDESTS),STAT=AllocErr)               ! Space for temporary buffers
   CALL CheckAllocation(AllocErr)                                    ! Check the success of the alloc
   ALLOCATE (DBuf(NumSectors*MEANDESTS),STAT=AllocErr)               ! Space for temporary buffers
   CALL CheckAllocation(AllocErr)                                    ! Check the success of the alloc   
   ALLOCATE (SortedDBuf(NumSectors*MEANDESTS),STAT=AllocErr)         ! Space for temporary buffers
   CALL CheckAllocation(AllocErr)                                    ! Check the success of the alloc
   ALLOCATE (SortedPBuf(NumSectors*MEANDESTS),STAT=AllocErr)         ! Space for temporary buffers
   CALL CheckAllocation(AllocErr)                                    ! Check the success of the alloc      
   ALLOCATE (CumDistances(NumSectors,MEANDESTS),STAT=AllocErr)       ! Space for temporary buffers
   CALL CheckAllocation(AllocErr)                                    ! Check the success of the alloc         
   ALLOCATE (CumDistProbs(NumSectors,MEANDESTS),STAT=AllocErr)       ! Space for temporary buffers
   CALL CheckAllocation(AllocErr)                                    ! Check the success of the alloc     
   CommHome   = 0                                                    ! Initialize all values to zero
   WBuf       = 0                                                    ! Initialize all values to zero
   PBuf       = 0.0                                                  ! Initialize all values to zero
   DBuf       = 0.0                                                  ! Initialize all values to zero
   DestFracIn = 0.0                                                  ! Initialize fraction work inside 
   CALL OpenInput(CommU,h=NO)                                        ! Open commuting input file
   DO                                                                ! Loop over input records
      READ(CommU,*,IOSTAT=Ios(CommU)) ID, Frac, Dist                 ! Read one line from input file
      IF (Ios(CommU)/=0) EXIT                                        ! Exit DO loop on end of file 
      IF (Keep==1) THEN                                              ! If already in a KEEP block...
         DO I=NumSectors,1,-1                                        ! Loop backward down sectors list
            IF (SList(I)%Name==ID) EXIT                              ! Exit loop if sector ID found
         ENDDO                                                       ! Try next sector in study area
         DNum = DNum+1                                               ! Increment destination counter
         Count = Count + 1                                           ! Incrementing destination num
         IF (DNum>MEANDESTS*NumSectors) Call Fatal(1)                ! Too many destinations in file
         DFrac = Frac                                                ! Delta for fraction of workers
         IF (DNum>CommHome(Home)) DFrac = DFrac-PBuf(DNum-1)         ! If past first one, subtract last 
         IF (I==0) DFrac = 0.0                                       ! If sector not on list, reset 
         DestFracIn(Home) = DestFracIn(Home)+DFrac                   ! Add fraction to accumulator
         PBuf(DNum) = Frac                                           ! Probability of this destination
         WBuf(DNum) = I                                              ! Destination sector number
         DBuf(DNum) = Dist                                           ! Distance of sector
         IF (Frac>=1.0) THEN                                         ! Frac=1 means last dest in block
            Keep=0                                                   ! Setting back to zero at end of block
            Last=0                                                   ! Initializing cumulative counter
            DO N = 1,Count                                           ! Cycling through all destinations
                SortedDBuf(N) = DBuf(N+DNum-Count)                   ! Assigning new cum dist array
                SortedPBuf(N) = PBuf(N+DNum-Count)-Last              ! Assigning new cum prob array
                Last=PBuf(N+DNum-Count)                              ! Incrementing cum value
            END DO                                                   ! Ending assignment logic
            DO N = 1,Count                                           ! Sorting algorithm
                DistLoc = MINLOC(SortedDBuf(N:Count),dim=1)          ! Finding minimum
                IF (N.NE.DistLoc+N-1) THEN                           ! If minimum not at current place
                    TmpDist = SortedDBuf(N)                          ! Assign current dist to tmp variable
                    TmpProb = SortedPBuf(N)                          ! Assign current prob to tmp variable
                    SortedDBuf(N) = SortedDBuf(DistLoc+N-1)          ! Move min to current
                    SortedPBuf(N) = SortedPBuf(DistLoc+N-1)          ! Move min to current
                    SortedDBuf(DistLoc+N-1) = TmpDist                ! Move current to min
                    SortedPBuf(DistLoc+N-1) = TmpProb                ! Move current to min
                ENDIF                                                ! End sorting
                IF (N == Count) THEN                                 ! If on last value
                    Last=0                                           ! Initializing cum variable
                    DO V = 1,Count                                   ! Cycling through all destinations
                        SortedPBuf(V) = SortedPBuf(V) + Last         ! Making distribution cumulative
                        Last=SortedPBuf(V)                           ! Incrementing cumulative value
                    END DO                                           ! Ending cumulative logic 
                    CCount = 1                                       ! Initializing cumulative dist counter
                    DO V = 1,Count                                   ! Cycling over all destinations
                        CumDistProbs(Home,CCount) = SortedPBuf(V)    ! Assigning probability
                        CumDistances(Home,CCount) = SortedDBuf(V)    ! Assigning distance
                        IF (V .NE. Count) THEN                       ! If not at last one
                            IF (SortedDBuf(V) .NE. SortedDBuf(V+1)) THEN ! Only if dists are different
                                CCount = CCount+1                    ! Should you increment the array
                            ENDIF                                    ! Ending dists logic
                        ENDIF                                        ! Ending logic
                    END DO                                           ! Ending loop over all counts
                ENDIF                                                ! Ending last count logic
            ENDDO                                                    ! End loop through counts
         ENDIF                                                       ! End last destination logic
      ELSEIF (Frac>=0.0) THEN                                        ! Dests for sectors outside study
         CYCLE                                                       ! Skip over them
      ELSE                                                           ! Else the start of a new block
         DO I=1,NumSectors                                           ! Loop over sectors in study area
            IF (Slist(I)%Name==ID) THEN                              ! If sector name is found ...
               CommHome(I) = DNum+1                                  ! Set the sector indexing vector
               Keep = 1                                              ! Keep this block of destinations
               Home = I                                              ! Set home sector value
               Count = 0                                             ! Count of each destination in sector
               EXIT                                                  ! Stop searching home sector list
            ENDIF                                                    ! End of matching ID logic
         ENDDO                                                       ! Continue with next home sector
      ENDIF                                                          ! End of input processing logic
   ENDDO                                                             ! Continue with next input record
   CLOSE(CommU)                                                      ! Close the commuting data file
   DO I=1,NumSectors-1                                               ! Loop over home sectors
      IF (CommHome(I)==CommHome(I+1)) CommHome(I)=0                  ! If sector has no dests, re-index
   ENDDO                                                             ! Continue with next home sector
   IF (CommHome(NumSectors)>DNum) CommHome(NumSectors)=0             ! This means last sector was empty
   ALLOCATE (CommProb(DNum),STAT=AllocErr)                           ! Set proper size for Probab vector
   CALL CheckAllocation(AllocErr)                                    ! Check the success of the alloc
   ALLOCATE (CommWork(DNum),STAT=AllocErr)                           ! Set proper size for WorkID vector 
   CALL CheckAllocation(AllocErr)                                    ! Check the success of the alloc
   ALLOCATE (CommDists(DNum),STAT=AllocErr)                          ! Set proper size for WorkID vector 
   CALL CheckAllocation(AllocErr)                                    ! Check the success of the alloc   
   CommProb = PBuf(1:DNum)                                           ! Copy probability data 
   CommWork = WBuf(1:DNum)                                           ! Copy work destination list
   CommDists = DBuf(1:DNum)                                          ! Copy work distances list   
   DEALLOCATE (WBuf, PBuf, DBuf, SortedPBuf, SortedDBuf)             ! Erase temporary vectors   
   PRINT*, 'Finished ReadCommute'                                    ! Status message for screen
   IF (DebugLevel>0) THEN                                            ! If debugging
      WRITE (LogU,*) "NumSectors*MEANDESTS=", NumSectors*MEANDESTS, &
         " Final commuting array size= ",DNum                        ! Print size of commuting arrays
      IF (DebugLevel>0)WRITE(LogU,LV_FMT) LV_MSG,ProcName(ProcLevel) ! Message for debugging
   ENDIF                                                             ! End debugging logic                 
   ProcLevel = ProcLevel-1                                           ! Decrement procedure depth
END SUBROUTINE ReadCommute                                           ! Return to ReadPopulation


SUBROUTINE ReadCommuteTimes                                          ! Called from ReadPopulation
   CHARACTER (LEN=LEN2)            :: ID                             ! Local variable for sector name   
   CHARACTER (LEN=LEN2)            :: Name                           ! Variable for sectors IDs 
   INTEGER   (KIND=IK)             :: I, N                           ! More local integer variables
   REAL      (KIND=RK),ALLOCATABLE :: CommBins(:)                    ! Sorted buffer for probabs   
   REAL      (KIND=RK)             :: Last                           ! Local temporary variable 
   ALLOCATE (CommBins(15),STAT=AllocErr)                             ! Space for temporary buffers
   CALL CheckAllocation(AllocErr)                                    ! Check the success of the alloc  
   ! Global constants: CommTU, IK, LEN2, LV_FMT, LV_MSG, NO, RK, ST_FMT, ST_MSG
   ! Global vars used: DebugLevel, NumSectors, SList
   ! Global vars set : Ios, ProcLevel, ProcName
   ! Module vars set : CumTimes
   ! Intrinsic procs : ALLOCATE, CLOSE, DEALLOCATE, PRINT, READ, WRITE
   ! APEX procedures : CheckAllocation, OpenInput, Fatal
   ProcLevel = ProcLevel+1                                           ! Increment procedure depth
   ProcName(ProcLevel) = "ReadCommuteTimes"                          ! Procedure name for messages
   IF (DebugLevel>0) WRITE(LogU,ST_FMT) ST_MSG, ProcName(ProcLevel)  ! Message for debugging
   PRINT*, 'Starting ReadCommuteTimes'                               ! Status message for screen
   ALLOCATE (CumTimes(NumSectors,15),STAT=AllocErr)                  ! Set proper size for bin array
   CALL CheckAllocation(AllocErr)                                    ! Check the success of the alloc   
   CALL OpenInput(CommTU,h=NO)                                       ! Open commuting input file
   DO                                                                ! Loop over input records
      READ(CommTU,*,IOSTAT=Ios(CommTU)) ID, CommBins(1:15)           ! Reading in line
      IF (Ios(CommTU)<0) EXIT                                        ! Exit DO loop when file ends
      DO I=1,NumSectors                                              ! Loop over sectors in study area
         IF (Slist(I)%Name==ID) THEN                                 ! If sector name is found ...
            Last = 0                                                 ! Initializing cumulative var
            CumTimes(I,1) = CommBins(1)                              ! Saving cumulative bin 1
            CumTimes(I,2) = CommBins(2)                              ! Saving cumulative bin 2
            CumTimes(I,15) = CommBins(15)                            ! Saving cumulative bin 3
            DO N = 3,15                                              ! Loop over all bins
                IF (CommBins(1) > 0) THEN                            ! If there are commuters in the tract
                    IF (N==3) THEN                                   ! Rearranging time bins 15->3
                       CumTimes(I,N) = CommBins(15)/CommBins(1)      ! Moving 15 (commutetime = 0)->3
                    ELSE                                             ! If not the first one...
                       CumTimes(I,N) = CommBins(N-1)/CommBins(1) + Last ! Write bins to cumulative array
                    ENDIF                                            ! End rearranging/assignment
                ELSE                                                 ! Else if no commuters
                    CumTimes(I,N) = (N-2)/12                         ! Assign equal prob to each bin
                ENDIF                                                ! Ending bin assignment
                Last = CumTimes(I,N)                                 ! Making bins cumulative          
            ENDDO                                                    ! End loop over bins
            EXIT                                                     ! Stop searching home sector list
         ENDIF                                                       ! End of matching ID logic
      ENDDO                                                          ! Continue with next home sector
   ENDDO                                                             ! Otherwise continue reading lines
   CLOSE(CommTU)                                                     ! Close the commuting data file
   DEALLOCATE (CommBins)                                             ! Erase temporary vectors
   PRINT*, 'Finished ReadCommuteTimes'                               ! Status message for screen
   IF (DebugLevel>0) THEN                                            ! If debugging
      IF (DebugLevel>0)WRITE(LogU,LV_FMT) LV_MSG,ProcName(ProcLevel) ! Message for debugging
   ENDIF                                                             ! End debugging logic  
   ProcLevel = ProcLevel-1                                           ! Decrement procedure depth
END SUBROUTINE ReadCommuteTimes                                      ! Return to ReadPopulation


SUBROUTINE ReadEmployment                                            ! Called from main APEX program 
   CHARACTER(LEN=1)    :: Gender(LEN4)                               ! Buffer for genders on header
   INTEGER (KIND=IK)   :: Buf1(LEN4),Buf2(LEN4)                      ! Buffers for integer input data
   CHARACTER(LEN=LEN4) :: String                                     ! Buffer for line from input file
   CHARACTER(LEN=LEN2) :: Name                                       ! Variable for sectors IDs 
   INTEGER (KIND=IK)   :: Ltype, Cols, I, J, Sex                     ! Local integer variables   
   INTEGER (KIND=IK)   :: LCount, Scount                             ! More local integer variables
   ! Global constants: EmpU, IK, LEN2, LEN4, LogU, LV_FMT, LV_MSG, NO, SEXTEXT, ST_FMT, ST_MSG
   ! Global vars used: DebugLevel, NumSectors, SList
   ! Global vars set : Ios, ProcLevel, ProcName
   ! Module vars used: NumPopAges, PopAges
   ! Module vars set : EmpGroup, EmpProbs 
   ! Intrinsic procs : ALLOCATE, ANY, CLOSE, COUNT, INDEX, MIN, PRINT, READ, WRITE
   ! APEX procedures : CheckAllocation, Fatal, GetLine, OpenInput, Up
   ProcLevel = ProcLevel+1                                           ! Increment procedure depth
   ProcName(ProcLevel) = "ReadEmployment"                            ! Procedure name for messages
   IF (DebugLevel>0) WRITE(LogU,ST_FMT) ST_MSG, ProcName(ProcLevel)  ! Message for debugging
   CALL OpenInput(EmpU,recl=LEN4,h=NO)                               ! Open agegroups/employment file
   CALL GetLine(EmpU,String,Ltype)                                   ! Read data from next input line
   String = Up(String(INDEX(String,'=')+1:))                         ! Drop the keywords
   READ(String,*,IOSTAT=Ios(EmpU)) Gender                            ! Extract gender settings
   Cols = COUNT(Gender=="M" .OR. Gender=="F")                        ! Sum of males + females
   IF (Cols<=0) CALL Fatal(1)                                        ! Stop if no columns of data
   IF (COUNT(Gender=="M")+COUNT(Gender=="F") /= Cols) CALL Fatal(2)  ! Stop if any unknown genders
   ALLOCATE (EmpProbs(NumSectors,0:Cols),STAT=AllocErr)              ! Dynamic space for prob array 
   CALL CheckAllocation(AllocErr)                                    ! Check the success of the alloc
   ALLOCATE (EmpGroup(0:PopAges(NumPopAges,2),2),STAT=AllocErr)      ! Dynamic space for index array
   CALL CheckAllocation(AllocErr)                                    ! Check the success of the alloc
   EmpGroup = 0                                                      ! Default is all indices are zero
   EmpProbs = 0.                                                     ! Default employment prob is zero
   CALL GetLine(EmpU,String,Ltype)                                   ! Read data from next input line 
   String = Up(String(INDEX(String,'=')+1:))                         ! Drop the keywords 
   READ(String,*,IOSTAT=Ios(EmpU)) Buf1(1:Cols)                      ! Read minimum ages for groups
   CALL GetLine(EmpU,String,Ltype)                                   ! Read data from next input line 
   String = Up(String(INDEX(String,'=')+1:))                         ! Drop the keywords 
   READ(String,*,IOSTAT=Ios(EmpU)) Buf2(1:Cols)                      ! Read maximum ages for groups
   WRITE(LogU,'(/A30)') ' Age Groups on Employment File'             ! Header for table on log file
   WRITE(LogU,*) ' Group#  Gender     MinAge   MaxAge'               ! Column descriptors for table
   DO I=1, Cols                                                      ! Loop over input data columns 
      Sex = INDEX("MF",Gender(I))                                    ! Gender for this column of data
      IF (Buf2(I) < Buf1(I)) CALL Fatal(3)                           ! Stop if max age less than min 
      IF (Buf1(I) > PopAges(NumPopAges,2)) CYCLE                     ! Drop data if age above pop limit
      Buf2(I) = MIN(Buf2(I),PopAges(NumPopAges,2))                   ! Reduce max age to pop age limit
      WRITE(LogU,'(I6,4X,A6,2I9)') I, SEXTEXT(Sex), Buf1(I), Buf2(I) ! Note data values on log file
      DO J=Buf1(I),Buf2(I)                                           ! Loop over age range for column
         EmpGroup(J,Sex) = I                                         ! Set group # for these ages
      ENDDO                                                          ! Continue with next year of age
   ENDDO                                                             ! Continue with next data column
   LCount = 0                                                        ! Initialize line counter
   SCount = 1                                                        ! Initialize sector counter
   DO                                                                ! Loop over input records
      CALL GetLine(EmpU,String,Ltype)                                ! Get one line from input file
      IF (Ios(EmpU)<0) EXIT                                          ! Exit DO loop when file ends
      LCount = LCount+1                                              ! Increment line counter 
      IF (LCount < SList(SCount)%Index) CYCLE                        ! Drop data for unwanted sectors
      READ(String,*,IOSTAT=Ios(EmpU)) Name                           ! Read sector ID
      IF (Name /= SList(SCount)%Name) CALL Fatal(4)                  ! Problem if name does not match
      READ(String,*,IOSTAT=Ios(EmpU)) Name,EmpProbs(SCount,1:Cols)   ! Get ID and counts from input
      IF (Ios(PopU)/=0) CALL Fatal(5)                                ! If not enough numbers on line
      IF (ANY(EmpProbs(SCount,1:)<0.)) CALL Fatal(6)                 ! Stop if negative probability
      IF (ANY(EmpProbs(SCount,1:)>1.)) CALL Fatal(7)                 ! Stop if probability exceeds one
      SCount = SCount+1                                              ! Increment sector counter
      IF (SCount>NumSectors) EXIT                                    ! When last sector complete, exit
   ENDDO                                                             ! Otherwise continue reading lines
   CLOSE(EmpU)                                                       ! Close input file
   PRINT*, 'Finished ReadEmployment'                                 ! Status message for screen
   IF (DebugLevel>0) WRITE(LogU,LV_FMT) LV_MSG, ProcName(ProcLevel)  ! Message for debugging
   ProcLevel = ProcLevel-1                                           ! Decrement procedure depth 
END SUBROUTINE ReadEmployment                                        ! Return to main APEX program

SUBROUTINE ReadDiseaseRate                                           ! Called from main APEX program 
   CHARACTER(LEN=1)    :: Gender(LEN4)                               ! Buffer for genders on header
   INTEGER (KIND=IK)   :: Buf1(LEN4),Buf2(LEN4)                      ! Buffers for integer input data
   CHARACTER(LEN=LEN4) :: String                                     ! Buffer for line from input file
   CHARACTER(LEN=LEN2) :: Name                                       ! Variable for sectors IDs 
   INTEGER (KIND=IK)   :: Ltype, Cols, I, J, Sex                     ! Local integer variables   
   INTEGER (KIND=IK)   :: LCount, Scount                             ! More local integer variables
   ! Global constants: PrevU, IK, LEN2, LEN4, LogU, LV_FMT, LV_MSG, NO, SEXTEXT, ST_FMT, ST_MSG
   ! Global vars used: DebugLevel, NumSectors, SList
   ! Global vars set : Ios, ProcLevel, ProcName
   ! Module vars used: NumPopAges, PopAges
   ! Module vars set : EmpGroup, EmpProbs 
   ! Intrinsic procs : ALLOCATE, ANY, CLOSE, COUNT, INDEX, MIN, PRINT, READ, WRITE
   ! APEX procedures : CheckAllocation, Fatal, GetLine, OpenInput, Up
   ProcLevel = ProcLevel+1                                           ! Increment procedure depth
   ProcName(ProcLevel) = "ReadDiseaseRate"                           ! Procedure name for messages
   IF (DebugLevel>0) WRITE(LogU,ST_FMT) ST_MSG, ProcName(ProcLevel)  ! Message for debugging
   CALL OpenInput(PrevU,recl=LEN4,h=NO)                              ! Open agegroups/employment file
   CALL GetLine(PrevU,String,Ltype)                                  ! Read data from next input line
   String = Up(String(INDEX(String,'=')+1:))                         ! Drop the keywords
   READ(String,*,IOSTAT=Ios(PrevU)) Gender                           ! Extract gender settings
   Cols = COUNT(Gender=="M" .OR. Gender=="F")                        ! Sum of males + females
   IF (Cols<=0) CALL Fatal(1)                                        ! Stop if no columns of data
   IF (COUNT(Gender=="M")+COUNT(Gender=="F") /= Cols) CALL Fatal(2)  ! Stop if any unknown genders
   ALLOCATE (DisProbs(NumSectors,0:Cols),STAT=AllocErr)              ! Dynamic space for prob array 
   CALL CheckAllocation(AllocErr)                                    ! Check the success of the alloc
   ALLOCATE (DisGroup(0:PopAges(NumPopAges,2),2),STAT=AllocErr)      ! Dynamic space for index array
   CALL CheckAllocation(AllocErr)                                    ! Check the success of the alloc
   DisGroup = 0                                                      ! Default is all indices are zero
   DisProbs = 0.                                                     ! Default employment prob is zero
   CALL GetLine(PrevU,String,Ltype)                                  ! Read data from next input line 
   String = Up(String(INDEX(String,'=')+1:))                         ! Drop the keywords 
   READ(String,*,IOSTAT=Ios(PrevU)) Buf1(1:Cols)                     ! Read minimum ages for groups
   CALL GetLine(PrevU,String,Ltype)                                  ! Read data from next input line 
   String = Up(String(INDEX(String,'=')+1:))                         ! Drop the keywords 
   READ(String,*,IOSTAT=Ios(PrevU)) Buf2(1:Cols)                     ! Read maximum ages for groups
   WRITE(LogU,'(/A38)') ' Age Groups on Disease Prevalence File'     ! Header for table on log file
   WRITE(LogU,*) ' Group#  Gender     MinAge   MaxAge'               ! Column descriptors for table
   DO I=1, Cols                                                      ! Loop over input data columns 
      Sex = INDEX("MF",Gender(I))                                    ! Gender for this column of data
      IF (Buf2(I) < Buf1(I)) CALL Fatal(3)                           ! Stop if max age less than min 
      IF (Buf1(I) > PopAges(NumPopAges,2)) CYCLE                     ! Drop data if age above pop limit
      Buf2(I) = MIN(Buf2(I),PopAges(NumPopAges,2))                   ! Reduce max age to pop age limit
      WRITE(LogU,'(I6,4X,A6,2I9)') I, SEXTEXT(Sex), Buf1(I), Buf2(I) ! Note data values on log file
      DO J=Buf1(I),Buf2(I)                                           ! Loop over age range for column
         DisGroup(J,Sex) = I                                         ! Set group # for these ages
      ENDDO                                                          ! Continue with next year of age
   ENDDO                                                             ! Continue with next data column
   LCount = 0                                                        ! Initialize line counter
   SCount = 1                                                        ! Initialize sector counter
   DO                                                                ! Loop over input records
      CALL GetLine(PrevU,String,Ltype)                               ! Get one line from input file
      IF (Ios(PrevU)<0) EXIT                                         ! Exit DO loop when file ends
      LCount = LCount+1                                              ! Increment line counter 
      IF (LCount < SList(SCount)%Index) CYCLE                        ! Drop data for unwanted sectors
      READ(String,*,IOSTAT=Ios(PrevU)) Name                          ! Read sector ID
      IF (Name /= SList(SCount)%Name) CALL Fatal(4)                  ! Problem if name does not match
      READ(String,*,IOSTAT=Ios(PrevU)) Name,DisProbs(SCount,1:Cols)  ! Get ID and counts from input
      IF (Ios(PopU)/=0) CALL Fatal(5)                                ! If not enough numbers on line
      IF (ANY(DisProbs(SCount,1:)<0.)) CALL Fatal(6)                 ! Stop if negative probability
      IF (ANY(DisProbs(SCount,1:)>1.)) CALL Fatal(7)                 ! Stop if probability exceeds one
      SCount = SCount+1                                              ! Increment sector counter
      IF (SCount>NumSectors) EXIT                                    ! When last sector complete, exit
   ENDDO                                                             ! Otherwise continue reading lines
   CLOSE(PrevU)                                                      ! Close input file
   PRINT*, 'Finished ReadDiseaseRate'                                ! Status message for screen
   IF (DebugLevel>0) WRITE(LogU,LV_FMT) LV_MSG, ProcName(ProcLevel)  ! Message for debugging
   ProcLevel = ProcLevel-1                                           ! Decrement procedure depth 
END SUBROUTINE ReadDiseaseRate                                       ! Return to main APEX program


SUBROUTINE ReadProfileFactors                                        ! Called from main APEX program 
   CHARACTER(LEN=1)    :: Gender(LEN4)                               ! Buffer for genders on header
   INTEGER (KIND=IK)   :: Buf1(LEN4),Buf2(LEN4)                      ! Buffers for integer input data
   CHARACTER(LEN=LEN4) :: String                                     ! Buffer for line from input file
   CHARACTER(LEN=LEN2) :: Sector, Name                               ! Variable for group, sectors IDs 
   INTEGER (KIND=IK)   :: Ltype, Cols, I, J, K, L, M, Sex            ! Local integer variables   
   INTEGER (KIND=IK)   :: Scount, NameCount                          ! More local integer variables
   ! Global constants: ProU, IK, LEN2, LEN4, LogU, LV_FMT, LV_MSG, NO, SEXTEXT, ST_FMT, ST_MSG
   ! Global vars used: DebugLevel, NumSectors, SList
   ! Global vars set : Ios, ProcLevel, ProcName
   ! Module vars used: NumPopAges, PopAges
   ! Module vars set : EmpGroup, EmpProbs 
   ! Intrinsic procs : ALLOCATE, ANY, CLOSE, COUNT, INDEX, MIN, PRINT, READ, WRITE
   ! APEX procedures : CheckAllocation, Fatal, GetLine, OpenInput, Up
   ProcLevel = ProcLevel+1                                           ! Increment procedure depth
   ProcName(ProcLevel) = "ReadProfileFactors"                        ! Procedure name for messages
   IF (DebugLevel>0) WRITE(LogU,ST_FMT) ST_MSG, ProcName(ProcLevel)  ! Message for debugging
   CALL OpenInput(ProU,recl=LEN4,h=NO)                               ! Open agegroups/employment file
   CALL GetLine(ProU,String,Ltype)                                   ! Read data from next input line
   IF (Up(String(1:6)) /= "LEVELS") CALL Fatal(10)                   ! Checking Levels keyword
   String = Up(String(INDEX(String,'=')+1:))                         ! Drop the keywords
   READ(String,*,IOSTAT=Ios(ProU)) NumLevels                         ! Extract # of levels
   IF (OccFactor==YES) NumLevels = NumLevels + 1                     ! Increment to pass Group 1
   CALL GetLine(ProU,String,Ltype)                                   ! Read data from next input line
   String = Up(String(INDEX(String,'=')+1:))                         ! Drop the keywords
   READ(String,*,IOSTAT=Ios(ProU)) Gender                            ! Extract gender settings
   Cols = COUNT(Gender=="M" .OR. Gender=="F")                        ! Sum of males + females
   IF (Cols<=0) CALL Fatal(1)                                        ! Stop if no columns of data
   IF (COUNT(Gender=="M")+COUNT(Gender=="F") /= Cols) CALL Fatal(2)  ! Stop if any unknown genders
   ALLOCATE (ProProbs(NumLevels,NumSectors,0:Cols),STAT=AllocErr)    ! Dynamic space for prob array 
   CALL CheckAllocation(AllocErr)                                    ! Check the success of the alloc
   ALLOCATE (ProGroup(0:PopAges(NumPopAges,2),2),STAT=AllocErr)      ! Dynamic space for index array
   CALL CheckAllocation(AllocErr)                                    ! Check the success of the alloc
   ALLOCATE (GroupNames(NumLevels),STAT=AllocErr)                    ! Dynamic space for name array
   CALL CheckAllocation(AllocErr)                                    ! Check the success of the alloc
   ProGroup = 0                                                      ! Default is all indices are zero
   ProProbs = 0.                                                     ! Default employment prob is zero
   CALL GetLine(ProU,String,Ltype)                                   ! Read data from next input line 
   String = Up(String(INDEX(String,'=')+1:))                         ! Drop the keywords 
   READ(String,*,IOSTAT=Ios(ProU)) Buf1(1:Cols)                      ! Read minimum ages for groups
   CALL GetLine(ProU,String,Ltype)                                   ! Read data from next input line 
   String = Up(String(INDEX(String,'=')+1:))                         ! Drop the keywords 
   READ(String,*,IOSTAT=Ios(ProU)) Buf2(1:Cols)                      ! Read maximum ages for groups
   WRITE(LogU,*) 'Number of Profile Factor Groups: ', NumLevels      ! Write out # of levels
   WRITE(LogU,'(/A40)') ' Age Groups on Profile Factors File'        ! Header for table on log file
   WRITE(LogU,*) ' Group#  Gender     MinAge   MaxAge'               ! Column descriptors for table
   DO I=1, Cols                                                      ! Loop over input data columns 
      Sex = INDEX("MF",Gender(I))                                    ! Gender for this column of data
      IF (Buf2(I) < Buf1(I)) CALL Fatal(3)                           ! Stop if max age less than min 
      IF (Buf1(I) > PopAges(NumPopAges,2)) CYCLE                     ! Drop data if age above pop limit
      Buf2(I) = MIN(Buf2(I),PopAges(NumPopAges,2))                   ! Reduce max age to pop age limit
      WRITE(LogU,'(I6,4X,A6,2I9)') I, SEXTEXT(Sex), Buf1(I), Buf2(I) ! Note data values on log file
      DO J=Buf1(I),Buf2(I)                                           ! Loop over age range for column
         ProGroup(J,Sex) = I                                         ! Set group # for these ages
      ENDDO                                                          ! Continue with next year of age
   ENDDO                                                             ! Continue with next data column   
   NameCount = 0                                                      ! Initializing number of ENDs
   DO K=1, NumLevels                                                 ! Loop through all sub groups\
      IF (K==1 .AND. OccFactor==YES) THEN                            ! Dealing with unemployed group
         GroupNames(K) = "UNEMPLOYED"
         ProProbs(K,:,1:Cols) = 0                                    ! Assigned probs to zero
         NameCount = NameCount+1                                     ! Incrementing group counter
      ELSE                                                           ! If not unemployed people
         SCount = 0                                                  ! Initialize sector counter
         Name = ""                                                ! Initializing name
         DO WHILE (Name == "")                                    ! While name isn't defined
            CALL GetLine(ProU,String,Ltype)                       ! Get name line
            IF (Ios(ProU)<0) EXIT                                 ! Exit DO loop when file ends  
            READ(String,*,IOSTAT=Ios(ProU)) Name                  ! Read group name        
         ENDDO                                                    ! End name grabbing loop                                            
         IF (Up(String(1:4)) /= "NAME") CALL Fatal(11)         ! 
         String = Up(String(INDEX(String,'=')+1:))             ! Drop the keywords
         READ(String,*,IOSTAT=Ios(ProU)) GroupNames(K)         ! Extract # of levels      
         NameCount = NameCount+1
         DO
            Sector = ""
            DO WHILE (Sector == "")                                  ! While name isn't defined
               CALL GetLine(ProU,String,Ltype)                       ! Get one line from input file
               IF (Ios(ProU)<0) EXIT                                 ! Exit DO loop when file ends     
               READ(String,*,IOSTAT=Ios(ProU)) Sector                ! Read sector ID          
            ENDDO                                                    ! End name grabbing loop            
               IF (Sector == "END") EXIT
               IF (ANY(SList(:)%Name == Sector)) THEN                   ! If names match
                  SCount = SCount+1                                     ! Increment sector counter
                  IF (SCount > NumSectors) CALL Fatal(8)                ! Stop if missing an END
                  READ(String,*,IOSTAT=Ios(ProU)) Sector,ProProbs(K,SCount,1:Cols) ! Get ID and counts from input
                  IF (ANY(ProProbs(K,SCount,1:)<0.)) CALL Fatal(6)      ! Stop if negative probability
               ENDIF                                                    ! End matching code
            ENDDO
         IF (SCount /= NumSectors) CALL Fatal(7)                     ! Checking to see all sectors
      ENDIF                                                          ! Ending logic
   ENDDO                                                             ! End loop over levels
   IF (NameCount /= NumLevels) CALL Fatal(9)                         ! If ENDs don't match levels 
   DO L=1, Cols                                                      ! Looping over each column
      DO M=1, NumSectors                                             ! Looping over each sector
         ProProbs(:,M,L) = ProProbs(:,M,L)/SUM(ProProbs(:,M,L))      ! Normalizing probabilities
      ENDDO                                                          ! End sector loop
   ENDDO                                                             ! End column loop
   CLOSE(ProU)                                                       ! Close input file
   PRINT*, 'Finished ReadProfileFactors'                             ! Status message for screen
   IF (DebugLevel>0) WRITE(LogU,LV_FMT) LV_MSG, ProcName(ProcLevel)  ! Message for debugging
   ProcLevel = ProcLevel-1                                           ! Decrement procedure depth 
END SUBROUTINE ReadProfileFactors                                    ! Return to main APEX program


SUBROUTINE StudyAreaPop                                              ! Called from ReadPopulation
   INTEGER (KIND=IK) :: I,J,K, Group, Sex, Range                     ! Local integer variables
   REAL    (KIND=RK) :: Workers, WorkIn, W                           ! Local worker counts  
   REAL    (KIND=RK) :: TotalPop, CorrPop, Pop, StudyPop             ! Local population counts
   REAL    (KIND=RK) :: CumWork=0., CumIn=0., CumCorr=0.             ! Local cumulative real counts
   REAL    (KIND=RK) :: CumTotal=0., CumStudy=0.
   ! Global constants: IK, LogU, LV_FMT, LV_MSG, RK, ST_FMT, ST_MSG
   ! Global vars used: CommuteFlag, DebugLevel, EmpProb, KeepLeavers, LogPopulate, NumAges,
   !                   NumPopTypes, NumSectors
   ! Global vars set : ProcLevel, ProcName, TotPop
   ! Module vars used: EmpGroup, EmpProbs, PopCounts
   ! Intrinsic procs : INDEX, NINT, REAL, TRIM, WRITE
   ! APEX procedures : Up
   ProcLevel = ProcLevel+1                                           ! Increment procedure depth
   ProcName(ProcLevel) = "StudyAreaPop"                              ! Procedure name for messages
   IF (DebugLevel>0) WRITE(LogU,ST_FMT) ST_MSG, ProcName(ProcLevel)  ! Message for debugging
   WRITE(LogU,'(/,A63)')" Sector   TotalPop   StudyPop   "          & 
       //" Workers  WorkInside   FinalPop"                           ! Header for table on log file
   DO I=1,NumSectors                                                 ! Loop over home sectors
      TotalPop = 0.                                                  ! Initialize total population
      StudyPop = 0.                                                  ! Initialize study population
      Workers  = 0.                                                  ! Initialize count of workers
      WorkIn   = 0.                                                  ! Number who work in study area
      DO J=1,NumPopTypes                                             ! Loop over Population types
         Sex = INDEX("MF",TRIM(Up(PopCat(J)%Gender(1:1))))           ! Identify gender for profile
         DO K=0,PopAges(NumPopAges,2)                                ! Loop over all ages
            Group   = PopGroup(K)                                    ! Group for this year of age
            IF (Group<1) CYCLE                                       ! If no pop data, skip this age 
            Range    = PopAges(Group,2)-PopAges(Group,1)+1           ! # of years in this group
            Pop      = REAL(PopCounts(J,I,Group))/Range              ! Each year in group has equal #
            TotalPop = TotalPop + Pop                                ! Add to total pop in study area 
            IF (K>=AgeMin .AND. K<=AgeMax) THEN                      ! If age is in simulation bounds 
               StudyPop = StudyPop + Pop                             ! Add to base pop for simulation
               W        = Pop*EmpProbs(I,EmpGroup(K,Sex))            ! # workers in age-type group
               Workers  = Workers + W                                ! Increment count of workers
               WorkIn   = WorkIn  + W*DestFracIn(I)                  ! Increment count in study area
            ENDIF                                                    ! End of logic for acceptable age
         ENDDO                                                       ! Continue with next age group
      ENDDO                                                          ! Continue with next pop type
      CorrPop = StudyPop                                             ! Either keep everyone
      IF (KeepLeavers==NO) CorrPop = StudyPop - Workers + WorkIn     ! Or exclude out-commuters
      IF (LogPopulate==YES) WRITE(LogU,'(I8,5F11.0)') I, TotalPop,  &
         StudyPop, Workers, WorkIn, CorrPop                          ! Report counts for this sector
      CumTotal = CumTotal + TotalPop                                 ! Add to cumulative pop total
      CumStudy = CumStudy + StudyPop                                 ! Add to cumulative age total  
      CumWork  = CumWork + Workers                                   ! Add to cumulative worker total
      CumIn    = CumIn  + WorkIn                                     ! Cumulative workers in study area
      CumCorr  = CumCorr + CorrPop                                   ! Corrected cumulative population
   ENDDO                                                             ! Continue with next sector
   WRITE(LogU,'(5X,A3,5F11.0)') 'ALL', CumTotal, CumStudy, CumWork, &
      CumIn, CumCorr                                                 ! Report study area totals
   TotPop = NINT(CumCorr)                                            ! Reset the total population
   IF (DebugLevel>0) WRITE(LogU,LV_FMT) LV_MSG, ProcName(ProcLevel)  ! Message for debugging
   ProcLevel = ProcLevel-1                                           ! Decrement procedure depth
END SUBROUTINE StudyAreaPop                                          ! Return to ReadPopulations

SUBROUTINE GenerateProfiles                                          ! Called from main APEX program
   INTEGER (KIND=IK), POINTER :: I1(:)                               ! Pointers used in distrib calls 
   INTEGER (KIND=IK), PARAMETER :: Gsize = 10000                     ! Size of profile groups 
   REAL    (KIND=RK)   :: R(8)                                       ! Local array for probabilities
   INTEGER (KIND=IK)   :: I, F, J, K, L, P, W, Extra, Age, Sex, NP   ! Local integer variables
   INTEGER (KIND=IK)   :: N, A, B, M, GroupNum                       ! Local integer variables
   INTEGER (KIND=IK)   :: Seed                                       ! Local rand number seed vars
   INTEGER (KIND=IK)   :: Emp, WorkSec, CommOut=0, AgeOut=0, Range   ! More local integer variables
   REAL    (KIND=RK)   :: WorkDist, WorkTime, MidProb, Diff, DiffC   ! Local temporary real variables
   REAL    (KIND=RK)   :: RatioC, CumProb, LastProb                  ! Local temporary real variables
   CHARACTER(LEN=LEN2) :: FN                                         ! Buffer for distribution name
   ! Global constants: IK, LEN2, LogU, LV_FMT, LV_MSG, NO, RK, ST_FMT, ST_MSG, YES 
   ! Global vars used: AgeMin, AgeMax, CommuteFlag, EmpProb, GroupYrs, KeepLeavers, KSeed, MinAges,
   !                   NumAges, NumDistricts, NumHours, NumPopTypes, NumProfiles, NumSectors, 
   !                   PopCat, SList
   ! Global vars set : EventSeq, HourSeq, ISeed, Person, PersonDay, ProcLevel, ProcName
   ! Module vars used: CommHome, CommProb, CommWork, EmpGroup, EmpProbs, PopAges, PopProbs
   ! Intrinsic procs : ALLOCATE, CEILING, DEALLOCATE, INDEX, INT, MIN, PRINT, RANDOM_SEED,
   !                   RANDOM_NUMBER, REAL, TRIM, WRITE
   ! APEX procedures : CheckAllocation, Distrib, Up
   ProcLevel = ProcLevel+1                                           ! Increment procedure depth
   ProcName(ProcLevel) = "GenerateProfiles"                          ! Procedure name for messages
   NP=NumPollutants                                                  ! Short form for# pollutants
   WRITE(LogU,ST_FMT) ST_MSG, ProcName(ProcLevel)                    ! Put status message on Log
   IF (TimeStepsPerDay>24)  EVENTSDAY=EVENTSDAY*TimeStepsPerHour     ! Update for longer events seq
   ALLOCATE(EventSeq(NumDays*EVENTSDAY),STAT=AllocErr)               ! Space for composite events 
   CALL CheckAllocation(AllocErr)                                    ! Check the success of the alloc
   ALLOCATE(EventExp(NumDays*EVENTSDAY,NP),STAT=AllocErr)            ! Space for composite events 
   CALL CheckAllocation(AllocErr)                                    ! Check the success of the alloc                  
   ALLOCATE(EventAmbHome(NumDays*EVENTSDAY,NP),STAT=AllocErr)        ! Space for composite events 
   CALL CheckAllocation(AllocErr)                                    ! Check the success of the alloc   
   ALLOCATE(EventAmb(NumDays*EVENTSDAY,NP),STAT=AllocErr)            ! Space for composite events 
   CALL CheckAllocation(AllocErr)                                    ! Check the success of the alloc    
   ALLOCATE(EventDose(NumDays*EVENTSDAY,NP),STAT=AllocErr)           ! Space for composite events 
   CALL CheckAllocation(AllocErr)                                    ! Check the success of the alloc
   ALLOCATE(EventInDose(NumDays*EVENTSDAY,NP),STAT=AllocErr)         ! Space for composite events 
   CALL CheckAllocation(AllocErr)                                    ! Check the success of the alloc
   ALLOCATE(EventDepDose(NumDays*EVENTSDAY,NP),STAT=AllocErr)        ! Space for composite events 
   CALL CheckAllocation(AllocErr)                                    ! Check the success of the alloc
   ALLOCATE(EventFDose(NumDays*EVENTSDAY,NP),STAT=AllocErr)          ! Space for composite events 
   CALL CheckAllocation(AllocErr)                                    ! Check the success of the alloc
   ALLOCATE(HourSeq(NumHours),STAT=AllocErr)                         ! Space for composite hours
   CALL CheckAllocation(AllocErr)                                    ! Check the success of the alloc
   ALLOCATE(HourExp(NumHours,NP),STAT=AllocErr)                      ! Space for hourly data 
   CALL CheckAllocation(AllocErr)                                    ! Check the success of the alloc
   ALLOCATE(HourDose(NumHours,NP),STAT=AllocErr)                     ! Space for hourly data 
   CALL CheckAllocation(AllocErr)                                    ! Check the success of the alloc
   ALLOCATE(HourIntake(NumHours,NP),STAT=AllocErr)                   ! Space for hourly data 
   CALL CheckAllocation(AllocErr)                                    ! Check the success of the alloc
   ALLOCATE(HourDep(NumHours,NP),STAT=AllocErr)                      ! Space for hourly data 
   CALL CheckAllocation(AllocErr)                                    ! Check the success of the alloc
   ALLOCATE(HourEF(NumHours,NP),STAT=AllocErr)                       ! Space for hourly data 
   CALL CheckAllocation(AllocErr)                                    ! Check the success of the alloc
   ALLOCATE(HourHomeEF(NumHours,NP),STAT=AllocErr)                   ! Space for hourly data 
   CALL CheckAllocation(AllocErr)                                    ! Check the success of the alloc   
   ALLOCATE(HourAmb(NumHours,NP),STAT=AllocErr)                      ! Space for hourly data 
   CALL CheckAllocation(AllocErr)                                    ! Check the success of the alloc
   ALLOCATE(HourHomeAmb(NumHours,NP),STAT=AllocErr)                  ! Space for hourly data 
   CALL CheckAllocation(AllocErr)                                    ! Check the success of the alloc
   ALLOCATE(HourConc(NumHours,NP,0:NumMicros),STAT=AllocErr)         ! Space for hourly data 
   CALL CheckAllocation(AllocErr)                                    ! Check the success of the alloc
   ALLOCATE(HourTime(NumHours,0:NumMicros),STAT=AllocErr)            ! Space for hourly data 
   CALL CheckAllocation(AllocErr)                                    ! Check the success of the alloc
   ALLOCATE(DayAvgExp(NumDays,NP),STAT=AllocErr)                     ! Space for daily data 
   CALL CheckAllocation(AllocErr)                                    ! Check the success of the alloc
   ALLOCATE(DayMax1Exp(NumDays,NP),STAT=AllocErr)                    ! Space for daily data 
   CALL CheckAllocation(AllocErr)                                    ! Check the success of the alloc
   ALLOCATE(DayMax8Exp(NumDays,NP),STAT=AllocErr)                    ! Space for daily data 
   CALL CheckAllocation(AllocErr)                                    ! Check the success of the alloc
   ALLOCATE(DayAvgDose(NumDays,NP),STAT=AllocErr)                    ! Space for daily data 
   CALL CheckAllocation(AllocErr)                                    ! Check the success of the alloc
   ALLOCATE(DayMax1Dose(NumDays,NP),STAT=AllocErr)                   ! Space for daily data 
   CALL CheckAllocation(AllocErr)                                    ! Check the success of the alloc
   ALLOCATE(DayMax8Dose(NumDays,NP),STAT=AllocErr)                   ! Space for daily data 
   CALL CheckAllocation(AllocErr)                                    ! Check the success of the alloc
   ALLOCATE(DayMax8EC(NumDays,NP),STAT=AllocErr)                     ! Space for daily data 
   CALL CheckAllocation(AllocErr)                                    ! Check the success of the alloc      
   ALLOCATE(DayHomeConc(NumDays,NP),STAT=AllocErr)                   ! Space for daily data 
   CALL CheckAllocation(AllocErr)                                    ! Check the success of the alloc      
   ALLOCATE(DayIntake(NumDays,NP),STAT=AllocErr)                     ! Space for daily data 
   CALL CheckAllocation(AllocErr)                                    ! Check the success of the alloc
   ALLOCATE(DayDep(NumDays,NP),STAT=AllocErr)                        ! Space for daily data 
   CALL CheckAllocation(AllocErr)                                    ! Check the success of the alloc
   ALLOCATE(DayMax1FDose(NumDays,NP),STAT=AllocErr)                  ! Space for daily data 
   CALL CheckAllocation(AllocErr)                                    ! Check the success of the alloc
   ALLOCATE(DayMaxTSDose(NumDays,NP),STAT=AllocErr)                  ! Space for daily data 
   CALL CheckAllocation(AllocErr)                                    ! Check the success of the alloc
   ALLOCATE(DayMaxTSExp(NumDays,NP),STAT=AllocErr)                   ! Space for daily data 
   CALL CheckAllocation(AllocErr)                                    ! Check the success of the alloc
   ALLOCATE (Person(NumProfiles),STAT=AllocErr)                      ! Space for profile data
   CALL CheckAllocation(AllocErr)                                    ! Check the success of the alloc
   ALLOCATE (PersonDay(NumDays),STAT=AllocErr)                       ! Space for daily varying data
   CALL CheckAllocation(AllocErr)                                    ! Check the success of the alloc
   ALLOCATE (TSSeq(NumTS),STAT=AllocErr)                             ! Space for timestep data
   CALL CheckAllocation(AllocErr)                                    ! Check the success of the alloc
   ALLOCATE (TSExp(NumTS,NP),STAT=AllocErr)                          ! Space for ts varying data
   CALL CheckAllocation(AllocErr)                                    ! Check the success of the alloc
   ALLOCATE (TSEF(NumTS,NP),STAT=AllocErr)                           ! Space for ts varying data
   CALL CheckAllocation(AllocErr)                                    ! Check the success of the alloc
   ALLOCATE (TSAmb(NumTS,NP),STAT=AllocErr)                          ! Space for ts varying data
   CALL CheckAllocation(AllocErr)                                    ! Check the success of the alloc
   ALLOCATE (TSDose(NumTS,NP),STAT=AllocErr)                         ! Space for ts varying data
   CALL CheckAllocation(AllocErr)                                    ! Check the success of the alloc
   ALLOCATE (TSInDose(NumTS,NP),STAT=AllocErr)                       ! Space for ts varying data
   CALL CheckAllocation(AllocErr)                                    ! Check the success of the alloc
   ALLOCATE (TSDepDose(NumTS,NP),STAT=AllocErr)                      ! Space for ts varying data
   CALL CheckAllocation(AllocErr)                                    ! Check the success of the alloc   
   EventExp   = 0                                                    ! Initialize array
   EventAmb   = 0                                                    ! Initialize array
   EventAmbHome = 0                                                  ! Initialize array
   EventDose  = 0                                                    ! Initialize array
   EventFDose = 0                                                    ! Initialize array                                                    ! Initialize array
   HourAmb    = 0                                                    ! Initialize array                                                      
   HourExp    = 0                                                    ! Initialize array                                                      
   HourDose   = 0                                                    ! Initialize array
   HourEF     = 0                                                    ! Initialize array
   HourHomeEF = 0                                                    ! Initialize array
   HourHomeAmb= 0                                                    ! Initialize array
   CALL RANDOM_SEED(Put=ISeed(1:KSeed))                              ! Set random number seed
   P = 0                                                             ! Initialize profile counter
   DO                                                                ! Loop over all profiles
      CALL RANDOM_NUMBER(R)                                          ! Get uniform randoms
      DO J=1,NumPopTypes-1                                           ! Loop over population types
         IF (R(1)<PopProbs(J,0,0)) EXIT                              ! Select type using pop. weights
      ENDDO                                                          ! Try next element in pop vector
      IF (NumPopTypes==1) J=1                                        ! Value in case DO loop is empty
      Sex = INDEX("MF",TRIM(Up(PopCat(J)%Gender(1:1))))              ! Identify gender for profile
      DO K=1,NumSectors-1                                            ! Loop over home sectors
         IF (R(2)<PopProbs(J,K,0)) EXIT                              ! Select sector using pop. weights
      ENDDO                                                          ! Try next sector
      IF (NumSectors==1) K=1                                         ! Value in case DO loop is empty
      DO L=1,NumPopAges-1                                            ! Loop over age groups
         IF (R(3)<PopProbs(J,K,L)) EXIT                              ! Select age group using weights
      ENDDO                                                          ! Try next age group
      IF (NumPopAges==1) L=1                                         ! Value in case DO loop is empty
      Range = PopAges(L,2)-PopAges(L,1)+1                            ! Size of age group  
      Extra = INT(R(4)*REAL(Range))                                  ! Years above group minimum
      Age   = PopAges(L,1)+MIN(Extra,Range-1)                        ! Generate age in years
      IF (Age<AgeMin.OR.Age>AgeMax) THEN                             ! Check if age within range
         AgeOut = AgeOut+1                                           ! If not, increment counter
         CYCLE                                                       ! If age out of range restart DO
      ENDIF                                                          ! End of age check logic block  
      Emp = NO                                                       ! Default is not employed
      IF (R(5)<EmpProbs(K,EmpGroup(Age,Sex))) Emp = YES              ! Generate employment status
      LastProb = 0                                                   ! Initializing lower dist bound
      CumProb = 0                                                    ! Initializing upper dist bound
      DO M=1,NumLevels                                               ! Looping through levels
         CumProb = ProProbs(M, K, ProGroup(Age,Sex)) + CumProb       ! Calculating upper bound
         IF (R(8)<CumProb .AND. R(8)>LastProb) EXIT                  ! If between bounds, exit
         LastProb = CumProb                                          ! Setting lower bound
      ENDDO                                                          ! End loop over levels
      GroupNum = M                                                   ! Setting group number
      IF (Emp == NO .AND. OccFactor == YES) GroupNum = 1             ! Unemployed defined to be 1
      IF (CumProb == 0) GroupNum = 0                                 ! If no probs found, undefined
      WorkSec = K                                                    ! Default work sector = home 
      WorkDist = 0                                                   ! Default work distance = 0
      WorkTime = 0                                                   ! Default work time = 0
      IF (CommuteFlag==YES.AND.Emp==YES) THEN                        ! If a commuter...
         W = CommHome(K)                                             ! Start of data for home sector K
         IF (W==0) THEN                                              ! If no data for home sector K
            WorkSec = K                                              ! Then profile does not commute
         ELSE                                                        ! Or else commuting data exists
            DO                                                       ! Loop over possible destinations
               IF (R(6)<CommProb(W)) EXIT                            ! Pick destination at random
               W = W+1                                               ! Move pointer to next destination
            ENDDO                                                    ! Continue loop until exit occurs
            WorkSec = CommWork(W)                                    ! Number of selected work sector
            WorkDist = CommDists(W)                                  ! Distance to selected work sector
            A = 1                                                    ! Initializing distance bin counter
            DO                                                       ! Entering distance bin logic
                IF (WorkDist <= CumDistances(K,A)) EXIT              ! If distance is met, exit
                A = A + 1                                            ! Else increment counter
            ENDDO                                                    ! Exiting distance bin logic
            IF (A == 1) THEN                                         ! If first one then
                MidProb = CumDistProbs(K,A)/2                        ! Midpoint is half bin
            ELSE                                                     ! If not first one
                MidProb = (CumDistProbs(K,A-1) + CumDistProbs(K,A))/2 ! Midpoint is (current - last) /2
            ENDIF                                                    ! End finding midpoint
            B = 3                                                    ! Initializing time bin counter
            DO                                                       ! Entering time bin logic
                IF (CumTimes(K,B) >= MidProb) EXIT                   ! If time is met, exit
                B = B + 1                                            ! Else increment to next bin
            ENDDO                                                    ! End time bin logic
            IF (B==1) THEN                                           ! If first bin...
                Diff = CumTimes(K,B)                                 ! Difference is just bin size
            ELSE                                                     ! If not first bin...
                Diff = CumTimes(K,B) - CumTimes(K,B-1)               ! Bin size is current - last
            ENDIF                                                    ! End finding bin size
            DiffC = CumTimes(K,B) - MidProb                          ! Difference midpoint from end of bin
            RatioC = DiffC/Diff                                      ! Placement in bin
            IF (B==3) WorkTime = 0                                   ! Setting bin #3
            IF (B==4) WorkTime = 4-FLOOR(5*RatioC)                   ! Setting bin #4
            IF (B==5) WorkTime = 9-FLOOR(5*RatioC)                   ! Setting bin #5
            IF (B==6) WorkTime = 14-FLOOR(5*RatioC)                  ! Setting bin #6
            IF (B==7) WorkTime = 19-FLOOR(5*RatioC)                  ! Setting bin #7
            IF (B==8) WorkTime = 24-FLOOR(5*RatioC)                  ! Setting bin #8
            IF (B==9) WorkTime = 29-FLOOR(5*RatioC)                  ! Setting bin #9
            IF (B==10) WorkTime = 34-FLOOR(5*RatioC)                 ! Setting bin #10
            IF (B==11) WorkTime = 39-FLOOR(5*RatioC)                 ! Setting bin #11
            IF (B==12) WorkTime = 44-FLOOR(5*RatioC)                 ! Setting bin #12
            IF (B==13) WorkTime = 59-FLOOR(15*RatioC)                ! Setting bin #13
            IF (B==14) WorkTime = 89-FLOOR(30*RatioC)                ! Setting bin #14
            IF (B==15) WorkTime = 120-FLOOR(31*RatioC)               ! Setting bin #15
            IF (WorkSec==0) THEN                                     ! If work sector not in study area
               CommOut = CommOut+1                                   ! Increment leavers counter
               IF (KeepLeavers==NO) CYCLE                            ! Start another profile 
            ENDIF                                                    ! End of work outside area logic
         ENDIF                                                       ! End of commuting data processing
      ENDIF                                                          ! End of commuting logic 
      Seed     = 2147483648*R(7)                                     ! Set each word in Seed for person
      P = P+1                                                        ! Increment profile counter
      Person(P)%Index       = P                                      ! Set the profile index
      Person(P)%Gender      = Sex                                    ! Set the profile gender
      Person(P)%PopCat      = J                                      ! Set the population category
      Person(P)%HomeSec     = K                                      ! Set the profile home sector
      Person(P)%HomeD       = SList(K)%NumD                          ! Air district for sector K
      Person(P)%RoadD       = SList(K)%NumRD                         ! Roadway district for sector K
      Person(P)%Employed    = Emp                                    ! Set profile employment status
      Person(P)%FactorGroup = GroupNum                               ! Set factor group number
      IF (UseProfileFactors==YES) THEN                               ! If profile factors are used
         Person(P)%GroupName   = GroupNames(GroupNum)                ! Set factor group name
      ELSE                                                           ! If not used, then...
         Person(P)%GroupName = "none"                                ! Set value to none
      ENDIF                                                          ! End profile factors logic
      Person(P)%WorkSec     = WorkSec                                ! Set profile work sector
      Person(P)%WorkD       = SList(WorkSec)%NumD                    ! Set profile work district 
      Person(P)%RoadWD      = SList(WorkSec)%NumRD                   ! Set profile road work district
      Person(P)%Age         = Age                                    ! Set the profile age
      Person(P)%Zone        = SList(K)%NumZ                          ! Set the profile temp zone
      Person(P)%Seed        = Seed                                   ! Set the profile initial seed
      Person(P)%CommDist    = WorkDist                               ! Set the profile commuting distance
      Person(P)%CommTime    = WorkTime                               ! Set the profile commuting time
      IF (P==NumProfiles) EXIT                                       ! Exit if enough profiles done
   ENDDO                                                             ! Continue with next profile
   WRITE(LogU,*) '# of generated ages outside range = ', AgeOut      ! # ages outside allowed range
   WRITE(LogU,*) '# commuting out of study area     = ', CommOut     ! # commuting outside study area
   IF (KeepLeavers==NO) WRITE(LogU,*) 'Commuters outside = deleted'  ! Rejected for leaving area
   WRITE(LogU,*) 'Base profiles ready'                               ! Write base profile summary
   DO I=1,CEILING(REAL(NumProfiles)/Gsize)                           ! Do Gsize profiles at a time
      F = 1+(I-1)*Gsize                                              ! First profile in group 
      L = MIN(I*Gsize,NumProfiles)                                   ! Last profile in group
      FN = "HASGASSTOVE"                                             ! Buffer for distribution name      
      IF (FuncNum(FN,NO)>0) THEN                                     ! If GasStove is defined
        CALL EvalFunc(Name=FN, IRes=Person(F:L)%GasStove)            ! Get gas stove status for all
        I1 => Person(F:L)%GasStove                                   ! Pointer to gas stove status
        FN = "HASGASPILOT"                                           ! Buffer for distribution name
        IF (FuncNum(FN,NO)>0) CALL EvalFunc(Name=FN,I1=I1,          &
        IRes=Person(F:L)%GasPilot)                                   ! Get pilot light status for all
      ENDIF                                                          ! End of GasStove logic block
      FN = "AC_HOME"                                                 ! Buffer for distribution name
      IF (FuncNum(FN,NO)>0) THEN                                     ! If AC_Home is defined 
         CALL EvalFunc(Name=FN,IRes=Person(F:L)%AC_Home)             ! Get air conditioning for all
      ENDIF                                                          ! End of AC_Home logic block 
      FN = "AC_CAR"                                                  ! Buffer for distribution name
      IF (FuncNum(FN,NO)>0) THEN                                     ! If AC_Car is defined    
         CALL EvalFunc(Name=FN,IRes=Person(F:L)%AC_Car)              ! Get car AC status for all 
      ENDIF                                                          ! End of AC_Car logic block 
      FN = "PROFILECONDITIONAL1"                                     ! Buffer for distribution name
      IF (FuncNum(FN,NO)>0) THEN                                     ! If PCond1 is defined    
         CALL EvalFunc(Name=FN,IRes=Person(F:L)%PCond1)              ! Get Pcond1 status for all 
      ENDIF                                                          ! End of PCond1 logic block 
      FN = "PROFILECONDITIONAL2"                                     ! Buffer for distribution name
      IF (FuncNum(FN,NO)>0) THEN                                     ! If PCond2 is defined    
         CALL EvalFunc(Name=FN,IRes=Person(F:L)%PCond2)              ! Get Pcond2 status for all 
      ENDIF                                                          ! End of PCond2 logic block 
      FN = "PROFILECONDITIONAL3"                                     ! Buffer for distribution name
      IF (FuncNum(FN,NO)>0) THEN                                     ! If PCond3 is defined    
         CALL EvalFunc(Name=FN,IRes=Person(F:L)%PCond3)              ! Get Pcond3 status for all 
      ENDIF                                                          ! End of PCond3 logic block 
      FN = "PROFILECONDITIONAL4"                                     ! Buffer for distribution name
      IF (FuncNum(FN,NO)>0) THEN                                     ! If PCond4 is defined    
         CALL EvalFunc(Name=FN,IRes=Person(F:L)%PCond4)              ! Get Pcond4 status for all 
      ENDIF                                                          ! End of PCond4 logic block 
      FN = "PROFILECONDITIONAL5"                                     ! Buffer for distribution name
      IF (FuncNum(FN,NO)>0) THEN                                     ! If PCond5 is defined    
         CALL EvalFunc(Name=FN,IRes=Person(F:L)%PCond5)              ! Get Pcond5 status for all 
      ENDIF                                                          ! End of PCond5 logic block 
      DO J=F,L                                                       ! Loop over persons in group 
         FN = "REGIONALCONDITIONAL1"                                 ! Buffer for distribution name
         IF (RegionFuncNum(FN)>0) THEN                               ! If RCond1 is defined       
            CALL EvalFunc(Name=FN,Profile=J,IRes=Person(J:J)%RCond1) ! Get Rcond1 status for all 
         ENDIF                                                       ! End of RCond1 logic block 
         FN = "REGIONALCONDITIONAL2"                                 ! Buffer for distribution name
         IF (RegionFuncNum(FN)>0) THEN                               ! If RCond2 is defined    
            CALL EvalFunc(Name=FN,Profile=J,IRes=Person(J:J)%RCond2) ! Get RCond2 status for all 
         ENDIF                                                       ! End of RCond2 logic block 
         FN = "REGIONALCONDITIONAL3"                                 ! Buffer for distribution name
         IF (RegionFuncNum(FN)>0) THEN                               ! If RCond3 is defined    
            CALL EvalFunc(Name=FN,Profile=J,IRes=Person(J:J)%RCond3) ! Get RCond3 status for all 
         ENDIF                                                       ! End of RCond3 logic block 
         FN = "REGIONALCONDITIONAL4"                                 ! Buffer for distribution name
         IF (RegionFuncNum(FN)>0) THEN                               ! If RCond4 is defined    
            CALL EvalFunc(Name=FN,Profile=J,IRes=Person(J:J)%RCond4) ! Get RCond4 status for all 
         ENDIF                                                       ! End of RCond4 logic block 
         FN = "REGIONALCONDITIONAL5"                                 ! Buffer for distribution name
         IF (RegionFuncNum(FN)>0) THEN                               ! If RCond5 is defined    
            CALL EvalFunc(Name=FN,Profile=J,IRes=Person(J:J)%RCond5) ! Get RCond5 status for all 
         ENDIF                                                       ! End of RCond5 logic block 
      ENDDO                                                          ! End loop over persons in group
   ENDDO                                                             ! Continue with next group
   IF (NumProfiles<=6) THEN                                          ! If there are few profiles...
      WRITE(LogU,*) 'Gender ', Person%Gender                         ! List genders on log file
      WRITE(LogU,*) 'Ages   ', Person%Age                            ! List ages on log file
      WRITE(LogU,*) 'PopCat ', Person%PopCat                         ! List pop categories on log file
      WRITE(logU,*) 'HSect  ', Person%HomeSec                        ! List home sectors on log file
      WRITE(logU,*) 'WSect  ', Person%WorkSec                        ! List work sectors on log file
      WRITE(LogU,*) 'HDist  ', Person%HomeD                          ! List home districts on log file
      WRITE(LogU,*) 'WorkD  ', Person%WorkD                          ! List work districts on log file
      WRITE(LogU,*) 'RoadD  ', Person%RoadD                          ! List roadway districts on log file      
      WRITE(LogU,*) 'RoadWD ', Person%RoadWD                         ! List work roadway district on log file
      WRITE(LogU,*) 'Zone   ', Person%Zone                           ! List temperature zones on log
      WRITE(logU,*) 'Emp    ', Person%Employed                       ! List employment status on log
      WRITE(LogU,*) 'Stove  ', Person%GasStove                       ! List gas stove status on log
      WRITE(LogU,*) 'Pilot  ', Person%GasPilot                       ! List pilot light status on log
      WRITE(LogU,*) 'ACHome ', Person%AC_Home                        ! List home AC status on log
      WRITE(LogU,*) 'ACCar  ', Person%AC_Car                         ! List car AC status on log
      WRITE(LogU,*) 'PSeed  ', Person%Seed                           ! List personal seed on log
      WRITE(LogU,*) 'PCond1 ', Person%PCond1                         ! List conditional var on log
      WRITE(LogU,*) 'PCond2 ', Person%PCond2                         ! List conditional var on log
      WRITE(LogU,*) 'PCond3 ', Person%PCond3                         ! List conditional var on log
      WRITE(LogU,*) 'PCond4 ', Person%PCond4                         ! List conditional var on log
      WRITE(LogU,*) 'PCond5 ', Person%PCond5                         ! List conditional var on log
      WRITE(LogU,*) 'RCond1 ', Person%RCond1                         ! List conditional var on log
      WRITE(LogU,*) 'RCond2 ', Person%RCond2                         ! List conditional var on log
      WRITE(LogU,*) 'RCond3 ', Person%RCond3                         ! List conditional var on log
      WRITE(LogU,*) 'RCond4 ', Person%RCond4                         ! List conditional var on log
      WRITE(LogU,*) 'RCond5 ', Person%RCond5                         ! List conditional var on log
      WRITE(LogU,*) 'CommDist ', Person%CommDist                     ! List conditional var on log
      WRITE(LogU,*) 'CommTime ', Person%CommTime                     ! List conditional var on log
   ENDIF                                                             ! End of profile listing logic
   PRINT*, 'Finished GenerateProfiles'                               ! Put status message on screen
   WRITE(LogU,LV_FMT) LV_MSG, ProcName(ProcLevel)                    ! Put status message on Log file
   ProcLevel = ProcLevel-1                                           ! Decrement procedure depth
END SUBROUTINE GenerateProfiles                                      ! Return to main APEX program

SUBROUTINE GeneratePhysiology                                        ! Called from main APEX program
   INTEGER (KIND=IK) :: Age, Gender, I, P, K                         ! Local integer variables
   INTEGER (KIND=IK) :: MinAge, MaxAge, Ltype                        ! More local integer variables
   REAL    (KIND=RK) :: WEIGHT, RMR, Rand(3), Z(1)                   ! More local real variables
   TYPE(CDist), ALLOCATABLE :: NVO2MAX(:,:)                          ! VO2MAX distributions
   TYPE(CDist), ALLOCATABLE :: BMDist(:,:)                           ! Body mass distributions
   TYPE(CDist), ALLOCATABLE :: MAXOXD(:,:)                           ! Max oxygen deficit dists  
   TYPE(CDist), ALLOCATABLE :: HMG(:,:)                              ! Blood hemoglobin content dists
   TYPE(CDist), ALLOCATABLE :: RMRSLP(:,:),RMRINT(:,:)               ! Regress. slope, interc. for RMR 
   TYPE(CDist), ALLOCATABLE :: RMRERR(:,:)                           ! Regression error for RMR 
   TYPE(CDist), ALLOCATABLE :: HEIGHTSLP(:,:), HEIGHTINT(:,:)        ! Regress. slope, interc. for hgt
   TYPE(CDist), ALLOCATABLE :: HEIGHTERR(:,:)                        ! Regression SD for height
   TYPE(CDist), ALLOCATABLE :: BLDF1(:,:), BLDF2(:,:)                ! Blood volume parameter arrays
   TYPE(CDist), ALLOCATABLE :: BSAEXP1(:,:), BSAEXP2(:,:)            ! BSA exponents
   TYPE(CDist), ALLOCATABLE :: ENDGN1(:,:), ENDGN2(:,:)              ! Endogenous CO production
   TYPE(CDist), ALLOCATABLE :: ECF(:,:), RECTIME(:,:)                ! Blood volume parameter arrays
   TYPE(CDist), ALLOCATABLE :: B1(:,:), B2(:,:), B3(:,:), B4(:,:)    ! FEV1 Model parameters
   TYPE(CDist), ALLOCATABLE :: B5(:,:), B6(:,:), B7(:,:), VU(:,:)    ! FEV1 Model parameters
   TYPE(CDist), ALLOCATABLE :: FEVSLP(:,:), FEVINT(:,:)              ! FEV1 Model parameters   
   REAL    (KIND=RK) :: VEB0(0:100,2), VEB1(0:100,2), VEB2(0:100,2)  ! Arrays for VE/VO2 regression
   REAL    (KIND=RK) :: VEB3(0:100,2), VEEB(0:100), VEEW(0:100)      ! More VE/VO2 arrays
   REAL    (KIND=RK) :: buff(10), VEinter                            ! Local real ventilation vars
   REAL    (KIND=RK) :: prevbuff(2),Prev(0:100,2)                    ! Arrays for disease prevalence 
   REAL    (KIND=RK) :: HTS(1),HTI(1),HTE(1),RMRE(1),RMRS(1),RMRI(1) ! Local real variables 
   REAL    (KIND=RK) :: NVO2MX(1),BM(1),BF1(1),BF2(1),BSA1(1),BSA2(1)! Local real variables
   REAL    (KIND=RK) :: HG(1),MXD(1),E(1),REC(1),ENDCO1(1),ENDCO2(1) ! Local real variables
   REAL    (KIND=RK) :: B1VAL(1),B2VAL(1),B3VAL(1),B4VAL(1),B5VAL(1) ! Local real variables
   REAL    (KIND=RK) :: B6VAL(1),B7VAL(1),VUVAL(1), FSV(1), FIV(1)   ! Local real variables
   CHARACTER (LEN=5) :: Char1                                        ! Buffer for distribution part
   CHARACTER (LEN=LEN4) :: String, UpString                          ! Buffers for input records
   CHARACTER (LEN=9)   :: Var                                        ! Buffers for input records
   CHARACTER (LEN=1)    :: Gen                                       ! Buffers for input records
   ! Global constants: IK, LEN3, MALE, NO, PhysU, RK, LV_MSG, LV_FMT, ST_FMT, ST_MSG, VentU 
   ! Global vars used: DebugLevel, FName, KSeed, NumProfiles, Person
   ! Global vars set : Ios, ISEED, Phys, ProcLevel, ProcName
   ! Intrinsic procs : ALLOCATE, CLOSE, INT, LOG, MIN, PRINT, RANDOM_NUMBER,
   !                   RANDOM_SEED, READ, REAL, WRITE
   ! APEX procedures : CheckAllocation, Fatal, GetLine, OpenInput, RnNor
   ProcLevel = ProcLevel+1                                           ! Increment procedure depth
   ProcName(ProcLevel) = "GeneratePhysiology"                        ! Procedure name for messages
   IF (DebugLevel>0) WRITE(LogU,ST_FMT) ST_MSG, ProcName(ProcLevel)  ! Message for debugging
   CALL OpenInput(PhysU,h=NO)                                        ! Open physiology input file
   IF (Ios(PhysU)/=0) CALL Fatal(1)                                  ! Stop if file not available
   ALLOCATE (NVO2MAX(0:100,2),STAT=AllocErr)                         ! Space for NVO2MAX
   CALL CheckAllocation(AllocErr)                                    ! Check the success of the alloc
   ALLOCATE (BMDist(0:100,2),STAT=AllocErr)                          ! Space for BMDist
   CALL CheckAllocation(AllocErr)                                    ! Check the success of the alloc
   ALLOCATE (MAXOXD(0:100,2),STAT=AllocErr)                          ! Space for MAXOXD
   CALL CheckAllocation(AllocErr)                                    ! Check the success of the alloc
   ALLOCATE (HMG(0:100,2),STAT=AllocErr)                             ! Space for HMG
   CALL CheckAllocation(AllocErr)                                    ! Check the success of the alloc
   ALLOCATE (RMRSLP(0:100,2),STAT=AllocErr)                          ! Space for RMRSLP
   CALL CheckAllocation(AllocErr)                                    ! Check the success of the alloc
   ALLOCATE (RMRINT(0:100,2),STAT=AllocErr)                          ! Space for RMRINT
   CALL CheckAllocation(AllocErr)                                    ! Check the success of the alloc
   ALLOCATE (RMRERR(0:100,2),STAT=AllocErr)                          ! Space for RMRERR
   CALL CheckAllocation(AllocErr)                                    ! Check the success of the alloc
   ALLOCATE (HEIGHTSLP(0:100,2),STAT=AllocErr)                       ! Space for HEIGHTSLP
   CALL CheckAllocation(AllocErr)                                    ! Check the success of the alloc
   ALLOCATE (HEIGHTINT(0:100,2),STAT=AllocErr)                       ! Space for HEIGHTINT
   CALL CheckAllocation(AllocErr)                                    ! Check the success of the alloc
   ALLOCATE (HEIGHTERR(0:100,2),STAT=AllocErr)                       ! Space for HEIGHTERR
   CALL CheckAllocation(AllocErr)                                    ! Check the success of the alloc
   ALLOCATE (BLDF1(0:100,2),STAT=AllocErr)                           ! Space for BLDF1
   CALL CheckAllocation(AllocErr)                                    ! Check the success of the alloc
   ALLOCATE (BLDF2(0:100,2),STAT=AllocErr)                           ! Space for BLDF2
   CALL CheckAllocation(AllocErr)                                    ! Check the success of the alloc
   ALLOCATE (BSAEXP1(0:100,2),STAT=AllocErr)                         ! Space for BSAEXP1
   CALL CheckAllocation(AllocErr)                                    ! Check the success of the alloc
   ALLOCATE (BSAEXP2(0:100,2),STAT=AllocErr)                         ! Space for BSAEXP2
   CALL CheckAllocation(AllocErr)                                    ! Check the success of the alloc
   ALLOCATE (ENDGN1(0:100,2),STAT=AllocErr)                          ! Space for ENDGN1
   CALL CheckAllocation(AllocErr)                                    ! Check the success of the alloc
   ALLOCATE (ENDGN2(0:100,2),STAT=AllocErr)                          ! Space for ENDGN2
   CALL CheckAllocation(AllocErr)                                    ! Check the success of the alloc
   ALLOCATE (ECF(0:100,2),STAT=AllocErr)                             ! Space for ECF
   CALL CheckAllocation(AllocErr)                                    ! Check the success of the alloc
   ALLOCATE (RECTIME(0:100,2),STAT=AllocErr)                         ! Space for RECTIME
   CALL CheckAllocation(AllocErr)                                    ! Check the success of the alloc
   ALLOCATE (B1(0:100,2),STAT=AllocErr)                              ! Space for B1
   CALL CheckAllocation(AllocErr)                                    ! Check the success of the alloc
   ALLOCATE (B2(0:100,2),STAT=AllocErr)                              ! Space for B2
   CALL CheckAllocation(AllocErr)                                    ! Check the success of the alloc
   ALLOCATE (B3(0:100,2),STAT=AllocErr)                              ! Space for B3
   CALL CheckAllocation(AllocErr)                                    ! Check the success of the alloc
   ALLOCATE (B4(0:100,2),STAT=AllocErr)                              ! Space for B4
   CALL CheckAllocation(AllocErr)                                    ! Check the success of the alloc
   ALLOCATE (B5(0:100,2),STAT=AllocErr)                              ! Space for B5
   CALL CheckAllocation(AllocErr)                                    ! Check the success of the alloc
   ALLOCATE (B6(0:100,2),STAT=AllocErr)                              ! Space for B6
   CALL CheckAllocation(AllocErr)                                    ! Check the success of the alloc
   ALLOCATE (B7(0:100,2),STAT=AllocErr)                              ! Space for B7
   CALL CheckAllocation(AllocErr)                                    ! Check the success of the alloc
   ALLOCATE (VU(0:100,2),STAT=AllocErr)                              ! Space for FEVU
   CALL CheckAllocation(AllocErr)                                    ! Check the success of the alloc   
   ALLOCATE (VE1(0:100,2),STAT=AllocErr)                             ! Space for FEVE1
   CALL CheckAllocation(AllocErr)                                    ! Check the success of the alloc      
   ALLOCATE (VE2(0:100,2),STAT=AllocErr)                             ! Space for FEVE2
   CALL CheckAllocation(AllocErr)                                    ! Check the success of the alloc         
   ALLOCATE (FEVSLP(0:100,2),STAT=AllocErr)                          ! Space for FEVSLP
   CALL CheckAllocation(AllocErr)                                    ! Check the success of the alloc   
   ALLOCATE (FEVINT(0:100,2),STAT=AllocErr)                          ! Space for FEVINT
   CALL CheckAllocation(AllocErr)                                    ! Check the success of the alloc   
   DO                                                                ! Loop over records on input file
      IF (Ios(PhysU)<0) EXIT                                         ! Exit loop on end-of-file
      CALL GetLine(PhysU,String,Ltype)                               ! Get next data line from input
      UpString = Up(String)                                          ! Make uppercase version of line 
      READ(UpString,*) Var, MinAge, MaxAge, Gen, Char1               ! Get the phys input info
      IF (MinAge>MaxAge) CALL Fatal(2)                               ! Error if min age > max age
      IF (Gen=='M') THEN                                             ! If input is for males
         Gender = 1                                                  ! Set gender index to 1
      ELSEIF (Gen=='F') THEN                                         ! If input is for females
         Gender = 2                                                  ! Set gender index to 2
      ELSE                                                           ! Otherwise there is an error
         CALL Fatal(3)                                               ! Unrecognized gender
      ENDIF                                                          ! End gender logic
      UpString = UpString(INDEX(UpString,Char1):)                    ! Drop all but distribution info
      SELECT CASE(TRIM(ADJUSTL(Var)))                                ! Select input variable
      CASE ('NVO2MAX')                                               ! If input is NVO2MAX
         DO I=MinAge, MaxAge                                         ! Loop over input ages
            NVO2MAX(I,Gender) = ReadDist(UpString)                   ! Define the NVO2MAX dist   
         ENDDO                                                       ! End loop over ages
      CASE ('BM')                                                    ! If input is body mass
         DO I=MinAge, MaxAge                                         ! Loop over input ages
            BMDist(I,Gender) = ReadDist(UpString)                    ! Define the body mass dist   
         ENDDO                                                       ! End loop over ages
      CASE ('RMRINT')                                                ! If input is RMR intercept
         DO I=MinAge, MaxAge                                         ! Loop over input ages
            RMRINT(I,Gender) = ReadDist(UpString)                    ! Define the intercept dist   
         ENDDO                                                       ! End loop over ages
       CASE ('RMRSLP')                                               ! If input is RMR slope
         DO I=MinAge, MaxAge                                         ! Loop over input ages
            RMRSLP(I,Gender) = ReadDist(UpString)                    ! Define the slope dist   
         ENDDO                                                       ! End loop over ages
      CASE ('RMRERR')                                                ! If input is RMR error 
         DO I=MinAge, MaxAge                                         ! Loop over input ages
            RMRERR(I,Gender) = ReadDist(UpString)                    ! Define the error dist   
         ENDDO                                                       ! End loop over ages
      CASE ('BLDFAC1')                                               ! If input is blood factor 1
         DO I=MinAge, MaxAge                                         ! Loop over input ages
            BLDF1(I,Gender) = ReadDist(UpString)                     ! Define the factor dist   
         ENDDO                                                       ! End loop over ages
      CASE ('BLDFAC2')                                               ! If input is blood factor 2      
         DO I=MinAge, MaxAge                                         ! Loop over input ages
            BLDF2(I,Gender) = ReadDist(UpString)                     ! Define the factor dist   
         ENDDO                                                       ! End loop over ages
      CASE ('HMG')                                                   ! If input is hemoglobin content
         DO I=MinAge, MaxAge                                         ! Loop over input ages
            HMG(I,Gender) = ReadDist(UpString)                       ! Define the hemoglobin dist   
         ENDDO                                                       ! End loop over ages
      CASE ('BSAEXP1')                                               ! If input is BSA exp 1
         DO I=MinAge, MaxAge                                         ! Loop over input ages
            BSAEXP1(I,Gender) = ReadDist(UpString)                   ! Define the exponent dist   
         ENDDO                                                       ! End loop over ages
      CASE ('BSAEXP2')                                               ! If input is BSA exp 1
         DO I=MinAge, MaxAge                                         ! Loop over input ages
            BSAEXP2(I,Gender) = ReadDist(UpString)                   ! Define the exponent dist   
         ENDDO                                                       ! End loop over ages
      CASE ('HEIGHTINT')                                             ! If input is height inter                    
         DO I=MinAge, MaxAge                                         ! Loop over input ages
            HEIGHTINT(I,Gender) = ReadDist(UpString)                 ! Define the intercept dist   
         ENDDO                                                       ! End loop over ages
      CASE ('HEIGHTSLP')                                             ! If input is height slope
         DO I=MinAge, MaxAge                                         ! Loop over input ages
            HEIGHTSLP(I,Gender) = ReadDist(UpString)                 ! Define the slope dist   
         ENDDO                                                       ! End loop over ages
      CASE ('HEIGHTERR')                                             ! If input is height SD
         DO I=MinAge, MaxAge                                         ! Loop over input ages
            HEIGHTERR(I,Gender) = ReadDist(UpString)                 ! Define the SD dist   
         ENDDO                                                       ! End loop over ages
      CASE ('MAXOXD')                                                ! If input is max ox deficit
         DO I=MinAge, MaxAge                                         ! Loop over input ages
            MAXOXD(I,Gender) = ReadDist(UpString)                    ! Define the MOXD dist   
         ENDDO                                                       ! End loop over ages
      CASE ('ECF')                                                   ! If input is energy conversion
         DO I=MinAge, MaxAge                                         ! Loop over input ages
            ECF(I,Gender) = ReadDist(UpString)                       ! Define the ECF dist   
         ENDDO                                                       ! End loop over ages
      CASE ('RECTIME')                                               ! If input is recovery time
         DO I=MinAge, MaxAge                                         ! Loop over input ages
            RECTIME(I,Gender) = ReadDist(UpString)                   ! Define the rec time dist   
         ENDDO                                                       ! End loop over ages
      CASE ('B1')                                                    ! If input is B1
         DO I=MinAge, MaxAge                                         ! Loop over input ages
            B1(I,Gender) = ReadDist(UpString)                        ! Define the B1 dist  
         ENDDO                                                       ! End loop over ages
      CASE ('B2')                                                    ! If input is B2
         DO I=MinAge, MaxAge                                         ! Loop over input ages
            B2(I,Gender) = ReadDist(UpString)                        ! Define the B2 dist   
         ENDDO                                                       ! End loop over ages
      CASE ('B3')                                                    ! If input is B3
         DO I=MinAge, MaxAge                                         ! Loop over input ages
            B3(I,Gender) = ReadDist(UpString)                        ! Define the B3 dist   
         ENDDO                                                       ! End loop over ages
      CASE ('B4')                                                    ! If input is B4
         DO I=MinAge, MaxAge                                         ! Loop over input ages
            B4(I,Gender) = ReadDist(UpString)                        ! Define the B4 dist   
         ENDDO                                                       ! End loop over ages
      CASE ('B5')                                                    ! If input is B5
         DO I=MinAge, MaxAge                                         ! Loop over input ages
            B5(I,Gender) = ReadDist(UpString)                        ! Define the B5 dist   
         ENDDO                                                       ! End loop over ages
      CASE ('B6')                                                    ! If input is B6
         DO I=MinAge, MaxAge                                         ! Loop over input ages
            B6(I,Gender) = ReadDist(UpString)                        ! Define the B6 dist   
         ENDDO                                                       ! End loop over ages
      CASE ('B7')                                                    ! If input is B7
         DO I=MinAge, MaxAge                                         ! Loop over input ages
            B7(I,Gender) = ReadDist(UpString)                        ! Define the B7 dist   
         ENDDO                                                       ! End loop over ages         
      CASE ('FEVU')                                                  ! If input is FEVU
         DO I=MinAge, MaxAge                                         ! Loop over input ages
            VU(I,Gender) = ReadDist(UpString)                        ! Define the FEVU dist   
         ENDDO                                                       ! End loop over ages                                                                             
      CASE ('FEVE1')                                                 ! If input is FEVE1
         DO I=MinAge, MaxAge                                         ! Loop over input ages
            VE1(I,Gender) = ReadDist(UpString)                       ! Define the FEVE1 dist   
         ENDDO                                                       ! End loop over ages                                                                             
      CASE ('FEVE2')                                                 ! If input is FEVE2
         DO I=MinAge, MaxAge                                         ! Loop over input ages
            VE2(I,Gender) = ReadDist(UpString)                       ! Define the FEVE2 dist   
         ENDDO                                                       ! End loop over ages                                                                             
      CASE ('FEVSLP')                                                ! If input is FEVSLP
         DO I=MinAge, MaxAge                                         ! Loop over input ages
            FEVSLP(I,Gender) = ReadDist(UpString)                    ! Define the FEVSLP dist   
         ENDDO                                                       ! End loop over ages                                                                             
      CASE ('FEVINT')                                                ! If input is FEVINT
         DO I=MinAge, MaxAge                                         ! Loop over input ages
            FEVINT(I,Gender) = ReadDist(UpString)                    ! Define the FEVINT dist   
         ENDDO                                                       ! End loop over ages         
      CASE ('ENDGN1')                                                ! If input is endgn CO 1
         DO I=MinAge, MaxAge                                         ! Loop over input ages
            ENDGN1(I,Gender) = ReadDist(UpString)                    ! Define the endgn1 dist   
         ENDDO                                                       ! End loop over ages
      CASE ('ENDGN2')                                                ! If input is endgn CO 2
         DO I=MinAge, MaxAge                                         ! Loop over input ages
            ENDGN2(I,Gender) = ReadDist(UpString)                    ! Define the endgn2 dist   
         ENDDO                                                       ! End loop over ages
      CASE DEFAULT                                                   ! Variable name not found
         CALL Fatal(4)                                               ! Unrecognized variable   
      END SELECT                                                     ! End select input variable 
    ENDDO                                                            ! End loop over input records     
   CLOSE(PhysU)                                                      ! Close physiology input file
   IF (ANY(NVO2MAX(:,:)%P1  ==-9999)) CALL Fatal(5)                  ! Missing distributions in file
   IF (ANY(BMDist(:,:)%P1   ==-9999)) CALL Fatal(6)                  ! Missing distributions in file 
   IF (ANY(RMRINT(:,:)%P1   ==-9999)) CALL Fatal(7)                  ! Missing distributions in file
   IF (ANY(RMRSLP(:,:)%P1   ==-9999)) CALL Fatal(8)                  ! Missing distributions in file
   IF (ANY(RMRERR(:,:)%P1   ==-9999)) CALL Fatal(9)                  ! Missing distributions in file
   IF (ANY(HEIGHTINT(:,:)%P1==-9999)) CALL Fatal(10)                 ! Missing distributions in file
   IF (ANY(HEIGHTSLP(:,:)%P1==-9999)) CALL Fatal(11)                 ! Missing distributions in file
   IF (ANY(HEIGHTERR(:,:)%P1==-9999)) CALL Fatal(12)                 ! Missing distributions in file
   IF (ANY(BSAEXP1(:,:)%P1  ==-9999)) CALL Fatal(13)                 ! Missing distributions in file
   IF (ANY(BSAEXP2(:,:)%P1  ==-9999)) CALL Fatal(14)                 ! Missing distributions in file
   IF (ANY(ECF(:,:)%P1      ==-9999)) CALL Fatal(15)                 ! Missing distributions in file
   IF (ANY(RECTIME(:,:)%P1  ==-9999)) CALL Fatal(16)                 ! Missing distributions in file
   IF (ANY(MAXOXD(:,:)%P1   ==-9999)) CALL Fatal(17)                 ! Missing distributions in file
   IF (ANY(ENDGN1(:,:)%P1   ==-9999)) CALL Fatal(18)                 ! Missing distributions in file
   IF (ANY(ENDGN2(:,2)%P1   ==-9999)) CALL Fatal(19)                 ! Missing distributions in file
   IF (ANY(BLDF1(:,:)%P1    ==-9999)) CALL Fatal(20)                 ! Missing distributions in file
   IF (ANY(BLDF2(:,:)%P1    ==-9999)) CALL Fatal(21)                 ! Missing distributions in file
   IF (ANY(B1(:,:)%P1       ==-9999)) CALL Fatal(24)                 ! Missing distributions in file
   IF (ANY(B2(:,:)%P1       ==-9999)) CALL Fatal(25)                 ! Missing distributions in file
   IF (ANY(B3(:,:)%P1       ==-9999)) CALL Fatal(26)                 ! Missing distributions in file
   IF (ANY(B4(:,:)%P1       ==-9999)) CALL Fatal(27)                 ! Missing distributions in file
   IF (ANY(B5(:,:)%P1       ==-9999)) CALL Fatal(28)                 ! Missing distributions in file
   IF (ANY(B6(:,:)%P1       ==-9999)) CALL Fatal(29)                 ! Missing distributions in file  
   IF (ANY(B7(:,:)%P1       ==-9999)) CALL Fatal(30)                 ! Missing distributions in file             
   IF (ANY(VU(:,:)%P1       ==-9999)) CALL Fatal(31)                 ! Missing distributions in file         
   IF (ANY(VE1(:,:)%P1      ==-9999)) CALL Fatal(32)                 ! Missing distributions in file            
   IF (ANY(VE2(:,:)%P1      ==-9999)) CALL Fatal(33)                 ! Missing distributions in file               
   IF (ANY(FEVSLP(:,:)%P1   ==-9999)) CALL Fatal(34)                 ! Missing distributions in file         
   IF (ANY(FEVINT(:,:)%P1   ==-9999)) CALL Fatal(35)                 ! Missing distributions in file            
   CALL OpenInput(VentU,h=NO)                                        ! Open ventilation input file
   IF (Ios(VentU)/=0) CALL Fatal(22)                                 ! Stop if file not available 
   DO                                                                ! Loop over ventilation records
      CALL GetLine(VentU,String,Ltype)                               ! Get one line from input file
      IF (IOS(VentU)<0) EXIT                                         ! Exit loop at end of file
      READ(String,*) MinAge,MaxAge, buff                             ! Read ages and data from buffer
      DO I=MinAge, MIN(MaxAge,100)                                   ! Loop over age range
         VEB0(I,1:2) = buff(1:2)                                     ! Set intercept terms
         VEB1(I,1:2) = buff(3:4)                                     ! Set slope terms
         VEB2(I,1:2) = buff(5:6)                                     ! Set age terms
         VEB3(I,1:2) = buff(7:8)                                     ! Set gender terms
         VEEB(I)     = buff(9)                                       ! Set interpersonal variance
         VEEW(I)     = buff(10)                                      ! Set intrapersonal variance
      ENDDO                                                          ! Continue with next age 
   ENDDO                                                             ! Continue with next input record
   CLOSE(VentU)                                                      ! Close ventilation input file
   ! IF (DISEASE /= "") THEN                                           ! If modeling disease 
   !  CALL OpenInput(PrevU,h=NO)                                      ! Open prevalence input file
   !  IF (Ios(PrevU)/=0) CALL Fatal(23)                               ! Stop if file not available
   !  DO I=0,100                                                      ! Loop over age range
   !    CALL GetLine(PrevU,String,Ltype)                              ! Get one line from input file
   !    READ(String,*) MinAge, prevbuff                               ! Read ages and data from buffer
   !    Prev(I,1:2) = prevbuff(1:2)                                   ! Set prevalence rates
   !  ENDDO                                                           ! End loop over ages
   ! ENDIF                                                             ! End disease logic  
   ALLOCATE (Phys(NumProfiles),STAT=AllocErr)                        ! Dynamic array for phys. data
   CALL CheckAllocation(AllocErr)                                    ! Check the success of the alloc
   CALL RANDOM_SEED(Put=ISdPhys(1:KSeed))                            ! Initialize physiology seed
   DO P=1, NumProfiles                                               ! Loop over profiles
      CALL RnNor(1,Z,-4D0,4D0)                                       ! Pick a normal for Ve regression 
      CALL RANDOM_NUMBER(Rand)                                       ! Pick three uniform numbers
      Age    = MIN(Person(P)%Age,100)                                ! Set local age variable (<=100)
      Gender = Person(P)%Gender                                      ! Set local gender variable
      K      = Person(P)%HomeSec                                     ! Set local home sector variable
      CALL SampleDist(NVO2MAX(Age,Gender),NVO2MX)                    ! Sample NVO2MAX 
      CALL SampleDist(BMDist(Age,Gender),BM)                         ! Sample body mass 
      CALL SampleDist(RMRINT(Age,Gender),RMRI)                       ! Sample RMR intercept 
      CALL SampleDist(RMRSLP(Age,Gender),RMRS)                       ! Sample RMR slope 
      CALL SampleDist(RMRERR(Age,Gender),RMRE)                       ! Sample error term
      CALL SampleDist(HEIGHTINT(Age,Gender),HTI)                     ! Sample RMR intercept 
      CALL SampleDist(HEIGHTSLP(Age,Gender),HTS)                     ! Sample RMR slope 
      CALL SampleDist(HEIGHTERR(Age,Gender),HTE)                     ! Sample error term
      CALL SampleDist(BSAEXP1(Age,Gender),BSA1)                      ! Sample BSAEXP1
      CALL SampleDist(BSAEXP2(Age,Gender),BSA2)                      ! Sample BSAEXP2
      CALL SampleDist(ECF(Age,Gender),E)                             ! Sample ECF term
      CALL SampleDist(RECTIME(Age,Gender),REC)                       ! Sample recovery time
      CALL SampleDist(B1(Age,Gender),B1VAL)                          ! Sample B1 parameter
      CALL SampleDist(B2(Age,Gender),B2VAL)                          ! Sample B2 parameter
      CALL SampleDist(B3(Age,Gender),B3VAL)                          ! Sample B3 parameter
      CALL SampleDist(B4(Age,Gender),B4VAL)                          ! Sample B4 parameter
      CALL SampleDist(B5(Age,Gender),B5VAL)                          ! Sample B5 parameter
      CALL SampleDist(B6(Age,Gender),B6VAL)                          ! Sample B6 parameter
      CALL SampleDist(B7(Age,Gender),B7VAL)                          ! Sample B7 parameter
      CALL SampleDist(VU(Age,Gender),VUVAL)                          ! Sample FEVU parameter                                        
      CALL SampleDist(FEVSLP(Age,Gender),FSV)                        ! Sample FEVSLP parameter   
      CALL SampleDist(FEVINT(Age,Gender),FIV)                        ! Sample FEVINT parameter               
      CALL SampleDist(HMG(Age,Gender),HG)                            ! Sample max O2 deficit
      CALL SampleDist(MAXOXD(Age,Gender),MXD)                        ! Sample max O2 deficit
      CALL SampleDist(ENDGN1(Age,Gender),ENDCO1)                     ! Sample endogenous CO 1
      CALL SampleDist(ENDGN2(Age,Gender),ENDCO2)                     ! Sample endogenous CO 2  
      CALL SampleDist(BLDF1(Age,Gender), BF1)                        ! Sample blood factor 1 
      CALL SampleDist(BLDF2(Age,Gender), BF2)                        ! Sample blood factor 2  
      Phys(P)%VO2MAX  = NVO2MX(1) * BM(1) * 0.001                    ! Set VO2MX (l-O2/min)
      Phys(P)%BM      = BM(1)                                        ! Set global body mass parameter
      Phys(P)%Weight  = 2.2046 * BM(1)                               ! Set global body weight (pounds)
      WEIGHT          = Phys(P)%Weight                               ! Local variable for weight (lbs)
      RMR             = RMRS(1)*BM(1)+RMRI(1)+RMRE(1)                ! Resting metabolic rate (MJ/day)   
      Phys(P)%RMR     = 0.166 * RMR                                  ! Units (1 MJ/day=0.166 kcal/min)
      Phys(P)%RMR     = MAX(Phys(P)%RMR     , 0.1  )                 ! Absolute min on RMR
      Phys(P)%RMR     = MIN(Phys(P)%RMR     , 2.0  )                 ! Absolute max on RMR            
      Phys(P)%Endgn1  = ENDCO1(1)/60                                 ! Set endogenous CO rate 1
      Phys(P)%Endgn2  = ENDCO2(1)/60                                 ! Set endogenous CO rate 2   
      Phys(P)%MOXD    = MXD(1)                                       ! Set global variable for MOXD
      IF (Gender==MALE)  Phys(P)%Endgn2 = Phys(P)%Endgn1             ! Rates for males don't change
      IF (Age < 18 ) THEN                                            ! If person is a child   
         Phys(P)%Height  = HTS(1)*Age + HTI(1) + HTE(1)              ! Height is f(age) for children        
      ELSE                                                           ! Else if person is an adult
         Phys(P)%Height  = HTS(1)*LOG(WEIGHT)+ HTI(1) + HTE(1)       ! Height is f(weight) for adults        
      ENDIF                                                          ! End height logic     
      Phys(P)%Height   = MAX(Phys(P)%Height  , 22.0 )                ! Absolute min on Height
      Phys(P)%Height   = MIN(Phys(P)%Height  , 82.0 )                ! Absolute max on Height            
      Phys(P)%BMI     = (Phys(P)%Weight * 703)/((Phys(P)%Height)**2)  ! Calculate BMI
      Phys(P)%Blood  = WEIGHT*BF1(1)+ BF2(1)*Phys(P)%Height**3 - 30. ! Blood volume is f(height,weight)
      Phys(P)%Blood    = MAX(Phys(P)%Blood   , 300.0)                ! Absolute min on Blood
      Phys(P)%Blood    = MIN(Phys(P)%Blood   , 12000.0)              ! Absolute max on Blood             
      Phys(P)%BSA    = exp(BSA1(1))*(BM(1)**(BSA2(1)))               ! Body surface area (m^2)   
      Phys(P)%ECF    = E(1)                                          ! Energy conversion (l-O2/kcal)
      Phys(P)%MetsMax = Phys(P)%VO2Max / (Phys(P)%ECF*Phys(P)%RMR)   ! Maximum allowed METS value
      Phys(P)%METSMax  = MAX(Phys(P)%METSMax  , 5.0 )                ! Absolute min on METSMax
      Phys(P)%METSMax  = MIN(Phys(P)%METSMax  , 20.0)                ! Absolute max on METSMax            
      Phys(P)%RecTime = REC(1)                                       ! Hours to recover maximum O2 Debt
      Phys(P)%Hmglb   = HG(1)                                        ! Set hemoglobin content  
      Phys(P)%Beta1   = B1VAL(1)                                     ! Set B1 parameter
      Phys(P)%Beta2   = B2VAL(1)                                     ! Set B2 parameter
      Phys(P)%Beta3   = B3VAL(1)                                     ! Set B3 parameter
      Phys(P)%Beta4   = B4VAL(1)                                     ! Set B4 parameter
      Phys(P)%Beta5   = B5VAL(1)                                     ! Set B5 parameter
      Phys(P)%Beta6   = B6VAL(1)                                     ! Set B6 parameter
      Phys(P)%Beta7   = B7VAL(1)                                     ! Set B7 parameter
      Phys(P)%FEVU    = VUVAL(1)                                     ! Set FEVU parameter  
      Phys(P)%FEVSLP  = FSV(1)                                       ! Set FEVSLP parameter
      Phys(P)%FEVINT  = FIV(1)                                       ! Set FEVINT parameter
      Phys(P)%VEslope = VEB1(Age,1)                                  ! Slope for VE/VO2 regression
      Phys(P)%VEresid = VEEW(Age)                                    ! Residual for VE/VO2 regression
      VEinter = VEB0(Age,1)+VEEB(Age)*Z(1)+VEB2(Age,1)*LOG(1.+Age)   ! Constant terms in regression      
      IF(Gender==MALE) THEN                                          ! Physiology for males
         Phys(P)%Diff = .361*Phys(P)%Height - .232*REAL(Age) + 16.3  ! Pulmonary diffusion rate
         Phys(P)%VEinter= VEinter-VEB3(Age,1)                        ! Adjust constant term for gender
      ELSE                                                           ! Else physiology for females
         Phys(P)%Diff = .556*Phys(P)%Height - .115*REAL(Age) - 5.97  ! Pulmonary diffusion rate
         IF(Age>=12.AND.Age<=50) Phys(P)%Start = 1+INT(28.*Rand(2))  ! Phase for menstrual cycle
         Phys(P)%VEinter= VEinter+VEB3(Age,1)                        ! Adjust constant term for gender
      ENDIF                                                          ! End of physiology for females
      Phys(P)%VEinter  = MAX(Phys(P)%VEinter , 1.1  )                ! Absolute min on VEinter
      Phys(P)%Diff     = MAX(Phys(P)%Diff    , 7.0  )                ! Absolute min on Diff
      Phys(P)%VEinter  = MIN(Phys(P)%VEinter , 5.0  )                ! Absolute max on VEinter
      Phys(P)%Diff     = MIN(Phys(P)%Diff    , 42.0 )                ! Absolute max on Diff            
      IF (Disease /= "") THEN                                        ! If modeling disease
        IF(Rand(3)<= DisProbs(K,DisGroup(Age,Gender))) THEN          ! If within prevalence probab.
           Phys(P)%Ill =YES                                          ! Person is ill  
        ELSE                                                         ! Otherwise
           Phys(P)%Ill =NO                                           ! Person is normal
        ENDIF                                                        ! End prevalence logic
      ENDIF                                                          ! End disease logic
   ENDDO                                                             ! Continue with next profile
   DEALLOCATE (NVO2MAX, BMDist, MAXOXD, HMG, RMRSLP, RMRINT, RMRERR) ! Deallocate internal arrays
   DEALLOCATE (HEIGHTSLP, HEIGHTINT, HEIGHTERR, BLDF1, BLDF2)        ! Deallocate internal arrays
   DEALLOCATE (BSAEXP1, BSAEXP2, ENDGN1, ENDGN2, ECF, RECTIME)       ! Deallocate internal arrays
   DEALLOCATE (B1, B2, B3, B4, B5, B6, B7, VU, FEVSLP, FEVINT)       ! Deallocate internal arrays
   PRINT*, "Finished GeneratePhysiology"                             ! Status message for screen
   IF (DebugLevel>0) WRITE(LogU,LV_FMT) LV_MSG, ProcName(ProcLevel)  ! Message for debugging
   ProcLevel = ProcLevel-1                                           ! Decrement procedure depth
END SUBROUTINE GeneratePhysiology                                    ! Return to main APEX program

SUBROUTINE FEVEEval(Age, Gender)                                     ! Called from CompositeDiary
   REAL    (KIND=RK) :: VE1VAL(1), VE2VAL(1)                         ! Local real variables
   INTEGER (KIND=IK) :: I                                            ! Local integer variables
   INTEGER (KIND=IK), INTENT(IN) :: Age, Gender                      ! Input variable (age, gender)
   ! Global constants: IK, RK, SL_FM3, ST_MS3, LV_MS3
   ! Global vars used: DebugLevel, NumHours, NumDays
   ! Global vars set : HourSeq, ProcLevel, ProcName
   ! Module constants: POINT, UNIFORM, NORMAL, LOGNORMAL, TRIANGLE, EXPONENTIAL
   ! Module vars used: VE1, VE2
   ! Intrinsic procs : EXP, LOG, SQRT, RANDOM_NUMBER, RANDOM_SEED, WRITE
   ProcLevel = ProcLevel+1                                           ! Increment procedure depth
   ProcName(ProcLevel) = "FEVEEval"                                  ! Procedure name for messages
   IF (DebugLevel>1) WRITE(LogU,SL_FM3) ST_MS3, ProcName(ProcLevel)  ! Message for debugging
   IF (HourlyFEVE1Flag == YES) THEN                                  ! If sampling hourly
      DO I=1,NumHours                                                ! Loop over hours
         CALL SampleDist(VE1(Age,Gender),VE1VAL)                     ! Sample FEVE1 parameter       
         HourSeq(I)%FEVE1 = VE1VAL(1)                                ! Set E1 hourly value
      ENDDO                                                          ! End loop over hours
   ELSE                                                              ! Else if value equals no
      DO I=1,NumDays                                                 ! Loop over days
         CALL SampleDist(VE1(Age,Gender),VE1VAL)                     ! Sample FEVE1 parameter       
         HourSeq(1+(I-1)*24:I*24)%FEVE1 = VE1VAL(1)                  ! Set E1 daily value
      ENDDO                                                          ! End loop over days
   ENDIF                                                             ! End E1 term logic
   IF (HourlyFEVE2Flag == YES) THEN                                  ! If sampling hourly
      DO I=1,NumHours                                                ! Loop over hours
         CALL SampleDist(VE2(Age,Gender),VE2VAL)                     ! Sample FEVE2 parameter       
         HourSeq(I)%FEVE2 = VE2VAL(1)                                ! Set E2 hourly value
      ENDDO                                                          ! End loop over hours
   ELSE                                                              ! Else if value equals no
      DO I=1,NumDays                                                 ! Loop over days
         CALL SampleDist(VE2(Age,Gender),VE2VAL)                     ! Sample FEVE2 parameter       
         HourSeq(1+(I-1)*24:I*24)%FEVE2 = VE2VAL(1)                  ! Set E2 daily value
      ENDDO                                                          ! End loop over days
   ENDIF                                                             ! End E2 term logic
   IF (DebugLevel>1) WRITE(LogU,SL_FM3) LV_MS3, ProcName(ProcLevel)  ! Message for debugging
   ProcLevel = ProcLevel-1                                           ! Decrement procedure depth   
END SUBROUTINE FEVEEval                                              ! Return to CompositeDiary


SUBROUTINE GenerateDailyVars(P)                                      ! Called from main APEX program
   INTEGER (KIND=IK), INTENT(IN) :: P                                ! Input variable (person number)
   INTEGER (KIND=IK), POINTER :: I1(:), I2(:), I3(:)                 ! Pointers for use with Distrib
   INTEGER (KIND=IK), TARGET  :: ACBuffer(NumDays)                   ! Automatic array for AC status
   CHARACTER(LEN=LEN2) :: FN                                         ! Buffer for distribution name
   ! Global constants: IK, LEN2, LogU, LV_MS2, SL_FM2, ST_MS2
   ! Global vars used: DebugLevel, KSeed, Person, SList, TemData
   ! Global vars set : PersonDay
   ! Intrinsic procs : RANDOM_SEED, WRITE
   ! APEX procedures : Distrib
   ProcLevel = ProcLevel+1                                           ! Increment procedure depth
   ProcName(ProcLevel) = "GenerateDailyVars"                         ! Procedure name for messages
   IF (DebugLevel>1) WRITE(LogU,SL_FM2) ST_MS2, ProcName(ProcLevel)  ! Message for debugging
   CALL RANDOM_SEED(Put=ISdDay(1:KSeed))                             ! Set random number seed
   FN = "WINDOWRES"                                                  ! Set WindowRes distribution name
   IF (FuncNum(FN,NO)>0) THEN                                        ! If WindowRes is defined 
      ACBuffer(:) = Person(P)%AC_Home                                ! One copy of AC_Home per day
      I1 => ACBuffer                                                 ! Point to daily AC_Home data
      I2 => TemData(:,SList(Person(P)%HomeSec)%NumZ)%MaxTemp         ! Point to daily MaxTemp data
      I3 => TemData(:,SList(Person(P)%HomeSec)%NumZ)%AvgTemp         ! Point to daily AvgTemp data
      CALL EvalFunc(Name=FN,I1=I1,I2=I2,I3=I3,                      &
         IRes=PersonDay(1:NumDays)%WindowRes)                        ! Determine daily window positions
   ENDIF                                                             ! End of WindowRes logic block 
   FN = "WINDOWCAR"                                                  ! Set WINDOWCAR distribution name
   IF (FuncNum(FN,NO)>0)  THEN                                       ! If WindowCar is defined 
      ACBuffer(:) = Person(P)%AC_Car                                 ! One copy of AC_Car per day
      I1 => ACBuffer                                                 ! Point to daily AC_Car data
      I2 => TemData(:,SList(Person(P)%HomeSec)%NumZ)%MaxTemp         ! Point to daily MaxTemp data
      I3 => TemData(:,SList(Person(P)%HomeSec)%NumZ)%AvgTemp         ! Point to daily AvgTemp data
      CALL EvalFunc(Name=FN,I1=I1,I2=I2,I3=I3,                      &
         IRes=PersonDay(1:NumDays)%WindowCar)                        ! Determine daily window positions
   ENDIF                                                             ! End of WindowCar logic block
   FN = "SPEEDCAT"                                                   ! Set SPEEDCAT distribution name 
   IF (FuncNum(FN,NO)>0) THEN                                        ! If SpeedCat is defined 
      CALL EvalFunc(Name=FN,IRes=PersonDay(1:NumDays)%SpeedCat)      ! Determine daily speed categories
   ENDIF                                                             ! End of SpeedCat logic block
   FN = "DAILYCONDITIONAL1"                                          ! Set COND1 distribution name 
   IF (FuncNum(FN,NO)>0) THEN                                        ! If Conditional1 is defined 
      CALL EvalFunc(Name=FN,IRes=PersonDay(1:NumDays)%DCond1)        ! Determine daily cond1 categories  
   ENDIF                                                             ! End of Conditional1 logic block
   FN = "DAILYCONDITIONAL2"                                          ! Set COND2 distribution name 
   IF (FuncNum(FN,NO)>0) THEN                                        ! If Conditional2 is defined 
      CALL EvalFunc(Name=FN,IRes=PersonDay(1:NumDays)%DCond2)        ! Determine daily cond2 categories
   ENDIF                                                             ! End of Conditional2 logic block
   FN = "DAILYCONDITIONAL3"                                          ! Set COND3 distribution name 
   IF (FuncNum(FN,NO)>0) THEN                                        ! If Conditional3 is defined 
      CALL EvalFunc(Name=FN,IRes=PersonDay(1:NumDays)%DCond3)        ! Determine daily cond3 categories
   ENDIF                                                             ! End of Conditional3 logic block
   IF (DebugLevel>1) WRITE(LogU,SL_FM2) LV_MS2, ProcName(ProcLevel)  ! Message for debugging
   ProcLevel = ProcLevel-1                                           ! Decrement procedure depth
END SUBROUTINE GenerateDailyVars                                     ! Return to main APEX program

END MODULE ProfileModule                                             ! End of ProfileModule source code
