MODULE UtilityModule                                                 ! Compile after GlobalModule
   ! Latest revisions by MCN on November 10, 2011
   USE GlobalModule                                                  ! Compile this module first
   IMPLICIT NONE                                                     ! Must declare all variables

CONTAINS                                                             ! Routines internal to this module

SUBROUTINE OpenInput(U,recl,name,preblank,postblank,h,header)        ! Called from many places
   INTEGER (KIND=IK),             INTENT(IN) :: U                    ! Input file unit number
   INTEGER (KIND=IK),   OPTIONAL, INTENT(IN) :: recl                 ! Input record length (optional)
   CHARACTER(LEN=LEN3), OPTIONAL, INTENT(IN) :: name                 ! Input file name (optional)
   INTEGER (KIND=IK),   OPTIONAL, INTENT(IN) :: preblank             ! YES/NO for blank line on log
   INTEGER (KIND=IK),   OPTIONAL, INTENT(IN) :: postblank            ! Same, but afterwards
   INTEGER (KIND=IK),             INTENT(IN) :: h                    ! YES/NO for returning header
   CHARACTER(LEN=LEN3), OPTIONAL,INTENT(OUT) :: header               ! Header line from file
   INTEGER (KIND=IK)   :: len, blank1, blank2,nin                    ! Local variables for arguments
   CHARACTER(LEN=LEN3) :: file, line                                 ! Local variable for file name
   CHARACTER(LEN=LEN4) :: Buff, Wrap                                 ! Local buffers for input strings
   CHARACTER(LEN=6)    :: Indent                                     ! Padding to indent text on log
   ! Global constants: FMT4, IK, LEN3, NO, YES  
   ! Global vars used: FName
   ! Global vars set : Ios, ProcLevel, ProcName
   ! Intrinsic procs : ADJUSTL, BACKSPACE, LEN_TRIM, OPEN, PRESENT, READ, TRIM, WRITE
   ! APEX procedures : Fatal
   ProcLevel = ProcLevel+1                                           ! Increment procedure depth
   ProcName(ProcLevel) = "OpenInput"                                 ! Procedure name for messages
   indent = '      '                                                 ! Set padding to blank
   nin = 3*(ProcLevel-2)                                             ! Amount of indentation    
   len = LEN3                                                        ! Default record length is LEN3
   IF (PRESENT(recl)) len = recl                                     ! Set record length if supplied
   file = FName(U)                                                   ! Default file name for unit #
   IF (PRESENT(name)) file = name                                    ! Set file name (pop files only)
   blank1 = NO                                                       ! Default is preblank=NO
   IF (PRESENT(preblank))  blank1 = preblank                         ! Set preblank if requested 
   blank2 = NO                                                       ! Default is postblank=NO
   IF (PRESENT(postblank)) blank2 = postblank                        ! Set postblank if requested 
   IF (LogIsOpen==YES) THEN                                          ! If log file has been opened
      IF (blank1==YES) WRITE(LogU,*)                                 ! Write blank line to log
      WRITE(LogU,*) '--------------------------------------------'   ! Separate log file visually
      WRITE(LogU,*) indent(1:nin),'Opening input file, unit # ',U    ! Report unit number
      WRITE(LogU,*) indent(1:nin),'Filename = ', TRIM(file)          ! Report file name
   ENDIF                                                             ! End log file open logic           
   OPEN (U,FILE=file,IOSTAT=Ios(U),ACTION="READ",RECL=len)           ! OPEN the file
   IF (Ios(U)/=0) CALL Fatal(1)                                      ! Stop if unable to open file
   line = ""                                                         ! Initialize header to blank line
   DO                                                                ! Loop until EXIT
      READ(U,FMT4) Buff                                              ! Read line from file
      Wrap = ADJUSTL(Buff)                                           ! Trim leading spaces
      IF (LEN_TRIM(Wrap)==0) CYCLE                                   ! If line is blank, get another
      IF (line=="") line = Wrap                                      ! Set header if not already set  
      IF (Wrap(1:1)/="!") EXIT                                       ! Exit loop if not a comment 
      IF (LogIsOpen==YES) WRITE(LogU,*) indent(1:nin),TRIM(Buff)     ! Write comment line to log
   ENDDO                                                             ! Continue with next input line
   IF (blank2==YES .AND. LogIsOpen==YES) WRITE(LogU,*)               ! Put blank line on log 
   BACKSPACE(U)                                                      ! Position input one line back
   IF (h==YES) header = line                                         ! Save header line if requested
   ProcLevel = ProcLevel-1                                           ! Decrement procedure depth
END SUBROUTINE OpenInput                                             ! Return to calling program

SUBROUTINE OpenOutput(U,recl,PL)                                     ! Called from many places
   INTEGER (KIND=IK),             INTENT(IN) :: U                    ! Input file unit number
   INTEGER (KIND=IK),   OPTIONAL, INTENT(IN) :: recl                 ! Input record length (optional)
   INTEGER (KIND=IK),   OPTIONAL, INTENT(IN) :: PL                   ! Pollutant # (optional)
   INTEGER (KIND=IK)   :: len, N, I, FILEU                           ! Local integer variables
   CHARACTER(LEN=LEN3) :: Header(7)                                  ! Store six header lines
   CHARACTER(LEN=LEN3) :: FN                                         ! Local copy of file name
   CHARACTER(LEN=LEN4) :: TabHeader(10)                              ! Store table header lines
   CHARACTER(LEN=7)    :: wr                                         ! Status for file writeability
   CHARACTER(LEN=40)   :: FMT                                        ! Write format for table header
   ! Global vars used: FName       
   ! Global vars set : Ios, ProcLevel, ProcName
   ProcLevel = ProcLevel+1                                           ! Increment procedure depth
   ProcName(ProcLevel) = "OpenOutput"                                ! Set procedure name 
   len = LEN3                                                        ! Default record length
   IF (PRESENT(recl)) len=recl                                       ! Use supplied record length
   IF (U==LogU)   Header(1) = " APEX Log File"                       ! Header for log file 
   IF (U==EventU) Header(1) = " APEX Events File"                    ! Header for events file
   IF (U==PSumU)  Header(1) = " APEX Personal Summary File"          ! Header for psum file
   IF (U==MSumU)  Header(1) = " APEX Microenvironmental Summary File"! Header for msum file
   IF (U==MResU)  Header(1) = " APEX Microenvironmental Results File"! Header for mres file
   IF (U==TabU)   Header(1) = " APEX Tables File"                    ! Header for table file
   IF (U==SiteU)  Header(1) = " APEX Sites File"                     ! Header for sites file
   IF (U==DayU)   Header(1) = " APEX Daily File"                     ! Header for daily file
   IF (U==HourU)  Header(1) = " APEX Hourly File"                    ! Header for hourly file
   IF (U==TSU)    Header(1) = " APEX Timestep File"                  ! Header for timestep file
   WRITE(Header(2),'(A, A, A, A, A, A6)') " APEX Version ",         &
   TRIM(Version), "  Run Date = ", TRIM(DateTime(1)),"  Time = ",   &
   DateTime(2)                                                       ! Save header line 1
   Header(3) = " Location    = "//ADJUSTL(Location)                  ! Save header line 2
   Header(4) = " Scenario    = "//ADJUSTL(Scenario)                  ! Save header line 4
   Header(5) = " Simulation  = "//ADJUSTL(ParamsHeader(2:))          ! Save header line 5
   Header(6) = " Pollutant   = "//ADJUSTL(TRIM(PolList))             ! Save header line 7                                                             ! End of log file logic
   IF (U==TabU.OR.U==MResU.OR.U==MSumU) THEN                         ! If pollutant specific output              
      Header(6) = " Pollutant   = "//ADJUSTL(TRIM(Pol(PL)%Name))     ! Write pollutant name
   ELSE                                                              ! For all other files
      Header(6) = " Pollutant   = "//ADJUSTL(TRIM(PolList))          ! Write pollutant list
   ENDIF                                                             ! End file logic                                                                   
   FILEU = U                                                         ! Initialize unit #
   FN    = FName(U)                                                  ! Initialize file name
   IF (U==MSumU) THEN                                                ! If this is a MSumU file
      FILEU = Pol(PL)%MSumU                                          ! Pollutant-specific unit #  
      FN    = Pol(PL)%MSumName                                       ! Pollutant-specific name                           
   ENDIF                                                             ! End MSumU file logic
   IF (U==MResU) THEN                                                ! If this is a MResU file
      FILEU = Pol(PL)%MResU                                          ! Pollutant-specific unit #  
      FN    = Pol(PL)%MResName                                       ! Pollutant-specific name   
   ENDIF                                                             ! End MResU file logic
   IF (U==TabU)  THEN                                                ! If this is a TabU file
      FILEU = Pol(PL)%TabU                                           ! Pollutant-specific unit #  
      FN    = Pol(PL)%TabName                                        ! Pollutant-specific name  
   ENDIF                                                             ! End TabU file logic
   DO                                                                ! Loop until file opens correctly
      OPEN(FILEU,File=FN,IOSTAT=Ios(U),RECL=len,ACTION='WRITE')      ! Open output file
      INQUIRE(FILE=FN,WRITE=wr)                                      ! Check if file can be written
      IF (wr=="YES" .AND. Ios(U)==0)  EXIT                           ! If so, then exit DO loop 
      CALL Fatal(U)                                                  ! Write error    
   ENDDO                                                             ! End of logic block  
   len = MIN(len,LEN3)                                               ! Length of header lines needed
   WRITE(FILEU,'(A)') TRIM(Header(1)(1:len))                         ! Write header 1 to output
   WRITE(FILEU,'(A)') TRIM(Header(2)(1:len))                         ! Write header 2 to output
   WRITE(FILEU,'(A)') TRIM(Header(3)(1:len))                         ! Write header 3 to output
   WRITE(FILEU,'(A)') TRIM(Header(4)(1:len))                         ! Write header 4 to output
   WRITE(FILEU,'(A)') TRIM(Header(5)(1:len))                         ! Write header 5 to output
   WRITE(FILEU,'(A)') TRIM(Header(6)(1:len))                         ! Write header 6 to output
   IF (FILEU > MAXU) THEN                                            ! If this is a pollut-spec file
      Header(7) = " Air Quality = "//ADJUSTL(AirHeader(PL))          ! Save AQ header
      WRITE(FILEU,'(A)') TRIM(Header(7)(1:len))                      ! Write AQ header to output
   ELSE                                                              ! Otherwise file is not specific   
      DO I=1,NumPollutants                                           ! Loop over pollutants
         Header(7) = " Air Quality = "//ADJUSTL(AirHeader(I))        ! Save AQ header
         WRITE(FILEU,'(A)') TRIM(Header(7)(1:len))                   ! Write all AQ headers to output
      ENDDO                                                          ! End loop over pollutants
   ENDIF                                                             ! End pollut-specific logic
   IF (U==TabU) THEN                                                 ! For table file only
      WRITE(TabHeader(1),'(A,I)')" Number of Micros   = ",          &
         NumMicros                                                   ! Write num micros to header  
      WRITE(TabHeader(2),'(A,I)')" Number of Profiles = ",          &
         NumProfiles                                                 ! Write num prof to header 
      WRITE(TabHeader(3),'(A,I)')" Area Population    = " ,         &
         TotPop                                                      ! Write area pop to header
      WRITE(TabHeader(4),'(A,A)')" Exposure Units    = ",           &
      "        "//UNITTEXT(Pol(PL)%OutUnits)                         ! Write units to header
      N  = Pol(PL)%N_Pctl
      WRITE(FMT, '(A,I,A)') "(A25,",N,"F10.3)"                       ! Write format string
      WRITE(TabHeader(5), FMT) " Percentiles            = ",        &
      (Pctl(PL,I),I=1,N)                                             ! Write percentiles  
      N  = Pol(PL)%N_DM1HExpLev                                      ! Number of 1-hour levels
      WRITE(FMT, '(A,I,A)') "(A25,",N,"F10.3)"                       ! Write format string 
      WRITE(TabHeader(6), FMT) " 1-Hour Exposure Levels =",         &
         (DM1HExpLev(PL,I),I=1,N)                                    ! Write levels  
      N  = Pol(PL)%N_DM8HExpLev                                      ! Number of levels to print
      WRITE(FMT, '(A,I,A)') "(A25,",N,"F10.3)"                       ! Write format string
      WRITE(TabHeader(7), FMT) " 8-Hour Exposure Levels =",         &
         (DM8HExpLev(PL,I),I=1,N)                                    ! Write levels  
      N  = Pol(PL)%N_DAvgExpLev                                      ! Number of levels to print
      WRITE(FMT, '(A,I,A)') "(A25,",N,"F10.3)"                       ! Write format string
      WRITE(TabHeader(8), FMT) " Daily  Exposure Levels =",         & 
      (DAvgExpLev(PL,I),I=1,N)                                       ! Write levels    
      N  = Pol(PL)%N_SAvgExpLev                                      ! Number of level to print
      WRITE(FMT, '(A,I,A)') "(A25,",N,"F10.3)"                       ! Write format string
      WRITE(TabHeader(9), FMT) " Annual Exposure Levels =",         & 
         (SAvgExpLev(PL,I),I=1,N)                                    ! Write levels         
      N  = Pol(PL)%N_TimeExpLev                                      ! Number of level to print
      WRITE(FMT, '(A,I,A)') "(A25,",N,"F10.3)"                       ! Write format string
      WRITE(TabHeader(10), FMT)" Time   Exposure Levels =",         & 
         (TimeExpLev(PL,I),I=1,N)                                    ! Write levels
      WRITE(FILEU,'(A)') TRIM(TabHeader(1)(1:len))                   ! Write table head 1 to output
      WRITE(FILEU,'(A)') TRIM(TabHeader(2)(1:len))                   ! Write table head 2 to output
      WRITE(FILEU,'(A)') TRIM(TabHeader(3)(1:len))                   ! Write table head 3 to output
      WRITE(FILEU,'(A)') TRIM(TabHeader(4)(1:len))                   ! Write table head 4 to output
      WRITE(FILEU,'(A)') TRIM(TabHeader(5)(1:len))                   ! Write table head 5 to output
      WRITE(FILEU,'(A)') TRIM(TabHeader(6)(1:len))                   ! Write table head 6 to output
      WRITE(FILEU,'(A)') TRIM(TabHeader(7)(1:len))                   ! Write table head 7 to output
      WRITE(FILEU,'(A)') TRIM(TabHeader(8)(1:len))                   ! Write table head 8 to output
      WRITE(FILEU,'(A)') TRIM(TabHeader(9)(1:len))                   ! Write table head 9 to output
      WRITE(FILEU,'(A)') TRIM(TabHeader(10)(1:len))                  ! Write table head 10 to output
   ENDIF                                                             ! End table file logic
   IF (U==MResU) THEN                                                ! For micro results file only
      WRITE(FILEU,'(A, I8)') " Simulation Start Date = ", StartDate  ! Write run start date    
   ENDIF                                                             ! End micro results file logic
   WRITE(FILEU,*)                                                    ! Write blank line to output 
   ProcLevel = ProcLevel-1                                           ! Decrement procedure depth
END SUBROUTINE OpenOutput                                            ! Return to calling routine

SUBROUTINE GetLine(UnitNum,String,LType)                             ! Called from many places
   INTEGER   (KIND=IK),  INTENT(IN)  :: UnitNum                      ! Input is unit number for read
   CHARACTER (LEN=LEN4), INTENT(OUT) :: String                       ! Output is one record
   INTEGER   (KIND=IK),  INTENT(OUT) :: LType                        ! Output is record type 
   INTEGER   (KIND=IK) :: I                                          ! Local integer variable
   ! Global constants: FMT4, IK, LEN4  
   ! Global vars set : Ios
   ! Intrinsic procs : ADJUSTL, INDEX, LEN_TRIM, READ
   ! Apex procedures : Fatal
   DO                                                                ! Loop until RETURN or EXIT
      READ(UnitNum,FMT4,IOSTAT=Ios(UnitNum)) String                  ! Read one line from input file
      IF (MEFlag == 1) LineNumM = LineNumM+1                         ! Incrementing ME file line counter
      IF (Ios(UnitNum)<0) RETURN                                     ! Exit subroutine on end-of-file 
      IF (Ios(UnitNum)>0) CALL Fatal(0)                              ! Error message for file reading
      String = ADJUSTL(String)                                       ! Trim leading blanks
      I = INDEX(String,'!')                                          ! Locate '!' in input line
      IF (I==1) CYCLE                                                ! Skip comment lines
      IF (I>1) String = String(1:I-1)                                ! Strip off trailing comments
      IF (LEN_TRIM(String)>0) EXIT                                   ! If anything found, exit loop
   ENDDO                                                             ! Continue to read input file
   LType = -1                                                        ! Default line type is numeric
   IF (INDEX("-.1234567890",String(1:1))>0) RETURN                   ! Return if first char is number
   LType = INDEX(String,"=")                                         ! Else locate '=' sign in line
END SUBROUTINE GetLine                                               ! Return to calling routine

SUBROUTINE GetLineLong(UnitNum,String,LType)                         ! Called from ReadAQ
   INTEGER   (KIND=IK),  INTENT(IN)  :: UnitNum                      ! Input is unit number for read
   CHARACTER (LEN=LEN5), INTENT(OUT) :: String                       ! Output is one record
   INTEGER   (KIND=IK),  INTENT(OUT) :: LType                        ! Output is record type 
   INTEGER   (KIND=IK) :: I                                          ! Local integer variable
   ! Global constants: FMT4, IK, LEN4  
   ! Global vars set : Ios
   ! Intrinsic procs : ADJUSTL, INDEX, LEN_TRIM, READ
   ! Apex procedures : Fatal
   DO                                                                ! Loop until RETURN or EXIT
      READ(UnitNum,FMT5,IOSTAT=Ios(UnitNum)) String                  ! Read one line from input file
      IF (Ios(UnitNum)<0) RETURN                                     ! Exit subroutine on end-of-file 
      IF (Ios(UnitNum)>0) CALL Fatal(0)                              ! Error message for file reading
      String = ADJUSTL(String)                                       ! Trim leading blanks
      I = INDEX(String,'!')                                          ! Locate '!' in input line
      IF (I==1) CYCLE                                                ! Skip comment lines
      IF (I>1) String = String(1:I-1)                                ! Strip off trailing comments
      IF (LEN_TRIM(String)>0) EXIT                                   ! If anything found, exit loop
   ENDDO                                                             ! Continue to read input file
   LType = -1                                                        ! Default line type is numeric
   IF (INDEX("-.1234567890",String(1:1))>0) RETURN                   ! Return if first char is number
   LType = INDEX(String,"=")                                         ! Else locate '=' sign in line
END SUBROUTINE GetLineLong                                           ! Return to calling routine

CHARACTER (LEN=LEN4) FUNCTION Up(String)                             ! Called from many places
   CHARACTER (LEN=*), INTENT(IN) :: String                           ! Input is a character string
   INTEGER   (KIND=IK) :: I, J                                       ! Local integer variables
   ! Global constants: IK, LEN3
   ! Intrinsic procs : ACHAR, IACHAR, LEN_TRIM
   Up=""                                                             ! Default return value is null
   DO I=1, LEN_TRIM(String)                                          ! Loop over chars in input
      J = IACHAR(String(I:I))                                        ! ASCII code for current char
      IF (J>96.AND.J<123) J=J-32                                     ! Make lowercase letters uppercase
      Up(I:I) = ACHAR(J)                                             ! Translate ASCII back to char
   ENDDO                                                             ! Continue with next char
END FUNCTION Up                                                      ! Return to calling routine

CHARACTER (LEN=LEN5) FUNCTION UpLong(String)                         ! Called from ReadAirQuality
   CHARACTER (LEN=*), INTENT(IN) :: String                           ! Input is a character string
   INTEGER   (KIND=IK) :: I, J                                       ! Local integer variables
   ! Global constants: IK, LEN3
   ! Intrinsic procs : ACHAR, IACHAR, LEN_TRIM
   UpLong=""                                                         ! Default return value is null
   DO I=1, LEN_TRIM(String)                                          ! Loop over chars in input
      J = IACHAR(String(I:I))                                        ! ASCII code for current char
      IF (J>96.AND.J<123) J=J-32                                     ! Make lowercase letters uppercase
      UpLong(I:I) = ACHAR(J)                                         ! Translate ASCII back to char
   ENDDO                                                             ! Continue with next char
END FUNCTION UpLong                                                  ! Return to calling routine

INTEGER (KIND=IK) FUNCTION CountNums(String)                         ! Counts numbers on an input line
   CHARACTER (LEN=*), INTENT(IN) :: String                           ! Input is a character string
   INTEGER   (KIND=IK) :: I, J, K                                    ! Local integer variables 
   ! Global constants: IK
   ! Intrinsic procs : IACHAR, LEN_TRIM
   CountNums = 0                                                     ! Default return value is zero
   K = 32                                                            ! Start with ASCII for a space
   DO I=1, LEN_TRIM(String)                                          ! Loop over chars in input
      J = IACHAR(String(I:I))                                        ! ASCII code for current char
      IF (K==32.AND.J>44.AND.J<58.AND.J.NE.47) CountNums=CountNums+1 ! Start of another number
      K = J                                                          ! Reset value for prior cell
   ENDDO                                                             ! Continue with next char
END FUNCTION CountNums                                               ! Return to calling routine

INTEGER (KIND=IK) FUNCTION CountWords(String)                        ! Counts words on an input line
   CHARACTER (LEN=*), INTENT(IN) :: String                           ! Input is a character string
   INTEGER   (KIND=IK) :: I, J, K                                    ! Local integer variables 
   ! Global constants: IK
   ! Intrinsic procs : IACHAR, LEN_TRIM
   CountWords = 0                                                    ! Default return value is zero
   K = 32                                                            ! Start with ASCII for a space
   DO I=1, LEN_TRIM(String)                                          ! Loop over chars in input
      J = IACHAR(String(I:I))                                        ! ASCII code for current char
      IF (K==32.AND.J.NE.32)    CountWords=CountWords+1              ! Start of another word
      K = J                                                          ! Reset value for prior cell
   ENDDO                                                             ! Continue with next char
END FUNCTION CountWords    

SUBROUTINE Fatal(N)                                                  ! Called from many places
   INTEGER  (KIND=IK), INTENT(IN) :: N                               ! Input value is error number 
   INTEGER  (KIND=IK)             :: I                               ! Local variable for DO loop index
   CHARACTER(LEN=LEN3)            :: C                               ! Message to be printed out
   ! Global constants: IK, LEN3, LogU
   ! Global vars used: ProcLevel, ProcName, NumPopTypes, PopCat
   ! Intrinsic procs : PRINT, WRITE
   PRINT*, '** FATAL ERROR **'                                       ! Indicate problem on screen  
   DO I=1, ProcLevel                                                 ! Loop over nested procedures
      PRINT*,       'In Procedure: ', ProcName(I)                    ! Put procedure name on screen
      IF (LogIsOpen==YES) WRITE(LogU,*) 'In Procedure: ',ProcName(I) ! Put procedure name on log file
   ENDDO                                                             ! Continue with next nesting level
   PRINT*,       'Error number: ', N                                 ! Put error number on screen 
   IF (LogIsOpen==YES) WRITE(LogU,*) 'Error number: ', N             ! Put error number on log file
   C = "Unknown error"                                               ! Default message  
   ! Main APEX module
   IF (ProcName(ProcLevel)=="ReadParameters") THEN                   ! Subroutine ReadParameters
      IF (N== 0)  C="Error reading input with GetLine"               ! Getline error in ReadParameters
      IF (N== 1)  C="Control file cannot be read"                    ! ReadParameters error #1
      IF (N== 2)  C="# of Pollutant definitions exceeds #Pollutants" ! ReadParameters error #2
      IF (N== 3)  C="# of micros either undefined or 0 "             ! ReadParameters error #3
      IF (N== 4)  C="# of pollutants undefined"                      ! ReadParameters error #4
      IF (N== 5)  C="Table levels not in ascending sequence"         ! ReadParameters error #5
      IF (N== 6)  C="AQ file keywords must be comma-separated"       ! ReadParameters error #6
      IF (N== 7)  C="Undefined pollutant in AQ file keyword"         ! ReadParameters error #7
      IF (N== 8)  C="Pop file keywords must be comma separated"      ! ReadParameters error #8 
      IF (N== 9)  C="Gender must start with M or F"                  ! ReadParameters error #9
      IF (N== 10) C="Race is missing or blank"                       ! ReadParameters error #10  
      IF (N== 11) C="Maximum number of counties/tracts exceeded"     ! ReadParameters error #11
      IF (N== 12) C="Requested percentiles out of range"             ! ReadParameters error #12
      IF (N== 13) C="Randomseed must be < 2147483648"                ! ReadParameters error #13
      IF (N== 14) C="Number of AQ files must equal pollutant number" ! ReadParameters error #14
      IF (N== 15) C="Pollutant value defined out of sequence"        ! ReadParameters error #15
   ENDIF                                                             ! End of main APEX module errors
   IF (ProcName(ProcLevel)=="CheckParameters") THEN                  ! Subroutine CheckParameters
      IF (N== 1)  C="NumProfiles missing or invalid"                 ! CheckParameters error #1
      IF (N== 2)  C="NumMicros missing or invalid"                   ! CheckParameters error #2
      IF (N== 3)  C="NumSources missing or invalid"                  ! CheckParameters error #3
      IF (N== 4)  C="Start date missing or invalid"                  ! CheckParameters error #4
      IF (N== 5)  C="End date missing or invalid"                    ! CheckParameters error #5
      IF (N== 6)  C="End date of simulation is before start date"    ! CheckParameters error #6
      IF (N== 7)  C="City radius must be greater than 0"             ! CheckParameters error #7 
      IF (N== 8)  C="Zone radius must be greater than 0"             ! CheckParameters error #8
      IF (N== 9)  C="Air radius must be greater than 0"              ! CheckParameters error #9
      IF (N== 10) C="Requested maximum age < minimum age"            ! CheckParameters error #11
      IF (N== 11) C="Minimum age must not be negative"               ! CheckParameters error #12
      IF (N== 12) C="LeaverMult must not be negative"                ! CheckParameters error #13
      IF (N== 13) C="LeaverAdd must not be negative"                 ! CheckParameters error #14
      IF (N== 14) C="PPMFactor must be greater than 0"               ! CheckParameters error #15
      IF (N== 15) C="AgeCutPct must not be negative"                 ! CheckParameters error #16
      IF (N== 16) C="AlertThresh must be greater than 0"             ! CheckParameters error #17   
      IF (N== 17) C="RandomSeed must not be negative"                ! CheckParameters error #18
      IF (N== 18) C="EventSample must not be negative"               ! CheckParameters error #19
      IF (N== 19) C="Latitude missing from PARAMS file"              ! CheckParameters error #20
      IF (N== 20) C="Longitude missing from PARAMS file"             ! CheckParameters error #21
      IF (N== 21) C="Invalid latitude value in PARAMS file"          ! CheckParameters error #22
      IF (N== 22) C="Invalid longitude value in PARAMS file"         ! CheckParameters error #23
      IF (N== 23) C="MissGender must be between 0 and 1"             ! CheckParameters error #24
      IF (N== 24) C="MissEmpl must be between 0 and 1"               ! CheckParameters error #25
      IF (N== 25) C="MissAge must be between 0 and 1"                ! CheckParameters error #26
      IF (N== 26) C="Age2Prob must be between 0 and 1"               ! CheckParameters error #27
      IF (N== 27) C="Rollback Background > Max"                      ! CheckParameters error #28
      IF (N== 28) C="Rollback Target > Max"                          ! CheckParameters error #29
      IF (N== 29) C="Rollback Background > Target"                   ! CheckParameters error #30
      IF (N== 30) C="DebugLevel must be 0, 1, or 2"                  ! CheckParameters error #31
      IF (N== 31) C="All EVR parameters in Control File must be >= 0"! CheckParameters error #32
      IF (N== 32) C="Child ages must be 0 or greater"                ! CheckParameters error #33
      IF (N== 33) C="Active PAI must be 1 or greater"                ! CheckParameters error #34        
      IF (N== 34) C="DiaryAutoC must be between -1 and 1"            ! CheckParameters error #35
      IF (N== 35) C="DiaryD must be between 0 and 1"                 ! CheckParameters error #36
      IF (N== 36) C="#OtherDistricts must be defined for this run"   ! CheckParameters error #38
      IF (N== 37) C="Bad micro # in MResMicros"                      ! CheckParameters error #39 
      IF (N== 38) C="Bad number of timesteps in day"                 ! CheckParameters error #38
      IF (N== 39) C="24 steps/day req'd when modeling AQ variability"! CheckParameters error #38
      IF (N== 40) C="No size given for PM pollutant"                 ! CheckParameters error #40
      IF (N== 41) C="No density given for PM pollutant"              ! CheckParameters error #41   
      IF (N== 42) C="Only one diary assembly method may be chosen"   ! CheckParameters error #42            
      IF (N== 43) C="ComProbab1, ComProbab2 must be between 0 and 1" ! CheckParameters error #43
      IF (N== 44) C="Nearby radius for sampling must be positive"    ! CheckParameters error #44         
      IF (N== 45) C="MissOcc must be between 0 and 1"                ! CheckParameters error #45
      IF (N== 46) C="Matching by occupation requires occup. groups"  ! CheckParameters error #46
   ENDIF                                                             ! End of CheckParameters errors
   ! DiaryModule
   IF (ProcName(ProcLevel)=="ReadCommmute") THEN                     ! Subroutine ReadCommute
      IF (N== 1) C="# Destinations exceeds MEANDESTS*NumSectors"     ! ReadCommute error #1 
      IF (N== 2) C="Problem with commuting distance distributions"   ! ReadCommute error #2
   ENDIF                                                             ! End of ReadCommute errors 
   IF (ProcName(ProcLevel)=="ReadCommmuteTimes") THEN                ! Subroutine ReadCommuteTimes
      IF (N== 1) C="Problem with commuting time distributions"       ! ReadCommuteTimes error #1 
   ENDIF                                                             ! End of ReadCommuteTimes errors    
   IF (ProcName(ProcLevel)=="ReadDiaries") THEN                      ! Subroutine ReadDiaries
      IF (N== 1) C="Problem reading CHAD Event File"                 ! ReadDiaries error #1 
      IF (N== 2) C="Cannot read CHAD location code as integer"       ! ReadDiaries error #2 
      IF (N== 3) C="Cannot read CHAD activity code as integer"       ! ReadDiaries error #3 
      IF (N== 4) C="Pool number out of range"                        ! ReadDiaries error #4
      IF (N== 5) C="At least one diary pool is empty"                ! ReadDiaries error #5
      IF (N== 6) C="Unknown CHAD Activity Codes"                     ! ReadDiaries error #6 
      IF (N== 7) C="Unknown CHAD Location Codes"                     ! ReadDiaries error #7
      IF (N== 8) C="# of diaries exceeds maximum"                    ! ReadDiaries error #8
      IF (N== 9) C="# of events exceeds maximum"                     ! ReadDiaries error #9
      IF (N== 10)C="DiaryStat ID does not match DiaryQuest ID"       ! ReadDiaries error #10
      IF (N== 11)C="Unknown Cluster CHAD Location Codes"             ! ReadDiaries error #11
      IF (N== 12)C="DiaryOcc ID does not match DiaryQuest ID"        ! ReadDiaries error #12
   ENDIF                                                             ! End of ReadDiaries errors 
   IF (ProcName(ProcLevel)=="ReadMicroMap") THEN                     ! Subroutine ReadMicroMap
      IF (N== 1) C="MicroMap Micro# is out of range"                 ! ReadMicroMap error #1 
      IF (N== 2) C="Missing APEX location in MicroMap file"          ! ReadMicroMap error #2 
      IF (N== 3) C="Location in MicroMap file must be H,W,O,R,RW,NH,NW,L or U" ! ReadMicroMap error #3 
      IF (N== 4) C="Roadway locations in MEMap, but RoadWay not specified" ! ReadMicroMap error #4
      IF (N== 5) C="Resampling specified, but no NH/NW locs in MEMap" ! ReadMicroMap error #5
   ENDIF                                                             ! End of ReadMicroMap errors 
   IF (ProcName(ProcLevel)=="ReadMETS") THEN                         ! Subroutine ReadMETS 
      IF (N== 1) C="Data on METS file are not readable"              ! ReadMETS error #1 
      IF (N== 2) C="METS file may be outdated, Row must be positive" ! ReadMETS error #2 
      IF (N== 3) C="Number of occupations exceeds maximum"           ! ReadMETS error #3 
      IF (N== 4) C="CHADMap Occupation name mismatch"                ! ReadMETS error #4    
   ENDIF                                                             ! End of ReadMETS errors 
   IF (ProcName(ProcLevel)=="DiaryProbabilities") THEN               ! Subroutine DiaryProbabilities  
      IF (N== 1) C="At least one diary pool was empty"               ! DiaryProbabilities error #1
   ENDIF                                                             ! End of DiaryProbabilities errors
   IF (ProcName(ProcLevel)=="CompositeDiary") THEN                   ! Subroutine CompositeDiary  
      IF (N== 1) C="No events in sequence, check cluster file"       ! CompositeDiary error #1
      IF (N== 2) C="Missing days in CHAD diaries"                    ! CompositeDiary error #2
   ENDIF                                                             ! End of CompositeDiary errors
   IF (ProcName(ProcLevel)=="METSEval") THEN                         ! Subroutine METSEval 
      IF (N== 1) C="Unknown METS distribution type"                  ! MetEval error #1
   ENDIF                                                             ! End of DiaryModule errors  
   ! DistributionModule
   IF (ProcName(ProcLevel)=="CVValue") THEN                          ! Function CVValue 
      IF (N== 1) C="Number of conditional variable is out of range"  ! CVValue error #1
      IF (N== 2) C="Value of conditional variable is out of range"   ! CVValue error #2
   ENDIF                                                             ! End of CVValue errors 
   IF (ProcName(ProcLevel)=="ReadProfileFunctions") THEN             ! Subroutine ReadProfileFunctions 
      IF (N== 1) C="Too many user-defined functions defined"         ! ReadProfileFunctions error #1
      IF (N== 2) C="Invalid input line type for function name"       ! ReadProfileFunctions error #2
      IF (N== 3) C="Input file ends in middle of definition"         ! ReadProfileFunctions error #3
      IF (N== 4) C="Unreadable input line for profile func"          ! ReadProfileFunctions error #4
      IF (N== 5) C="Maximum number of input variables exceeded"      ! ReadProfileFunctions error #5
      IF (N== 6) C="Unreadable parameters for input variable"        ! ReadProfileFunctions error #6
      IF (N== 7) C="Input variables out of sequence"                 ! ReadProfileFunctions error #7
      IF (N== 8) C="First variable cannot be conditional"            ! ReadProfileFunctions error #8
      IF (N== 9) C="Only 1 conditional variable allowed in function" ! ReadProfileFunctions error #9
      IF (N==10) C="CONDITIONAL extents do not conform"              ! ReadProfileFunctions error #10
      IF (N==11) C="Invalid CONDITIONAL probability data"            ! ReadProfileFunctions error #11
      IF (N==12) C="Invalid PROBABILITY probability data"            ! ReadProfileFunctions error #12  
      IF (N==13) C="Invalid REALRANGE cutpoint data"                 ! ReadProfileFunctions error #13 
      IF (N==14) C="Invalid INTRANGE cutpoint data"                  ! ReadProfileFunctions error #14
      IF (N==15) C="Invalid INTVALUE input data"                     ! ReadProfileFunctions error #15
      IF (N==16) C="Invalid CHARVALUE input data"                    ! ReadProfileFunctions error #16
      IF (N==17) C="Unknown type for input variable"                 ! ReadProfileFunctions error #17 
      IF (N==18) C="Unreadable RESULT type or extent"                ! ReadProfileFunctions error #18
      IF (N==19) C="Conditional variables require INTEGER results"   ! ReadProfileFunctions error #19
      IF (N==20) C="RESULT length does not match input extents"      ! ReadProfileFunctions error #20
      IF (N==21) C="Histograms must have only one index"             ! ReadProfileFunctions error #21
      IF (N==22) C="RESULT type is not recognized"                   ! ReadProfileFunctions error #22
      IF (N==23) C="RESULT data are unreadable"                      ! ReadProfileFunctions error #23 
      IF (N==24) C="Extra data found on profile function"            ! ReadProfileFunctions error #24 
      IF (N==25) C="Distribution type is not recognized"             ! ReadProfileFunctions error #25
      IF (N==26) C="Too many regional CVs defined"                   ! ReadProfileFunctions error #26
      IF (N==27) C="Number of regions for CV 0 or missing"           ! ReadProfileFunctions error #27
      IF (N==28) C="BY statement missing for regional CV"            ! ReadProfileFunctions error #28
      IF (N==29) C="Regional CV must be of type PROBABILITY OR REGIONINDEX" ! ReadProfileFunctions error #29
      IF (N==30) C="Region missing for some study area sectors"      ! ReadProfileFunctions error #30      
      IF (N==31) C="Too few regions in list"                         ! ReadProfileFunctions error #31      
      IF (N==31) C="RESULT missing or too many regions in list"      ! ReadProfileFunctions error #31     
      IF (N==32) C="The DIARYPOOLS profile requires INPUTS1-3"       ! ReadProfileFunctions error #32
      IF (N==33) C="RegionIndex value not in bounds"                 ! ReadProfileFunctions error #33
      IF (N==34) C="RESULTS line missing from file"                  ! ReadProfileFunctions error #34
   ENDIF                                                             ! End of ReadProfileFunctions errors 
   IF (ProcName(ProcLevel)=="Distrib") THEN                          ! Subroutine Distrib 
      IF (N== 1) C="Distribution name not found on the list"         ! Distrib error #1 
      IF (N== 2) C="Value for TABLE not on integer value list"       ! Distrib error #2
      IF (N== 3) C="Value for TABLE not on character string list"    ! Distrib error #3
      IF (N== 4) C="Variable type for TABLE not found on list"       ! Distrib error #4 
      IF (N== 5) C="Result type not recognized"                      ! Distrib error #5
      IF (N== 6) C="Distribution type not found on list"             ! Distrib error #6
   ENDIF                                                             ! End of Distrib errors 
   IF (ProcName(ProcLevel)=="FuncNum") THEN                          ! Subroutine FuncNum 
      IF (N== 1) C="Required distribution missing in file"           ! FuncNum error #1 
   ENDIF                                                             ! End of FuncNum errors 
   IF (ProcName(ProcLevel)=="BetaInv") THEN                          ! Function BetaInv 
      IF (N== 1) C="Invalid input to BetaInv"                        ! BetaInv error #1 
   ENDIF                                                             ! End of DistributionModule errors
   IF (ProcName(ProcLevel)=="ReadDist") THEN                         ! Function ReadDist
      IF (ProcName(Proclevel-1)=="ReadMicroData") THEN               ! If problem in readmicrodata
         PRINT *, "Problem with line number ", LineNumM              ! Printing out line number
      ENDIF                                                          ! End printing logic
      IF (N==1) C="Too many input parameters to distribution"        ! ReadDist error #1
      IF (N==2) C="Missing parameters or placeholders for dist"      ! ReadDist error #2
      IF (N==10+BETA) C="Missing/invalid parameter in BETA dist"     ! ReadDist error 
      IF (N==10+BURR) C="Missing/invalid parameter in BURR dist"     ! ReadDist error 
      IF (N==10+CAUCHY) C="Missing/invalid parameter in CAUCHY dist" ! ReadDist error 
      IF (N==10+EXPONENTIAL)                                        &
         C="Missing or invalid parameter in EXPONENTIAL dist"        ! ReadDist error 
      IF (N==10+EVALUE) C="Missing/invalid parameter in EVALUE dist" ! ReadDist error 
      IF (N==10+GAMMA) C="Missing/invalid parameter in GAMMA dist"   ! ReadDist error 
      IF (N==10+LGT) C="Missing/invalid parameter in LGT dist"       ! ReadDist error 
      IF (N==10+LOGNORMAL)                                          &
         C="Missing/invalid parameter in LOGNORMAL dist"             ! ReadDist error 
      IF (N==10+LUNIFORM)                                           & 
         C="Missing/invalid parameter in LUNIFORM dist"              ! ReadDist error 
      IF (N==10+NORMAL) C="Missing/invalid parameter in NORMAL dist" ! ReadDist error 
      IF (N==10+OFFON) C="Missing/invalid parameter in OFFON dist"   ! ReadDist error 
      IF (N==10+PARETO) C="Missing/invalid parameter in PARETO dist" ! ReadDist error 
      IF (N==10+POINT) C="Missing/invalid parameter in POINT dist"   ! ReadDist error 
      IF (N==10+TRIANGLE)                                           & 
         C="Missing/invalid parameter in TRIANGLE dist"              ! ReadDist error 
      IF (N==10+UNIFORM)                                            &
         C="Missing/invalid parameter in UNIFORM dist"               ! ReadDist error 
      IF (N==10+WEIBULL)                                            &
         C="Missing/invalid parameter in WEIBULL dist"               ! ReadDist error 
   ENDIF                                                             ! End of ReadDist errors
   IF (ProcName(ProcLevel)=="GammaCFD") THEN                         ! Subroutine GammaCFD 
      IF (N== 1) C="Bad input to GammaCDF"                           ! GammaCFD error #1 
   ENDIF                                                             ! End of GammaCFD errors 
   IF (ProcName(ProcLevel)=="GammaInv") THEN                         ! Subroutine GammaInv 
      IF (N== 1) C="Bad input to GammaInv"                           ! GammaInv error #1 
      IF (N== 2) C="Error encountered in GammaInv"                   ! GammaInv error #2 
      IF (N== 3) C="Convergence failed in GammaInv"                  ! GammaInv error #3 
     ENDIF                                                           ! End of GammaInv errors 
   IF (ProcName(ProcLevel)=="BetaInv") THEN                          ! Subroutine BetaInv 
      IF (N== 1) C="Input parameters out of range in BetaInv"        ! BetaInv error #1 
   ENDIF                                                             ! End of NormInv errors
   IF (ProcName(ProcLevel)=="NormInv") THEN                          ! Subroutine BetaInv 
      IF (N== 1) C="Input parameter out of range in NormInv"         ! NormInv error #1 
   ENDIF                                                             ! End of NormInv errors
   IF (ProcName(ProcLevel)=="BetaFrac")THEN                          ! Subroutine BetaFrac 
      IF (N== 1) C="Convergence failed in BetaFrac"                  ! BetaFrac error #1 
   ENDIF                                                             ! End of BetaFrac errors
   IF (ProcName(ProcLevel)=="EvalFunc")THEN                          ! Subroutine BetaFrac 
      IF (N== 1) C="Error in EvalFunc"                               ! EvalFunc error #1 
      IF (N== 2) C="No match in INTVALUE distribution"               ! EvalFunc error #2 
      IF (N== 3) C="No match in CHARVALUE distribution"              ! EvalFunc error #3 
      IF (N== 4) C="Unknown profile variable type"                   ! EvalFunc error #4 
      IF (N== 5) C="Unknown profile result type"                     ! EvalFunc error #5 
      IF (N== 6) C="Profile INT distribution array out of bounds"    ! EvalFunc error #6 
      IF (N== 7) C="Profile INT distribution array out of bounds"    ! EvalFunc error #7                                     
   ENDIF      
   ! ExposureDoseModule
   IF (ProcName(ProcLevel)=="Exposure") THEN                         ! Subroutine Exposure
      IF (N== 1) C="A negative exposure has been generated"          ! Exposure error #1   
      IF (N== 2) C="Durations not equal to 60 minutes per hour"      ! Exposure error #2
      IF (N== 3) C="Duration not = to correct minutes per timestep"  ! Exposure error #3
   ENDIF                                                             ! End of Exposure errors     
   IF (ProcName(ProcLevel)=="Dose") THEN                             ! Subroutine dose 
      IF (N== 1) C="Duration not = to correct minutes per timestep"  ! Dose error #1   
   ENDIF                                                             ! End of Dose errors 
   IF (ProcName(ProcLevel)=="Ventilation")  THEN                     ! Subroutine Ventilation 
      IF (N== 1) C="Maximum number of events exceeded"               ! Ventilation error #1   
   ENDIF                                                             ! End of Ventilation errors
   ! MicroEnvModule
   IF (ProcName(ProcLevel)=="ReadMicroData") THEN                    ! Subroutine ReadMicroData
      PRINT*, "Problem with line number ", LineNumM                  ! Printing out line number
      IF (N== 0) C="Problem reading input with GetLine"              ! GetLine error in ReadMicroData 
      IF (N== 1) C="Micro number is out of range (1-127)"            ! ReadMicroData error #1
      IF (N== 2) C="Micro number exceeds limit on PARAMS file"       ! ReadMicroData error #2
      IF (N== 3) C="Micro is neither Mass Balance nor Factors"       ! ReadMicroData error #3
      IF (N== 4) C="Micro is not listed in first section of file"    ! ReadMicroData error #4
      IF (N== 5) C="Too few distributions for microparameter"        ! ReadMicroData error #5
      IF (N== 6) C="Pollutant number out of range in micros file"    ! ReadMicroData error #6
      IF (N== 7) C="Unknown MP parameter type"                       ! ReadMicroData error #7
      IF (N== 8) C="Multiple definitions for AER in micros file"     ! ReadMicroData error #8
      IF (N== 9) C="Multiple definitions for VOL in micros file"     ! ReadMicroData error #9
      IF (N==10) C="Source number exceeds limit on PARAMS file"      ! ReadMicroData error #10
      IF (N==11) C="Problem reading 24 values for Hours-Block map"   ! ReadMicroData error #11
      IF (N==12) C="Non-positive Block number found"                 ! ReadMicroData error #12
      IF (N==13) C="Cannot read 7 values for Weekday-Daytype map"    ! ReadMicroData error #13
      IF (N==14) C="Non-positive Daytype number found"               ! ReadMicroData error #14
      IF (N==15) C="Cannot read 12 values for Month-Season map"      ! ReadMicroData error #15
      IF (N==16) C="Non-positive Season number found"                ! ReadMicroData error #16
      IF (N==17) C="Cannot read values for District-Area map"        ! ReadMicroData error #17
      IF (N==18) C="Non-positive Area number found"                  ! ReadMicroData error #18 
      IF (N==19) C="No # sign in Condition keyword"                  ! ReadMicroData error #19
      IF (N==20) C="Condition number on MP file unreadable"          ! ReadMicroData error #20
      IF (N==21) C="Condition number out of range (1-3)"             ! ReadMicroData error #21
      IF (N==22) C="Invalid conditional variable in micros file"     ! ReadMicroData error #22
      IF (N==23) C="Micro uses an undefined conditional variable"    ! ReadMicroData error #23
      IF (N==24) C="Random number seed is unreadable"                ! ReadMicroData error #24
      IF (N==25) C="Distribution data on MP file are unreadable"     ! ReadMicroData error #25
      IF (N==26) C="Found distribution index less than one"          ! ReadMicroData error #26
      IF (N==27) C="Distribution index is too large"                 ! ReadMicroData error #27
      IF (N==28) C="Too many distributions listed for MP"            ! ReadMicroData error #28
      IF (N==29) C="Distribution index number out of range"          ! ReadMicroData error #29 
      IF (N==30) C="Same set of indices defined twice"               ! ReadMicroData error #30
      IF (N==31) C="Unrecognized input line on MP file"              ! ReadMicroData error #31
      IF (N==32) C="At least one MP distribution not defined"        ! ReadMicroData error #32 
      IF (N==33) C="MP counter does not total correctly"             ! ReadMicroData error #33
      IF (N==34) C="Distribution count differs from expected total"  ! ReadMicroData error #34
      IF (N==35) C="Same source number for both ES and CS sources"   ! ReadMicroData error #35
      IF (N==36) C="Mass balance micro requires air exchange rate"   ! ReadMicroData error #36
      IF (N==37) C="Micro with ESource is missing volume"            ! ReadMicroData error #37
      IF (N==38) C="Missing pollutant number for parameter"          ! ReadMicroData error #37
   ENDIF                                                             ! End of ReadMicroData errors
   IF (ProcName(ProcLevel)=="SetupMicroArrays") THEN                 ! Subroutine SetupMicroArrays
      IF (N== 1) C="No micros are defined"                           ! SetupMicroArrays error #1 
      IF (N== 2) C="No air quality districts found"                  ! SetupMicroArrays error #2 
   ENDIF                                                             ! End of SetupMicroArrays errors  
   IF (ProcName(ProcLevel)=="MicroConcs") THEN                       ! Subroutine MicroConcs
      IF (N== 1) C="Volume for micro is not positive"                ! MicroConcs error #1 
      IF (N== 2) C="Negative ESource total found"                    ! MicroConcs error #2
      IF (N== 3) C="Negative CSource total found"                    ! MicroConcs error #3 
      IF (N== 4) C="Negative MP value"                               ! MicroConcs error #4 
      IF (N== 5) C="Negative concentration found in MASSBAL micro"   ! MicroConcs error #5
   ENDIF                                                             ! End of MicroEnvModule errors
   ! ProfileModule
   IF (ProcName(ProcLevel)=="ReadPopulation") THEN                   ! Subroutine ReadPopulation
      IF (N== 0) C="Problem reading input with GetLine"              ! GetLine error in ReadPopulation
      IF (N== 1) C="No age groups on population files"               ! ReadPopulation error #1
      IF (N== 2) C="Pop file gender does not match PARAMS status"    ! ReadPopulation error #2
      IF (N== 3) C="Pop file race does not match PARAMS status"      ! ReadPopulation error #3
      IF (N== 4) C="Inconsistent # age groups across pop files"      ! ReadPopulation error #4
      IF (N== 5) C="Population age group has min age > max age"      ! ReadPopulation error #5
      IF (N== 6) C="Age groups overlap on pop file"                  ! ReadPopulation error #6 
      IF (N== 7) C="Inconsistent age ranges across pop files"        ! ReadPopulation error #7
      IF (N== 8) C="Sector list on employmemt and pop files differ"  ! ReadPopulation error #8 
      IF (N== 9) C="Problem reading input record on pop file"        ! ReadPopulation error #9
      IF (N==10) C="Negative population found"                       ! ReadPopulation error #10
      IF (N==12) C="No persons within requested age range"           ! ReadPopulation error #12
      IF (N==13) C="Race name in pop file must have at least 5 chars"! ReadPopulation error #13
      IF (N>100) THEN                                                ! Errors over 100->sector miss.
         C="Sector missing from pop file "//TRIM(PopCat(N-100)%FName)! ReadPopulation errors >100
      ENDIF                                                          ! End of sector err in pop fl  
   ENDIF                                                             ! End of ReadPopulation errors
   IF (ProcName(ProcLevel)=="ReadEmployment") THEN                   ! Subroutine ReadEmployment
      IF (N== 0) C="Problem reading input with GetLine"              ! GetLine error in ReadEmployment
      IF (N== 1) C="No age groups on employment files"               ! ReadEmployment error #1
      IF (N== 2) C="Not all genders on emp file are M or F"          ! ReadEmployment error #2
      IF (N== 3) C="Employment age group has min age > max age"      ! ReadEmployment error #3
      IF (N== 4) C="Sector list on prevalence and pop files differ"  ! ReadEmployment error #4
      IF (N== 5) C="Problem reading input record on emp file"        ! ReadEmployment error #5
      IF (N== 6) C="Employment probability <0 on Employment file"    ! ReadEmployment error #6
      IF (N== 7) C="Employment probability >1 on Employment file"    ! ReadEmployment error #7
   ENDIF                                                             ! End of ReadEmployment errors
   IF (ProcName(ProcLevel)=="ReadDiseaseRate") THEN                  ! Subroutine ReadDiseaseRate
      IF (N== 0) C="Problem reading input with GetLine"              ! GetLine error in ReadDiseaseRate
      IF (N== 1) C="No age groups on prevalence files"               ! ReadDiseaseRate error #1
      IF (N== 2) C="Not all genders on prevalence file are M or F"   ! ReadDiseaseRate error #2
      IF (N== 3) C="Prevalence age group has min age > max age"      ! ReadDiseaseRate error #3
      IF (N== 4) C="Sector name not in right place on preval. file"  ! ReadDiseaseRate error #4
      IF (N== 5) C="Problem reading input record on preval. file"    ! ReadDiseaseRate error #5
      IF (N== 6) C="Prevalence probability <0 on Prevalence file"    ! ReadDiseaseRate error #6
      IF (N== 7) C="Prevalence probability >1 on Prevalence file"    ! ReadDiseaseRate error #7
   ENDIF                                                             ! End of ReadDiseaseRate errors
   IF (ProcName(ProcLevel)=="ReadProfileFactors") THEN               ! Subroutine ReadProfileFactors
      IF (N== 0) C="Problem reading input with GetLine"              ! GetLine error in ReadProfileFactors
      IF (N== 1) C="No age groups on profile factors files"          ! ReadProfileFactors error #1
      IF (N== 2) C="Not all genders on pro file are M or F"          ! ReadProfileFactors error #2
      IF (N== 3) C="Profile age group has min age > max age"         ! ReadProfileFactors error #3
      IF (N== 4) C="Sector name not in right place on Profile file"  ! ReadProfileFactors error #4
      IF (N== 5) C="Problem reading input record on Profile file"    ! ReadProfileFactors error #5
      IF (N== 6) C="Probability <0 on Profile Factors file"          ! ReadProfileFactors error #6
      IF (N== 7) C="Missing sectors in Profile Factors file"         ! ReadProfileFactors error #7
      IF (N== 8) C="More than one match per sector name"             ! ReadProfileFactors error #8
      IF (N== 9) C="Missing information in Profile Factors file"     ! ReadProfileFactors error #9
      IF (N==10) C="Missing levels keyword in Profile Factors file"  ! ReadProfileFactors error #10
      IF (N==11) C="Missing Profile Factor Group name"               ! ReadProfileFactors error #11
   ENDIF                                                             ! End of ReadProfileFactors errors
   IF (ProcName(ProcLevel)=="GeneratePhysiology") THEN               ! Subroutine GeneratePhysiology
      IF (N== 1) C="Unable to open physiology input file"            ! GeneratePhysiology error #1
      IF (N== 2) C="Min age greater than max age in Physiology file" ! GeneratePhysiology error #2
      IF (N== 3) C="Gender must be M or F in Physiology file"        ! GeneratePhysiology error #3
      IF (N== 4) C="Unrecognized variable in Physiology file"        ! GeneratePhysiology error #4
      IF (N== 5) C="NVO2MAX has undefined cohorts in Phys file"      ! GeneratePhysiology error #5
      IF (N== 6) C="BM has undefined cohorts in Phys file"           ! GeneratePhysiology error #6
      IF (N== 7) C="RMRINT has undefined cohorts in Phys file"       ! GeneratePhysiology error #7
      IF (N== 8) C="RMRSLP has undefined cohorts in Phys file"       ! GeneratePhysiology error #8
      IF (N== 9) C="RMRERR has undefined cohorts in Phys file"       ! GeneratePhysiology error #9
      IF (N== 10) C="HEIGHTINT has undefined cohorts in Phys file"   ! GeneratePhysiology error #10
      IF (N== 11) C="HEIGHTSLP has undefined cohorts in Phys file"   ! GeneratePhysiology error #11
      IF (N== 12) C="HEIGHTERR has undefined cohorts in Phys file"   ! GeneratePhysiology error #12
      IF (N== 13) C="BSAEXP1 has undefined cohorts in Phys file"     ! GeneratePhysiology error #13
      IF (N== 14) C="BSAEXP2 has undefined cohorts in Phys file"     ! GeneratePhysiology error #14
      IF (N== 15) C="ECF has undefined cohorts in Phys file"         ! GeneratePhysiology error #15
      IF (N== 16) C="RECTIME has undefined cohorts in Phys file"     ! GeneratePhysiology error #16
      IF (N== 17) C="MAXOXD has undefined cohorts in Phys file"      ! GeneratePhysiology error #17
      IF (N== 18) C="ENDGN1 has undefined cohorts in Phys file"      ! GeneratePhysiology error #18
      IF (N== 19) C="ENDGN2 has undefined cohorts in Phys file"      ! GeneratePhysiology error #19
      IF (N== 20) C="BLDF1 has undefined cohorts in Phys file"       ! GeneratePhysiology error #20
      IF (N== 21) C="BLDF2 has undefined cohorts in Phys file"       ! GeneratePhysiology error #21
      IF (N== 22) C="Unable to open ventilation input file"          ! GeneratePhysiology error #22
      IF (N== 23) C="Unable to open prevalence input file"           ! GeneratePhysiology error #23
      IF (N== 24) C="B1 has undefined cohorts in Phys file"          ! GeneratePhysiology error #24
      IF (N== 25) C="B2 has undefined cohorts in Phys file"          ! GeneratePhysiology error #25
      IF (N== 26) C="B3 has undefined cohorts in Phys file"          ! GeneratePhysiology error #26
      IF (N== 27) C="B4 has undefined cohorts in Phys file"          ! GeneratePhysiology error #27
      IF (N== 28) C="B5 has undefined cohorts in Phys file"          ! GeneratePhysiology error #28
      IF (N== 29) C="B6 has undefined cohorts in Phys file"          ! GeneratePhysiology error #29
      IF (N== 30) C="B7 has undefined cohorts in Phys file"          ! GeneratePhysiology error #30      
      IF (N== 31) C="FEVU has undefined cohorts in Phys file"        ! GeneratePhysiology error #31
      IF (N== 32) C="FEVE1 has undefined cohorts in Phys file"       ! GeneratePhysiology error #32      
      IF (N== 33) C="FEVE2 has undefined cohorts in Phys file"       ! GeneratePhysiology error #33      
      IF (N== 34) C="FEVSLP has undefined cohorts in Phys file"      ! GeneratePhysiology error #34
      IF (N== 35) C="FEVINT has undefined cohorts in Phys file"      ! GeneratePhysiology error #35                  
   ENDIF                                                             ! End of ProfileModule errors  
   ! SpaceTimeModule
   IF (ProcName(ProcLevel)=="CalcDates") THEN                        ! Subroutine CalcDates 
   ENDIF                                                             ! End of CalcDates errors
   IF (ProcName(ProcLevel)=="JulianDay") THEN                        ! Function JulianDay 
      IF (N== 1) C="Earliest possible date is in year 1"             ! JulianDay error #1 
      IF (N== 2) C="Month not between 1 and 12 inclusive"            ! JulianDay error #2 
      IF (N== 3) C="Day of month out of range"                       ! JulianDay error #3 
   ENDIF                                                             ! End of JulianDay errors
   IF (ProcName(ProcLevel)=="YearMonthDay") THEN                     ! Function YearMonthDay
      IF (N== 1) C="Minimum Julian day is 1 (Jan 1,0001)"            ! YearMonthDay error #1
   ENDIF                                                             ! End of YearMonthDay errors
   IF (ProcName(ProcLevel)=="ReadSiteLists") THEN                    ! Subroutine ReadSiteLists
      IF (N== 0) C="Problem reading input with GetLine"              ! GetLine error in SelectSites 
      IF (N== 1) C="No appropriate sectors found"                    ! ReadSiteLists error #1 
      IF (N== 2) C="No appropriate districts found"                  ! ReadSiteLists error #2
      IF (N== 3) C="No appropriate zones found"                      ! ReadSiteLists error #3 
      IF (N== 4) C="No roadway concentrations found"                 ! ReadSiteLists error #4
      IF (N== 5) C="Negative district radii found."                  ! ReadSiteLists error #5
   ENDIF                                                             ! End of SelectSites errors 
   IF (ProcName(ProcLevel)=="ReadMeteorologyData") THEN              ! Subroutine ReadMeteorologyData
      IF (N== 1) C="Meteorology site name is missing"                ! ReadMeteorologyData error #1
      IF (N== 2) C="Problem reading meteorology data"                ! ReadMeteorologyData error #2
      IF (N== 3) C="Hours out of range in meteorology data file"     ! ReadMeteorologyData error #3
      IF (N== 4) C="Missing meteorology data"                        ! ReadMeteorologyData error #4
   ENDIF                                                             ! End of ReadMeteorologyData errors   
   IF (ProcName(ProcLevel)=="ReadAirQuality") THEN                   ! Subroutine ReadAirQuality
      IF (N== 0) C="Problem reading input with GetLine"              ! GetLine error in ReadAirQuality
      IF (N== 1) C="Air quality dists missing in AQ file"            ! ReadAirQuality error #1
      IF (N== 2) C="Air quality site name is missing"                ! ReadAirQuality error #2
      IF (N== 3) C="Problem reading air quality data"                ! ReadAirQuality error #3
      IF (N== 4) C="Missing or negative values in air quality data"  ! ReadAirQuality error #4
      IF (N== 5) C="Adjustments for commuters give negative AQ"      ! ReadAirQuality error #5
      IF (N== 6) C="AQ has unexpected format: check ModelAQVar flag" ! ReadAirQuality error #6
      IF (N== 7) C="Date in AQ file in incorrect format."            ! ReadAirQuality error #6
   ENDIF                                                             ! End of SpaceTimeModule errors  
   IF (ProcName(ProcLevel)=="GenerateAirQuality") THEN               ! Subroutine GenerateAirQuality
     IF (N== 1) C="Adjustments for commuters give negative AQ"       ! GenerateAirQuality error #1
   ENDIF                                                             ! End of SpaceTimeModule errors  
  ! UtilityModule
   IF (ProcName(ProcLevel)=="OpenInput") THEN                        ! Subroutine OpenInput
      IF (N== 1) C="Unable to open input file"                       ! OpenInput error #1  
   ENDIF                                                             ! End of OpenInput errors   
   IF (ProcName(ProcLevel)=="CheckAllocation") THEN                  ! Subroutine CheckAllocation
      IF (N== 1) C="Out of memory: array allocation failed"          ! CheckAllocation error #1  
   ENDIF                                                             ! End of OpenInput errors   
   IF (ProcName(ProcLevel)=="ManageFiles") THEN                      ! Subroutine ManageFiles 
      IF (N== SectU) C="Sectors filename not set in Control File"    ! ManageFiles Error 
      IF (N== MonU)  C="Districts filename not set in Control File"  ! ManageFiles Error 
      IF (N== ZoneU) C="Zones filename not set in Control File"      ! ManageFiles Error 
      IF (N== CommU) C="Commuting filename not set in Control File"  ! ManageFiles Error 
      IF (N== CommTU) C="Commuting time filename not set in Control File" ! ManageFiles Error      
      IF (N== TempU) C="Meteorology flname not set in Control File"  ! ManageFiles Error 
      IF (N== AQU)   C="Air qual filename not set in Control File"   ! ManageFiles Error 
      IF (N== MetsU) C="Metabolic filename not set in Control File"  ! ManageFiles Error 
      IF (N== PhysU) C="Physiol filename not set in Control File"    ! ManageFiles Error  
      IF (N== DistU) C="Functions filename not set in Control File"  ! ManageFiles Error 
      IF (N== ChMapU)C="MEMap filename not set in Control File"      ! ManageFiles Error
      IF (N== ClMapU)C="Cluster Diary Map file does not exist"       ! ManageFiles Error       
      IF (N== MMapU) C="Metsmap filename not set in Control File"    ! ManageFiles Error 
      IF (N== ChQuU) C="DiaryQuest filename not set in Control File" ! ManageFiles Error 
      IF (N== ChEvU) C="Diaryev filename not set in Control File"    ! ManageFiles Error 
      IF (N== MicroU)C="Microenv filename not set in Control File"   ! ManageFiles Error
      IF (N== EmpU)  C="Employ filename not set in Control File"     ! ManageFiles Error 
      IF (UseProfileFactors == YES) THEN                            
         IF (N== ProU)  C="Profile Factors filename not set in Control File"  ! ManageFiles Error          
      ENDIF
      IF (N== VentU) C="Ventil filename not set in Control File"     ! ManageFiles Error 
      IF (N== StatU) C="Diary stat filename not set in Control File" ! ManageFiles Error 
      IF (N== PrevU) C="Prevalence filename not set in Control File" ! ManageFiles Error 
      IF (N== LogU)  C="Log filename not set in Control File"        ! ManageFiles Error 
      IF (N== PSumU) C="Persons filename not set in Control File"    ! ManageFiles Error 
      IF (N== MSumU) C="Microsum filename not set in Control File"   ! ManageFiles Error 
      IF (N== TabU)  C="Tables filename not set in Control File"     ! ManageFiles Error 
      IF (N== SiteU) C="Site filename not set in Control File"       ! ManageFiles Error 
      IF (N== EventU)C="Events filename not set in Control File"     ! ManageFiles Error 
      IF (N== HourU) C="Hourly filename not set in Control File"     ! ManageFiles Error 
      IF (N== DayU)  C="Daily filename not set in Control File"      ! ManageFiles Error 
      IF (N== TSU)   C="Timestep filename not set in Control File"   ! ManageFiles Error 
      IF (N== MResU) C="Microres filename not set in Control File"   ! ManageFiles Error 
      IF (N== 100+SectU) C="Sectors input file does not exist"       ! ManageFiles Error 
      IF (N== 100+MonU)  C="Districts input file does not exist"     ! ManageFiles Error 
      IF (N== 100+ZoneU) C="Zones input file does not exist"         ! ManageFiles Error 
      IF (N== 100+CommU) C="Commuting input file does not exist"     ! ManageFiles Error 
      IF (N== 100+CommTU) C="Commuting time input file does not exist" ! ManageFiles Error       
      IF (N== 100+TempU) C="Meteorology input file does not exist"   ! ManageFiles Error 
      IF (N== 100+AQU)   C="Air quality input file does not exist"   ! ManageFiles Error 
      IF (N== 100+MetsU) C="METS distribution file does not exist"   ! ManageFiles Error 
      IF (N== 100+PhysU) C="Physiology input file does not exist"    ! ManageFiles Error 
      IF (N== 100+DistU) C="Functions input file does not exist"     ! ManageFiles Error  
      IF (N== 100+ChMapU)C="MEMap input file does not exist"         ! ManageFiles Error 
      IF (N== 100+ChQuU) C="DiaryQuest input file does not exist"    ! ManageFiles Error 
      IF (N== 100+ChEvU) C="Diaryevent input file does not exist"    ! ManageFiles Error 
      IF (N== 100+MicroU)C="Microenv input file does not exist"      ! ManageFiles Error 
      IF (N== 100+EmpU)  C="Employment input file does not exist"    ! ManageFiles Error 
      IF (N== 100+ProU)  C="Profile factors input file does not exist" ! ManageFiles Error       
      IF (N== 100+VentU) C="Ventilation input file does not exist"   ! ManageFiles Error 
      IF (N== 100+StatU) C="Diarystat input file does not exist"     ! ManageFiles Error 
      IF (N== 100+PrevU) C="Prevalence input file does not exist"    ! ManageFiles Error 
      IF (N== 100+MMapU) C="METSMap input file does not exist"       ! ManageFiles Error 
      IF (N== 100+ClMapU)C="Cluster Diary Map file does not exist"   ! ManageFiles Error         
      IF (N>200) THEN                                                ! Pop file errors > 200
         C="The input population file "//TRIM(PopCat(N-200)%FName)  &  
            //" does not exist"                                      ! Pop file error #=200+unit#          
      ENDIF                                                          ! End pop file logic   
   ENDIF                                                             ! End of ManageFiles errors  
   IF (ProcName(ProcLevel)=="OpenOutput") THEN                       ! Subroutine OpenOutput
      IF (N== 30) C="Log creation failed: Output dir may not exist"  ! OpenOutput Error #30
      IF (N== 31) C="Cannot create/access output persons file"       ! OpenOutput Error #31
      IF (N== 32) C="Cannot create/access output microsum file "     ! OpenOutput Error #32
      IF (N== 33) C="Cannot create/access output tables file"        ! OpenOutput Error #33
      IF (N== 34) C="Cannot create/access output site file"          ! OpenOutput Error #34
      IF (N== 35) C="Cannot create/access output events file"        ! OpenOutput Error #35
      IF (N== 36) C="Cannot create/access output hourly file"        ! OpenOutput Error #36
      IF (N== 37) C="Cannot create/access output microres file"      ! OpenOutput Error #37
      IF (N== 38) C="Cannot create/access output daily file"         ! OpenOutput Error #38
      IF (N== 39) C="Cannot create/access output timestep file"      ! OpenOutput Error #38
   ENDIF                                                             ! End of OpenOutput errors  
   ! All modules
   PRINT*, TRIM(C)                                                   ! Put message on screen
   IF (LogIsOpen==YES) THEN                                          ! If log file is open            
      WRITE(LogU,*) TRIM(C)                                          ! Put message on log file     
      WRITE(LogU,'(2I10)') ((I,Ios(I)),I=1,MAXU)                     ! Put IO status values on log
      WRITE(LogU,*) 'APEX finished with completion code 1'           ! Put completion code on log
   ENDIF                                                             ! End log file open if block 
   STOP 1                                                            ! Return code '1' to system 
END SUBROUTINE Fatal                                                 ! End of code (does not return)


SUBROUTINE Warning(N)                                                ! Called from many places
   INTEGER  (KIND=IK), INTENT(IN) :: N                               ! Input value is warning number  
   INTEGER  (KIND=IK)             :: I                               ! Local variable for DO loop index 
   CHARACTER(LEN=LEN3)            :: C,line                          ! String for warning message
   ! Global constants: IK, LEN3, LogU
   ! Global vars used: ProcLevel, ProcName, PopCat
   ! Intrinsic procs : PRINT, WRITE
   PRINT*,       '* WARNING *'                                       ! Put proc names on screen 
   IF (LogIsOpen==YES) WRITE(LogU,*) '* WARNING *'                   ! Put proc names on log 
   DO I=1, ProcLevel                                                 ! Loop over nested procedures
      PRINT*,       'In Procedure: ', ProcName(I)                    ! Put procedure name on screen
      IF (LogIsOpen==YES) WRITE(LogU,*) 'In Procedure: ',ProcName(I) ! Put procedure name on log file
   ENDDO                                                             ! Continue with next nesting level
   PRINT*,       'Warning  #  : ', N                                 ! Put error number on screen 
   IF (LogIsOpen==YES) WRITE(LogU,*) 'Warning  #  : ', N             ! Put error number on log file
   C = "Unknown error"                                               ! Default message
   ! In main APEX module
   IF (ProcName(ProcLevel)=="ReadParameters") THEN                   ! Subroutine ReadParameters
      IF (N<0) C="Missing data for one gender for race type "//     & 
         TRIM(PopCat(0-N)%Race)                                      ! ReadParameter warn <0 
      IF (N>0) THEN                                                  ! ReadParameter warn #N
         WRITE(line,'(I3)') N                                        ! Write line num to string       
         C="Unrecognized info at parameters file data line #"       &
            //TRIM(line)                                             ! Write warning                                              
      ENDIF                                                          ! End of warning N logic   
   ENDIF                                                             ! End of ReadParameters warnings
   IF (ProcName(ProcLevel)=="CheckParameters") THEN                  ! Subroutine CheckParameters
      IF (N== 1)  C= "County list used, but no counties found"       ! CheckParameters warning #1 
      IF (N== 2)  C= "Tract list used, but no tracts found"          ! CheckParameters warning #2 
      IF (N== 3)  C= "Using default maximum age=0"                   ! CheckParameters warning #3 
      IF (N== 4)  C= "PM pollutants must use ug/m3 for in/out units" ! CheckParameters warning #3 
   ENDIF                                                             ! End CheckParameters warnings
   ! DistributionModule
   IF (ProcName(ProcLevel)=="SetCVLevels") THEN                      ! Subroutine SetCVLevels
      IF (N== 1) C="Potential conditional variables are not defined" ! SetCVLevels warning #1
   ENDIF                                                             ! End SetCVLevels warnings
   IF (ProcName(ProcLevel)=="BetaInv") THEN                          ! Subroutine BetaInv
      IF (N== 1) C="Maximum number of iterations reached in BetaInv" ! BetaInv warning #1
   ENDIF                                                             ! End BetaInv warnings
   IF (ProcName(ProcLevel)=="BetaFrac") THEN                         ! Subroutine BetaFrac
      IF (N== 1) C="Maximum number of iterations reached in BetaFrac"! BetaFrac warning #1
   ENDIF                                                             ! End BetaFrac warnings
   IF (ProcName(ProcLevel)=="ReadDist") THEN                         ! Function ReadDist
      IF (N==1)  C="Too many input parameters for distribution"      ! ReadDist warning #1       
      IF (N==2)  C="Too few input parameters for distribution"       ! ReadDist warning #2
      IF (N==3)  C="To many values for DISCRETE dist: max is 100 "   ! ReadDist warning #3            
      IF (N==4)  C="Must have at least one value for DISCRETE dist"  ! ReadDist warning #2
      IF (N==10+BETA)                                               &
         C="Too many parameters given for BETA dist: ignored"        ! ReadDist warning 
      IF (N==10+CAUCHY)                                             &
         C="Too many parameters given for CAUCHY dist: ignored"      ! ReadDist warning 
      IF (N==10+EXPONENTIAL)                                        &
         C="Too many parameters given for EXPONENTIAL dist: ignored" ! ReadDist warning 
      IF (N==10+EVALUE)                                             &
         C="Too many parameters given for EVALUE dist: ignored"      ! ReadDist warning 
      IF (N==10+GAMMA)                                              &
         C="Too many parameters given for GAMMA dist: ignored"       ! ReadDist warning 
      IF (N==10+LGT)                                                &
         C="Too many parameters given for LGT dist: ignored"         ! ReadDist warning 
      IF (N==10+LOGNORMAL)                                          &
         C="Too many parameters given for LOGNORMAL dist: ignored"   ! ReadDist warning 
      IF (N==10+LUNIFORM)                                           & 
          C="Too many parameters given for LUNIFORM dist: ignored"   ! ReadDist warning 
      IF (N==10+NORMAL)                                             &
         C="Too many parameters given for NORMAL dist: ignored"      ! ReadDist warning 
      IF (N==10+OFFON)                                              & 
         C="Too many parameters given for OFFON dist: ignored"       ! ReadDist warning 
      IF (N==10+PARETO)                                             &  
         C="Too many parameters given for PARETO dist: ignored"      ! ReadDist warning 
      IF (N==10+POINT)                                              &
         C="Too many parameters given for POINT dist: ignored"       ! ReadDist warning 
      IF (N==10+TRIANGLE)                                           & 
         C="Too many parameters given for TRIANGLE dist: ignored"    ! ReadDist warning 
      IF (N==10+UNIFORM)                                            &
         C="Too many parameters given for UNIFORM dist: ignored"     ! ReadDist warning 
      IF (N==10+WEIBULL)                                            &
         C="Too many parameters given for WEIBULL dist: ignored"     ! ReadDist warning 
   ENDIF                                                             ! End of ReadDist warnings
   ! MicroEnvModule
   IF (ProcName(ProcLevel)=="ReadMicroData") THEN                    ! Subroutine ReadMicrodata
      IF (N== 1) C="Micro number previously defined"                 ! ReadMicroData warning #1 
      IF (N== 2) C="Inconsistent mappings for correlated parameters" ! ReadMicroData warning #1 
      IF (N== 3) C="Inconsistent cond vars for correlated parameters"! ReadMicroData warning #1 
      IF (N== 4) C="Inconsistent resamp for correlated parameters"   ! ReadMicroData warning #1 
   ENDIF                                                             ! End of ReadMicroData warnings 
   IF (ProcName(ProcLevel)=="SetupMicArrays") THEN                   ! Subroutine SetupMicroArrays
      IF (N== 1) C="No MP defined - all micros at ambient levels"    ! SetupMicroArrays warning #1
      IF (N== 2) C="No MP distributions"                             ! SetupMicroArrays warning #2 
   ENDIF                                                             ! End of SetupMicroArrays warnings
   IF (ProcName(ProcLevel)=="MicroConcs") THEN                       ! Subroutine MicroConcs
      IF (N== 1) C="Negative CSource total found"                    ! MicroConcs warning #1 
      IF (N== 2) C="Negative Total ME Concentration found"           ! MicroConcs warning #2
   ENDIF                                                             ! End of MicroConcs warnings 
   ! SpaceTimeModule
   IF (ProcName(ProcLevel)=="MatchSites") THEN                       ! Subroutine MatchSites  
      IF (N== 1) C="RoadLast ignored: roadway not specified"         ! MatchSites warning #1
   ENDIF                                                             ! End of MatchSites warnings   
   IF (ProcName(ProcLevel)=="ReadMeteorologyData") THEN              ! Subroutine ReadMeteorologyData  
      IF (N== 1) C="In meteorology data: hours or days out of seq"   ! ReadMeteorologyData warning #1
   ENDIF                                                             ! End of ReadMeteorologyData warnings
   IF (ProcName(ProcLevel)=="ReadAirQuality") THEN                   ! Subroutine ReadAirQuality 
      IF (N== 1) C="Some data missing on air quality file"           ! ReadAirQuality warning #1  
      IF (N== 2) C="Extra data found on air quality file"            ! ReadAirQuality warning #2
   ENDIF                                                             ! End of ReadAirQuality warnings
   ! DiaryModule
   IF (ProcName(ProcLevel)=="ReadMicroMap") THEN                     ! Subroutine ReadMicroMap
      IF (N== 1) C="No roadway concentrations specified in MEMap file" ! ReadMicroMap Warning #1
   ENDIF                                                             ! End of ReadMicroMap warnings   
   IF (ProcName(ProcLevel)=="ReadDiaries") THEN                      ! Subroutine ReadDiaries
      IF (N>=100) C="No CHAD diaries found with occupation "//TRIM(GroupNames(N-100))           ! ReadDiaries Warning #1
   ENDIF                                                             ! End of ReadDiaries warnings    
   ! ProfileModule
   IF (ProcName(ProcLevel)=="CVLevels") THEN                         ! Subroutine CVLevels
      IF (N== 1) C="Not all CV Functions have defined levels"        ! CVLevels Warning #1
   ENDIF                                                             ! End of CVLevels warnings
   IF (ProcName(ProcLevel)=="ReadPopulation") THEN                   ! Subroutine ReadPopulation
      IF (N== 1) C="Population data start above AgeMin"              ! ReadPopulation warning #1
      IF (N== 2) C="Population data do not extend to AgeMax"         ! ReadPopulation warning #2
      IF (N== 3) C="Gaps found between population group ages"        ! ReadPopulation warning #3
      IF (N>=100) THEN                                               ! Sector order errors > 100
         C="Sectors out of order in "//TRIM(PopCat(N-100)%FName)     ! ReadPopulation error >100                                                                       
      ENDIF                                                          ! End sector order errors
   ENDIF                                                             ! End of ReadPopulation warnings
    IF (ProcName(ProcLevel)=="ReadProfileFunctions") THEN            ! Subroutine ReadProfileFunctions
      IF (N== 1) C="Region does not have an associated sector"       ! ReadProfileFunctions warning #1
   ENDIF                                                             ! End ReadProfileFunctions warns
   PRINT*,  TRIM(C)                                                  ! Put message on screen
   IF (LogIsOpen==YES) WRITE(LogU,*) TRIM(C)                         ! Put message on log file 
END SUBROUTINE Warning                                               ! Return to calling routine

SUBROUTINE ManageFiles(Action)                                       ! Called from ReadParameters
   INTEGER  (KIND=IK), INTENT(IN) :: Action                          ! Input value is OPN/CLSE  
   INTEGER  (KIND=IK)             :: I,J,K                           ! Loop index variable
   LOGICAL                        :: there                           ! Local var for file existence
   CHARACTER(LEN=4)               :: OutExt           = ""           ! File extention for output files
   CHARACTER(LEN=LEN3)            :: FN                              ! Local var 
   CHARACTER*21 :: HM1='   Person Mic Minutes'                       ! Part 1 of Micro file header          
   CHARACTER*28 :: HM2='    MeanConc     MaxConc'                    ! Part 2 of Micro file header
   ! Global vars used: FName, PopCat, NumPopTypes, PopU, MAXINU                       
   ! Intrinsic procs : INQUIRE, OPEN
   ! APEX procedures : Fatal  
   ProcLevel = ProcLevel+1                                           ! Increment procedure depth
   ProcName(ProcLevel) = "ManageFiles"                               ! Set procedure name 
   IF (Action==CLSE) THEN                                            ! If closing files
      CLOSE(SiteU)                                                   ! Close site file
      CLOSE(PSumU)                                                   ! Close personal file
      IF (EventsOut ==YES) CLOSE(EventU)                             ! Close events file
      IF (HourlyOut ==YES) CLOSE (HourU)                             ! Close hourly file
      IF (TimeStepOut==YES)CLOSE (TSU)                               ! Close ts file
      IF (DailyOut  ==YES) CLOSE (DayU)                              ! Close daily file
      DO I=1, NumPollutants                                          ! Loop over pollutants
         IF (MResOut       == YES) CLOSE(Pol(I)%MResU)               ! Close micro results file
         IF (MSumOut       == YES) CLOSE(Pol(I)%MSumU)               ! Close micro sum file
         IF (FirstProfile  == YES) CLOSE(Pol(I)%TabU)                ! Close tables file
      ENDDO                                                          ! End loop over pollutants
      RETURN                                                         ! Return to calling routine
   ENDIF                                                             ! End of file close logic 
   IF (FName(LogU)   =="") CALL Fatal(LogU)                          ! Filename not given in PARAMS
   IF (FName(ClMapU) =="" .AND. CLUSTDIARYA==YES) CALL Fatal(ClMapU) ! Filename not given in PARAMS
   IF (FName(ChQuU)  =="") CALL Fatal(ChQuU)                         ! Filename not given in PARAMS
   IF (FName(ChEvU)  =="") CALL Fatal(ChEvU)                         ! Filename not given in PARAMS
   IF (FName(ChMapU) =="") CALL Fatal(ChMapU)                        ! Filename not given in PARAMS
   IF (FName(DistU)  =="") CALL Fatal(DistU)                         ! Filename not given in PARAMS
   IF (FName(MetsU)  =="") CALL Fatal(MetsU)                         ! Filename not given in PARAMS
   IF (FName(MMapU)  =="") CALL Fatal(MMapU)                         ! Filename not given in PARAMS
   IF (FName(CommU)  =="" .AND. CommuteFlag==YES) CALL Fatal(CommU)  ! Filename not given in PARAMS
   IF (FName(CommTU) =="" .AND. CommuteFlag==YES) CALL Fatal(CommTU) ! Filename not given in PARAMS   
   IF (FName(TempU)  =="") CALL Fatal(TempU)                         ! Filename not given in PARAMS
   IF (FName(SectU)  =="") CALL Fatal(SectU)                         ! Filename not given in PARAMS
   IF (FName(MonU)   =="") CALL Fatal(MonU)                          ! Filename not given in PARAMS
   IF (FName(ZoneU)  =="") CALL Fatal(ZoneU)                         ! Filename not given in PARAMS
   IF (FName(EmpU)   =="") CALL Fatal(EmpU)                          ! Filename not given in PARAMS
   IF (UseProfileFactors == YES) THEN     
      IF (FName(ProU)   =="") CALL Fatal(ProU)                       ! Filename not given in PARAMS   
   ENDIF
   IF (FName(VentU)  =="") CALL Fatal(VentU)                         ! Filename not given in PARAMS
   IF (FName(TabU)   =="") CALL Fatal(TabU)                          ! Filename not given in PARAMS
   IF (FName(StatU)  =="" .AND. LongitDiary==YES) CALL Fatal(StatU)  ! Filename not given in PARAMS
   IF (FName(HourU)  =="" .AND. HourlyOut==YES) CALL Fatal(HourU)    ! Filename not given in PARAMS
   IF (FName(DayU)   =="" .AND. DailyOut==YES)  CALL Fatal(DayU)     ! Filename not given in PARAMS
   IF (FName(TSU)    =="" .AND. TimeStepOut==YES) CALL Fatal(TSU)    ! Filename not given in PARAMS
   IF (FName(PSumU)  =="") CALL Fatal(PSumU)                         ! Filename not given in PARAMS
   IF (FName(SiteU)  =="") CALL Fatal(SiteU)                         ! Filename not given in PARAMS
   IF (FName(EventU) =="" .AND. EventsOut==YES) CALL Fatal(EventU)   ! Filename not given in PARAMS
   IF (FName(MSumU)  =="" .AND. MSumOut==YES)   CALL Fatal(MSumU)    ! Filename not given in PARAMS
   IF (FName(MResU)  =="" .AND. MResOut==YES)   CALL Fatal(MResU)    ! Filename not given in PARAMS
   IF (FName(PrevU)  =="" .AND. Disease/="")    CALL Fatal(PrevU)    ! Filename not given in PARAMS
   IF (FName(MicroU) =="") CALL Fatal(MicroU)                        ! Filename not given in PARAMS
   IF (FName(PhysU)  =="") CALL Fatal(PhysU)                         ! Filename not given in PARAMS
   IF (ANY(Pol(:)%AQName=="")) CALL Fatal(AQU)                       ! Filename not given in PARAMS
   DO I=11,MAXINU                                                    ! Loop over the input unit #s
      IF (I==PopU) CYCLE                                             ! Continue if pop file
      IF (I==CommU .AND. CommuteFlag==YES) THEN                      ! If this is a commut file
            INQUIRE(FILE=FName(I),EXIST=there)                       ! Inquire if input file exists
            IF (.NOT.(there)) CALL Fatal(100+I)                      ! Write error
      ELSEIF (I==CommTU .AND. CommuteFlag==YES) THEN                 ! If this is a diarystat file
         INQUIRE(FILE=FName(I),EXIST=there)                          ! Inquire if input file exists
         IF (.NOT.(there)) CALL Fatal(100+I)                         ! Write error            
      ELSEIF (I==StatU .AND. LongitDiary==YES) THEN                  ! If this is a diarystat file
         INQUIRE(FILE=FName(I),EXIST=there)                          ! Inquire if input file exists
         IF (.NOT.(there)) CALL Fatal(100+I)                         ! Write error
      ELSEIF (I==PrevU .AND. Disease/="") THEN                       ! If this is a prev file
         INQUIRE(FILE=FName(I),EXIST=there)                          ! Inquire if input file exists
         IF (.NOT.(there)) CALL Fatal(100+I)                         ! Write error
      ELSEIF (I==StatU.OR.I==PrevU.OR.I==CommU.OR.I==CommTU) THEN    ! These files
         CYCLE                                                       !    are not needed
      ELSEIF (I==AQU) THEN                                           ! If this is an AQ file   
         DO J=1, NumPollutants                                       ! Loop over pollutants
            INQUIRE(FILE=Pol(J)%AQName,EXIST=there)                  ! Inquire if input file exits
            IF (.NOT.(there)) CALL Fatal(100+I)                      ! If not, call fatal (print msg)
         ENDDO                                                       ! End loop over pollutants
      ELSE                                                           ! For rest of the files   
         INQUIRE(FILE=FName(I),EXIST=there)                          ! Inquire if input file exits
         IF (.NOT.(there)) CALL Fatal(100+I)                         ! If not, call fatal (print msg)
      ENDIF                                                          ! End of file type logic
   ENDDO                                                             ! End of check of non-pop files
   DO I=1,NumPopTypes                                                ! Loop over existing pop files 
      INQUIRE(FILE=PopCat(I)%FName,EXIST=there)                      ! Check if file exists
      IF (.NOT.(there)) CALL Fatal(200+I)                            ! If not, call fatal (print msg)
   ENDDO                                                             ! End check of pop files 
   DO I=1, NumPollutants                                             ! Loop over pollutants
      CALL OpenInput(AQU,name=Pol(I)%AQName,h=YES,                  &
        header=AirHeader(I))                                         ! Get AQ header for pollutant
      CLOSE(AQU)                                                     ! Close AQ file
   ENDDO                                                             ! End of pollutant logic
   CALL OpenOutput(LogU,recl=LENTAB)                                 ! Open Log file for output
   IF (FName(LogU)  =="")  CALL Fatal(LogU)                          ! Filename not given in PARAMS
   IF (Ios(LogU)==0) LogIsOpen = YES                                 ! Log file is now open
   CALL OpenOutput(PSumU,recl=1000)                                  ! Open personal summary data file
   CALL OpenOutput(SiteU,recl=244)                                   ! Open site list output file
   IF (EventsOut==YES) CALL OpenOutput(EventU,recl=500)              ! Open the events output file 
   IF (HourlyOut==YES) CALL OpenOutput(HourU,recl=1000)              ! If user wants hourly file
   IF (DailyOut==YES)  CALL OpenOutput(DayU,recl=1000)               ! If user wants daily file
   IF (TimeStepOut==YES .AND. DoTimeSteps==YES)                     &
      CALL OpenOutput(TSU,recl=1000)                                 ! If user wants ts file
    DO I=1, NumPollutants                                            ! Loop over pollutants
      Pol(I)%MResU  = MResU  + 100*I                                 ! Unit # for pollutant-spec file  
      Pol(I)%MSumU  = MSumU  + 100*I                                 ! Unit # for pollutant-spec file    
      Pol(I)%TabU   = TabU   + 100*I                                 ! Unit # for pollutant-spec file    
      IF (FirstProfile==1) THEN                                      ! If sim starts on 1st person
         FN=FName(TabU)                                              ! Name given in control file
         K=LEN(TRIM(FN))                                             ! Length of filename
         IF (FN(K-3:K-3)==".") THEN                                  ! If extension exists
            OutExt = TRIM(ADJUSTL(FN(K-3:)))                         ! Set extension
            FN = TRIM(ADJUSTL(FN(1:K-4)))                            ! Prefix is part to left of "."
         ENDIF                                                       ! End extension logic
         Pol(I)%TabName = TRIM(ADJUSTL(FN))//"_"//                  & 
            TRIM(ADJUSTL(Pol(I)%Name))//TRIM(ADJUSTL(OutExt))        ! Construct file name
         CALL OpenOutput(TabU,LENTAB, PL=I)                          ! Open tables file for output
      ENDIF                                                          ! End first person logic
      IF (MResOut == YES) THEN                                       ! If printing micro res file
         FN=FName(MResU)                                             ! Name given in control file
         K=LEN(TRIM(FN))                                             ! Length of filename
         IF (FN(K-3:K-3)==".") THEN                                  ! If extension exists
            OutExt = TRIM(ADJUSTL(FN(K-3:)))                         ! Set extension
            FN = TRIM(ADJUSTL(FN(1:K-4)))                            ! Prefix is part to left of "."
         ENDIF                                                       ! End extension logic
         Pol(I)%MResName = TRIM(ADJUSTL(FN))//"_"//                 & 
            TRIM(ADJUSTL(Pol(I)%Name))//TRIM(ADJUSTL(OutExt))        ! Construct file name
         CALL OpenOutput(MResU,recl=1000,PL=I)                       ! Open micro results output file 
      ENDIF                                                          ! End micro res file logic
      IF (MSumOut==YES) THEN                                         ! If user wants micro summary  
         FN=FName(MSumU)                                             ! Name given in control file
         K=LEN(TRIM(FN))                                             ! Length of filename
         IF (FN(K-3:K-3)==".") THEN                                  ! If extension exists
            OutExt = TRIM(ADJUSTL(FN(K-3:)))                         ! Set extension
            FN = TRIM(ADJUSTL(FN(1:K-4)))                            ! Prefix is part to left of "."
         ENDIF                                                       ! End extension logic
         Pol(I)%MSumName = TRIM(ADJUSTL(FN))//"_"//                 & 
            TRIM(ADJUSTL(Pol(I)%Name))//TRIM(ADJUSTL(OutExt))        ! Construct file name
         CALL OpenOutput(MSumU,recl=250,PL=I)                        ! Open micro summary output file 
         WRITE(Pol(I)%MSumU, '(A)' ) HM1//HM2                        ! Write micro file header
      ENDIF                                                          ! End of MSumOut logic block
   ENDDO                                                             ! End loop over pollutants   
   ProcLevel = ProcLevel-1                                           ! Decrement proc level 
END SUBROUTINE ManageFiles                                           ! Return to calling routine


SUBROUTINE CheckAllocation(Status)                                   ! Called from many places
   INTEGER  (KIND=IK)             :: Status                          ! Allocation status
   ProcLevel=ProcLevel+1                                             ! Increment procedure level 
   ProcName(ProcLevel) = "CheckAllocation"                           ! Procedure name for messages
   IF (Status/=0) CALL Fatal(1)                                      ! Allocation failed, exit   
   ProcLevel=ProcLevel-1                                             ! Decrement procedure level 
END SUBROUTINE CheckAllocation                                       ! Return to calling routine 


SUBROUTINE WriteJobTime(P)                                           ! Called from multiple places
   INTEGER (KIND=IK), INTENT(IN) :: P                                ! Input variable (profile number)
   ! Global constants:  LogU
   ! Intrinsic procs : WRITE, SYSTEM_CLOCK                      
   INTEGER (KIND=IK) :: ClockRate, ClockMax                          ! Local integer variables
   REAL    (KIND=RK) :: JobTime                                      ! Local real variables   
   WRITE(LogU,*)                                                     ! Write blank line to log
   CALL SYSTEM_CLOCK(Clock(2),ClockRate, ClockMax)                   ! Get the system time  
   JobTime = REAL(Clock(2)-Clock(1))/REAL(ClockRate)                 ! Calculate job time
   IF (Jobtime<0) JobTime=JobTime+REAL(ClockMax)/REAL(ClockRate)     ! Adjust job time 
   IF (P==0) THEN                                                    ! If called from main prog       
      WRITE(LogU,*) ' Job Initialization Duration   =',             &
         JobTime, 'seconds'                                          ! Write initialization time
      WRITE(LogU,'(31X,A1,F8.2,A6)') '=',JobTime/3600.,' hours'      ! Write time in hours
      WRITE(LogU,*)                                                  ! Write blank line   
   ELSE                                                              ! Else called from Output  
      WRITE(LogU,*) ' Profile #     =', P                            ! Write profile number
      WRITE(LogU,*) ' Job Duration  =',JobTime, 'seconds'            ! Job duration
      WRITE(LogU,'(16X,A1,F8.2,A6)') '=',JobTime/3600.,' hours'      ! Write time in hours
      WRITE(LogU,*)                                                  ! Write blank line   
   ENDIF                                                             ! End profile logic
END SUBROUTINE WriteJobTime                                          ! Return to calling routine

END MODULE UtilityModule                                             ! End of UtilityModule code 